
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const Gateway_cxx_Version =
    "$Id: Gateway.cxx,v 1.47 2001/01/23 22:22:34 cktam Exp $";
#include <signal.h>
#include <VTime.hxx>
#include <sys/types.h>
#include <typeinfo>
#include <unistd.h>
#include "Gateway.h"
#include "Endpoint.h"
#include "NullHardware.h"
#include "parse3tuple.h"
#include "support.hxx"
#include "SystemInfo.hxx"

#ifdef __vxworks
#include <hostLib.h>
#include <resolvLib.h>
#include "SH3Hardware.h"
#else
#include "TrunkHardware.h"
#include "IPJackHardware.h"
#endif


/*********************************************************************
 *  Gateway Extras
 *
 *  Need to clean up but it works for now.
 *
 ********************************************************************/

// global configuration data
GatewayConfigData configData;

// global ptr to the gateway
Gateway* theGateway;

/** Callback function to start thread for hardware.
 */
void* hardwareCallback (void* objectPtr)
{
    Hardware* hardwarePtr = static_cast < Hardware * > (objectPtr);
    hardwarePtr->hardwareMain(NULL) ;
    return NULL;
} // end hardwareCallback()

/** Callback function to start thread for each endpoint.
 */
void* endpointCallback (void* objectPtr)
{
    Endpoint* endpointPtr = static_cast < Endpoint * > (objectPtr);
    endpointPtr->endpointMain(NULL);
    return NULL;
} // end endpointCallback()


/** Callback function to start thread for MGCP stack of agent.
 */
void* agentCallback (void* objectPtr)
{
    MgAgent* agentPtr = static_cast < MgAgent * > (objectPtr);
    agentPtr->threadProcess();
    return NULL;
} // end agentCallback()


/**  NOTE:  This is for internal use only!
 *   gwNum is used for the mta project.  
 *   This was used as a parameter for distinuishing the gateway 
 *   (gateway 1, gateway 2, etc.)  This parameter is set in main() 
 *   for the vxworks implementation by the boardNum.
 */
int gwNum = -1;


/**  NOTE:  This is for internal use only!
 *   boardNum is used for the mta project.   
 *   boardNum is used to determine which board is used.  See main()
 *   for more details.
 */
int boardNum = -1;


/**  NOTE:  This is for internal use only!
 *   callAgentHostName is used for the mta project.
 *   callAgentHostName is used at start up to store the host name
 *   of the call agent.  See main() for more details.  
 */
char* callAgentHostName;


/**  Constants for the device.
 */
const char* DEVICE_QUICKNET0 = "/dev/phone0";

/**  Constants for the device.
 */
const char* DEVICE_QUICKNET1 = "/dev/phone1";

/**  Constants for the device.
 */
const char* DEVICE_NULL = "\0";

/*********************************************************************
Gateway
*********************************************************************/

Gateway::Gateway (void)
        : cidMutex(), rtpMutex()
{
    int i;


    // configure gateway according to configuration data
    myPort = configData.gatewayPort;
    if (configData.firstRtpPort % 2)
    {
        currentRtpPort = configData.firstRtpPort + 1;
    }
    else
    {
        currentRtpPort = configData.firstRtpPort;
    }

    // set the global ptr to this gateway
    theGateway = this;


    // create MGCP domain name from standard hostname

#ifdef __vxworks
    char hostName[256];
    // for sh3 boards.
    // set hostname based on the input parameter boardNum.

    if (boardNum == 82)
    {
        memcpy (&hostName[0], "hitachi-x82.private.vovida.com", 33);
    }
    else if (boardNum == 83)
    {
        memcpy (&hostName[0], "hitachi-x83.private.vovida.com", 33);
    }
    else if (boardNum == 0)
    {
        memcpy (&hostName[0], "hitachi-x00.private.vovida.com", 33);
    }
    else if (boardNum == 1)
    {
        memcpy (&hostName[0], "hitachi-x01.private.vovida.com", 33);
    }


    cpLog(LOG_DEBUG, "my hostname = %s\n", hostName);

    // initialize the resolver library
    resolvInit("192.168.5.254", "private.vovida.com", 0);

#else
    string hostName = theSystem.gethostAddress();
    printf("my hostname = %s\n", hostName.c_str());
#endif


    myDomainName = hostName;

    // create agent
    myAgent = 0;
    setAgent(configData.agentHostname, configData.agentPort);
    assert(myAgent != 0);

    // create queues
    hardwareQ = new Fifo < HardwareMsg > ;
    assert(hardwareQ != NULL);
    for (i = 0; i < configData.numEndpoints; i++)
    {
        endpointQ[i] = new Fifo < EndpointMsg > ;
        assert(endpointQ[i] != NULL);
    }

    // create the hardware
    createHardware();

    // create the endpoints
    createEndpoints();
    // send restart in progress to agent.
    const string restart = "restart";

    for (i = 0; i < configData.numEndpoints; i++)
    {
        myEndpoint[i]->restartInProgress(restart);
    }

} 


Gateway::~Gateway (void)
{
    int numEndpoints = activeEndpoints;

    deleteAgent();
    deleteHardware();
    for (int i = 0; i < numEndpoints; i++)
    {
        deleteEndpoint(i);
    }
    delete [] myEndpointId;

}


void
Gateway::createHardware()
{
    int errorcode;

    // create hardware

#ifdef __vxworks

    if (configData.deviceName == "sh")
    {
        // NOTE:  for the 1st ARG, 0 = CH_1 (line 1)
        //                         1 = CH_2 (line 2)

        myHardware = new SH3Hardware(DEVICE_NULL, 2, hardwareQ, endpointQ);
    }

    myHardware->setEndpointIndex(configData.endpointId,
                                 configData.numEndpoints);


    errorcode = myHardwareThread.spawn(hardwareCallback,
                                       (void *) myHardware);
    assert(errorcode >= 0);

#else

    if (configData.deviceName == "ipjack")
    {
        myHardware = new IPJackHardware(DEVICE_QUICKNET0, hardwareQ, endpointQ[0]);
    }
    else if (configData.deviceName == "ipjack1")
    {

        myHardware = new IPJackHardware(DEVICE_QUICKNET1, hardwareQ, endpointQ[0]);
    }
    else if (configData.deviceName == "trunk")
    {
        myHardware = new TrunkHardware(DEVICE_QUICKNET0, 24, hardwareQ, endpointQ);
    }
    else
    {
        myHardware = new NullHardware(DEVICE_NULL, hardwareQ, endpointQ[0]);
    }
    assert(myHardware != NULL);

    myHardware->setEndpointIndex(configData.endpointId,
                                 configData.numEndpoints);

    errorcode = myHardwareThread.spawn(hardwareCallback,
                                       (void *) myHardware);
    assert(!errorcode);

#endif

}


void
Gateway::createEndpoints()
{
    int i;
    int errorcode;
    // create all endpoints, this MUST be done after creating hardware
    activeEndpoints = 0;
    for (i = 0; i < configData.numEndpoints; i++)
    {
        myEndpointId[i] = configData.endpointId[i];

        myEndpoint[i] = new Endpoint(myEndpointId[i] + "@" + myDomainName,
                                     configData.cntsPerEndpt,
                                     myAgent,
                                     *myHardware,
                                     endpointQ[i],
                                     hardwareQ);

        assert(myEndpoint[i] != NULL);

        errorcode = myEndpointThread[i].spawn(endpointCallback,
                                              (void *) myEndpoint[i]);

#ifdef __vxworks
        taskDelay(100);
        assert(errorcode >= 0);
#else
        assert(!errorcode);
#endif

        activeEndpoints++;
    }
}



MgAgent*
Gateway::setAgent (string agentHostname, int agentPort)
{
    int errorcode;

    MgAgent* newAgent = new MgAgent(agentHostname.c_str(), agentPort, myPort);

    if (newAgent == NULL)
    {
        cpLog(LOG_ERR, "Could not create new agent, keeping old agent\n");
    }
    else
    {
        // deleteAgent() should be call in the scenerio where the
        // gateway can change call agents on the fly.  this is not
        // supported.
 
        // deleteAgent();
        myAgent = 0;
        myAgent = newAgent;
    }

    // create thread for mgcp stack to communicate with agent
    errorcode = myAgentThread.spawn(agentCallback, (void *) myAgent );

#ifdef __vxworks
    assert(errorcode >= 0);
#else
    assert(!errorcode);
#endif 

    return newAgent;
}


int
Gateway::getEndpointIndex (string id)
{
    // residential and trunk gateways should not have to deal with
    // wildcards; the use of "*" for ALL and "$" for ANY is reserved
    // for media gateways

    // locate matching endpoint id from list
    for (int i = 0; i < activeEndpoints; i++)
    {
        //	if (id == myEndpointId[i])
        unsigned int j = id.find(myEndpointId[i]);
        if (j != string::npos)
        {
            return i;
        }
    }
    return GW_INV_ENDPOINT_INDEX;
} 


void
Gateway::forwardCommand (CMD cmd, MgCommandType type,
                         Fifo < EndpointMsg > * outputQ)
{
    EndpointMsg msg;
    EndpointAgentCommand command;

    command.type = type;
    command.cmd = cmd;

    msg.type = EndpointAgentCommandType;
    msg.command = command;
    outputQ->add(msg);
} 


void
Gateway::deleteAgent (void)
{
    myAgentThread.join();

    if (myAgent)
    {
        printf("deleting Agent now!\n");
        delete myAgent;
        myAgent = 0;
    }
}


void
Gateway::deleteHardware (void)
{
    myHardwareThread.join();
    if (myHardware)
    {
        delete myHardware;
        myHardware = 0;
    }

    // flush hardware queue, then delete it
    while (hardwareQ->size() != 0)
    {
        hardwareQ->getNext();
    }
    delete hardwareQ;
    hardwareQ = NULL;

}


void
Gateway::deleteEndpoint (int index)
{
    if (index < activeEndpoints)
    {
        myEndpointThread[index].join();
        if (myEndpoint[index])
        {
            delete myEndpoint[index];
            myEndpoint[index] = 0;
        }
        activeEndpoints--;
    }

    // flush endpoint queue, then delete it
    while (endpointQ[index]->size() != 0)
    {
        endpointQ[index]->getNext();
    }
    delete endpointQ[index];
    endpointQ[index] = NULL;

}


void
Gateway::gatewayMain (void)
{
    CMD cmd;
    MgCommandType cmdType;
    int endpointIndex = GW_INV_ENDPOINT_INDEX;

    while (1)
    {
        cmd = myAgent->receive();
        cmdType = cmd->getType();


        // if command is for a given endpoint, forward it
        switch (cmdType)
        {
            case MgEndpointConfigurationType:
            {
                EPCF epcf;
                epcf.dynamicCast(cmd);
                endpointIndex = getEndpointIndex(epcf->getEndpointId());
            }
            break;

            case MgCreateConnectionType:
            {
                CRCX crcx;
                crcx.dynamicCast(cmd);
                endpointIndex = getEndpointIndex(crcx->getEndpointId());
            }
            break;

            case MgModifyConnectionType:
            {
                MDCX mdcx;
                mdcx.dynamicCast(cmd);
                endpointIndex = getEndpointIndex(mdcx->getEndpointId());
            }
            break;

            case MgDeleteConnectionAgentType:
            {
                DLCX dlcx;
                dlcx.dynamicCast(cmd);
                endpointIndex = getEndpointIndex(dlcx->getEndpointId());
            }
            break;

            case MgNotificationRequestType:
            {
                RQNT rqnt;
                rqnt.dynamicCast(cmd);
                endpointIndex = getEndpointIndex(rqnt->getEndpointId());
            }
            break;

            case MgAuditEndpointType:
            {
                AUEP auep;
                auep.dynamicCast(cmd);
                endpointIndex = getEndpointIndex(auep->getEndpointId());
            }
            break;

            case MgAuditConnectionType:
            {
                AUCX aucx;
                aucx.dynamicCast(cmd);
                endpointIndex = getEndpointIndex(aucx->getEndpointId());
            }
            break;

            default:
            {
                cpLog(LOG_ERR, "Rcvd unknown cmd, type = %d",
                      cmdType);
            }
        } // end switch

        // send cmd to appropriate endpoint
        if (endpointIndex != GW_INV_ENDPOINT_INDEX)
        {
            forwardCommand(cmd, cmdType, endpointQ[endpointIndex]);
        }
        else
        {
            cpLog(LOG_DEBUG, "Gateway needes to handle command, type = %d",
                  cmdType);
        }

    } // end while

}



int
Gateway::newRtpPort (void)
{
    int newPort;

    // lock
    rtpMutex.lock();

    if ( currentRtpPort >= (configData.firstRtpPort + GW_NUM_RTP_PORTS) )
    {
        currentRtpPort = configData.firstRtpPort;
    }

    // intentionally increment after assigning
    newPort = currentRtpPort;
    currentRtpPort += 2;

    // unlock
    rtpMutex.unlock();

    return newPort;
} 


void
Gateway::deleteRtpPort (int port)
{
    // nothing needed for this simplfied scheme
}



string
Gateway::newConnectionId (void)
{
    // cycle through numbers and assume connection id will be free
    // before wrapping around
    static unsigned long connectionNumber = 1;
    string id;

    // lock
    cidMutex.lock();

    id = "c" + itos(connectionNumber++);

    // unlock
    cidMutex.unlock();

    return id;
} 


/*********************************************************************
Configuration File Handling
*********************************************************************/

/// Enumeration of valid tags in gateway configuration file
enum
{
    GatewayConfigTagHardware = 0,
    GatewayConfigTagGatewayPort,
    GatewayConfigTagAgentHostname,
    GatewayConfigTagAgentPort,
    GatewayConfigTagNumEndpoints,
    GatewayConfigTagEndpoint,
    GatewayConfigTagCntsPerEndpt,
    GatewayConfigTagFirstRtpPort,
    GatewayConfigTagMax,
    GatewayConfigInvalid
} GatewayConfigTag;

/// Valid tag strings for gateway configuration file
static char* gatewayConfigTagString[GatewayConfigTagMax] =
    {
        "GatewayHardware",
        "GatewayPort",
        "GatewayAgentHostname",
        "GatewayAgentPort",
        "GatewayNumEndpoints",
        "GatewayEndpoint",
        "GatewayCntsPerEndpt",
        "GatewayFirstRtpPort",
    };


/** Converts a tag string to an index, takes a string and returns the
 ** corresponding index, or GatewayConfigInvalid if matching tag is
 ** not found.
 */
int
gatewayConfigTag2i (char* tag)
{
    int i;

    // search until matching tag is found
    for (i = 0; i < GatewayConfigTagMax; i++)
    {
        if (strcmp(tag, gatewayConfigTagString[i]) == 0)
        {
            //cpLog (LOG_DEBUG, "Matched tag = %s", tag);
            return i;
        }
    }

    // no match found, report error
    cpLog (LOG_ERR, "Unknown tag = %s", tag);
    return GatewayConfigInvalid;

}


/** Initializes gateway configuration data based on valid tags
 ** extracted from configuration file. Takes a valid tag/type/value
 ** and sets the corresponding field in the configuration data.
 */
void
setGatewayConfigParameter (char* tag, char* type, char* value)
{
    static int index;

    switch (gatewayConfigTag2i(tag))
    {
        case GatewayConfigTagHardware:
        {
            configData.deviceName = value;
            //cpLog(LOG_DEBUG, "Device name = %s",
            //  configData.deviceName.c_str());
        }

        break;

        case GatewayConfigTagGatewayPort:
        {
            configData.gatewayPort = atoi(value);
            //cpLog(LOG_DEBUG, "Gateway port = %d", configData.gatewayPort);
        }

        break;

        case GatewayConfigTagAgentHostname:
        {
            configData.agentHostname = value;
            //cpLog(LOG_DEBUG, "Agent Hostname = %s",
            //  configData.agentHostname.c_str());
        }

        break;

        case GatewayConfigTagAgentPort:
        {
            configData.agentPort = atoi(value);
            //cpLog(LOG_DEBUG, "Agent port = %d", configData.agentPort);
        }

        break;

        case GatewayConfigTagNumEndpoints:
        {
            configData.numEndpoints = atoi(value);
            //cpLog(LOG_DEBUG, "Number of endpoints = %d",
            //  configData.numEndpoints);
        }

        break;

        case GatewayConfigTagEndpoint:
        {
            if (index < GW_ENDPOINT_MAX)
            {
                configData.endpointId[index] = value;
                //cpLog(LOG_DEBUG, "EndpointId[%d] = %s",
                //    index, configData.endpointId[index].c_str());
                index++;
            }
        }
        break;

        case GatewayConfigTagCntsPerEndpt:
        {
            configData.cntsPerEndpt = atoi(value);
            //cpLog(LOG_DEBUG, "Number of endpoints = %d",
            //  configData.numEndpoints);
        }

        break;

        case GatewayConfigTagFirstRtpPort:
        {
            configData.firstRtpPort = atoi(value);
            //cpLog(LOG_DEBUG, "Number of endpoints = %d",
            //  configData.numEndpoints);
        }

        break;

        default:
        {
            cpLog (LOG_ERR, "Invalid tag index");
        }
        break;
    } // end switch

}



void
showConfigData (void)
{
    printf("\n-------------------------------------------------");
    printf("\nGateway Configuration Data");
    printf("\nDevice Name:    %s", configData.deviceName.c_str());
    printf("\nGateway Port#:  %d", configData.gatewayPort);
    printf("\nAgent Hostname  %s", configData.agentHostname.c_str());
    printf("\nAgent Port#:    %d", configData.agentPort);
    printf("\nEndpoints:      %d", configData.numEndpoints);
    printf("\nCntsPerEndpt:   %d", configData.cntsPerEndpt);
    printf("\nRtp port range: %d - %d", configData.firstRtpPort,
           (configData.firstRtpPort + GW_NUM_RTP_PORTS - 1) );
    for (int i = 0; i < configData.numEndpoints; i++)
    {
        if (i < 10)
        {
            printf("\nEndpointId[%d]:  %s", i,
                   configData.endpointId[i].c_str());
        }
        else
        {
            printf("\nEndpointId[ %d]:  %s", i,
                   configData.endpointId[i].c_str());
        }
    }
    printf("\n-------------------------------------------------\n");

} 




void* gatewayCallback(void* objectPtr)
{

    // setup logfile
    cpLogSetLabel ("gw");
    cpLogSetPriority (LOG_DEBUG);


    // TBD: Get configuration data (tftp?) here.


    // Hardcode for testing.

    configData.deviceName = "sh";
    configData.gatewayPort = 5050 + gwNum;
    configData.agentHostname = callAgentHostName;
    // configData.agentPort         = 2427 + gwNum;
    configData.agentPort = 2427;
    configData.numEndpoints = 2;
    configData.endpointId[0] = "ep0";
    configData.endpointId[1] = "ep1";
    configData.cntsPerEndpt = 1;
    configData.firstRtpPort = 6050 + (gwNum * GW_NUM_RTP_PORTS);

    showConfigData();

    // create new gateway according to configuration data
    Gateway gateway;

    // startup gateway, should never return
    gateway.gatewayMain();

    return 0;
}




/*********************************************************************
main
*********************************************************************/


#ifdef __vxworks

// ********************************
// vxworks Main
// ********************************

int main(int board, char* callAgent)
{
    int errorcode = 0;
    VThread gwThread;

    if (board == 82)
    {
        gwNum = 1;
    }
    else if (board == 83)
    {
        gwNum = 0;
    }
    else if (board == 0)
    {
        gwNum = 2;
    }
    else if (board == 1)
    {
        gwNum = 3;
    }

    boardNum = board;

    callAgentHostName = callAgent;

    printf("gwNum = %d, boardNum = %d, callAgent = %s\n",
           gwNum,
           boardNum,
           callAgentHostName);

    errorcode = gwThread.spawn(gatewayCallback,
                               (void *) 0);
    gwThread.join();

    return errorcode;

};


#else

// ********************************
// Main
// ********************************

/** Reads configuration file into global data structure and launches
** the gateway
*/

int
main (int argc, char* argv[])
{
    string gwConfigFile;

    // setup logfile
    cpLogSetLabel ("gw");
    cpLogSetPriority (LOG_DEBUG);

    if (argc == 1)
    {
        // This is the default configuration file.
        gwConfigFile = "gw.cfg";
    }
    else if (argc == 2)
    {
        // Use what is specified by user
        gwConfigFile = argv[1];
    }
    else
    {
        // err.
        cerr << "Usage:  ./Gateway" << endl;
        cerr << "        ./Gateway [config file]" << endl;
        return 0;
    }

    // read configuration file
    printf("\n\n");
    cpLog(LOG_DEBUG, "Loading gateway configuration from: %s", gwConfigFile.c_str());

    if (parse3tuple (gwConfigFile.c_str(), setGatewayConfigParameter) == false)
    {
        cpLog(LOG_ERR, "Error parsing configuration file");
    }

    // show config data
    showConfigData();

    // create new gateway according to configuration data
    Gateway gateway;

    // startup gateway, should never return
    gateway.gatewayMain();

    return 0;
} 


#endif   // __vxworks
