
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const Endpoint_cxx_Version =
    "$Id: Endpoint.cxx,v 1.37 2001/01/18 02:45:15 liuhong Exp $";
#include "common.h"
#include <signal.h>
#include <sys/time.h>
#include <sys/types.h>
#include <typeinfo>
#include <unistd.h>
#include "MgAgent.hxx"
//#include "sdp.hxx"
#include "Connection.h"
#include "DigitMap.h"
#include "Hardware.h"
#include "Endpoint.h"
#include "Gateway.h"

extern Gateway* theGateway;
extern int numOfBadStates;
extern int numOfAttCalls;
extern int numOfRevCalls;
extern int numOfBusy;
extern int numOfRestart;
extern GatewayConfigData configData;


extern timeval RQNTTime[GW_ENDPOINT_MAX];
extern timeval CRCXTime[GW_ENDPOINT_MAX];
extern timeval MDCXTime[GW_ENDPOINT_MAX];
extern timeval DLCXTime[GW_ENDPOINT_MAX];

extern long RQNT_TO_CRCX[GW_ENDPOINT_MAX];
extern long CRCX_TO_MDCX[GW_ENDPOINT_MAX];
extern long MDCX_TO_DLCX[GW_ENDPOINT_MAX];
extern long DLCX_TO_RQNT[GW_ENDPOINT_MAX];

extern int numOfRQNT_TO_CRCX[GW_ENDPOINT_MAX];
extern int numOfCRCX_TO_MDCX[GW_ENDPOINT_MAX];
extern int numOfMDCX_TO_DLCX[GW_ENDPOINT_MAX];
extern int numOfDLCX_TO_RQNT[GW_ENDPOINT_MAX];

extern long max_RQNT_TO_CRCX[GW_ENDPOINT_MAX];

extern const string restart = "restart";

Endpoint::Endpoint ( string endpointId,
                     char* digitPtr,
                     string runmode,
                     int endpointIndex,
                     int numConnections,
                     MgAgent* agent,
                     const Hardware& hardware,
                     Fifo < EndpointMsg > * inputQ,
                     Fifo < HardwareMsg > * outputQ )
        : myId(endpointId)

{
    myAgent = agent;
    myQ = inputQ;
    myEndpointIndex = endpointIndex;
    hardwareQ = outputQ;
    testMarker = false;
    BAD_STATE = false;
    RECOVER_FLAG = false;
    BeginNewCall = true;
    myHangUp = false;

    if (runmode == "CallingGW")
    {
        cpLog (LOG_DEBUG, "Run Mode:  CallingGW");
        myTestType = CallingGW;
    }
    else if (runmode == "ReceiveGW")
    {
        cpLog (LOG_DEBUG, "Run Mode:  ReceiveGW");
        myTestType = ReceiveGW;
        testMarker = true;
        BeginNewCall = false;
    }


    for (int i = 0; i < TimeoutMaxTimer; i++)
    {
        myTimerEventId[i] = (FifoEventId) NULL;
    }

    myHookstate = HookstateOnhook;

    activeReqEvs = new list < ReqEv* > ;
    activeReqSigs = new list < ReqSig* > ;

    myLocalSD.setSessionName("Vovida Gateway");
    myLocalSD.setUserName("-");
    myLocalSD.setVersion(0);
    myLocalSD.setAddress(theGateway->getDomainName());

    // preallocate all connections
    myMaxConnections = numConnections;
    /*
        for (int j = 0; j < myMaxConnections; j++)
        {
            cpLog (LOG_DEBUG, "Creating Connection %d with endpointIndex = %d", 
                   j, myEndpointIndex);
    	myConnections[j] = new Connection(myAgent, hardwareQ, myEndpointIndex );
    	assert(myConnections[j] != NULL);
        }
    */
    myDigits = new DigitMap(digitPtr);

    // initialize myReqEv arrays
    initializeGenericEvents();
    initializeLineEvents();
    initializeDtmfEvents();

    // initialize myReqSig arrays
    initializeGenericSignals(hardware);
    initializeLineSignals(hardware);
    initializeDtmfSignals(hardware);

} // end Endpoint::Endpoint()


Endpoint::~Endpoint (void)
{
    int i;

    for (i = 0; i < GenericReqEvMaxEvent; i++)
    {
        delete myGenericReqEv[i];
        myGenericReqEv[i] = NULL;
    }
    for (i = 0; i < GenericReqSigMaxSignal; i++)
    {
        delete myGenericReqSig[i];
        myGenericReqSig[i] = NULL;
    }

    for (i = 0; i < LineReqEvMaxEvent; i++)
    {
        delete myLineReqEv[i];
        myLineReqEv[i] = NULL;
    }
    for (i = 0; i < LineReqSigMaxSignal; i++)
    {
        delete myLineReqSig[i];
        myLineReqSig[i] = NULL;
    }

    for (i = 0; i < DtmfReqEvMaxEvent; i++)
    {
        delete myDtmfReqEv[i];
        myDtmfReqEv[i] = NULL;
    }
    for (i = 0; i < DtmfReqSigMaxSignal; i++)
    {
        delete myDtmfReqSig[i];
        myDtmfReqSig[i] = NULL;
    }

    // individual Connections should already be deleted but may need
    // to double check here by iterating through list
    //delete myConnections;

} // end Endpoint::~Endpoint()



void*
Endpoint::endpointMain(void* parms)
{
    cerr << "BEGIN:  endpointMain" << endl;
    EndpointMsg msg;

    while (1)
    {

        cerr << "Endpoint::endpointMain" << endl;
        msg = myQ->getNext();
        // cerr << msg.type << endl;
        switch (msg.type)
        {
            case EndpointAgentCommandType:
            {
                cpLog (LOG_DEBUG, "%s <- Agent Cmd", myId.data());
                processAgentCommand(msg.command);
            }
            break;

            case EndpointHardwareEventType:
            {
                cpLog (LOG_DEBUG, "%s <- HW Event", myId.data());
                processHardwareEvent(msg.event);
            }
            break;

            case EndpointTimeoutType:
            {
                cpLog (LOG_DEBUG, "%s <- Timeout", myId.data());
                processTimeout(msg.timeout);
            }
            break;

            default:
            {
                cpLog (LOG_DEBUG, "%s <- Unknown Msg", myId.data());
            }
            break;
        } // end switch
    }

    cerr << "END:  endpointMain" << endl;
    return NULL;
}
;  // end Endpoint::EndpointMain()


void
Endpoint::processAgentCommand (const EndpointAgentCommand& cmd)
{
    const EndpointAgentCommand* command = &cmd;
    switch (command->type)
    {
        case MgEndpointConfigurationType:
        {
            EPCF epcf;
            epcf.dynamicCast(command->cmd);
            processEPCF( epcf );
        }
        break;

        case MgCreateConnectionType:
        {

            CRCX crcx;
            crcx.dynamicCast(command->cmd);
            processCRCX( crcx );

            stopTimer(TimeoutStuckState);
            startTimer(TimeoutStuckState, 20000);


            if (myTestType == CallingGW)
            {
                stopTimer(TimeoutBadState);

                if (RECOVER_FLAG == false)
                {
                    // Time statistics
                    gettimeofday(&CRCXTime[myEndpointIndex], NULL);

                    long temptime = tv_diff(CRCXTime[myEndpointIndex],
                                            RQNTTime[myEndpointIndex]);

                    RQNT_TO_CRCX[myEndpointIndex] += temptime;

                    if (max_RQNT_TO_CRCX[myEndpointIndex] < temptime)
                        max_RQNT_TO_CRCX[myEndpointIndex] = temptime;

                    numOfRQNT_TO_CRCX[myEndpointIndex] ++;

                }
                if (RECOVER_FLAG == true)
                {
                    cpLog(LOG_ERR, "ep %d:  ### Recovered!  CRCX received.  ###",
                          myEndpointIndex);

                    // Assume that we've fully recovered by now.
                    stopTimer(TimeoutRecover);
                    RECOVER_FLAG = false;
                }
            }
            else if (myTestType == ReceiveGW)
            {
                // If receiveGWHU is set, then the ReceiveGW is the one that
                // hangs up.

                if (configData.receiveGWHU == 1)
                {
                    myHangUp = true;
                }
            }

        }
        break;

        case MgModifyConnectionType:
        {
            if (myTestType == CallingGW)
            {
                if (RECOVER_FLAG == false)
                {
                    gettimeofday(&MDCXTime[myEndpointIndex], NULL);
                    CRCX_TO_MDCX[myEndpointIndex] +=
                        tv_diff(MDCXTime[myEndpointIndex],
                                CRCXTime[myEndpointIndex]);
                    numOfCRCX_TO_MDCX[myEndpointIndex] ++;
                };

                // If receiveGWHU is not set then the CallingGW is the one
                // that will hang up.

                if (configData.receiveGWHU == 0)
                {
                    stopTimer(TimeoutEndCall);
                    startTimer(TimeoutEndCall, configData.callDuration);
                }
            }

            MDCX mdcx;
            mdcx.dynamicCast(command->cmd);
            processMDCX( mdcx );

            stopTimer(TimeoutStuckState);
            startTimer(TimeoutStuckState, 20000);


        }
        break;

        case MgDeleteConnectionAgentType:
        {

            if (myTestType == CallingGW)
            {
                if (RECOVER_FLAG == false)
                {
                    gettimeofday(&DLCXTime[myEndpointIndex], NULL);

                    MDCX_TO_DLCX[myEndpointIndex] +=
                        tv_diff(DLCXTime[myEndpointIndex],
                                MDCXTime[myEndpointIndex]);

                    numOfMDCX_TO_DLCX[myEndpointIndex] ++;
                }

                // Got DLCX... set flag for starting the next call when we
                // get the RQNT.

                BeginNewCall = true;
            };

            DLCX dlcx;
            dlcx.dynamicCast(command->cmd);
            processDLCX( dlcx );

            // Hang up.  Doesn't matter who we are.

            if (myHookstate == HookstateOffhook)
            {
                processHardwareEvent(EventOnhook);
            }

            stopTimer(TimeoutStuckState);
            startTimer(TimeoutStuckState, 20000);

        }
        break;

        case MgNotificationRequestType:
        {
            RQNT rqnt;
            rqnt.dynamicCast(command->cmd);
            processRQNT( rqnt );


            if (myTestType == CallingGW)
            {
                // Did we get a digit map?

                if (!(rqnt->getDigitMap() == ""))
                {
                    // Got digit map.  Now check hookstate.

                    if (myHookstate == HookstateOffhook)
                    {
                        cpLog(LOG_DEBUG, "ep %d:  TimeoutSendDigits", myEndpointIndex);

                        if (RECOVER_FLAG == false)
                        {
                            gettimeofday(&RQNTTime[myEndpointIndex], NULL);
                            if (DLCXTime[myEndpointIndex].tv_sec != 0)
                            {
                                DLCX_TO_RQNT[myEndpointIndex] += tv_diff(RQNTTime[myEndpointIndex],
                                                                 DLCXTime[myEndpointIndex]);
                                numOfDLCX_TO_RQNT[myEndpointIndex]++;
                            };
                        }

                        if (RECOVER_FLAG == true)
                        {
                            cpLog(LOG_ERR, "ep %d:  ##### Recovering!  Send digits.  #####",
                                  myEndpointIndex);
                            stopTimer(TimeoutRecover);
                            startTimer(TimeoutRecover, 10000);
                        };

                        // Send down the digits.

                        myDigits->sendTestDigits(myAgent,
                                                 myDtmfReqEv,
                                                 myId);

                        // Start timer in case we don't get anything back
                        stopTimer(TimeoutBadState);
                        startTimer(TimeoutBadState, 10000);
                    }
                    else
                    {
                        cpLog(LOG_ERR, "ep %d:  NOT OFFHOOK!  CANNOT SEND DIGITS!", myEndpointIndex);
                    }
                }
                else if (BeginNewCall == true)
                {
                    // Didn't get a digit map.

                    // Making a call for the first time.
                    if (testMarker == false)
                    {
                        cpLog (LOG_DEBUG, "starting timer for the first time" );
                        stopTimer (TimeoutStartCall);
                        startTimer(TimeoutStartCall, (5000 + (20 *myEndpointIndex) ) );
                        testMarker = true;
                    }
                    else
                    {
                        if (RECOVER_FLAG == true)
                        {
                            cpLog(LOG_ERR, "ep %d:  ##### Recovering!  Making new call.  #####",
                                  myEndpointIndex);
                            stopTimer(TimeoutRecover);
                            startTimer(TimeoutRecover, 10000);
                        }

                        if (BAD_STATE == true)
                        {
                            cpLog(LOG_ERR, "ep %d:  ##### Making new call.  #####",
                                  myEndpointIndex);
                            BAD_STATE = false;
                        };

                        // Start a new call.
                        cpLog (LOG_DEBUG, "ep %d:  ***** Start a new call *****", myEndpointIndex);
                        stopTimer(TimeoutStartCall);
                        startTimer(TimeoutStartCall, configData.callDelay);

                        BeginNewCall = false;
                    };

                }
            }
            else if (myTestType == ReceiveGW)
            {
                if (myHangUp == true)
                {
                    // printf( "ep %d: start timer to end call\n", myEndpointIndex);
                    stopTimer(TimeoutEndCall);
                    startTimer(TimeoutEndCall, configData.callDuration);
                }
            }

            stopTimer(TimeoutStuckState);
            startTimer(TimeoutStuckState, 20000);

        }
        break;

        case MgAuditEndpointType:
        {
            AUEP auep;
            auep.dynamicCast(command->cmd);
            processAUEP( auep );
        }
        break;

        case MgAuditConnectionType:
        {
            AUCX aucx;
            aucx.dynamicCast(command->cmd);
            processAUCX( aucx );
        }
        break;

        default:
        {
            //	    cpLog(LOG_ERR, "Rcvd unknown cmd from gateway, type = %d",
            //		  command->type);
        }

        break;

    } // end switch

} // end Endpoint::processAgentCommand()



bool testDigitFlag = false;


void
Endpoint::processHardwareEvent (HardwareEvent event)
{
    ReqEv* reqEv = NULL;
    switch (event)
    {
        case EventOffhook:
        {
            cpLog(LOG_DEBUG, "ep %d:  Received offhook", myEndpointIndex);

            cpLog(LOG_DEBUG, "ep %d:  Hookstate -> offhook", myEndpointIndex);
            reqEv = myLineReqEv[LineReqEvOffhook];
            myHookstate = HookstateOffhook;



            if (configData.runmode == "CallingGW")
                numOfAttCalls++;
            else if (configData.runmode == "ReceiveGW")
                numOfRevCalls++;

        }
        break;

        case EventOnhook:
        {
            cpLog(LOG_DEBUG, "Received onhook");
            reqEv = myLineReqEv[LineReqEvOnhook];
            myHookstate = HookstateOnhook;

            cpLog(LOG_DEBUG, "ep %d:  Hanging up!", myEndpointIndex);

            if (myTestType == CallingGW)
            {
                stopTimer(TimeoutBadState);
            }
            else
            {
                // printf ("ep %d:  ReceiveGW got Onhook...\n", myEndpointIndex);
            }



            if (BAD_STATE == true)
            {
                // start the calls again.  Hope that someone likes us.
                // If not, then we will be in a bad state again.
                cpLog(LOG_DEBUG, "*********    BAD STATE!     BAD STATE!    *********");
                cpLog(LOG_DEBUG, "********* Try to make a call in 5 sec    *********");
                cpLog(LOG_DEBUG, "*********    BAD STATE!     BAD STATE!    *********");

                stopTimer(TimeoutStartCall);
                startTimer(TimeoutStartCall, 5000 + (10* myEndpointIndex));
            };

        }
        break;

        case EventFlashhook:
        {
            cpLog(LOG_DEBUG, "ep %d:  Received flashhook", myEndpointIndex);
            reqEv = myLineReqEv[LineReqEvFlashhook];
            myHookstate = HookstateOffhook;
            cpLog(LOG_DEBUG, "ep %d:  Hookstate -> offhook", myEndpointIndex);
        }
        break;

        default:
        {
            cpLog(LOG_ERR, "Rcvd unknown HW event = %d", event);
        }
        break;

    } // end switch



    if (reqEv != NULL)
    {
        switch (reqEv->getAction())
        {
            case ActionNotify:
            {
                if (myTestType == ReceiveGW && reqEv == myLineReqEv[LineReqEvOnhook])
                {
                    // printf ("ep %d:  Sending NTFY for Onhook\n", myEndpointIndex);
                }


                // create and send a Notify msg regarding the event

                cpLog(LOG_DEBUG, "ep %d:  Send Notify to agent", myEndpointIndex);

                MgNotify ntfy(myId, reqEv->getRequestId(),
                              reqEv->getObservedEvent());
                //                myAgent->send(ntfy);
                myAgent->asyncSend(ntfy);

                // TBD, process response
                //cerr << "Agent rspCode: " << ntfy.getResponseCode() << '\n';

                //if (!(rqnt->keepSignalsActive()))
                clearAllSignals();

                // TBD, for now, continue to process future events based
                // on previously received RQNT, but eventually need to
                // enhance to store events until next RQNT arrives, then
                // process them according to that list of ReqEv

            }

            break;

            case ActionAccumulate:
            {
                cpLog(LOG_DEBUG, "Accumulate");
            }
            break;

            case ActionDigitMap:
            {
                cpLog(LOG_DEBUG, "Accumulate according to digit map");

                myDigits->handledigits(myAgent,
                                       myDtmfReqEv,
                                       myId,
                                       event);

            }
            break;

            case ActionIgnore:
            {
                cpLog(LOG_DEBUG, "ActionIgnore");
            }
            break;

            default:
            {
                cpLog(LOG_DEBUG, "Invalid Action %d", reqEv->getAction());
            }
            break;
        }

    } // end reqEv
} // end Endpoint::processHardwareEvent()



void
Endpoint::processTimeout (EndpointTimeout timeout)
{

    ReqEv* reqEv = NULL;

    cpLog(LOG_DEBUG, "Processing timer %d for timeout event = %d", timeout, myTimerEventId[timeout]);
    myTimerEventId[timeout] = (FifoEventId) NULL;
    switch (timeout)
    {
        case TimeoutStartCall:
        {
            processHardwareEvent(EventOffhook);
        }
        break;

        case TimeoutEndCall:
        {
            // printf("ep %d: Got timer to end call\n", myEndpointIndex);

            processHardwareEvent(EventOnhook);
        }
        break;

        case TimeoutAnswerCall:
        {
            processHardwareEvent(EventOffhook);

        }
        break;

        case TimeoutStuckState:
        {

            cpLog(LOG_ERR, "*** ep%d:  Stuck...  Go onhook then recover with RSIP ***", myEndpointIndex);
            // restartInProgress(restart);

            numOfRestart++;

            if (myTestType == CallingGW)
            {
                processHardwareEvent(EventOnhook);

                RECOVER_FLAG = true;
                startTimer(TimeoutRecover, 20000);

                BeginNewCall = true;
            }
            else
            {
                restartInProgress(restart);
            }

        }
        break;


        case TimeoutRecover:
        {

            if (myTestType == CallingGW)
            {
                cpLog(LOG_ERR, "***  ep%d:  Recovery failed!  Send RSIP ***",
                      myEndpointIndex);

                BeginNewCall = true;
                RECOVER_FLAG = true;
                startTimer(TimeoutRecover, 100);
            }


            restartInProgress(restart);
            numOfRestart++;

        }
        break;

        case TimeoutBadState:
        {
            cpLog(LOG_ERR, "***  ep %d:  Sent digits failed!  hu, 5+ sec, make call  ***",
                  myEndpointIndex);

            processHardwareEvent(EventOnhook);
            BAD_STATE = true;

            if (myTestType == CallingGW)
            {
                BeginNewCall = true;
            }

            numOfBadStates++;

        }
        break;


        case TimeoutInterdigit:
        {
            cpLog(LOG_DEBUG, "TimeoutInterdigit");

            myDigits->handleDigitTimer(myAgent,
                                       myDtmfReqEv,
                                       myId);

        }
        break;

        case TimeoutFlashhook:
        {
            cpLog(LOG_DEBUG, "TimeoutFlashhook");
            if (myHookstate == HookstateFlashhook)
            {
                // treat previously received onhook as a real onhook
                cpLog(LOG_DEBUG, "Hookstate -> onhook");
                reqEv = myLineReqEv[LineReqEvOnhook];
                myHookstate = HookstateOnhook;
            }
            else
            {
                // ignore timeout because timer should have been stopped
                cpLog(LOG_DEBUG, "Ignore TimeoutFlashhook");
            }
        }
        break;

        default:
        {
            cpLog(LOG_ERR, "Rcvd unknown timeout = %d", timeout);
        }
        break;

    } // end switch

    if (reqEv != NULL)
    {
        if (reqEv->isRequested())
        {
            if (reqEv->getAction() == ActionNotify)
            {
                // create and send a Notify msg regarding the event
                cpLog(LOG_DEBUG, "Send Notify to agent");
                MgNotify ntfy(myId, reqEv->getRequestId(),
                              reqEv->getObservedEvent());
                //		myAgent->send(ntfy);
                myAgent->asyncSend(ntfy);

                // TBD, comment out until blocking API is working
                // check response
                // cerr << "Agent Response Code: " << ntfy.getResponseCode() << '\n';
                //if (!(rqnt->keepSignalsActive()))
                clearAllSignals();
            } // end if ActionNotify
            else
            {
                // TBD need to add handling for this case
                cpLog(LOG_ERR, "Event requested but not ActionNotify");
            }
        } // end if isRequested
    } // end reqEv
} // end Endpoint::processTimeout()



void
Endpoint::processEPCF (EPCF epcf)
{
    MgcpResponseCode responseCode;

    cpLog(LOG_DEBUG, "Endpoint::processEPCF");

    // TBD
    //bearerInfo = epcf->getBearerInformation();
    responseCode = TransactionExecuted;

    // respond to agent with result
    //epcf->setResponseCode(responseCode);
    //    myAgent->reply(epcf);
    myAgent->asyncReply(epcf);

} // end Endpoint::processEPCF()


void
Endpoint::processCRCX (CRCX crcx)
{
    MgcpResponseCode responseCode;

    responseCode = TransactionExecuted;
    crcx->setConnectionId("1");
    crcx->setLocalConnectionDescriptor(myLocalSD);

    // respond to agent with result
    crcx->setResponseCode(responseCode);
    //    myAgent->reply(crcx);
    myAgent->asyncReply(crcx);

} // end Endpoint::processCRCX


void
Endpoint::processMDCX (MDCX mdcx)
{
    MgcpResponseCode responseCode;

    responseCode = TransactionExecuted;

    mdcx->setResponseCode(responseCode);
    //    myAgent->reply(mdcx);
    myAgent->asyncReply(mdcx);

} // end Endpoint::processMDCX()


void
Endpoint::processDLCX (DLCX dlcx)
{
    MgcpResponseCode responseCode;

    cpLog(LOG_DEBUG, "Endpoint::processDLCX");

    // We only have one connection for testing
    // responseCode = myConnections[0]->close(dlcx);

    responseCode = ConnectionDeleted;

    // respond to agent with result
    dlcx->setResponseCode(responseCode);
    //    myAgent->reply(dlcx);
    myAgent->asyncReply(dlcx);

} // end Endpoint::processDLCX()


void
Endpoint::clearAllReqEvs (void)
{
    // clear any outstanding requested events
    while (!(activeReqEvs->empty()))
    {
        ( activeReqEvs->front() )->stop();
        activeReqEvs->pop_front();
    }
} // end Endpoint::clearAllReqEvs()


void
Endpoint::clearAllSignals (void)
{
    // clear any outstanding requested events
    while (!(activeReqSigs->empty()))
    {
        ( activeReqSigs->front() )->stop();
        activeReqSigs->pop_front();
    }
} // end Endpoint::clearAllSignals()



void
Endpoint::restartInProgress(const string & RestartMethod, const string& RestartDelay)
{
    //cerr<<  "Got restartInProgress" << endl;


    // Stop all timers!
    stopTimer (TimeoutRecover);
    stopTimer (TimeoutStartCall);
    stopTimer (TimeoutEndCall);
    stopTimer (TimeoutAnswerCall);
    stopTimer (TimeoutBadState);

    MgRestartInProgress myRestart(myId, RestartMethod, RestartDelay);

    //    myAgent->send (myRestart);
    myAgent->asyncSend(myRestart);
}



void
Endpoint::processRQNT (RQNT rqnt)
{
    string requestId = rqnt->getRequestIdentifier();
    int numRequests;
    int i;  // index to access requested events

    cpLog(LOG_DEBUG, "Endpoint::processRQNT");

    // clear any outstanding requested events
    clearAllReqEvs();

    // If the call agent sent down a digit map, use it.
    if (!(rqnt->getDigitMap() == ""))
    {
        myDigits->setDigitMap(rqnt->getDigitMap() );
    }

    // process new requested events
    numRequests = rqnt->getNumRequestedEvents();
    for (i = 0; i < numRequests; i++)
    {
        //addReqEv((rqnt->getItemRequestedEvents(i))->getEvent(), rqnt);
        addReqEv(requestId, rqnt->getItemRequestedEvents(i));
    }

    // clear any active signals
    clearAllSignals();

    // process new requested signals
    numRequests = rqnt->getNumSignalRequests();
    for (i = 0; i < numRequests; i++)
    {
        activateSignal((rqnt->getItemSignalRequests(i))->getEvent(), rqnt);
    }

    // reply with successful response code and string,
    rqnt->setResponseCode(ReasonTransactionExecuted);  // TBD MGCP API constant
    rqnt->setResponseString(OKResponse);
    //    myAgent->reply(rqnt);
    myAgent->asyncReply(rqnt);

} // end Endpoint::processRQNT()


void
Endpoint::processAUEP (AUEP auep)
{
    cpLog(LOG_DEBUG, "Endpoint::processAUEP");
}


void
Endpoint::processAUCX (AUCX aucx)
{
    cpLog(LOG_DEBUG, "Endpoint::processAUCX");
}


void
Endpoint::addReqEv (string requestId, Sptr < MgRequestedEvent > event)
{
    ReqEv* reqEv = NULL;

    cpLog(LOG_DEBUG, "Endpoint::addReqEv");
    switch (event->getPackageType())
    {
        case MgcpGenericMediaPackageType:
        {
            GenericReqEv index = getGenericReqEvIndex(
                                     (event->getEvent()).getGenericPackageType());

            if (index != GenericReqEvInvalidEvent)
            {
                reqEv = myGenericReqEv[index];
            }
        }
        break;

        case MgcpLinePackageType:
        {
            LineReqEv index = getLineReqEvIndex(
                                  (event->getEvent()).getLinePackageType());

            if (index != LineReqEvInvalidEvent)
            {
                reqEv = myLineReqEv[index];
            }
        }
        break;

        case MgcpDTMFPackageType:
        {
            DtmfReqEv index = getDtmfReqEvIndex(
                                  (event->getEvent()).getDTMFPackageType());

            if (index != DtmfReqEvInvalidEvent)
            {
                reqEv = myDtmfReqEv[index];
            }

            // check for special case of wildcard (aka any digit 0-9)
            if (index == DtmfReqEvAnyDigit)
            {
                // request all digits 0-9
                myDtmfReqEv[DtmfReqEv0]->start(requestId, event);
                activeReqEvs->push_back(myDtmfReqEv[DtmfReqEv0]);
                myDtmfReqEv[DtmfReqEv1]->start(requestId, event);
                activeReqEvs->push_back(myDtmfReqEv[DtmfReqEv1]);
                myDtmfReqEv[DtmfReqEv2]->start(requestId, event);
                activeReqEvs->push_back(myDtmfReqEv[DtmfReqEv2]);
                myDtmfReqEv[DtmfReqEv3]->start(requestId, event);
                activeReqEvs->push_back(myDtmfReqEv[DtmfReqEv3]);
                myDtmfReqEv[DtmfReqEv4]->start(requestId, event);
                activeReqEvs->push_back(myDtmfReqEv[DtmfReqEv4]);
                myDtmfReqEv[DtmfReqEv5]->start(requestId, event);
                activeReqEvs->push_back(myDtmfReqEv[DtmfReqEv5]);
                myDtmfReqEv[DtmfReqEv6]->start(requestId, event);
                activeReqEvs->push_back(myDtmfReqEv[DtmfReqEv6]);
                myDtmfReqEv[DtmfReqEv7]->start(requestId, event);
                activeReqEvs->push_back(myDtmfReqEv[DtmfReqEv7]);
                myDtmfReqEv[DtmfReqEv8]->start(requestId, event);
                activeReqEvs->push_back(myDtmfReqEv[DtmfReqEv8]);
                myDtmfReqEv[DtmfReqEv9]->start(requestId, event);
                activeReqEvs->push_back(myDtmfReqEv[DtmfReqEv9]);
                // Any Digit is 0-9 but need to add the hash to support VM.
                myDtmfReqEv[DtmfReqEvHash]->start(requestId, event);
                activeReqEvs->push_back(myDtmfReqEv[DtmfReqEvHash]);
            }
        }
        break;

        case MgcpMFPackageType:
        case MgcpTrunkPackageType:
        case MgcpHandsetPackageType:
        case MgcpRTPPackageType:
        case MgcpNetworkAccessServerPackageType:
        case MgcpAnnouncementPackageType:
        case MgcpScriptPackageType:
        case MgcpXVoiceMailPackageType:
        default:
        {
            cpLog(LOG_ERR, "Unsupported req event package = %d",
                  event->getPackageType());
        }
        break;
    } // end switch

    if (reqEv != NULL)
    {
        // mark event as requested and store associated paramters
        reqEv->start(requestId, event);

        // add to list of currently requested events
        activeReqEvs->push_back(reqEv);
    }

} // end Endpoint::addReqEv()


void
Endpoint::activateSignal (const MgcpGenericEvent& event,
                          const RQNT rqnt)
{
    ReqSig* signal = NULL;

    cpLog(LOG_DEBUG, "Endpoint::activateSignal");
    switch (event.getPackageType())
    {
        case MgcpGenericMediaPackageType:
        {
            GenericReqSig index =
                getGenericReqSigIndex(event.getGenericPackageType());

            cpLog (LOG_DEBUG, "GenericReqSig index = %d", index);

            if (index != GenericReqSigInvalidSignal)
            {
                signal = myGenericReqSig[index];
            }
        }
        break;

        case MgcpLinePackageType:
        {
            LineReqSig index = getLineReqSigIndex(event.getLinePackageType());


            // check to see if we are ringing.  if so, answer the call.

            if (index == LineReqSigRinging)
            {

                cpLog(LOG_DEBUG, "ep %d:  #####   Start ringing   #####", myEndpointIndex);

                stopTimer (TimeoutAnswerCall);
                startTimer(TimeoutAnswerCall, configData.answerDelay);

            }
            if (index == LineReqSigBusyTone)
            {
                // Got busy tone.  Hang up and try again.

                cpLog(LOG_ERR, "ep %d:  ##### Got Busy Tone #####", myEndpointIndex);

                numOfBusy++;
                BeginNewCall = true;
                RECOVER_FLAG = true;
                processHardwareEvent(EventOnhook);

                stopTimer (TimeoutRecover);
                startTimer(TimeoutRecover, 10000);
            }

            if (index == LineReqSigErrorTone)
            {
                cpLog(LOG_ERR, "ep %d:  ##### Got Error Tone Sig #####", myEndpointIndex);
                cpLog(LOG_ERR, "ep %d:  ##### Send RSIP #####", myEndpointIndex);

                stopTimer (TimeoutRecover);
                startTimer(TimeoutRecover, 1);
            }

            cpLog (LOG_DEBUG, "LineReqSig index = %d", index);


            if (index != LineReqSigInvalidSignal)
            {
                signal = myLineReqSig[index];
            }
        }
        break;

        case MgcpDTMFPackageType:
        {
            DtmfReqSig index = getDtmfReqSigIndex(event.getDTMFPackageType());

            cpLog (LOG_DEBUG, "DtmfReqSig index = %d", index);
            if (index != DtmfReqSigInvalidSignal)
            {
                signal = myDtmfReqSig[index];
            }
        }
        break;

        case MgcpMFPackageType:
        case MgcpTrunkPackageType:
        case MgcpHandsetPackageType:
        case MgcpRTPPackageType:
        case MgcpNetworkAccessServerPackageType:
        case MgcpAnnouncementPackageType:
        case MgcpScriptPackageType:
        case MgcpXVoiceMailPackageType:
        default:
        {
            cpLog(LOG_ERR, "Unsupported req signal package = %d",
                  event.getPackageType());
        }
        break;
    } // end switch

    if (signal != NULL)
    {
        // mark signal as active and configure hardware
        signal->start(rqnt);

        //signal->show();  // debug purposes only

        // add to list of active signals
        activeReqSigs->push_back(signal);
    }

} // end Endpoint::activateSignal()


GenericReqEv
Endpoint::getGenericReqEvIndex (MgcpGenericPackageItemType type)
{
    GenericReqEv index = GenericReqEvInvalidEvent;

    cpLog(LOG_DEBUG, "Endpoint::getGenericReqEvIndex");
    switch (type)
    {
        case MgcGenericPackageModemDetect:
        {
            index = GenericReqEvModemDetected;
        }
        break;

        case MgcGenericPackageFaxToneDetect:
        {
            index = GenericReqEvFaxToneDetected;
        }
        break;

        case MgcGenericPackageLongDurationConnection:
        {
            index = GenericReqEvLongDurationConnection;
        }
        break;

        case MgcGenericPackagePatternDetected:
        {
            index = GenericReqEvPatternDetected;
        }
        break;

        case MgcGenericPackageReportFailure:
        {
            index = GenericReqEvReportFailure;
        }
        break;

        default:
        {
            cpLog(LOG_ERR, "Unknown generic req event = %d", type);
        }
    } // end switch

    return index;

} // end Endpoint::getGenericReqEvIndex()


GenericReqSig
Endpoint::getGenericReqSigIndex (MgcpGenericPackageItemType type)
{
    GenericReqSig index = GenericReqSigInvalidSignal;

    cpLog(LOG_DEBUG, "Endpoint::getGenericSignalIndex");
    switch (type)
    {
        case MgcGenericPackagePatternDetected:
        {
            index = GenericReqSigPatternDetected;
        }
        break;

        case MgcGenericPackageRingBackTone:
        {
            index = GenericReqSigRingbackTone;
        }
        break;

        case MgcGenericPackageRingBackOnConnection:
        {
            index = GenericReqSigRingbackOnConnection;
        }
        break;

        case MgcGenericPackageConfirmTone:
        {
            index = GenericReqSigConfirmTone;
        }
        break;

        case MgcGenericPackageNetworkCongestionTone:
        {
            index = GenericReqSigNetworkCongestionTone;
        }
        break;

        case MgcGenericPackageInterceptTone:
        {
            index = GenericReqSigInterceptTone;
        }
        break;

        case MgcGenericPackagePreemtionTone:
        {
            index = GenericReqSigPreemptionTone;
        }
        break;

        default:
        {
            cpLog(LOG_ERR, "Unknown generic req signal = %d", type);
        }
    } // end switch

    return index;

} // end Endpoint::getGenericSignalIndex()


LineReqEv
Endpoint::getLineReqEvIndex (MgcpLinePackageItemType type)
{
    LineReqEv index = LineReqEvInvalidEvent;

    cpLog(LOG_DEBUG, "Endpoint::getLineReqEvIndex");
    switch (type)
    {
        case MgcLinePackageOffHookTransition:
        {
            index = LineReqEvOffhook;
        }
        break;

        case MgcLinePackageOnHookTransition:
        {
            index = LineReqEvOnhook;
        }
        break;

        case MgcLinePackageFlashHook:
        {
            index = LineReqEvFlashhook;
        }
        break;

        case MgcLinePackageAnswerTone:
        {
            index = LineReqEvAnswerTone;
        }
        break;

        case MgcLinePackageNetworkBusy:
        {
            index = LineReqEvNetworkBusy;
        }
        break;

        case MgcLinePackagePromptTone:
        {
            index = LineReqEvPromptTone;
        }
        break;

        case MgcLinePackageErrorTone:
        {
            index = LineReqEvErrorTone;
        }
        break;

        case MgcLinePackageReportOnCompletion:
        {
            index = LineReqEvRptOnComplete;
        }
        break;

        case MgcLinePackageDistinctiveTonePattern:
        {
            index = LineReqEvDsnctTonePattern;
        }
        break;

        case MgcLinePackageReportFailure:
        {
            index = LineReqEvFailureRpt;
        }
        break;

        default:
        {
            cpLog(LOG_ERR, "Unknown line req event = %d", type);
        }
    } // end switch

    return index;

} // end Endpoint::getLineReqEvIndex()


LineReqSig
Endpoint::getLineReqSigIndex (MgcpLinePackageItemType type)
{
    LineReqSig index = LineReqSigInvalidSignal;

    cpLog(LOG_DEBUG, "Endpoint::getLineReqSigIndex");
    switch (type)
    {
        case MgcLinePackageAdsiDisplay:
        {
            index = LineReqSigAdsiDisplay;
        }
        break;

        case MgcLinePackageVisualMessageWaitingIndicator:
        {
            index = LineReqSigVisualMsg;
        }
        break;

        case MgcLinePackageAnswerTone:
        {
            index = LineReqSigAnswerTone;
        }
        break;

        case MgcLinePackageBusyTone:
        {
            index = LineReqSigBusyTone;
        }
        break;

        case MgcLinePackageCallerId:
        {
            index = LineReqSigCallerId;
        }
        break;

        case MgcLinePackageCallWaitingTone:
        {
            index = LineReqSigCallWaitTone;
        }
        break;

        case MgcLinePackageDialTone:
        {
            index = LineReqSigDialTone;
        }
        break;

        case MgcLinePackageMessageWaitingInd:
        {
            index = LineReqSigMsgWaitInd;
        }
        break;

        case MgcLinePackageNetworkBusy:
        {
            index = LineReqSigNetworkBusy;
        }
        break;

        case MgcLinePackageRinging:
        {
            index = LineReqSigRinging;
        }
        break;

        case MgcLinePackageDistinctiveRinging:
        {
            // TBD need separate mgcp types for the others
            index = LineReqSigDsnctRing0;
        }
        break;

        case MgcLinePackageRingsplash:
        {
            index = LineReqSigRingSplash;
        }
        break;

        case MgcLinePackagePromptTone:
        {
            index = LineReqSigPromptTone;
        }
        break;

        case MgcLinePackageErrorTone:
        {
            index = LineReqSigErrorTone;
        }
        break;

        case MgcLinePackageStutterDialtone:
        {
            index = LineReqSigStutterTone;
        }
        break;

        case MgcLinePackageAlertingTone:
        {
            index = LineReqSigAlertTone;
        }
        break;

        case MgcLinePackageRecorderWarningTone:
        {
            index = LineReqSigRecordWarnTone;
        }
        break;

        case MgcLinePackageCallingCardServiceTone:
        {
            index = LineReqSigCallCardSrvTone;
        }
        break;

        case MgcLinePackageOffHookWarningTone:
        {
            index = LineReqSigOffhookWarnTone;
        }
        break;

        case MgcLinePackageDistinctiveTonePattern:
        {
            index = LineReqSigDsnctTonePattern;
        }
        break;

        default:
        {
            cpLog(LOG_ERR, "Unknown line req signal = %d", type);
        }
    } // end switch

    return index;

} // end Endpoint::getLineReqSigIndex()


DtmfReqEv
Endpoint::getDtmfReqEvIndex (MgcpDTMFPackageItemType type)
{
    DtmfReqEv index = DtmfReqEvInvalidEvent;

    cpLog(LOG_DEBUG, "Endpoint::getDtmfReqEvIndex");
    switch (type)
    {
        case MgcDTMFPackageDTMF_0:
        {
            index = DtmfReqEv0;
        }
        break;

        case MgcDTMFPackageDTMF_1:
        {
            index = DtmfReqEv1;
        }
        break;

        case MgcDTMFPackageDTMF_2:
        {
            index = DtmfReqEv2;
        }
        break;

        case MgcDTMFPackageDTMF_3:
        {
            index = DtmfReqEv3;
        }
        break;

        case MgcDTMFPackageDTMF_4:
        {
            index = DtmfReqEv4;
        }
        break;

        case MgcDTMFPackageDTMF_5:
        {
            index = DtmfReqEv5;
        }
        break;

        case MgcDTMFPackageDTMF_6:
        {
            index = DtmfReqEv6;
        }
        break;

        case MgcDTMFPackageDTMF_7:
        {
            index = DtmfReqEv7;
        }
        break;

        case MgcDTMFPackageDTMF_8:
        {
            index = DtmfReqEv8;
        }
        break;

        case MgcDTMFPackageDTMF_9:
        {
            index = DtmfReqEv9;
        }
        break;

        case MgcDTMFPackageDTMF_Hash:
        {
            index = DtmfReqEvHash;
        }
        break;

        case MgcDTMFPackageDTMF_Star:
        {
            index = DtmfReqEvStar;
        }
        break;

        case MgcDTMFPackageDTMF_A:
        {
            index = DtmfReqEvA;
        }
        break;

        case MgcDTMFPackageDTMF_B:
        {
            index = DtmfReqEvB;
        }
        break;

        case MgcDTMFPackageDTMF_C:
        {
            index = DtmfReqEvC;
        }
        break;

        case MgcDTMFPackageDTMF_D:
        {
            index = DtmfReqEvD;
        }
        break;

        case MgcDTMFPackageLongDurationIndicator:
        {
            index = DtmfReqEvLongDurInd;
        }
        break;

        case MgcDTMFPackageWildcardMatchAnyDigit_0_9:
        {
            index = DtmfReqEvAnyDigit;
        }
        break;

        case MgcDTMFPackageInterdigitTimer:
        {
            index = DtmfReqEvInterdigitTmr;
        }
        break;

        case MgcDTMFPackageReportFailure:
        {
            index = DtmfReqEvRptFailure;
        }
        break;

        default:
        {
            cpLog(LOG_ERR, "Unknown dtmf req event = %d", type);
        }
    } // end switch

    return index;

} // end Endpoint::getDtmfReqEvIndex()


DtmfReqSig
Endpoint::getDtmfReqSigIndex (MgcpDTMFPackageItemType type)
{
    DtmfReqSig index = DtmfReqSigInvalidSignal;

    cpLog(LOG_DEBUG, "Endpoint::getDtmfReqSigIndex");
    switch (type)
    {
        case MgcDTMFPackageDTMF_0:
        {
            index = DtmfReqSig0;
        }
        break;

        case MgcDTMFPackageDTMF_1:
        {
            index = DtmfReqSig1;
        }
        break;

        case MgcDTMFPackageDTMF_2:
        {
            index = DtmfReqSig2;
        }
        break;

        case MgcDTMFPackageDTMF_3:
        {
            index = DtmfReqSig3;
        }
        break;

        case MgcDTMFPackageDTMF_4:
        {
            index = DtmfReqSig4;
        }
        break;

        case MgcDTMFPackageDTMF_5:
        {
            index = DtmfReqSig5;
        }
        break;

        case MgcDTMFPackageDTMF_6:
        {
            index = DtmfReqSig6;
        }
        break;

        case MgcDTMFPackageDTMF_7:
        {
            index = DtmfReqSig7;
        }
        break;

        case MgcDTMFPackageDTMF_8:
        {
            index = DtmfReqSig8;
        }
        break;

        case MgcDTMFPackageDTMF_9:
        {
            index = DtmfReqSig9;
        }
        break;

        case MgcDTMFPackageDTMF_Hash:
        {
            index = DtmfReqSigHash;
        }
        break;

        case MgcDTMFPackageDTMF_Star:
        {
            index = DtmfReqSigStar;
        }
        break;

        case MgcDTMFPackageDTMF_A:
        {
            index = DtmfReqSigA;
        }
        break;

        case MgcDTMFPackageDTMF_B:
        {
            index = DtmfReqSigB;
        }
        break;

        case MgcDTMFPackageDTMF_C:
        {
            index = DtmfReqSigC;
        }
        break;

        case MgcDTMFPackageDTMF_D:
        {
            index = DtmfReqSigD;
        }
        break;

        default:
        {
            cpLog(LOG_ERR, "Unknown dtmf req event = %d", type);
        }
    } // end switch

    return index;

} // end Endpoint::getDtmfReqSigIndex()


void
Endpoint::initializeGenericEvents (void)
{
    // create all requested events
    myGenericReqEv[GenericReqEvModemDetected] =
        new ReqEv(MgGenericPackage::notifyModemDetect());
    myGenericReqEv[GenericReqEvFaxToneDetected] =
        new ReqEv(MgGenericPackage::notifyFaxToneDetect());
    myGenericReqEv[GenericReqEvLongDurationConnection] =
        new ReqEv(MgGenericPackage::notifyLongDurationConnection());
    // TBD, how do we handle initialization with unknown parameter?
    myGenericReqEv[GenericReqEvPatternDetected] =
        // TBD need to fake until API works for parameterized observed events
        //new ReqEv(MgGenericPackage::notifyPatternDetected(*pattern));
        new ReqEv(MgGenericPackage::notifyReportFailure());
    myGenericReqEv[GenericReqEvReportFailure] =
        new ReqEv(MgGenericPackage::notifyReportFailure());

    // check that all events were created successfully
    for (int i = 0 ; i < GenericReqEvMaxEvent; i++)
    {
        assert(myGenericReqEv[i] != NULL);
    }

} // end Endpoint::initializeGenericEvents()


void
Endpoint::initializeLineEvents (void)
{
    // initialize line signals
    myLineReqEv[LineReqEvOffhook] =
        new ReqEv(MgLinePackage::notifyOffHookTransition());
    myLineReqEv[LineReqEvOnhook] =
        new ReqEv(MgLinePackage::notifyOnHookTransition());
    myLineReqEv[LineReqEvFlashhook] =
        new ReqEv(MgLinePackage::notifyFlashHook());
    myLineReqEv[LineReqEvAnswerTone] =
        new ReqEv(MgLinePackage::notifyAnswerTone());
    myLineReqEv[LineReqEvNetworkBusy] =
        new ReqEv(MgLinePackage::notifyNetworkBusy());
    myLineReqEv[LineReqEvPromptTone] =
        new ReqEv(MgLinePackage::notifyPromptTone());
    myLineReqEv[LineReqEvErrorTone] =
        new ReqEv(MgLinePackage::notifyErrorTone());
    myLineReqEv[LineReqEvRptOnComplete] =
        new ReqEv(MgLinePackage::notifyReportOnCompletion());
    //myLineReqEv[LineReqEvDsnctTonePattern] =
    //new ReqEv(MgLinePackage::notifyDistinctiveTonePattern());
    myLineReqEv[LineReqEvFailureRpt] =
        new ReqEv(MgLinePackage::notifyReportFailure());

    // check that all events were created successfully
    for (int i = 0 ; i < LineReqEvMaxEvent; i++)
    {
        //assert(myLineReqEv[i] != NULL);
    }


} // end Endpoint::initializeLineEvents()


void
Endpoint::initializeDtmfEvents (void)
{
    // create all requested events
    myDtmfReqEv[DtmfReqEv0] = new ReqEv(MgDTMFPackage::notifyDTMF_0());
    myDtmfReqEv[DtmfReqEv1] = new ReqEv(MgDTMFPackage::notifyDTMF_1());
    myDtmfReqEv[DtmfReqEv2] = new ReqEv(MgDTMFPackage::notifyDTMF_2());
    myDtmfReqEv[DtmfReqEv3] = new ReqEv(MgDTMFPackage::notifyDTMF_3());
    myDtmfReqEv[DtmfReqEv4] = new ReqEv(MgDTMFPackage::notifyDTMF_4());
    myDtmfReqEv[DtmfReqEv5] = new ReqEv(MgDTMFPackage::notifyDTMF_5());
    myDtmfReqEv[DtmfReqEv6] = new ReqEv(MgDTMFPackage::notifyDTMF_6());
    myDtmfReqEv[DtmfReqEv7] = new ReqEv(MgDTMFPackage::notifyDTMF_7());
    myDtmfReqEv[DtmfReqEv8] = new ReqEv(MgDTMFPackage::notifyDTMF_8());
    myDtmfReqEv[DtmfReqEv9] = new ReqEv(MgDTMFPackage::notifyDTMF_9());
    myDtmfReqEv[DtmfReqEvHash] = new ReqEv(MgDTMFPackage::notifyDTMF_Hash());
    myDtmfReqEv[DtmfReqEvStar] = new ReqEv(MgDTMFPackage::notifyDTMF_Star());
    myDtmfReqEv[DtmfReqEvA] = new ReqEv(MgDTMFPackage::notifyDTMF_A());
    myDtmfReqEv[DtmfReqEvB] = new ReqEv(MgDTMFPackage::notifyDTMF_B());
    myDtmfReqEv[DtmfReqEvC] = new ReqEv(MgDTMFPackage::notifyDTMF_C());
    myDtmfReqEv[DtmfReqEvD] = new ReqEv(MgDTMFPackage::notifyDTMF_D());
    myDtmfReqEv[DtmfReqEvLongDurInd] =
        new ReqEv(MgDTMFPackage::notifyLongDurationIndicator());
    myDtmfReqEv[DtmfReqEvAnyDigit] =
        new ReqEv(MgDTMFPackage::notifyWildcardMatch());
    myDtmfReqEv[DtmfReqEvInterdigitTmr] =
        new ReqEv(MgDTMFPackage::notifyInterdigitTimer());
    myDtmfReqEv[DtmfReqEvRptFailure] =
        new ReqEv(MgDTMFPackage::notifyReportFailure());

    // check that all events were created successfully
    for (int i = 0 ; i < DtmfReqEvMaxEvent; i++)
    {
        assert(myDtmfReqEv[i] != NULL);
    }

} // end Endpoint::initializeDtmfEvents()


void
Endpoint::initializeGenericSignals (const Hardware& hw)
{
    // initialize signals
    myGenericReqSig[GenericReqSigPatternDetected] =
        new ReqSig(hw, MgcpGenericMediaPackageType,
                   GenericReqSigPatternDetected, myEndpointIndex, SignalOnOff);
    myGenericReqSig[GenericReqSigRingbackTone] =
        new ReqSig(hw, MgcpGenericMediaPackageType,
                   GenericReqSigRingbackTone, myEndpointIndex, SignalTimeout);
    myGenericReqSig[GenericReqSigRingbackOnConnection] =
        new ReqSig(hw, MgcpGenericMediaPackageType,
                   GenericReqSigRingbackOnConnection, myEndpointIndex, SignalTimeout, 180);
    myGenericReqSig[GenericReqSigConfirmTone] =
        new ReqSig(hw, MgcpGenericMediaPackageType,
                   GenericReqSigConfirmTone, myEndpointIndex, SignalBrief);
    myGenericReqSig[GenericReqSigNetworkCongestionTone] =
        new ReqSig(hw, MgcpGenericMediaPackageType,
                   GenericReqSigNetworkCongestionTone, myEndpointIndex, SignalTimeout);
    myGenericReqSig[GenericReqSigInterceptTone] =
        new ReqSig(hw, MgcpGenericMediaPackageType,
                   GenericReqSigInterceptTone, myEndpointIndex, SignalOnOff);
    myGenericReqSig[GenericReqSigPreemptionTone] =
        new ReqSig(hw, MgcpGenericMediaPackageType,
                   GenericReqSigPreemptionTone, myEndpointIndex, SignalOnOff);

    // check that all events were created successfully
    for (int i = 0 ; i < GenericReqSigMaxSignal; i++)
    {
        assert(myGenericReqSig[i] != NULL);
    }

} // end Endpoint::initializeGenericSignals()


void
Endpoint::initializeLineSignals (const Hardware& hw)
{
    // initialize signals
    myLineReqSig[LineReqSigAdsiDisplay] =
        new ReqSig(hw, MgcpLinePackageType,
                   LineReqSigAdsiDisplay, myEndpointIndex, SignalBrief);
    myLineReqSig[LineReqSigVisualMsg] =
        new ReqSig(hw, MgcpLinePackageType,
                   LineReqSigVisualMsg, myEndpointIndex, SignalTimeout);
    myLineReqSig[LineReqSigAnswerTone] =
        new ReqSig(hw, MgcpLinePackageType,
                   LineReqSigAnswerTone, myEndpointIndex, SignalOnOff);
    myLineReqSig[LineReqSigBusyTone] =
        new ReqSig(hw, MgcpLinePackageType,
                   LineReqSigBusyTone, myEndpointIndex, SignalTimeout, 30);
    myLineReqSig[LineReqSigCallerId] =
        new ReqSig(hw, MgcpLinePackageType,
                   LineReqSigCallerId, myEndpointIndex, SignalBrief);
    myLineReqSig[LineReqSigCallWaitTone] =
        new ReqSig(hw, MgcpLinePackageType,
                   LineReqSigCallWaitTone, myEndpointIndex, SignalTimeout, 30);
    myLineReqSig[LineReqSigDialTone] =
        new ReqSig(hw, MgcpLinePackageType,
                   LineReqSigDialTone, myEndpointIndex, SignalTimeout, 16);
    myLineReqSig[LineReqSigMsgWaitInd] =
        new ReqSig(hw, MgcpLinePackageType,
                   LineReqSigMsgWaitInd, myEndpointIndex, SignalTimeout, 16);
    myLineReqSig[LineReqSigNetworkBusy] =
        new ReqSig(hw, MgcpLinePackageType,
                   LineReqSigNetworkBusy, myEndpointIndex, SignalOnOff);
    myLineReqSig[LineReqSigRinging] =
        new ReqSig(hw, MgcpLinePackageType,
                   LineReqSigRinging, myEndpointIndex, SignalTimeout, 180);
    myLineReqSig[LineReqSigDsnctRing0] =
        new ReqSig(hw, MgcpLinePackageType,
                   LineReqSigDsnctRing0, myEndpointIndex, SignalTimeout, 180);
    myLineReqSig[LineReqSigDsnctRing1] =
        new ReqSig(hw, MgcpLinePackageType,
                   LineReqSigDsnctRing1, myEndpointIndex, SignalTimeout, 180);
    myLineReqSig[LineReqSigDsnctRing2] =
        new ReqSig(hw, MgcpLinePackageType,
                   LineReqSigDsnctRing2, myEndpointIndex, SignalTimeout, 180);
    myLineReqSig[LineReqSigDsnctRing3] =
        new ReqSig(hw, MgcpLinePackageType,
                   LineReqSigDsnctRing3, myEndpointIndex, SignalTimeout, 180);
    myLineReqSig[LineReqSigDsnctRing4] =
        new ReqSig(hw, MgcpLinePackageType,
                   LineReqSigDsnctRing4, myEndpointIndex, SignalTimeout, 180);
    myLineReqSig[LineReqSigDsnctRing5] =
        new ReqSig(hw, MgcpLinePackageType,
                   LineReqSigDsnctRing5, myEndpointIndex, SignalTimeout, 180);
    myLineReqSig[LineReqSigDsnctRing6] =
        new ReqSig(hw, MgcpLinePackageType,
                   LineReqSigDsnctRing6, myEndpointIndex, SignalTimeout, 180);
    myLineReqSig[LineReqSigDsnctRing7] =
        new ReqSig(hw, MgcpLinePackageType,
                   LineReqSigDsnctRing7, myEndpointIndex, SignalTimeout, 180);
    myLineReqSig[LineReqSigRingSplash] =
        new ReqSig(hw, MgcpLinePackageType,
                   LineReqSigRingSplash, myEndpointIndex, SignalBrief);
    myLineReqSig[LineReqSigPromptTone] =
        new ReqSig(hw, MgcpLinePackageType,
                   LineReqSigPromptTone, myEndpointIndex, SignalBrief);
    myLineReqSig[LineReqSigErrorTone] =
        new ReqSig(hw, MgcpLinePackageType,
                   LineReqSigErrorTone, myEndpointIndex, SignalBrief);
    myLineReqSig[LineReqSigStutterTone] =
        new ReqSig(hw, MgcpLinePackageType,
                   LineReqSigStutterTone, myEndpointIndex, SignalTimeout, 16);
    myLineReqSig[LineReqSigAlertTone] =
        new ReqSig(hw, MgcpLinePackageType,
                   LineReqSigAlertTone, myEndpointIndex, SignalOnOff);
    myLineReqSig[LineReqSigRecordWarnTone] =
        new ReqSig(hw, MgcpLinePackageType,
                   LineReqSigRecordWarnTone, myEndpointIndex, SignalOnOff);
    myLineReqSig[LineReqSigCallCardSrvTone] =
        new ReqSig(hw, MgcpLinePackageType,
                   LineReqSigCallCardSrvTone, myEndpointIndex, SignalOnOff);
    myLineReqSig[LineReqSigOffhookWarnTone] =
        new ReqSig(hw, MgcpLinePackageType,
                   LineReqSigOffhookWarnTone, myEndpointIndex, SignalTimeout, 180);
    myLineReqSig[LineReqSigDsnctTonePattern] =
        new ReqSig(hw, MgcpLinePackageType,
                   LineReqSigDsnctTonePattern, myEndpointIndex, SignalBrief);
    // check that all events were created successfully
    for (int i = 0 ; i < LineReqSigMaxSignal; i++)
    {
        assert(myLineReqSig[i] != NULL);
    }

} // end Endpoint::initializeLineSignals()


void
Endpoint::initializeDtmfSignals (const Hardware& hw)
{
    // initialize signals
    myDtmfReqSig[DtmfReqSig0] = new ReqSig(hw, MgcpDTMFPackageType,
                                           DtmfReqSig0, myEndpointIndex, SignalBrief);
    myDtmfReqSig[DtmfReqSig1] = new ReqSig(hw, MgcpDTMFPackageType,
                                           DtmfReqSig1, myEndpointIndex, SignalBrief);
    myDtmfReqSig[DtmfReqSig2] = new ReqSig(hw, MgcpDTMFPackageType,
                                           DtmfReqSig2, myEndpointIndex, SignalBrief);
    myDtmfReqSig[DtmfReqSig3] = new ReqSig(hw, MgcpDTMFPackageType,
                                           DtmfReqSig3, myEndpointIndex, SignalBrief);
    myDtmfReqSig[DtmfReqSig4] = new ReqSig(hw, MgcpDTMFPackageType,
                                           DtmfReqSig4, myEndpointIndex, SignalBrief);
    myDtmfReqSig[DtmfReqSig5] = new ReqSig(hw, MgcpDTMFPackageType,
                                           DtmfReqSig5, myEndpointIndex, SignalBrief);
    myDtmfReqSig[DtmfReqSig6] = new ReqSig(hw, MgcpDTMFPackageType,
                                           DtmfReqSig6, myEndpointIndex, SignalBrief);
    myDtmfReqSig[DtmfReqSig7] = new ReqSig(hw, MgcpDTMFPackageType,
                                           DtmfReqSig7, myEndpointIndex, SignalBrief);
    myDtmfReqSig[DtmfReqSig8] = new ReqSig(hw, MgcpDTMFPackageType,
                                           DtmfReqSig8, myEndpointIndex, SignalBrief);
    myDtmfReqSig[DtmfReqSig9] = new ReqSig(hw, MgcpDTMFPackageType,
                                           DtmfReqSig9, myEndpointIndex, SignalBrief);
    myDtmfReqSig[DtmfReqSigHash] = new ReqSig(hw, MgcpDTMFPackageType,
                                   DtmfReqSigHash, myEndpointIndex, SignalBrief);
    myDtmfReqSig[DtmfReqSigStar] = new ReqSig(hw, MgcpDTMFPackageType,
                                   DtmfReqSigStar, myEndpointIndex, SignalBrief);
    myDtmfReqSig[DtmfReqSigA] = new ReqSig(hw, MgcpDTMFPackageType,
                                           DtmfReqSigA, myEndpointIndex, SignalBrief);
    myDtmfReqSig[DtmfReqSigB] = new ReqSig(hw, MgcpDTMFPackageType,
                                           DtmfReqSigB, myEndpointIndex, SignalBrief);
    myDtmfReqSig[DtmfReqSigC] = new ReqSig(hw, MgcpDTMFPackageType,
                                           DtmfReqSigC, myEndpointIndex, SignalBrief);
    myDtmfReqSig[DtmfReqSigD] = new ReqSig(hw, MgcpDTMFPackageType,
                                           DtmfReqSigD, myEndpointIndex, SignalBrief);

    // check that all events were created successfully
    for (int i = 0 ; i < DtmfReqSigMaxSignal; i++)
    {
        assert(myDtmfReqSig[i] != NULL);
    }

} // end Endpoint::initializeDtmfSignals()


void
Endpoint::startTimer (EndpointTimeout timer, int msec)
{
    EndpointMsg msg;

    // stop timer if already running
    if (myTimerEventId[timer])
    {
        stopTimer(timer);
    }

    // send message to start timer
    msg.type = EndpointTimeoutType;
    msg.timeout = timer;
    msg.endpointIndex = myEndpointIndex;
    myTimerEventId[timer] = myQ->addDelayMs(msg, msec);
    cpLog(LOG_DEBUG, "Start timer %d for %d ms as event %d",
          timer, msec, myTimerEventId[timer] );
} // end Endpoint::startTimer()


void
Endpoint::stopTimer (EndpointTimeout timer)
{
    // stop timer if running
    if (myTimerEventId[timer] != (FifoEventId) NULL)
    {
        cpLog(LOG_DEBUG, "Stop timer %d as event %d",
              timer, myTimerEventId[timer] );
        myQ->cancel( myTimerEventId[timer] );
        myTimerEventId[timer] = (FifoEventId) NULL;
    }
} // end Endpoint::stopTimer()
