#ifndef REQ_EV_H_
#define REQ_EV_H_

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const ReqEv_h_Version =
    "$Id: ReqEv.h,v 1.15 2001/01/18 02:45:27 liuhong Exp $";

#include "MgCommand.hxx"
#include "MgcEvent.hxx"

/** Action to be taken if ReqestedEvent occurs. Note, the actions swap
 ** audio, keep signals active, and embedded notification request are
 ** handled independently.
 */
enum ReqEvAction
{
    ActionNotify,
    ActionAccumulate,
    ActionDigitMap,
    ActionIgnore,
};

/** Generic Media Package requested events enumeration.  Provides
 ** indices with which to access array of Generic Media Package
 ** requested events. An entry exists for each event the call agent
 ** can ask the gateway to detect.
  */
enum GenericReqEv
{
    GenericReqEvModemDetected,
    GenericReqEvFaxToneDetected,
    GenericReqEvLongDurationConnection,
    GenericReqEvPatternDetected,
    GenericReqEvReportFailure,
    GenericReqEvMaxEvent,
    GenericReqEvInvalidEvent
};


/** Line Package requested events enumeration.  Provides indices with
 ** which to access array of Line Package requested events. An entry
 ** exists for each event the call agent can ask the gateway to
 ** detect.
  */
enum LineReqEv
{
    LineReqEvOffhook,
    LineReqEvOnhook,
    LineReqEvFlashhook,
    LineReqEvAnswerTone,
    LineReqEvNetworkBusy,
    LineReqEvPromptTone,
    LineReqEvErrorTone,
    LineReqEvRptOnComplete,
    LineReqEvDsnctTonePattern,
    LineReqEvFailureRpt,
    LineReqEvMaxEvent,
    LineReqEvInvalidEvent
};

/** DTMF Package requested events enumeration.  Provides indices with
 ** which to access array of DTMF Package requested events. An entry
 ** exists for each event the call agent can ask the gateway to
 ** detect.
 */
enum DtmfReqEv
{
    DtmfReqEv0,
    DtmfReqEv1,
    DtmfReqEv2,
    DtmfReqEv3,
    DtmfReqEv4,
    DtmfReqEv5,
    DtmfReqEv6,
    DtmfReqEv7,
    DtmfReqEv8,
    DtmfReqEv9,
    DtmfReqEvHash,
    DtmfReqEvStar,
    DtmfReqEvA,
    DtmfReqEvB,
    DtmfReqEvC,
    DtmfReqEvD,
    DtmfReqEvLongDurInd,
    DtmfReqEvAnyDigit,
    DtmfReqEvInterdigitTmr,
    DtmfReqEvRptFailure,
    DtmfReqEvMaxEvent,
    DtmfReqEvInvalidEvent
};


/** Class of used to store and track a call agent requested event
 */
class ReqEv
{
    public:
        /** Creates a ReqEv and initializes all fields
         */
        ReqEv (MgNotificationEvent event);

        /** Destructor
         */
        ~ReqEv (void);

        /** Marks the event as requested and sets the event fields
         ** according to the supplied parameters.
         */
        void
        start (const string& requestId, Sptr < MgRequestedEvent > event);

        /** Marks the events as NOT requested
         */
        void
        stop (void);

        /** For debugging, dump all fields of requested event to standard out
         */
        void
        show (void);

        /** Acceess routines for private data
         */
        inline bool
        isRequested (void)
        {
            return requested;
        };

        inline string
        getRequestId (void)
        {
            return myRequestId;
        };

        inline const MgNotificationEvent&
        getObservedEvent (void)
        {
            return observedEvent;
        };

        inline ReqEvAction
        getAction (void)
        {
            return action;
        };

    private:
        /// true if currently being requested by call agent
        bool requested;

        /// RequestIdentifier supplied by call agent
        string myRequestId;

        /// call agent or as specified in RQNT
        string notifiedEntity;

        /// observedEvent to be sent to agent if requestedEvent is observed
        MgNotificationEvent observedEvent;

        /// true if to swap audion if event occurs
        bool swapAudio;

        /// true if to keep signals active if event occurs
        bool keepSignalsActive;

        /// true if to process embedded request if event occurs
        bool embeddedRequest;

        /// action to be taken if event occurs
        ReqEvAction action;
};

#endif // REQ_EV_H_
