
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const gateway_asend_cxx_Version =
    "$Id: gateway.asend.cxx,v 1.9 2001/02/27 05:59:23 icahoon Exp $";
#include <stdio.h>
#include <signal.h>
#include <sys/time.h>
#include <sys/types.h>
#include <typeinfo>
#include <unistd.h>
#include "Gateway.h"
#include "Endpoint.h"
#include "NullHardware.h"
#include "TrunkHardware.h"
#include "TestHardware.h"
#include "IPJackHardware.h"
// #include "vmHardware.h"
#include "parse3tuple.h"
#include "support.hxx"
#include "VCondition.h"

// global benchmark testing data
int numOfRevCalls;
int numOfAttCalls;
time_t startTime;
time_t endTime;

int numOfBadStates;

/*********************************************************************
 
Gateway Extras
 
*********************************************************************/

// global configuration data
GatewayConfigData configData;

// global ptr to the gateway
Gateway* theGateway;

/** Callback function to start pthread for hardware.
 */
void* hardwareCallback (void* objectPtr)
{
    Gateway* object = static_cast < Gateway* > (objectPtr);

    object->handleHardware(NULL);
    return NULL;
} // end hardwareCallback()

/** Callback function to start pthread for each endpoint.
 */
void* endpointCallback (void* objectPtr)
{
    Gateway* object = static_cast < Gateway* > (objectPtr);

    object->handleEndpoint(NULL);
    return NULL;
} // end endpointCallback()


/** Callback function to start pthread for MGCP stack of agent.
 */
void* agentCallback (void* objectPtr)
{
    MgAgent* object = static_cast < MgAgent* > (objectPtr);

    // object->threadProcess();
    object->asyncProcess();
    return NULL;
} // end agentCallback()



/*********************************************************************
 
Gateway
 
*********************************************************************/

Gateway::Gateway (void)
{
    int errorcode;
    int i;

    numOfAttCalls = 0;
    numOfRevCalls = 0;
    numOfBadStates = 0;

    // configure gateway according to configuration data
    myPort = configData.gatewayPort;
    if (configData.firstRtpPort % 2)
    {
        currentRtpPort = configData.firstRtpPort + 1;
    }
    else
    {
        currentRtpPort = configData.firstRtpPort;
    }

    // set the global ptr to this gateway
    theGateway = this;

    // create mutexs
    errorcode = pthread_mutex_init(&cid_mutex, NULL);
    assert(!errorcode);
    errorcode = pthread_mutex_init(&rtp_mutex, NULL);
    assert(!errorcode);

    // create MGCP domain name from standard hostname
    char hostName[256];
    gethostname(hostName, 256);
    myDomainName = hostName;

    // create agent
    myAgent = NULL;
    setAgent(configData.agentHostname, configData.agentPort);
    assert(myAgent != NULL);

    // create queues
    hardwareQ = new Fifo < HardwareMsg > ;
    assert(hardwareQ != NULL);


    // use only one endpointQ to handle all the endpoints.
    endpointQ[0] = new Fifo < EndpointMsg > ;
    assert(endpointQ[0] != NULL);


    // create hardware
    if (configData.deviceName == "ipjack")
    {
        myHardware[0] = new IPJackHardware("/dev/ixj0", hardwareQ, endpointQ[0]);
    }
    else if (configData.deviceName == "ipjack1")
    {
        myHardware[0] = new IPJackHardware("/dev/ixj1", hardwareQ, endpointQ[0]);
    }
    else if (configData.deviceName == "trunk")
    {
        myHardware[0] = new TrunkHardware("/dev/ixj0", 24, hardwareQ, endpointQ);
    }
    else if (configData.deviceName == "test")
    {

        // Create a hardware object for each endpoint.

        for (i = 0; i < configData.numEndpoints; i++)
        {
            //            dialNumber = getDialNumber(i, configData.dialNumber, digitArray[i]);

            cpLog(LOG_DEBUG, "Creating Hardware: %d", i);

            myHardware[i] = new TestHardware("/dev/ixj0",
                                             i,
                                             hardwareQ,
                                             endpointQ[0],
                                             configData.runmode,
                                             configData.dialNumber,
                                             configData.callDuration,
                                             configData.callDelay,
                                             configData.answerDelay);

        };

        // Set the rest of them to NULL

        while (i < GW_ENDPOINT_MAX)
        {
            myHardware[i] = NULL;
            i++;
        }
    }
    else
    {
        myHardware[0] = new NullHardware("/dev/ixj0", hardwareQ, endpointQ[0]);
    }
    assert(myHardware[0] != NULL);

    errorcode = pthread_create(&myHardwareThread, NULL,
                               hardwareCallback, (void *) theGateway );
    assert(!errorcode);


    // create all endpoints, this MUST be done after creating hardware
    activeEndpoints = 0;
    for (i = 0; i < configData.numEndpoints; i++)
    {
        digitString[i] = digitArray[i];

        dialNumber[i] = getDialNumber(i, configData.dialNumber, digitString[i]);

        cpLog(LOG_DEBUG, "Creating Endpoint: %d, dial num = %s", i, dialNumber[i]);

        myEndpointId[i] = configData.endpointId[i];
        myEndpoint[i] = new Endpoint(myEndpointId[i] + "@" + myDomainName,
                                     dialNumber[i],
                                     configData.runmode,
                                     i,
                                     configData.cntsPerEndpt,
                                     myAgent,
                                     *myHardware[i],
                                     endpointQ[0],
                                     hardwareQ);

        assert(myEndpoint[i] != NULL);
        activeEndpoints++;
    }

    // Set the rest of them to NULL.

    while (i < GW_ENDPOINT_MAX )
    {
        myEndpoint[i] = NULL;
        i++;
    }

    errorcode = pthread_create( &(myEndpointThread[0]), NULL,
                                endpointCallback,
                                (void *) theGateway );
    assert(!errorcode);


    // send restart in progress to agent.

    const string restart = "restart";
    /*
        MgRestartInProgress myRestart("*", restart, "none");
        myAgent->send(myRestart);
    */

    cout << "Delay = " << configData.betweenEndpointDelay << endl;

    sleep (1);

    for (i = 0; i < configData.numEndpoints; i++)
    {
        myEndpoint[i]->restartInProgress(restart);
        vusleep (configData.betweenEndpointDelay);
    }

} // end Gateway::Gateway()


Gateway::~Gateway (void)
{
    int numEndpoints = activeEndpoints;
    int errorcode;

    deleteAgent();
    deleteHardware();
    for (int i = 0; i < numEndpoints; i++)
    {
        deleteEndpoint(i);
    }
    delete [] myEndpointId;

    errorcode = pthread_mutex_destroy(&cid_mutex);
    assert(errorcode != EBUSY /* some thread has cid_mutex locked */);
    assert(!errorcode);
    errorcode = pthread_mutex_destroy(&rtp_mutex);
    assert(errorcode != EBUSY /* some thread has rtp_mutex locked */);
    assert(!errorcode);

} // end Gateway::~Gateway()


MgAgent*
Gateway::setAgent (string agentHostname, int agentPort)
{
    int errorcode;

    MgAgent* newAgent = new MgAgent(agentHostname.c_str(), agentPort, myPort);
    if (newAgent == NULL)
    {
        cpLog(LOG_ERR, "Could not create new agent, keeping old agent\n");
    }
    else
    {
        deleteAgent();
        myAgent = newAgent;
    }

    // create thread for mgcp stack to communicate with agent
    errorcode = pthread_create( &myAgentThread, NULL,
                                agentCallback, (void *) myAgent );

    assert(!errorcode);
    return newAgent;
} // end Gateway::setAgent()


int
Gateway::getEndpointIndex (string id)
{
    // residential and trunk gateways should not have to deal with
    // wildcards; the use of "*" for ALL and "$" for ANY is reserved
    // for media gateways

    // locate matching endpoint id from list
    for (int i = 0; i < activeEndpoints; i++)
    {
        //	if (id == myEndpointId[i]+ "@" + myDomainName)
        //	if (id == myEndpointId[i])
        int j = id.find(myEndpointId[i]);
        if (j != string::npos)
        {
            return i;
        }
    }
    return GW_INV_ENDPOINT_INDEX;
} // end Gateway::getEndpointIndex()


void
Gateway::forwardCommand (CMD cmd, MgCommandType type,
                         Fifo < EndpointMsg > * outputQ,
                         int endpointIndex)
{

    /**********************************************************************
     
        // this is comment is incorrect:
     
        // reuse same msg each time, fifo makes its own copy and there is
        // only one gateway thread
        static EndpointMsg msg;
        static EndpointAgentCommand command;
        
    **********************************************************************/

    // reuse same msg each time, fifo makes its own copy and there is
    // only one gateway thread
    static EndpointMsg msg;
    EndpointAgentCommand command;

    command.type = type;
    command.cmd = cmd;

    msg.type = EndpointAgentCommandType;
    msg.command = command;
    msg.endpointIndex = endpointIndex;

    cpLog(LOG_DEBUG, "forwardCommand,   type =  %d\n", type);

    outputQ->add(msg);
} // end Gateway::forwardCommand()


void
Gateway::deleteAgent (void)
{
    if (myAgentThread != (pthread_t)NULL)
    {
        pthread_join( myAgentThread, NULL );
        myAgentThread = (pthread_t)NULL;
    }

    if (myAgent)
    {
        delete myAgent;
        myAgent = NULL;
    }
}


void
Gateway::deleteHardware (void)
{
    if (myHardwareThread != (pthread_t)NULL)
    {
        pthread_join( myHardwareThread, NULL );
        myHardwareThread = (pthread_t)NULL;
    }

    for (int i = 0; i < GW_ENDPOINT_MAX; i++)
    {
        if (myHardware[i])
        {
            delete myHardware[i];
            myHardware[i] = NULL;
        }
    };
    // flush hardware queue, then delete it
    while (hardwareQ->size() != 0)
    {
        hardwareQ->getNext();
    }
    delete hardwareQ;
    hardwareQ = NULL;

} // end Gateway::deleteHardware()


void
Gateway::deleteEndpoint (int index)
{
    if (index < activeEndpoints)
    {
        if (myEndpointThread[index] != (pthread_t)NULL)
        {
            pthread_join( myEndpointThread[index], NULL );
            myEndpointThread[index] = (pthread_t)NULL;
        }

        if (myEndpoint[index] != NULL)
        {
            delete myEndpoint[index];
            myEndpoint[index] = NULL;
        }
        activeEndpoints--;
    }

    // flush endpoint queue, then delete it
    while (endpointQ[index]->size() != 0)
    {
        endpointQ[index]->getNext();
    }
    delete endpointQ[index];
    endpointQ[index] = NULL;

} // end Gateway::deleteEndpoint()


void
Gateway::handleHardware(void* params)
{
    // process forever on behalf of test hardware
    fd_set readfds;
    bool operational = true;
    int dummy = 0;

    while (operational)
    {
        // reset file descriptor
        FD_ZERO(&readfds);
        // addToFdSet(&readfds);
        //if (maxFd <= gateway.myAgent->getFd())
        //maxFd =  gateway.myAgent->getFd() + 1;

        // block on select but timeout needed to service myQ

        if (dummy == 1)
        {
            cpLog(LOG_ERR, "Select returned with an error - ignore");
        }
        else // if (retval > 0)
        {
            for (int i = 0; i < GW_ENDPOINT_MAX; i++)
            {
                if (myHardware[i])
                {
                    // hardware input received
                    if (myHardware[i]->process(&readfds) < 0)
                    {
                        cpLog(LOG_ERR, "Hardware failed - shutting down");
                        operational = false;
                    }
                }
            }

        }

        // process msgs from endpoints (if any)
        while (hardwareQ->size() > 0)
        {
            HardwareMsg msg;
            msg = hardwareQ->getNext();

            cpLog (LOG_DEBUG, "HW Q:    msg.type = %d    msg.endpointIndex = %d",
                   msg.type, msg.endpointIndex);

            if (msg.endpointIndex < GW_ENDPOINT_MAX && msg.endpointIndex >= 0)
            {
                if (myHardware[msg.endpointIndex])
                {
                    myHardware[msg.endpointIndex]->processEndpointMsg(msg);
                };
            }
            else
            {
                cerr << "Invalid endpointIndex!!!" << endl;
            };
        }

    } // end while

}
;


void
Gateway::handleEndpoint(void* params)
{
    EndpointMsg msg;

    while (1)
    {
        msg = endpointQ[0]->getNext();

        cpLog (LOG_DEBUG, "EP Q:    msg.type = %d    msg.endpointIndex = %d",
               msg.type, msg.endpointIndex);

        if (msg.endpointIndex < GW_ENDPOINT_MAX && msg.endpointIndex >= 0)
        {
            switch (msg.type)
            {
                case EndpointAgentCommandType:
                {
                    cpLog (LOG_DEBUG, "Got Agent msg" );
                    myEndpoint[msg.endpointIndex]->processAgentCommand(msg.command);
                }
                break;

                case EndpointHardwareEventType:
                {
                    cpLog (LOG_DEBUG, "Got hardware msg");
                    myEndpoint[msg.endpointIndex]->processHardwareEvent(msg.event);

                    // Look for the Offhook Event to determine the number of calls
                    // made and received.

                    if (msg.event == EventOffhook)
                    {
                        if (configData.runmode == "CallingGW")
                            numOfAttCalls++;
                        else if (configData.runmode == "ReceiveGW")
                            numOfRevCalls++;
                    }
                }
                break;

                case EndpointTimeoutType:
                {
                    cpLog (LOG_DEBUG, "Timeout");
                    myEndpoint[msg.endpointIndex]->processTimeout(msg.timeout);
                }
                break;

                default:
                {
                    cpLog (LOG_DEBUG, "Unknown Msg");
                }
                break;
            }
        }
        else
        {
            cerr << "Invalid endpointIndex!!!" << endl;
        }

    };

}

Gateway* fakeGateway;

void killGatewayGlobal(int sig)
{
    fakeGateway->killGateway(sig);
}

void
Gateway::gatewayMain (void)
{
    CMD cmd;
    MgCommandType cmdType;
    int endpointIndex = GW_INV_ENDPOINT_INDEX;


    // Intercept the Interrupt signal from user and send to killGateway()
    fakeGateway = this;

    signal(SIGINT, killGatewayGlobal);

    fd_set fds0;
    myAgent->addToFdSet (&fds0);

    int retval;
    fd_set fds;
    struct timeval tv;

    while (1)
    {
        fds = fds0;
        tv.tv_sec = 0;
        tv.tv_usec = 300;

        select(129, &fds, NULL, NULL, &tv);

        cpLog(LOG_DEBUG, "*** Begin Loop:  gatewayMain ***");
        if (asyncStack->checkMsgReady (&fds) == true)
        {
            cmd = myAgent->asyncReceive();
            cmdType = cmd->getType();

            // if command is for a given endpoint, forward it
            switch (cmdType)
            {
                case MgEndpointConfigurationType:
                {
                    EPCF epcf;
                    epcf.dynamicCast(cmd);
                    endpointIndex = getEndpointIndex(epcf->getEndpointId());
                }
                break;

                case MgCreateConnectionType:
                {
                    CRCX crcx;
                    crcx.dynamicCast(cmd);
                    endpointIndex = getEndpointIndex(crcx->getEndpointId());
                }
                break;

                case MgModifyConnectionType:
                {
                    MDCX mdcx;
                    mdcx.dynamicCast(cmd);
                    endpointIndex = getEndpointIndex(mdcx->getEndpointId());
                }
                break;

                case MgDeleteConnectionAgentType:
                {
                    DLCX dlcx;
                    dlcx.dynamicCast(cmd);
                    endpointIndex = getEndpointIndex(dlcx->getEndpointId());
                }
                break;

                case MgNotificationRequestType:
                {
                    RQNT rqnt;
                    rqnt.dynamicCast(cmd);
                    endpointIndex = getEndpointIndex(rqnt->getEndpointId());
                }
                break;

                case MgAuditEndpointType:
                {
                    AUEP auep;
                    auep.dynamicCast(cmd);
                    endpointIndex = getEndpointIndex(auep->getEndpointId());
                }
                break;

                case MgAuditConnectionType:
                {
                    AUCX aucx;
                    aucx.dynamicCast(cmd);
                    endpointIndex = getEndpointIndex(aucx->getEndpointId());
                }
                break;

                default:
                {
                    cpLog(LOG_ERR, "Rcvd unknown cmd, type = %d",
                          cmdType);
                }
            } // end switch

            // send cmd to appropriate endpoint
            if (endpointIndex != GW_INV_ENDPOINT_INDEX)
            {
                // NOTE:  Use only endpointQ[0]!!!
                // Only one queue is created!!
                forwardCommand(cmd, cmdType, endpointQ[0], endpointIndex);
            }
            else
            {
                cpLog(LOG_DEBUG, "Gateway needes to handle command, type = %d",
                      cmdType);
            }
        }
        cpLog(LOG_DEBUG, "*** End Loop:  gatewayMain ***");
    } // end while

} // end Gateway::gatewayMain()



int
Gateway::newRtpPort (void)
{
    int newPort;
    int errorcode;
    // lock
    errorcode = pthread_mutex_lock(&rtp_mutex);
    assert( !errorcode );

    if ( currentRtpPort >= (configData.firstRtpPort + GW_NUM_RTP_PORTS) )
    {
        currentRtpPort = configData.firstRtpPort;
    }

    // intentionally increment after assigning
    newPort = currentRtpPort;
    currentRtpPort += 2;

    // unlock
    errorcode = pthread_mutex_unlock(&rtp_mutex);
    assert( !errorcode );
    return newPort;
} // end Gateway::newRtpPort()


void
deleteRtpPort (int port)
{
    // nothing needed for this simplfied scheme
} // end Gateway::deleteRtpPort()



string
Gateway::newConnectionId (void)
{
    // cycle through numbers and assume connection id will be free
    // before wrapping around
    static unsigned long connectionNumber = 1;
    string id;
    int errorcode;
    // lock
    errorcode = pthread_mutex_lock(&cid_mutex);
    assert( !errorcode );

    id = "c" + itos(connectionNumber++);


    // unlock
    errorcode = pthread_mutex_unlock(&cid_mutex);
    assert( !errorcode );

    return id;

} // end Gateway::newConnectionId()



char*
Gateway::getDialNumber(int index, string startNum, char* digitString)
{
    int tempNum;
    /*
        for (int i = 0; i < MAX_DIGIT_BUFFER; i++)
        {
            digitArray[i] = '\0';
        }
    */
    tempNum = atoi(startNum.c_str()) + index;

    sprintf(digitString, "%d", tempNum);

    /*
        if (tempNum >=  100000)
            sprintf (digitString[myEndpointIndex], "0%d", tempNum);
        else if (tempNum >= 10000)
            sprintf (digitString[myEndpointIndex], "00%d", tempNum);
        else if (tempNum >= 1000)
            sprintf (digitString[myEndpointIndex], "000%d", tempNum);
        else if (tempNum >= 100)
            sprintf (digitString[myEndpointIndex], "0000%d", tempNum);
        else if (tempNum >= 10)
            sprintf (digitString[myEndpointIndex], "00000%d", tempNum);
        else if (tempNum >= 0)
            sprintf (digitString[myEndpointIndex], "000000%d", tempNum);
    */ 
    return digitString;
}


void
Gateway::killGateway(int sig)
{
    time(&endTime);

    printf("\n\n\n\n******************************************************\n");

    for (int i = 0; i < 3; i++)
        printf("***************   GOT KILL SIGNAL!!!   ***************\n");

    printf("******************************************************\n");
    cerr << "      - STATS -" << endl;
    cerr << "      Attempted Calls   = " << numOfAttCalls << endl;
    cerr << "      Received Calls    = " << numOfRevCalls << endl;
    cerr << "      Bad States        = " << numOfBadStates << endl;
    cerr << "      Total time (sec)  = " << difftime(endTime, startTime) << endl;
    printf("******************************************************\n");
    printf("******************************************************\n");
    exit(0);
}


/*********************************************************************
 
Configuration File Handling
 
*********************************************************************/

/// Enumeration of valid tags in gateway configuration file
enum
{
    GatewayConfigTagHardware = 0,
    GatewayConfigTagGatewayPort,
    GatewayConfigTagAgentHostname,
    GatewayConfigTagAgentPort,
    GatewayConfigTagNumEndpoints,
    GatewayConfigTagEndpoint,
    GatewayConfigTagCntsPerEndpt,
    GatewayConfigTagFirstRtpPort,
    GatewayConfigTagRunMode,
    GatewayConfigTagCallDuration,
    GatewayConfigTagCallDelay,
    GatewayConfigTagAnswerDelay,
    GatewayConfigTagDialNumber,
    GatewayConfigTagEndpointDelay,
    GatewayConfigTagMax,
    GatewayConfigInvalid
} GatewayConfigTag;

/// Valid tag strings for gateway configuration file
static char* gatewayConfigTagString[GatewayConfigTagMax] =
    {
        "GatewayHardware",
        "GatewayPort",
        "GatewayAgentHostname",
        "GatewayAgentPort",
        "GatewayNumEndpoints",
        "GatewayEndpoint",
        "GatewayCntsPerEndpt",
        "GatewayFirstRtpPort",
        "GatewayRunMode",
        "GatewayCallDuration",
        "GatewayCallDelay",
        "GatewayAnswerDelay",
        "DialNumber",
        "BetweenEndpointDelay"
    };


/** Converts a tag string to an index, takes a string and returns the
 ** corresponding index, or GatewayConfigInvalid if matching tag is
 ** not found.
 */
int
gatewayConfigTag2i (char* tag)
{
    int i;

    // search until matching tag is found
    for (i = 0; i < GatewayConfigTagMax; i++)
    {
        if (strcmp(tag, gatewayConfigTagString[i]) == 0)
        {
            //cpLog (LOG_DEBUG, "Matched tag = %s", tag);
            return i;
        }
    }

    // no match found, report error
    cpLog (LOG_ERR, "Unknown tag = %s", tag);
    return GatewayConfigInvalid;

} // end gatewayConfigTag2i()


/** Initializes gateway configuration data based on valid tags
 ** extracted from configuration file. Takes a valid tag/type/value
 ** and sets the corresponding field in the configuration data.
 */
void
setGatewayConfigParameter (char* tag, char* type, char* value)
{
    // static int index;

    switch (gatewayConfigTag2i(tag))
    {
        case GatewayConfigTagHardware:
        {
            configData.deviceName = value;
            //cpLog(LOG_DEBUG, "Device name = %s",
            //  configData.deviceName.c_str());
        }

        break;

        case GatewayConfigTagGatewayPort:
        {
            configData.gatewayPort = atoi(value);
            //cpLog(LOG_DEBUG, "Gateway port = %d", configData.gatewayPort);
        }

        break;

        case GatewayConfigTagAgentHostname:
        {
            configData.agentHostname = value;
            //cpLog(LOG_DEBUG, "Agent Hostname = %s",
            //  configData.agentHostname.c_str());
        }

        break;

        case GatewayConfigTagAgentPort:
        {
            configData.agentPort = atoi(value);
            //cpLog(LOG_DEBUG, "Agent port = %d", configData.agentPort);
        }

        break;

        case GatewayConfigTagNumEndpoints:
        {
            configData.numEndpoints = atoi(value);
            //cpLog(LOG_DEBUG, "Number of endpoints = %d",
            //  configData.numEndpoints);
        }

        break;

        case GatewayConfigTagEndpointDelay:
        {
            configData.betweenEndpointDelay = atoi(value);
            //cpLog(LOG_DEBUG, "Number of endpoints = %d",
            //  configData.numEndpoints);
        }

        break;

        case GatewayConfigTagEndpoint:
        {
            configData.startEndpoint = atoi(value);

            /*
            if (index < GW_ENDPOINT_MAX)
            {
            configData.endpointId[index] = value;
            //cpLog(LOG_DEBUG, "EndpointId[%d] = %s",
            //    index, configData.endpointId[index].c_str());
            index++;
        }
            */
        }
        break;

        case GatewayConfigTagCntsPerEndpt:
        {
            configData.cntsPerEndpt = atoi(value);
            //cpLog(LOG_DEBUG, "Number of endpoints = %d",
            //  configData.numEndpoints);
        }

        break;

        case GatewayConfigTagFirstRtpPort:
        {
            configData.firstRtpPort = atoi(value);
            //cpLog(LOG_DEBUG, "Number of endpoints = %d",
            //  configData.numEndpoints);
        }

        break;

        case GatewayConfigTagRunMode:
        {
            configData.runmode = value;
        }
        break;


        case GatewayConfigTagCallDuration:
        {
            configData.callDuration = atoi(value);
        }
        break;

        case GatewayConfigTagCallDelay:
        {
            configData.callDelay = atoi(value);

        }
        break;

        case GatewayConfigTagAnswerDelay:
        {
            configData.answerDelay = atoi(value);
        }
        break;

        case GatewayConfigTagDialNumber:
        {
            configData.dialNumber = value;
        }
        break;

        default:
        {
            cpLog (LOG_ERR, "Invalid tag index");
        }
        break;
    } // end switch

} // end setGatewayConfigParameter()



void
showConfigData (void)
{
    cout << "\n-------------------------------------------------\n";
    cout << "Gateway Configuration Data";
    cout << "\nDevice Name:    " << configData.deviceName;
    cout << "\nGateway Port#:  " << configData.gatewayPort;
    cout << "\nAgent Hostname  " << configData.agentHostname;
    cout << "\nAgent Port#:    " << configData.agentPort;
    cout << "\nEndpoints:      " << configData.numEndpoints;
    cout << "\nEndpointDelay:  " << configData.betweenEndpointDelay;
    cout << "\nCntsPerEndpt:   " << configData.cntsPerEndpt;
    cout << "\nRtp port range: " << configData.firstRtpPort
    << "-"
    << (configData.firstRtpPort + GW_NUM_RTP_PORTS - 1);

    cout << "\nRun Mode:       " << configData.runmode;
    cout << "\nCallDuration:   " << configData.callDuration;
    cout << "\nCallDelay:      " << configData.callDelay;
    cout << "\nAnswerDelay:    " << configData.answerDelay;
    cout << "\nDialNumber:     " << configData.dialNumber;
    cout << "\nStartEndpoint   " << configData.startEndpoint;

    for (int i = 0; i < configData.numEndpoints; i++)
    {
        if (i < 10)
        {
            cout << "\nEndpointId[" << i << "]:  " << configData.endpointId[i];
        }
        else
        {
            cout << "\nEndpointId[" << i << "]: " << configData.endpointId[i];
        }
    }
    cout << "\n-------------------------------------------------\n";

} // end showConfigData()


/*********************************************************************
 
main
 
*********************************************************************/

/** Reads configuration file into global data structure and launches
 ** the gateway
 */
int
main (int argc, char* argv[])
{
    // setup logfile
    cpLogSetLabel ("gw");

    int GW_DEBUG_LEVEL = atoi(getenv("GW_DEBUG_LEVEL") ? getenv("GW_DEBUG_LEVEL") : "0");

    if (GW_DEBUG_LEVEL > 0)
    {
        cpLogSetPriority (LOG_DEBUG);
    }

    // read configuration file
    cpLog(LOG_DEBUG, "Loading gateway configuration from 'gw.cfg'");
    cpLog(LOG_DEBUG, "Override file with runtime args as follows:");
    cpLog(LOG_DEBUG, "%s [gateway#]  [null|trunk|ipjack|ipjack1|vm]  [agent hostname]", argv[0]);
    if (parse3tuple ("gw.cfg", setGatewayConfigParameter) == false)
    {
        cpLog(LOG_ERR, "Error parsing configuration file");
    }


    // modify configuration based on runtime args
    // use argv[1] (aka gateway#) as offset for port numbers
    if (argc >= 2)
    {
        configData.gatewayPort += atoi(argv[1]);
        configData.agentPort += atoi(argv[1]);
        configData.firstRtpPort += (atoi(argv[1]) * GW_NUM_RTP_PORTS);
        cpLog(LOG_DEBUG, "Override config file, set gateway port = %d",
              configData.gatewayPort);
        cpLog(LOG_DEBUG, "Override config file, set agent port = %d",
              configData.agentPort);
        cpLog(LOG_DEBUG, "Override config file, set first rtp port = %d",
              configData.firstRtpPort);
    }

    // use argv[2] as device name
    if (argc >= 3)
    {
        if (strcmp("ipjack", argv[2]) == 0)
        {
            configData.deviceName = "ipjack";
        }
        else if (strcmp("vm", argv[2]) == 0)
        {
            configData.deviceName = "vm";
        }
        else if (strcmp("ipjack1", argv[2]) == 0)
        {
            configData.deviceName = "ipjack1";
        }
        else if (strcmp("trunk", argv[2]) == 0)
        {
            configData.deviceName = "trunk";
            configData.numEndpoints = GW_ENDPOINT_MAX;
            cpLog(LOG_DEBUG, "Override config file, set num endpoints = %d",
                  configData.numEndpoints);

        }
        else if (strcmp("test", argv[2]) == 0)
        {
            configData.deviceName = "test";
            // create the endpoint ids
            cerr << configData.numEndpoints << endl;


            for (int i = 0; i < configData.numEndpoints; i ++)
            {
                char* tempString;
                char tempArray[10];
                tempString = tempArray;
                // sprintf(tempString, "%s", "testid");
                sprintf(tempString, "%s%d", "ep", i + configData.startEndpoint);
                configData.endpointId[i] = tempString;
            }

        }
        else if (strcmp("null", argv[2]) == 0)
        {
            configData.deviceName = "null";
        }
        cpLog(LOG_DEBUG, "Override config file, set device name = %s",
              configData.deviceName.c_str());
    }

    // use argv[3] as agent hostname
    if (argc >= 4)
    {
        configData.agentHostname = argv[3];
        cpLog(LOG_DEBUG, "Override config file, set agent hostname = %s",
              configData.agentHostname.c_str());
    }

    // show config data
    showConfigData();

    // create new gateway according to configuration data
    Gateway gateway;

    time(&startTime);

    // startup gateway, should never return
    gateway.gatewayMain();

    return 0;
} // end main()
