#if !defined(VOCAL_HTTP_HEADER_HXX)
#define VOCAL_HTTP_HEADER_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const HttpHeader_hxx_Version =
    "$Type: HttpHeader.hxx,v 1.1 2001/04/01 12:14:56 icahoon Exp $";


#include "Http.hxx"
#include "Writer.hxx"
#include <map>


/** Infrastructure common to VOCAL.<br><br>
 */
namespace Vocal 
{


/** Http Protocol Libraries<br><br>
 */
namespace Http
{


/** Implementation of RFC 2616, Sections 4-2, 5-3, 6-2 and 7-1.<br><br>
 */
class Header : public Vocal::IO::Writer
{
    public:

        ///
        typedef     Token   Value;
    
        // General headers
        //
        static const Type   CacheControl;
        static const Type   Connection;
        static const Type   Date;
        static const Type   Pragma;
        static const Type   Trailer;
        static const Type   TransferEncoding;
        static const Type   Upgrade;
        static const Type   Via;
        static const Type   Warning;
                      
        // Request headers
        //
        static const Type   Accept;
        static const Type   AcceptCharset;
        static const Type   AcceptEncoding;
        static const Type   AcceptLanguage;
        static const Type   Authorization;
        static const Type   Expect;
        static const Type   From;
        static const Type   Host;
        static const Type   IfMatch;
        static const Type   IfModifiedSince;
        static const Type   IfNoneMatch;
        static const Type   IfRange;
        static const Type   IfUnmodifiedSince;
        static const Type   MaxForwards;
        static const Type   ProxyAuthorizatiation;
        static const Type   Range;
        static const Type   Referer;
        static const Type   TE;
        static const Type   UserAgent;


        // Response headers
        //
        static const Type   AcceptRanges;
        static const Type   Age;
        static const Type   ETag;
        static const Type   Location;
        static const Type   ProxyAuthentication;
        static const Type   RetryAfter;
        static const Type   Server;
        static const Type   Vary;
        static const Type   WWWAuthenticate;


        // Entity headers
        //
        static const Type   Allow;
        static const Type   ContentEncoding;
        static const Type   ContentLanguage;
        static const Type   ContentLength;
        static const Type   ContentLocation;
        static const Type   ContentMD5;
        static const Type   ContentRange;
        static const Type   ContentType;
        static const Type   Expires;
        static const Type   LastModified;

                
        static const Token      TokenUnknown;


        // General headers
        //
        static const Token      TokenCacheControl;
        static const Token      TokenConnection;
        static const Token      TokenDate;
        static const Token      TokenPragma;
        static const Token      TokenTrailer;
        static const Token      TokenTransferEncoding;
        static const Token      TokenUpgrade;
        static const Token      TokenVia;
        static const Token      TokenWarning;

                      
        // Request headers
        //
        static const Token      TokenAccept;
        static const Token      TokenAcceptCharset;
        static const Token      TokenAcceptEncoding;
        static const Token      TokenAcceptLanguage;
        static const Token      TokenAuthorization;
        static const Token      TokenExpect;
        static const Token      TokenFrom;
        static const Token      TokenHost;
        static const Token      TokenIfMatch;
        static const Token      TokenIfModifiedSince;
        static const Token      TokenIfNoneMatch;
        static const Token      TokenIfRange;
        static const Token      TokenIfUnmodifiedSince;
        static const Token      TokenMaxForwards;
        static const Token      TokenProxyAuthorizatiation;
        static const Token      TokenRange;
        static const Token      TokenReferer;
        static const Token      TokenTE;
        static const Token      TokenUserAgent;


        // Response headers
        //
        static const Token      TokenAcceptRanges;
        static const Token      TokenAge;
        static const Token      TokenETag;
        static const Token      TokenLocation;
        static const Token      TokenProxyAuthentication;
        static const Token      TokenRetryAfter;
        static const Token      TokenServer;
        static const Token      TokenVary;
        static const Token      TokenWWWAuthenticate;


        // Entity headers
        //
        static const Token      TokenAllow;
        static const Token      TokenContentEncoding;
        static const Token      TokenContentLanguage;
        static const Token      TokenContentLength;
        static const Token      TokenContentLocation;
        static const Token      TokenContentMD5;
        static const Token      TokenContentRange;
        static const Token      TokenContentType;
        static const Token      TokenExpires;
        static const Token      TokenLastModified;


        /** Create the header given the type
         */
        Header(Type = EXTENSION);
        

        /** Create the header given the token
         */
        Header(const Token &);
        

        /** Create the header given the type and value
         */ 
        Header(Type, const Value &);


        /** Create the header given the token and value
         */ 
        Header(const Token &, const Value &);


        /** Copy constructor
         */
        Header(const Header &);


        /** Virtual destructor
         */
        virtual ~Header();

        
        /** Assignment operator
         */
        const Header &  operator=(const Header &);
        
        
        /** Equality operator
         */
        bool    operator==(const Header &) const;
        
        
        /** Less than relational operator
         */
        bool    operator<(const Header &) const;


        /** Type accessor
         */
        Type    type() const;    
        
        
        /** Type mutator
         */
        void    type(Type);    
        

        /** Provide the type given the token 
         */
        void    type(const Token &) throw ( Vocal::Http::Exception );
        
        
        /** Header value accessor
         */
        const Value &  value() const;

        
        /** Header value mutator
         */
        void    value(const Value &);
        
        
        /** A header value may come in multiple parts. Use this to
         *  concatenate multiple value lines.
         */
        void    continueValue(const Value &);
        
        
        /** Provide the token text for this header
         */
        const Token &  token() const;

        
        /** An alias for token
         */
        const Token &  name() const;
        
        
        /** The total number of header types
         */
        static size_t   totalTypes();


        /** Returns true if this header is a general header
         */
        virtual bool    isGeneral() const;


        /** Returns true if this header is a request header
         */
        virtual bool    isRequest() const;


        /** Returns true if this header is a response header
         */
        virtual bool    isResponse() const;


        /** Returns true if this header is a entity header
         */
        virtual bool    isEntity() const;        
        

        /** Returns true if this header is an extension
         */
        virtual bool    isExtension() const;        
        

        /** Write this header to an ostream
         */
        ostream &       writeTo(ostream &) const;


    protected:
    
        
        void    init();


        /** Create the header type to token table
         */
        void    initTable();
        
        typedef map<Type, const Token *>  Table;

        static Table        myTable;

        /** Create the token to header type table 
         */
        void    initDecodeTable();

        typedef map<Token, Type>           DecodeTable;

        static DecodeTable  myDecodeTable;


        static size_t       myTotal;
        
        Type                myType;
        Token               myToken;
        Value               myValue;
};


} // namespace Http
} // namespace Vocal


#endif // !defined(VOCAL_HTTP_HEADER_HXX)
