
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

#include "LGCommandLine.hxx"
#include "Data.hxx"
#include "LGParam.hxx"
#include "SystemInfo.hxx"
#include "LGUserGroup.hxx"
#include "LGUserRange.hxx"
#include <iostream>
#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <fstream>
#include "cpLog.h"

using namespace Vocal;

const char * LGCommandLine::usage =
"[-i <sampleMsg>] {{<source user#@host:port> <dest usre#@host:port>} | -f <config file>}";
const int LGCommandLine::maxCount = 5;    /// usage count
const int LGCommandLine::minCount = 3;    /// usage count


bool
LGCommandLine::parse(int c, char **v)
{
    if(c < LGCommandLine::minCount || c > LGCommandLine::maxCount)
    {
	cerr << "usage: " << v[0] << " " << LGCommandLine::usage << endl;
	return false;
    }
    
    /// parse and set the params here...
    if( v[1][1] == 'i')
    {
	/// read the sample msg
	if (!readSampleMsg(v[2]))
	{
	    return false;
	}
    }
    if((c==3 && v[1][1] == 'f')||(c==5 && v[3][1] == 'f'))
    {
	/// parse the config file
	return parseFile((c==3?v[2]:v[4]));
    }
    else if(c==3 || (c == 5 && v[1][1] == 'i'))
	return parseCL((c==3?&v[1]:&v[3]));
    else
	cerr << "ERROR: cannot parse parameters" <<endl;
    return false;
}

bool
LGCommandLine::readSampleMsg(char * val)
{
    ifstream inputFile(val);
    if ( !inputFile )
    {
	cerr << "Error: Cannot open sample msg file " <<
	    val << endl;
	return false;
    }
    string str;
    char ch;
    while ( inputFile.get( ch ) )
    {
	str += ch;
    }
//    cerr << "gonna do " << (str);
    LGParam::MsgData = new Data(str.c_str());
    return true;
}

bool
LGCommandLine::parseCL(char **v)
{
    char *params = v[0];

    while (*params == ' ') params++;

    char * curr = params;
    for(;*curr && *curr!='@' && *curr!=' '; curr++);
    int from;
    if(*curr == '@')
    {
	*(curr++) = 0;
	from = atoi(params); // <<<<<<<<<<<-----------
	for(params = curr;*curr && *curr!=':' && *curr!=' ';curr++);
	if(*curr)
	{
	    *(curr++) = 0;
	    LGParam::FromHost = new char[curr-params];
	    strcpy(LGParam::FromHost,params);
	    
	    for(params = curr;*curr && *curr!=' ';curr++);
	    *(curr++) = 0;
	    int port = atoi(params);
	    if(port)
		LGParam::UDPPort = port;
	}
    }
    else
	from = atoi(params); // <<<<<<<<<<<-----------


    if(!LGParam::FromHost)
    {
	LGParam::FromHost = new char[256];
	strcpy(LGParam::FromHost,theSystem.gethostAddress());
    }

    params = v[1];
    while (*params == ' ') params++;
    for(curr=params;*curr && *curr!='@'; curr++);
    *(curr++) = 0;
    int to = atoi(params); // <<<<<<<<<<<-----------
    for(params = curr;*curr && *curr!=':' && *curr!=' ';curr++);
    if(params<curr)
    {
	*(curr++) = 0;
	LGParam::ToHost = new char[curr-params];
	strcpy(LGParam::ToHost,params);
	LGParam::NextHopHost = LGParam::ToHost;

	for(params = curr;*curr && *curr!=' ';curr++);
	*(curr++) = 0;

	if(atoi(params))
	{
	    LGParam::ToPort = new char[curr-params];
	    strcpy(LGParam::ToPort,params);
	    LGParam::NextHopPort = LGParam::ToPort;
	}
    }
    else
    {
	cerr << "ERROR: needed To host" << endl;
	return false;
    }

    if(from && to)
	LGUserGroup::uses(new LGUserRange(from,from,to,to));
    else
    {
	cerr << "ERROR: illegal From and/or To" << endl;
	return false;
    }

    return true;
}

bool
LGCommandLine::parseFile(char * fname)
{
    while (fname && *fname == ' ') fname++;

    FILE * file = fopen(fname,"r");
    if(!file)
    {
	cerr << "Error: Cannot open config file " <<
	    fname << endl;
	return false;
    }

    char line[256], arg[256], val[256];
    VeryLong fromStart = 0, fromEnd = 0, toStart = 0, toEnd = 0;
    char * fromFile = 0, * toFile = 0;

    while(!feof(file))
    {
	fgets(line, 256, file);
	char * curr = line;
	while(*curr && (*curr==' ' || *curr == '\t')) curr++;
	if(*curr != '#')
	{
	    switch(sscanf(curr,"%s = %s", arg, val))
	    {
		case 1: //special cases, eg., "." in secret
		{
		    cpLog(LOG_DEBUG,"Doing special val parsing");
		    char * tmp = curr;
		    for(;*tmp && *tmp != '='; tmp++);
		    for(;*tmp && (*tmp == ' ' || *tmp == '\t'); tmp++);
		    int i = 0;
		    for(;*tmp;tmp++)
			val[i++] = *tmp;
		    val[i]=0;
		}
		    break;

		case 0:
		case 2:
		default:
		    cpLog(LOG_DEBUG,"Doing normal val parsing");
		    break;
	    }
	    if(!strcmp(arg,"MSG_FILE"))
	    {
		/// don't overrid explicit -i from commandline
		if(!LGParam::MsgData)
		{
		    if ( !readSampleMsg(val) )
		    {
			fclose(file);
			return false;
		    }
		}
	    }
	    else if(!strcmp(arg,"RUNNING_MODE"))
	    {
		/// set the debug log level
		if(!strcmp(val,"ORIGINATING"))
		    LGParam::Function = LGParam::ORIGINATING;
		else if(!strcmp(val,"TERMINATING"))
		    LGParam::Function = LGParam::TERMINATING;
		else if(!strcmp(val,"REGISTRATION"))
		    LGParam::Function = LGParam::REGISTRATION;
		else
		{
		    cerr << "ERROR: Illegal running mode ["<<val<<"]" << endl;
		    fclose(file);
		    return false;
		}
		    
	    }
	    else if(!strcmp(arg,"USER"))
	    {
		/// set the user type
		if(!strcmp(val,"IP"))
		    LGParam::User = LGParam::IP;
		else if(!strcmp(val,"E164"))
		    LGParam::User = LGParam::E164;
		else if(!strcmp(val,"PHONE"))
		    LGParam::User = LGParam::PHONE;
		else
		{
		    cerr << "ERROR: Illegal user type ["<<val<<"]" << endl;
		    fclose(file);
		    return false;
		}
		    
	    }
	    else if(!strcmp(arg,"TRANSPORT"))
	    {
		/// set the user type
		if(!strcmp(val,"UDP"))
		    LGParam::Transport = LGParam::UDP;
		else if(!strcmp(val,"TCP"))
		    LGParam::Transport = LGParam::TCP;
		else
		{
		    cerr << "ERROR: Illegal transport ["<<val<<"]" << endl;
		    fclose(file);
		    return false;
		}
		    
	    }
	    else if(!strcmp(arg,"FROM_FILE"))
	    {
		/// prepare the From user group from file
		fromFile = new char[strlen(val)+1];
		strcpy(fromFile,val);
	    }
	    else if(!strcmp(arg,"TO_FILE"))
	    {
		/// prepare the To user group from file
		toFile = new char[strlen(val)+1];
		strcpy(toFile,val);
	    }
	    else if(!strcmp(arg,"FROM_RANGE"))
	    {
		fromStart = atovl(val);
		for(curr = val;*curr && *curr!='.';curr++);
		if(*(curr++) == '.' && *(curr++) == '.')
		{
		    fromEnd = atovl(curr);
		}
		else
		{
		    fromEnd = fromStart;
		}
	    }
	    else if(!strcmp(arg,"TO_RANGE"))
	    {
		toStart = atovl(val);
		for(curr = val;*curr && *curr!='.';curr++);
		if(*(curr++) == '.' && *(curr++) == '.')
		{
		    toEnd = atovl(curr);
		}
		else
		{
		    toEnd = toStart;
		}
	    }
	    else if(!strcmp(arg,"NEXT_HOST"))
	    {
		/// set the next hop host
		for(curr = val;*curr && *curr!=':';curr++);
		LGParam::NextHopHost = new char[curr - val +1];
		strncpy(LGParam::NextHopHost,val, curr-val);
		LGParam::NextHopHost [curr - val] = 0;
		if(*curr)
		{
		    int size = strlen(++curr);
		    if(size>1)
		    {
			LGParam::NextHopPort = new char[size+1];
			strncpy(LGParam::NextHopPort, curr, size);
			LGParam::NextHopPort[size]=0;
		    }
		}
	    }
	    else if(!strcmp(arg,"NEXT_PORT"))
	    {
		/// set the next hop port
		int size = strlen(val);
		if(size>1)
		{
		    LGParam::NextHopPort = new char[size+1];
		    strncpy(LGParam::NextHopPort, val, size);
		    LGParam::NextHopPort[size]=0;
		}
	    }
	    else if(!strcmp(arg,"TO_HOST"))
	    {
		/// set the To host
		for(curr = val;*curr && *curr!=':';curr++);
		LGParam::ToHost = new char[curr - val +1];
		strncpy(LGParam::ToHost, val, curr-val);
		LGParam::ToHost [curr - val] = 0;
		if(*curr)
		{
		    int size = strlen(++curr);
		    if(size>1)
		    {
			LGParam::ToPort = new char[size+1];
			strncpy(LGParam::ToPort, curr, size);
			LGParam::ToPort[size]=0;
		    }
		}
	    }
	    else if(!strcmp(arg,"TO_PORT"))
	    {
		/// set the To port
		int size = strlen(val);
		if(size>1)
		{
		    LGParam::ToPort = new char[size+1];
		    strncpy(LGParam::ToPort, val, size);
		    LGParam::ToPort[size]=0;
		}
	    }
	    else if(!strcmp(arg,"CONTACT_HOST"))
	    {
		/// set the Contact host
		for(curr = val;*curr && *curr!=':';curr++);
		LGParam::ContactHost = new char[curr - val +1];
		strncpy(LGParam::ContactHost, val, curr-val);
		LGParam::ContactHost [curr - val] = 0;
		if(*curr)
		{
		    int size = strlen(++curr);
		    if(size>1)
		    {
			LGParam::ContactPort = new char[size+1];
			strncpy(LGParam::ContactPort, curr, size);
			LGParam::ContactPort[size]=0;
		    }
		}
	    }
	    else if(!strcmp(arg,"LEASE_TIME"))
	    {
		/// set the lease duration
		int size = strlen(val);
		if(size)
		{
		    LGParam::LeaseTime = new char[size+1];
		    strncpy(LGParam::LeaseTime, val, size);
		    LGParam::LeaseTime[size]=0;
		}
	    }
	    else if(!strcmp(arg,"REFRESH_MODE"))
	    {
		/// set the lease refresh mode
		if(!strcmp(val,"REMOVE"))
		    LGParam::Refresh = LGParam::REMOVE;
		else
		    LGParam::Refresh = LGParam::OVERWRITE;
	    }
	    else if(!strcmp(arg,"SECRET"))
	    {
		/// set the user secret
		int size = strlen(val);
		if(size>1)
		{
		    LGParam::Secret = new char[size+1];
		    strncpy(LGParam::Secret, val, size);
		    LGParam::Secret[size]=0;
		}
	    }
	    else if(!strcmp(arg,"CPS"))
	    {
		/// set the call to call delay
		int cps = atoi(val);
		if(cps)
		{
		    LGParam::CPS = cps;
		}
	    }
	    else if(!strcmp(arg,"MAX_RETRY"))
	    {
		/// set the call hold time
		int delay = atoi(val);
		if(delay || val[0] == '0')
		{
		    LGParam::MaxRetry = delay;
		}
	    }
	    else if(!strcmp(arg,"CALL_HOLD"))
	    {
		/// set the call hold time
		int delay = atoi(val);
		if(delay || val[0] == '0')
		{
		    LGParam::CallHoldTime = delay*1000;
		}
	    }
	    else if(!strcmp(arg,"RING_TIME"))
	    {
		/// set the ring time
		int delay = atoi(val);
		if(delay || val[0] == '0')
		{
		    LGParam::RingTime = delay*1000;
		}
	    }
	    else if(!strcmp(arg,"REQUEST_TIMEOUT"))
	    {
		/// set the call hold time
		int delay = atoi(val);
		if(delay || val[0] == '0')
		{
		    LGParam::RequestTimeOut = delay*1000;
		}
	    }
	    else if(!strcmp(arg,"INVITE_TIMEOUT"))
	    {
		/// set the call hold time
		int delay = atoi(val);
		if(delay || val[0] == '0')
		{
		    LGParam::InviteTimeOut = delay*1000;
		}
	    }
	    else if(!strcmp(arg,"TOTAL_CALLS"))
	    {
		/// set the total calls to make
		int num = atoi(val);
		if(num)
		{
		    LGParam::TotalCalls = num;
		}
	    }
	    else if(!strcmp(arg,"BURST_RATE"))
	    {
		/// set the total calls to make
		int num = atoi(val);
		if(num)
		{
		    LGParam::BurstRate = num;
		}
	    }
	    else if(!strcmp(arg,"LOG_FILE"))
	    {
		/// set the cpLog o/p device
		if(!cpLogOpen(val))
		{
		    cerr << "ERROR: could not open log file \""
			 << val << "\"" << endl;
		    fclose(file);
		    return false;
		}
	    }
	    else if(!strcmp(arg,"LOG_LEVEL"))
	    {
		/// set the debug log level
		if(!strcmp(val,"STACK"))
		    cpLogSetPriority(LOG_DEBUG_STACK);
		else if(!strcmp(val,"ALERT"))
		    cpLogSetPriority(LOG_ALERT);
		else if(!strcmp(val,"INFO"))
		    cpLogSetPriority(LOG_INFO);
		else if(!strcmp(val,"DEBUG"))
		    cpLogSetPriority(LOG_DEBUG);
		else if(!strcmp(val,"CRIT"))
		    cpLogSetPriority(LOG_CRIT);
		else if(!strcmp(val,"ERR"))
		    cpLogSetPriority(LOG_ERR);
		else
		    cpLogSetPriority(LOG_EMERG);
	    }
	    else if(!strcmp(arg,"UDP_PORT"))
	    {
		/// set the source udp port
		int port = atoi(val);
		if(port)
		{
		    LGParam::UDPPort = port;
		}
	    }
	    else if(!strcmp(arg,"RTP_PORT"))
	    {
		/// set the source udp port
		int port = atoi(val);
		if(port)
		{
		    LGParam::RTPPort = port;
		}
	    }
	    else if(!strcmp(arg,"SDP_MEDIA"))
	    {
		/// set the SDP media in invites
		if(!strcmp(val,"OFF"))
		{
		    LGParam::SDPOff = true;
		}
	    }
	    else if(!strcmp(arg,"RPORT"))
	    {
		/// set the rport tag in invites
		if(!strcmp(val,"ON"))
		{
		    LGParam::RPort = true;
		}
	    }
	    else if(!strcmp(arg,"RANDOM"))
	    {
		/// set the rport tag in invites
		if(!strcmp(val,"ON"))
		{
		    LGParam::Random = true;
		}
	    }
	    else if(!strcmp(arg, "FROM_HOST"))
	    {
		for(curr = val;*curr && *curr!=':';curr++);
		LGParam::FromHost = new char[curr - val +1];
		strncpy(LGParam::FromHost,val, curr-val);
		LGParam::FromHost [curr - val] = 0;
		if(*curr)
		{
		    int port = atoi(curr);
		    if(port)
			LGParam::UDPPort = port;
		}
	    }
	    else if(!strcmp(arg, "FROM_URL"))
	    {
		for(curr = val;*curr && *curr!=':';curr++);
		LGParam::FromUrl = new char[curr - val +1];
		strncpy(LGParam::FromUrl,val, curr-val);
		LGParam::FromUrl [curr - val] = 0;
		if(*curr)
		{
		    int port = atoi(curr);
		    if(port)
			LGParam::FromUrlPort = port;
		}
	    }
	}
    }

//// now take care of correlation between stuff...
    if(fromStart && fromEnd && toStart && toEnd)
    {
	switch(LGParam::Function)
	{
	    case LGParam::REGISTRATION:
////????	toStart = fromStart; toEnd = fromEnd;

	    case LGParam::ORIGINATING:
		LGUserGroup::uses(new LGUserRange(fromStart,fromEnd,
						  toStart,toEnd));
		break;
/*************  not needed, Sample message will set the callid *************
		LGUserGroup::uses(new LGRegistrationRange(fromStart,fromEnd,
						  toStart,toEnd));
		break;
****************************************************************************/
	    case LGParam::TERMINATING:
	    default:
		break;
	}
    }
    else if(fromFile && toFile)
    {
	/// construct user group from files...
	cerr << "SORRY: users cant be created from files yet!" << endl;
 	fclose(file);
	delete fromFile; delete toFile;
	return false;
    }
    else
    {
	cerr << "ERROR: not enough information for users" << endl;
 	fclose(file);
	delete fromFile; delete toFile;
	return false;
    }

    if(!LGParam::ToHost)
    {
	LGParam::ToHost = LGParam::NextHopHost;
	LGParam::ToPort = LGParam::NextHopPort;
    }

//     if(!LGParam::URIHost)
//     {
// 	LGParam::URIHost = LGParam::NextHopHost;
// 	LGParam::URIPort = LGParam::NextHopPort;
//     }

    if(!LGParam::FromHost)
    {
	LGParam::FromHost = new char[256];
	strcpy(LGParam::FromHost,theSystem.gethostAddress());
    }

    if(!LGParam::FromUrl)
    {
	LGParam::FromUrl = LGParam::FromHost;
	LGParam::FromUrlPort = LGParam::UDPPort;
    }
    else if(!LGParam::FromUrlPort)
	    LGParam::FromUrlPort = LGParam::UDPPort;

    if(!LGParam::ContactHost)
    {
	LGParam::ContactHost = LGParam::FromHost;
    }

    if(!LGParam::ContactPort)
    {
	LGParam::ContactPort = new char[8];
	sprintf(LGParam::ContactPort,"%d",LGParam::UDPPort);
    }

    fclose(file);
    return true;
}
