
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

#include "LGState.hxx"
#include "LGOperator.hxx"
#include "LGEngine.hxx"
#include "LGInitialize.hxx"
#include "LGTransient.hxx"
#include "LGOnHold.hxx"
#include "LGCleanUp.hxx"
#include "OpStartCall.hxx"
#include "OpAccepted.hxx"
#include "OpRejected.hxx"
#include "OpTimeOut.hxx"
#include "OpTerminate.hxx"
#include "OpTerminated.hxx"
#include "OpChallenged.hxx"
#include "OpRing.hxx"
#include "OpAnswer.hxx"
#include "LGListening.hxx"
#include "LGRinging.hxx"
#include "LGParam.hxx"

LGEngine::LGEngine()
    : start(0)
{
    switch(LGParam::Function)
    {
	case  LGParam::ORIGINATING:
	{
	    LGState * sInitialize = new LGInitialize();
	    LGState * sTransient = new LGTransient();
	    LGState * sOnHold = new LGOnHold();
	    LGState * sTeardown = new LGTransient();
	    LGState * sCleanup = new LGCleanup();
	    
	    LGOperator * op = 0;
	    
	    /// start state
	    op = new OpStartCall();
	    op->setNext(sTransient);
	    sInitialize->addOp(op);
	    
	    /// transient state
	    op = new OpAccepted();
	    op->setNext(sOnHold);
	    sTransient->addOp(op);
	    
	    op = new OpRejected();
	    op->setNext(sCleanup);
	    sTransient->addOp(op);
	    
	    op = new OpTimeOut();
	    op->setNext(sCleanup);
	    sTransient->addOp(op);
	    
	    op = new OpChallenged();
	    op->setNext(sTransient);
	    sTransient->addOp(op);
	    
	    /// hold state
	    op = new OpTerminate();
	    op->setNext(sTeardown);
	    sOnHold->addOp(op);
	    
	    op = new OpTerminated();
	    op->setNext(sCleanup);
	    sOnHold->addOp(op);
	    
	    /// teardown state
	    op = new OpAccepted();
	    op->setNext(sCleanup);
	    sTeardown->addOp(op);
	    
	    op = new OpRejected();
	    op->setNext(sCleanup);
	    sTeardown->addOp(op);
	    
	    op = new OpTimeOut();
	    op->setNext(sCleanup);
	    sTeardown->addOp(op);
	    
	    sTeardown->addOp(new OpChallenged()); //needed due to LGTransient's logic!!!

	    start = sInitialize;
	}
	break;

	case LGParam::REGISTRATION:
	{
	    LGState * sInitialize = new LGInitialize();
	    LGState * sTransient = new LGTransient();
	    LGState * sCleanup = new LGCleanup();
	    
	    LGOperator * op = 0;
	    
	    /// start state
	    op = new OpStartCall();
	    op->setNext(sTransient);
	    sInitialize->addOp(op);
	    
	    /// transient state
	    op = new OpAccepted();
	    op->setNext(sCleanup);
	    sTransient->addOp(op);
	    
	    op = new OpRejected();
	    op->setNext(sCleanup);
	    sTransient->addOp(op);
	    
	    op = new OpTimeOut();
	    op->setNext(sCleanup);
	    sTransient->addOp(op);
	    
	    op = new OpChallenged();
	    op->setNext(sTransient);
	    sTransient->addOp(op);
	    
	    
	    start = sInitialize;
	}
	break;

	case LGParam::TERMINATING:
	{
	    LGState * sListening = new LGListening();
	    LGState * sRinging = new LGRinging();
	    LGState * sOnHold = new LGOnHold();
	    LGState * sTeardown = new LGTransient();
	    LGState * sCleanup = new LGCleanup();
	    
	    LGOperator * op = 0;
	    
	    /// listening state
	    op = new OpRing();
	    op->setNext(sRinging);
	    sListening->addOp(op);
	    
	    /// ringing state
	    op = new OpAnswer();
	    op->setNext(sOnHold);
	    sRinging->addOp(op);
	    
	    /// hold state
	    op = new OpTerminate();
	    op->setNext(sTeardown);
	    sOnHold->addOp(op);
	    
	    op = new OpTerminated();
	    op->setNext(sCleanup);
	    sOnHold->addOp(op);
	    
	    /// teardown state
	    op = new OpAccepted();
	    op->setNext(sCleanup);
	    sTeardown->addOp(op);
	    
	    op = new OpRejected();
	    op->setNext(sCleanup);
	    sTeardown->addOp(op);
	    
	    op = new OpTimeOut();
	    op->setNext(sCleanup);
	    sTeardown->addOp(op);

	    sTeardown->addOp(new OpChallenged()); //bogus
	    start = sListening;
	}
	break;

	default:
	    break;
    }
    
}

LGState*
LGEngine::process(const Sptr<SipProxyEvent>& event)
{
    assert(start);
    return start->process(event);
}
