/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const LGFireThread_cxx_Version =
    "$Id: LGFireThread.cxx,v 1.6 2002/04/09 21:34:37 bhadoria Exp $";


#include "LGFireThread.hxx"
#include "Fifo.h"
#include "Sptr.hxx"
#include "LGEvent.hxx"
#include "LGParam.hxx"
#include "LGInfo.hxx"
#include <stdlib.h>
#include <time.h>
#include <iostream>

extern "C" {
     int nanosleep(const struct timespec *rqtp,  struct  timespec *rmtp);

};

LGFireThread::LGFireThread( 
    const Sptr < Fifo < Sptr < SipProxyEvent > > > fifo)
    : myFifo(fifo)
{
    assert ( myFifo != 0 );
}



LGFireThread::~LGFireThread()
{
}

int operator -(struct timeval& left, struct timeval& right)
{
    int delta = (left.tv_sec - right.tv_sec)*1000000;
    delta += (left.tv_usec - right.tv_usec);
    return delta/1000;
}

void
LGFireThread::thread()
{
    if(LGParam::Function == LGParam::TERMINATING)
    { // nothing to fire for this mode!!!
	shutdown();
	return;
    }

    int i,totCalls = 0;
    int burst = LGParam::BurstRate ? LGParam::BurstRate : LGParam::CPS;
    if (burst > LGParam::CPS) burst = LGParam::CPS;
    static struct timeval curr, prev;
    gettimeofday(&curr,NULL);
    int delay = burst<LGParam::CPS ? (1000*burst)/LGParam::CPS : 0;

    while ( true )
    {
	for(i=0;i<burst;i++)
	{
	    myFifo->add(new LGEvent(LGEvent::E_START_CALL, myFifo));
	    if(LGParam::TotalCalls > -1 &&
	       ++totCalls >= LGParam::TotalCalls)
	    {
		shutdown();
		break;
	    }
	}
        if ( isShutdown() == true )
        {
	    break;
        }
	if (!delay)
	{
	    sleep(1);
	}
	else
	{
	    prev = curr;
	    gettimeofday(&curr,NULL);
	    i = curr - prev;
	    if(i>delay)
		i = (i-delay)%1000;
	    else i = 0;
	    i = delay - i;
	    struct timespec req, rem;
	    rem.tv_sec = i/1000;
	    rem.tv_nsec = (i%1000)*1000000L;
	    do
	    {
		req.tv_sec = rem.tv_sec;
		req.tv_nsec = rem.tv_nsec;
		if(req.tv_nsec < 0) break;
	    }while(::nanosleep(&req,&rem));
	}
    }
    LGInfo::stop();
}
