
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

#include "LGOperator.hxx"
#include "OpRejected.hxx"
#include "OpAccepted.hxx"
#include "OpTimeOut.hxx"
#include "SipProxyEvent.hxx"
#include "SipEvent.hxx"
#include "TimerEvent.hxx"
#include "CallInfo.hxx"
#include "LGState.hxx"
#include "LGTearDown.hxx"
#include "StateMachnData.hxx"
#include "AckMsg.hxx"
#include "StatusMsg.hxx"

LGTearDown::LGTearDown()
    : opRejected(0), opAccepted(0), opTimeOut(0)
{}

LGTearDown::~LGTearDown()
{
    delete opRejected; delete opAccepted; delete opTimeOut;
}

bool
LGTearDown::addOp(LGOperator* nextOp)
{
    if(dynamic_cast<OpRejected*>(nextOp)!=0)
	opRejected = nextOp;
    else if(dynamic_cast<OpAccepted*>(nextOp)!=0)
	opAccepted = nextOp;
    else if(dynamic_cast<OpTimeOut*>(nextOp)!=0)
	opTimeOut = nextOp;
    else
	assert(0);
    return true;
}

LGState*
LGTearDown::process(const Sptr<SipProxyEvent>& event)
{
    assert(opRejected && opAccepted && opTimeOut);

    LGState * nextState = this;

    Sptr<SipEvent> sipEvent;
    if(sipEvent.dynamicCast(event)!=0)
    {
	// process sip event...

	Sptr<StatusMsg> response;
	response.dynamicCast(sipEvent->getSipMsg());
	if(!response)
	{
//	    assert(0);
	    return nextState;
	}

	SipStatusLine statusLine = response->getStatusLine();
	if(statusLine.getStatusCode() < 200)
	{
	    // nothing
	}
	else
	{
//	    event->getCallInfo()->getCallStateData()->
//		sipStack->sendAsync(AckMsg(*response));
	    sipEvent->getSipStack()->sendAsync(new AckMsg(*response));
	    
	    event->getCallInfo()->getTimer()->cancelTimer();

	    if(statusLine.getStatusCode() < 300)
		nextState = opAccepted->process(event);
	    else
		nextState = opRejected->process(event);
	}
    }
    else
    {
	Sptr<TimerEvent> timerEvent;
	if(timerEvent.dynamicCast(event)!=0)
	{
	    // process the timeout...
	    nextState = opTimeOut->process(event);
	}
	else
	{
	    assert(0);
	}
    }
    return nextState;
}
