/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const elgeeReceiver 
  = "$Id: elgeeReceiver.cxx,v 1.3 2002/02/20 18:34:22 chok Exp $";

#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <cstdlib>
//#include <fstream>
//#include <pthread.h>

#include "cpLog.h"
#include "Sptr.hxx"
#include "Data.hxx"
#include "VovidaSipStack.hxx"
#include "InviteMsg.hxx"
#include "AckMsg.hxx"
//#include "TimeFunc.hxx"
#include "ByeMsg.hxx"
#include "BaseUrl.hxx"
//#include "VThread.hxx"
//#include "VMutex.h"
#include "StatusMsg.hxx"

Sptr<StatusMsg> makeStatus(Data username, 
			   int localPort, 
			   InviteMsg& msg, 
			   int status)
{
    Sptr<StatusMsg> m = new StatusMsg (msg, status);
    if (status == 200)
    {
	Sptr< SipUrl > myUrl = new SipUrl;
	myUrl->setUserValue( username );
	myUrl->setHost( Data( theSystem.gethostAddress() ) );
	myUrl->setPort( localPort );
	SipContact me;
	me.setUrl(myUrl);
	m->setNumContact( 0 );    // Clear
	m->setContact(me);
    }
    return m;
}


///
int main( int argc, char* argv[] )
{
    if ( argc != 3)
    {
        cerr << "Usage: " << argv[0] << " <local_SIP_port> <username>" << endl;
        exit( -1 );
    }

    cout << "     Local SIP Port : " << argv[1] << endl;
    cout << "     Username : " << argv[2] << endl;
    cpLogSetPriority( LOG_ERR );

    int localport = atoi( argv[1] ) ;
    Data username(argv[2]);

    SipTransceiverFilter* stack = new SipTransceiverFilter( "elgeeReceiver",
                                                            localport,
                                                            0 /* 1 = NAT */,
                                                            APP_CONTEXT_GENERIC );

    while ( 1 )
    {
	Sptr < SipMsgQueue > sipRcv;

        sipRcv = stack->receive();

    	Sptr < SipMsg > sipMsg;
        sipMsg = sipRcv->back();
	if ( sipMsg == 0 )	continue;
	
    	Sptr < InviteMsg > inviteMsg;
        inviteMsg.dynamicCast( sipMsg );
        if ( inviteMsg != 0 )
        {
		Sptr<StatusMsg> status;

		status = makeStatus (username, 
				     localport, 
				     *inviteMsg, 
				     100 );
                stack->sendReply( *status );

		status = makeStatus (username,
				     localport,
				     *inviteMsg,
				     180);
                stack->sendReply( *status );

		status = makeStatus (username,
				     localport,
				     *inviteMsg,
				     200);
                stack->sendReply( *status );
		continue;
	}

    	Sptr < ByeMsg > byeMsg;
	byeMsg.dynamicCast( sipMsg);
        if (byeMsg != 0)
	{
		StatusMsg statusMsg( *byeMsg, 200 );
                stack->sendReply( statusMsg );
		continue;
	}

    	Sptr < AckMsg > ackMsg;
        ackMsg.dynamicCast( sipMsg );
	if ( ackMsg != 0 )
	{
		continue;
	}
    }

    return 0;
}
