
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const SipMsg_cxx_Version = 
    "$Id: SipMsg.cxx,v 1.1 2001/10/18 17:13:28 sprajpat Exp $";

#include "symbols.hxx"         
#include "InviteMsg.hxx"
#include "AckMsg.hxx"
#include "StatusMsg.hxx"
#include "CancelMsg.hxx"
#include "ByeMsg.hxx"
#include "StatusMsg.hxx"
#include "RegisterMsg.hxx"
#include "ReferMsg.hxx"
#include "OptionsMsg.hxx"
#include "InfoMsg.hxx"
#include "TransferMsg.hxx"
#include "SipSubsNotifyEvent.hxx"
#include "SubscribeMsg.hxx"
#include "NotifyMsg.hxx"
#include "UnknownExtensionMsg.hxx"
#include "SipContentDisposition.hxx"
#include "SipMsg.hxx"
#include "SipContentType.hxx"
#include "SipContentLength.hxx"
#include <cassert>

    
SipParserException::SipParserException( 
                              const string& msg,
                              const string& file,
                              const int line,
                              const int error /*Default Arguments*/ )
 : VException( msg, file, line, error )
{
}

string
SipParserException::getName( void ) const
{
    return "SipParserException";
}

SipMsg::SipMsg()
    :
    myVersion(),
    myReceivedAddress(),
    myHeaderList(),
#ifdef USE_BIND
    myMimeList(),
    dnsResolver(),
    dnscount(0)
#else
    myMimeList()
#endif
    
    
{
}


SipMsg::~SipMsg()
{
}
    
    
SipMsg& SipMsg::operator=(const SipMsg& newSipMsg)
{
    if ( &newSipMsg != this) 
    {
	myVersion = newSipMsg.myVersion;
	myReceivedAddress = newSipMsg.myReceivedAddress;
	myHeaderList = newSipMsg.myHeaderList;
        myMimeList = newSipMsg.myMimeList;
#ifdef USE_BIND
        dnsResolver = newSipMsg.dnsResolver;
        dnscount = newSipMsg.dnscount;
#endif
    }
    return *(this);
}


bool
SipMsg::operator<(const SipMsg& src) const
{
    cpLog(LOG_INFO,  "not verified!");
    assert(0);
#if 0

    if(
	( callId < src.callId ) ||
	( cseq < src.cseq ) ||
	( sipto < src.sipto ) ||
	( sipfrom < src.sipfrom ) ||
	( ltViaList(src) )
	)
    {
	return true;
    }
#endif
    return false;
}

   
bool
SipMsg::operator==(const SipMsg& src) const
{
    // i'm not sure if the functionality in the if 0 block is
    // fully implemented.
    bool equal = ((myVersion == src.myVersion) &&
                  (myReceivedAddress == src.myReceivedAddress) &&
                  (myHeaderList == src.myHeaderList));
    cpLog(LOG_DEBUG_STACK, "SipMsg::operator== returns %s", 
	  equal ? "true" : "false");
    return equal;
#if 0
  //compare in the order of importance.
    if ( 
	 ( callId == src.getCallId()) &&
	 ( cseq == src.getCSeq()) &&
	 ( getType() == src.getType()) &&
	 ( sipto == src.getTo()) &&
	 ( sipfrom == src.getFrom()) &&
	 ( mimeversion == src.getMimeVersion())&&
         ( sipdate == src.getDate()) &&
         ( encryption == src.getEncryption()) &&
         ( expires == src.getExpires()) &&
	 ( timestamp == src.getTimestamp()) &&
         ( retryafter == src.getRetryAfter()) &&
         ( retry == src.retry) &&
	 ( also == src.also) &&
         ( transferTo == src.transferTo) &&
	 ( referredBy == src.referredBy) &&
	 ( referTo == src.referTo) &&
	 ( contentLength == src.contentLength) &&
	 ( contentType == src.contentType ) &&
         ( version == src.getVersion()) &&
         ( recvdIPName == src.recvdIPName) &&
         ( recvdIPPort == src.recvdIPPort) &&
         ( compareViaList(src) ) &&
	 ( compareContact(src) ) &&
	 ( compareRecordRoute(src) ) &&
         ( compareAccept(src) ) &&
	 ( compareSupported(src))&&
         ( compareContentEncoding(src) ) &&
	 ( compareAcceptLanguage(src) ) &&
	 ( compareAcceptEncoding(src) ) &&
	 ( compareDiversionList(src) ) &&
	 ( compareSubsNotifyEvent(src) ) &&
	 ( compareAllow(src) ) &&
	 ( compareContentLanguage(src))
#if 0 
	 //this is commented out, this it takes a lot of time to compare the MIME objects. Any work around for this?
	 ( compareContentData(src) ) &&
#endif
       )
    {
	return true;
    }
    else
    {
	return false;
    }
#endif
}

    
SipMsg::SipMsg(const SipMsg& src)
    :
    myVersion(src.myVersion),
    myReceivedAddress(src.myReceivedAddress),
    myHeaderList(src.myHeaderList),
#ifdef USE_BIND
    myMimeList(src.myMimeList),
    dnsResolver(src.dnsResolver),
    dnscount(src.dnscount)
#else
    myMimeList(src.myMimeList)
#endif
{ 
}
    

#if 0
/// 
void SipMsg::copyContact(const SipMsg& src)
{
    vector<SipContact*> newItem = src.getContactList();

    if (newItem.size() > 0)
    {
        vector <SipContact*>::iterator iter;
        iter = newItem.begin();

        if (contactList.size() > 0)
        {
            flushContact();
        }

        //copy
        while (iter != newItem.end())
        {
            if (*iter)
            {
                SipContact* newObject = new SipContact( *(*iter));
                assert(newObject);

                //assign to the vector
	        contactList.push_back(newObject);
            }
            ++iter;
        }
    }
    else
    {
        if (contactList.size() > 0)
        {
            flushContact();
        }
    }
}


///
void SipMsg::flushContact()
{
    vector<SipContact*>::iterator iter = contactList.begin();
    
    while (iter != contactList.end())
    {
        if (*iter)
        {
            delete (*iter);
        }
        ++iter;
    }
    contactList.clear();
}
    
///
Data SipMsg::encodeContact() const
{
    Data encContact = EMPTY;

    //get each element in the . Call encode.
    vector<SipContact*>::const_iterator iter;
   
    iter = contactList.begin();
    if (iter)
    {
   
        while (iter != contactList.end())
        {
            //get the object.
            if  (*iter)
            {
                encContact += (*iter)->encode();
            }
            ++iter;
        }
    }
    return encContact;
}
    

void SipMsg::flushViaList()
{
    vector<SipVia*>::iterator iter = viaList.begin();
    
    while (iter != viaList.end())
    {
        if (*iter)
        {
	    delete (*iter);
        }
        ++iter;
    }
    viaList.clear();

}

const vector<SipVia*>& SipMsg::getViaList() const
{
    return (viaList);
}

void SipMsg::addVia(SipVia* sipvia)
{
    viaList.push_back(sipvia);
}

void SipMsg::copyViaList( const SipMsg& srcMsg) 
{
    
    vector<SipVia*> newviaList = srcMsg.getViaList();

    if (newviaList.size() > 0)
    {
        vector <SipVia*>::iterator iter;
        iter = newviaList.begin();
        
        if (viaList.size() > 0)
        {
            flushViaList();
        }

        //copy
        while (iter != newviaList.end())
        {
            if (*iter)
            {
                SipVia* sipvia = new SipVia;
                assert(sipvia);

                //copy the object
                *sipvia = *(*iter);

                //assign to the vector
                addVia(sipvia);
            }
            ++iter;
        }
    }
    else
    {
        if (viaList.size() > 0)
        {
            flushViaList();
        }
    }
}


Data SipMsg::encodeViaList() const
{
    Data encViaList;

    //get each element in the viaList. Call encode.
    vector<SipVia*>::const_iterator iter;
    
    iter = viaList.begin();
    while (iter != viaList.end())
    {
        //get the object.
        if  (*iter)
        {
            encViaList += (*iter)->encode();
        }
        ++iter;
    }
    return encViaList;
}


bool SipMsg::ltViaList(const SipMsg& src) const
{
    int srcViasize = src.getNumVia();
    int selfViasize = getNumVia();
    
    if ( srcViasize < selfViasize)
    {
        return true;
    }
    else if ( srcViasize > selfViasize )
    {
	return false;
    }
    else
    {
	// sizes are equal, so compare each
        for (int i = 0; i < srcViasize; i++)
	{
	    if ( (getVia(i) < src.getVia(i) ) )
	    {
		return true;
	    }
	    else if ( (src.getVia(i) < getVia(i) ))
	    {
		return false;
	    }
	}
        return false;
    }
}


bool SipMsg::compareViaList(const SipMsg& src) const
{
    int srcViasize = src.getNumVia();
    
    if ( srcViasize != getNumVia() )
    {
        return false;
    }
    else
    {
        for (int i = 0; i < srcViasize; i++)
	{
	    if ( !(getVia(i) == src.getVia(i)) )
	    {
		return false;
	    }
	}
        return true;   
    }
}

///------------------Accept functions-------------------------------

Data SipMsg::encodeAcceptList() const
{
    //get each element in the AcceptList. Call encode.
    vector<SipAccept*>::const_iterator iter;
    Data encAcceptList;

    if (acceptList.size() > 0)
    {
        encAcceptList = ACCEPT;
        encAcceptList+= SP;
        iter = acceptList.begin();

    
        while (iter != acceptList.end())
        {
            //get the object.
            if  (*iter)
            {
                //this is a SipAccept item.
                encAcceptList+= (*iter)->encode();
                //this should call encode of SipAccept.
            }
            ++iter;
            if (iter != acceptList.end())
            {
                encAcceptList+= COMMA;
            }
        }
        encAcceptList+= CRLF;
    }
    return encAcceptList;
}

#endif
    
/// get the number of Accept items 
int SipMsg::getNumAccept() const
{
    return myHeaderList.getNumHeaders(SIP_ACCEPT_HDR);
}
    
/// Get the i'th Accept item. If i is -1, it gets the last one 
const SipAccept&
SipMsg::getAccept( int i /*default argument*/) const
{
    Sptr<SipAccept> x;
    myHeaderList.getParsedHeader(x, SIP_ACCEPT_HDR, i);
    return *x;
#if 0
    int size = acceptList.size();

    if ( ( i < -1 ) || ( i > size) )
    {
        throw SipHeaderException(OUT_OF_RANGE);
    }
    else
    {
        if (i == -1)
        {
           
            return *(acceptList[size -1]);
        }
        else
        {
            return *(acceptList[i]);
        }
    }
#endif

}

/** set or add another Accept itme, if the index is -1, it is appended 
        to the current list */

void SipMsg::setAccept(SipAccept item, int index /*default argument*/ )
{
    Sptr<SipAccept> x = new SipAccept(item);
    myHeaderList.appendHeader(SIP_ACCEPT_HDR, x, index);
#if 0
    int size = acceptList.size();

    if ( (index >= -1) && (index <= size) )
    {
        SipAccept* accept = new SipAccept(item);
        assert(accept);

        if (index == -1)
        {
            acceptList.push_back(accept);
        }
        else if ( index <= size )
        {
	    vector<SipAccept*>::iterator iter;
	    
	    iter = acceptList.begin();
	    iter+=index;
	    //insert it at index.
	    acceptList.insert(iter, accept);
	}
     }
     else
     {
        cpLog(LOG_DEBUG, "index to insert Accept item not within scope");
     }
#endif
}
    
    
/** set or add another Accept itme, if the index is -1, it is appended 
        to the current list */

void SipMsg::setAccept(const Data&item, int index /*default argument*/ )
{
     Sptr<SipAcceptLanguage> x = new SipAcceptLanguage(item);
    myHeaderList.appendHeader(SIP_ACCEPT_LANGUAGE_HDR, x, index);
#if 0
    int size = acceptList.size();

    if ( (index >= -1) && (index <= size) )
    {
        SipAccept* accept = new SipAccept(item);
        assert(accept);

        if (index == -1)
        {
            acceptList.push_back(accept);
        }
        else if ( index <= size )
        {
	    vector<SipAccept*>::iterator iter;
	    
	    iter = acceptList.begin();
	    iter+=index;
	    //insert it at index.
	    acceptList.insert(iter, accept);
	}
     }
     else
     {
        cpLog(LOG_DEBUG, "index to insert Accept item not within scope");
     }
#endif
}
    


/** Set number of Accept items.
	If i is less than current number then the extras are deleted. */
void SipMsg::setNumAccept(int index)
{
    myHeaderList.setNumHeaders(SIP_ACCEPT_HDR, index);
#if 0
    if ( (index < 0) || (index > getNumAccept() ) )
    {
        throw SipHeaderException(OUT_OF_RANGE);
    }
    else
    {
        vector<SipAccept*>::iterator iter;
        iter = acceptList.begin();

        //increment to index
        iter+=index;

        //delete from iter
        while (iter != acceptList.end())
        {
            assert(*iter);
            delete (*iter);

            ++iter;
        }
    }
#endif
}

    
#if 0

void SipMsg::copyAcceptList(const SipMsg& srcMsg)
{

    vector<SipAccept*> newacceptList = srcMsg.getAcceptList();

    if (newacceptList.size() > 0)
    {
        vector <SipAccept*>::iterator iter;
        iter = newacceptList.begin();

        if (acceptList.size() > 0)
        {
            flushAcceptList();
        }

        //copy
        while (iter != newacceptList.end())
        {
            if (*iter)
            {
                SipAccept* sipaccept = new SipAccept;


                //copy the object
                *sipaccept = *(*iter);

                //assign to the vector
                addAccept(sipaccept);
            }
            ++iter;
        }
    }
    else
    {
        if (acceptList.size() > 0)
        {
            flushAcceptList();
        }
    }
}

const vector<SipAccept*>& SipMsg::getAcceptList() const
{

    return (acceptList);
}

void SipMsg::addAccept(SipAccept* sipaccept)
{
    acceptList.push_back(sipaccept);
}

void SipMsg::flushAcceptList()
{
    vector<SipAccept*>::iterator iter = acceptList.begin();
   
    while (iter != acceptList.end())
    {
        if (*iter)
        {
            delete (*iter);
        }
        ++iter;
    }
    acceptList.clear();
}

bool SipMsg::compareAccept(const SipMsg& src) const
{
    int srcAcceptsize = src.getNumAccept();
    
    if ( srcAcceptsize != getNumAccept() )
    {
        return false;
    }
    else
    {
        for (int i = 0; i < srcAcceptsize; i++)
	{
	    if ( !(getAccept(i) == src.getAccept(i)) )
	    {
		return false;
	    }
	}
        return true;   
    }
}


  


// ----------------- AcceptEncoding Header Methods  -----------------
Data SipMsg::encodeAcceptEncodingList() const
{
    //get each element in the AcceptEncodingList. Call encode.
    vector<SipAcceptEncoding*>::const_iterator iter;
    Data encAcceptEncodingList;

    if (acceptencodingList.size() > 0)
    {
        encAcceptEncodingList = ACCEPT_ENCODING;
        encAcceptEncodingList+= SP;
        iter = acceptencodingList.begin();


        while (iter != acceptencodingList.end())
        {
            //get the object.
            if  (*iter)
            {
                //this is a SipAcceptEncoding item.
                encAcceptEncodingList+= (*iter)->encode();
                //this should call encode of SipAcceptEncoding.
            }
            ++iter;
            if (iter != acceptencodingList.end())
            {
                encAcceptEncodingList+= COMMA;
            }
        }
        encAcceptEncodingList+= CRLF;
    }
    return encAcceptEncodingList;
}    
    
#endif
    
/// get the number of AcceptEncoding items 
int SipMsg::getNumAcceptEncoding() const
{ 
    return myHeaderList.getNumHeaders(SIP_ACCEPT_ENCODING_HDR);
}

/// Get the i'th AcceptEncoding item. If i is -1, it gets the last one 
const SipAcceptEncoding&
SipMsg::getAcceptEncoding( int i /*default argument*/ ) const
{
    Sptr<SipAcceptEncoding> x;
    myHeaderList.getParsedHeader(x, SIP_ACCEPT_ENCODING_HDR, i);
    return *x;
#if 0
    int size = acceptencodingList.size();

    if ( ( i < -1 ) || ( i > size)  )
    {
        throw SipHeaderException(OUT_OF_RANGE);
    }
    else
    {
        if (i == -1)
        {
           
            return *(acceptencodingList[size -1]);
        }
        else
        {
            return *(acceptencodingList[i]);
        }
    }
#endif
}

/** set or add another AcceptEncoding itme, if the index is -1, it is appended 
        to the current list */

void SipMsg::setAcceptEncoding(const Data& item, int index /*default argument*/)
{
    Sptr<SipAcceptEncoding> x = new SipAcceptEncoding(item);
    myHeaderList.appendHeader(SIP_ACCEPT_ENCODING_HDR, x, index); 

#if 0
    int size = acceptencodingList.size();

    if ( (index >= -1) && (index <= size) )
    {
        SipAcceptEncoding* acceptencoding = new SipAcceptEncoding(item);
        assert(acceptencoding);
    
        if (index == -1)
        {
            acceptencodingList.push_back(acceptencoding);
        }
        else if ( index <= size ) 
        {
            vector<SipAcceptEncoding*>::iterator iter;

            iter = acceptencodingList.begin();
            iter+=index;
            //insert it at index.
            acceptencodingList.insert(iter, acceptencoding);
        }
     }
#endif
}
    

/** set or add another AcceptEncoding itme, if the index is -1, it is appended 
        to the current list */

void SipMsg::setAcceptEncoding(SipAcceptEncoding item, int index /*default argument*/)
{
    Sptr<SipAcceptEncoding> x = new SipAcceptEncoding(item);
    myHeaderList.appendHeader(SIP_ACCEPT_ENCODING_HDR, x, index);
#if 0
    int size = acceptencodingList.size();

    if ( (index >= -1) && (index <= size) )
    {
        SipAcceptEncoding* acceptencoding = new SipAcceptEncoding(item);
        assert(acceptencoding);
    
        if (index == -1)
        {
            acceptencodingList.push_back(acceptencoding);
        }
        else if ( index <= size ) 
        {
            vector<SipAcceptEncoding*>::iterator iter;

            iter = acceptencodingList.begin();
            iter+=index;
            //insert it at index.
            acceptencodingList.insert(iter, acceptencoding);
        }
     }
#endif
}
    


/** Set number of AcceptEncoding items.
	If i is less than current number then the extras are deleted. */
void SipMsg::setNumAcceptEncoding(int index)
{
    myHeaderList.setNumHeaders(SIP_ACCEPT_ENCODING_HDR, index);
#if 0
    if ( (index < 0) || (index > getNumAccept() ) )
    {
        throw SipHeaderException(OUT_OF_RANGE);
    }
    else
    {
        vector<SipAcceptEncoding*>::iterator iter;
        iter = acceptencodingList.begin();

        //increment to index
        iter+=index;

        //delete from iter
        while (iter != acceptencodingList.end())
        {
            assert(*iter);
            delete (*iter);

            ++iter;
        }
    }
#endif
}

    
#if 0
bool SipMsg::compareAcceptEncoding (const SipMsg& src) const
{
    int srcAcceptEncodingsize = src.getNumAcceptEncoding();

    if ( srcAcceptEncodingsize != getNumAcceptEncoding())
    {
        return false;
    }
    else
    {
        for (int i = 0; i < srcAcceptEncodingsize; i++)
	{
	    if ( !(getAcceptEncoding(i) == src.getAcceptEncoding(i)) )
	    {
	        return false;
	    }
	}
	return true;
    }
}


const vector<SipAcceptEncoding*>& SipMsg::getAcceptEncodingList() const
{
    return (acceptencodingList);
}


void SipMsg::copyAcceptEncodingList( const SipMsg& srcMsg) 
{
    
    vector<SipAcceptEncoding*> newacceptEncList = srcMsg.getAcceptEncodingList();

    if (newacceptEncList.size() > 0)
    {
        vector <SipAcceptEncoding*>::iterator iter;
        iter = newacceptEncList.begin();
        
        if (acceptencodingList.size() > 0)
        {
            flushAcceptEncodingList();
        }

        //copy
        while (iter != newacceptEncList.end())
        {
            if (*iter)
            {
                SipAcceptEncoding* sipacceptEnc = new SipAcceptEncoding;

                //copy the object
                *sipacceptEnc = *(*iter);

                //assign to the vector
                addAcceptEncoding(sipacceptEnc);
            }
            ++iter;
        }
    }
    else
    {
        if (viaList.size() > 0)
        {
            flushAcceptEncodingList();
        }
    }
}


void SipMsg::addAcceptEncoding(SipAcceptEncoding* sipacceptEncoding)
{
    acceptencodingList.push_back(sipacceptEncoding);
}

#endif
    
void SipMsg::flushAcceptEncodingList()
{
    myHeaderList.setNumHeaders(SIP_ACCEPT_ENCODING_HDR, 0);
#if 0
	vector<SipAcceptEncoding*>::iterator iter = acceptencodingList.begin();
       
	//delete from iter
	while (iter != acceptencodingList.end())
	{
	    assert(*iter);
	    delete (*iter);
	    
	    ++iter;
	}
#endif
}

#if 0
// ----------------- AcceptLanguage Header Methods  -----------------
Data SipMsg::encodeAcceptLangList() const
{
    //get each element in the AcceptList. Call encode.
    vector<SipAcceptLanguage*>::const_iterator iter;
    Data encAcceptLangList;

    if (acceptlangList.size() > 0)
    {
        encAcceptLangList = ACCEPT_LANGUAGE;
        encAcceptLangList+= SP;
        iter = acceptlangList.begin();


        while (iter != acceptlangList.end())
        {
            //get the object.
            if  (*iter)
            {
                //this is a SipAcceptLanguage item.
                encAcceptLangList+= (*iter)->encode();
                //this should call encode of SipAcceptLanguage.
            }
            ++iter;
            if (iter != acceptlangList.end())
            {
                encAcceptLangList+= COMMA;
            }
        }
        encAcceptLangList+= CRLF;
    }
    return encAcceptLangList;
}
#endif
    
/// get the number of AcceptLanguage items 
int SipMsg::getNumAcceptLanguage() const
{
    return myHeaderList.getNumHeaders(SIP_ACCEPT_LANGUAGE_HDR);

}

/// Get the i'th AcceptLanguage item. If i is -1, it gets the last one 

const SipAcceptLanguage&
SipMsg::getAcceptLanguage( int i /*default argument*/ ) const
{
    
    Sptr<SipAcceptLanguage> x;
    myHeaderList.getParsedHeader(x, SIP_ACCEPT_LANGUAGE_HDR, i);
    return *x;
    
#if 0
    int size = acceptlangList.size();

    if ( ( i < -1 ) || ( i > size) )
    {
        throw SipHeaderException(OUT_OF_RANGE);
    }
    else
    {
        if (i == -1)
        {
            int size = acceptlangList.size();
            return *(acceptlangList[size -1]);
        }
        else
        {
            return *(acceptlangList[i]);
        }
    }
#endif
}

/** set or add another AcceptLanguage itme, if the index is -1, it is appended 
 to the current list */

void SipMsg::setAcceptLanguage(SipAcceptLanguage item, int index /*default argument*/)
{
    Sptr<SipAcceptLanguage> x = new SipAcceptLanguage(item);
    myHeaderList.appendHeader(SIP_ACCEPT_LANGUAGE_HDR, x, index);
    
#if 0
    int size = acceptlangList.size();

    if ( (index >= -1) && (index <= size) )
    {
        SipAcceptLanguage* acceptlang = new SipAcceptLanguage(item);
        assert(acceptlang);

        if (index == -1)
        {
            acceptlangList.push_back(acceptlang);
        }
        else if ( index <= size )
        {
            vector<SipAcceptLanguage*>::iterator iter;
           
            iter = acceptlangList.begin();
            iter+=index;
            //insert it at index.
            acceptlangList.insert(iter,acceptlang);
        }
    }
    else
    {
        cpLog(LOG_DEBUG, "index to insert AcceptLanguage item not within scope");
    }
#endif
}
    
    
void SipMsg::setAcceptLanguage(const Data& item, int index /*default argument*/)
{
    Sptr<SipAcceptLanguage> x = new SipAcceptLanguage(item);
    myHeaderList.appendHeader(SIP_ACCEPT_LANGUAGE_HDR, x, index);
    
#if 0
    int size = acceptlangList.size();

    if ( (index >= -1) && (index <= size) )
    {
        SipAcceptLanguage* acceptlang = new SipAcceptLanguage(item);
        assert(acceptlang);

        if (index == -1)
        {
            acceptlangList.push_back(acceptlang);
        }
        else if ( index <= size )
        {
            vector<SipAcceptLanguage*>::iterator iter;
           
            iter = acceptlangList.begin();
            iter+=index;
            //insert it at index.
            acceptlangList.insert(iter,acceptlang);
        }
    }
    else
    {
        cpLog(LOG_DEBUG, "index to insert AcceptLanguage item not within scope");
    }
#endif
}    



/** Set number of AcceptLanguage items.
	If i is less than current number then the extras are deleted. */
void SipMsg::setNumAcceptLanguage(int index)
{
    myHeaderList.setNumHeaders(SIP_ACCEPT_LANGUAGE_HDR, index);
#if 0
    int size = acceptlangList.size();

    if ( (index < 0) || (index > size) )
    {
        throw SipHeaderException(OUT_OF_RANGE);
    }
    else
    {
        vector<SipAcceptLanguage*>::iterator iter;
        iter = acceptlangList.begin();

        //increment to index
        iter+=index;

        //delete from iter
        while (iter != acceptlangList.end())
        {
            assert(*iter);
            delete (*iter);

            ++iter;
        }
    }
#endif
}

#if 0
const vector<SipAcceptLanguage*>& SipMsg::getAcceptLangList() const
{
    return (acceptlangList);
}

void SipMsg::addAcceptLang(SipAcceptLanguage* sipacceptlang)
{
    acceptlangList.push_back(sipacceptlang);
}
#endif

void SipMsg::flushAcceptLanguageList()
{
   myHeaderList.setNumHeaders(SIP_ACCEPT_LANGUAGE_HDR, 0);
    
#if 0
    vector<SipAcceptLanguage*>::iterator iter =  acceptlangList.begin();
    while (iter != acceptlangList.end())
    {
        if (*iter)
        {
            delete (*iter);
        }
        ++iter;
    }
    acceptlangList.clear();
#endif
}

#if 0
void SipMsg::copyAcceptLangList( const SipMsg& srcMsg)
{

    vector<SipAcceptLanguage*> newacceptlangList = srcMsg.getAcceptLangList();

    if (newacceptlangList.size() > 0)
    {
        vector <SipAcceptLanguage*>::iterator iter;
        iter = newacceptlangList.begin();

        if (acceptlangList.size() > 0)
        {
            flushAcceptLanguageList();
        }

        //copy
        while (iter != newacceptlangList.end())
        {
            if (*iter)
            {
                SipAcceptLanguage* sipacceptlang = new SipAcceptLanguage;


                //copy the object
                *sipacceptlang = *(*iter);

                //assign to the vector
                addAcceptLang(sipacceptlang);
            }
            ++iter;
        }
    }
    else
    {
        if (acceptlangList.size() > 0)
        {
            flushAcceptLanguageList();
        }
    }
}

bool SipMsg::compareAcceptLanguage (const SipMsg& src) const
{
    int srcAcceptLanguagesize = src.getNumAcceptLanguage();

    if ( srcAcceptLanguagesize != getNumAcceptLanguage())
    {
        return false;
    }
    else
    {
        for (int i = 0; i < srcAcceptLanguagesize; i++)
	{
	    if ( !(getAcceptLanguage(i) == src.getAcceptLanguage(i)) )
	    {
	        return false;
	    }
	}
	return true;
    }
}
#endif


/*--------------------------- SipSubsNotifyEvent ---------------------------*/


int SipMsg::getNumSubsNotifyEvent() const 
{
    return myHeaderList.getNumHeaders(SIP_SUBSCRIPTION_NOTIFY_HDR);
}


const SipSubsNotifyEvent&
SipMsg::getSubsNotifyEvent(int i) const
{
    Sptr<SipSubsNotifyEvent> x;
    myHeaderList.getParsedHeader(x, SIP_SUBSCRIPTION_NOTIFY_HDR, i);
    return *x;
}


void SipMsg::setSubsNotifyEvent(SipSubsNotifyEvent item, int index)
{
    Sptr<SipSubsNotifyEvent> x = new SipSubsNotifyEvent(item);
    myHeaderList.appendHeader(SIP_SUBSCRIPTION_NOTIFY_HDR, x, index);
}


void SipMsg::setSubsNotifyEvent(const Data& item, int index)
{
    Sptr<SipSubsNotifyEvent> x = new SipSubsNotifyEvent(item);
    myHeaderList.appendHeader(SIP_SUBSCRIPTION_NOTIFY_HDR, x, index);
}


void SipMsg::removeSubsNotifyEvent(int index)
{
    myHeaderList.removeHeader(SIP_SUBSCRIPTION_NOTIFY_HDR, index);
}


void SipMsg::setNumSubsNotifyEvent(int i)
{
    myHeaderList.setNumHeaders(SIP_SUBSCRIPTION_NOTIFY_HDR, i);
}


void SipMsg::flushSubsNotifyEvent()
{
    myHeaderList.setNumHeaders(SIP_SUBSCRIPTION_NOTIFY_HDR, 0);
}


#if 0
///
Data SipMsg::encodeSubsNotifyEvent() const
{
    Data encSubsNotifyEvent = EMPTY;

    //get each element in the . Call encode.
    vector<SipSubsNotifyEvent*>::const_iterator iter;
   
    iter = subsNotifyEventList.begin();
    if (iter)
    {
   
        while (iter != subsNotifyEventList.end())
        {
            //get the object.
            if  (*iter)
            {
                encSubsNotifyEvent += (*iter)->encode();
            }
            ++iter;
        }
    }
    return encSubsNotifyEvent;
}
#endif


void SipMsg::copySubsNotifyEvent(const SipMsg& srcObject)
{
    Sptr <SipRawHeader> hdr 
        = srcObject.myHeaderList.getHeader(SIP_SUBSCRIPTION_NOTIFY_HDR);
    Sptr <SipRawHeader> copy;
    if(hdr != 0)
    {
        copy = hdr->duplicate();
    }
    myHeaderList.replaceHeader(SIP_SUBSCRIPTION_NOTIFY_HDR, copy);
}


#if 0
///
const vector<SipSubsNotifyEvent*>& SipMsg::getSubsNotifyEventList() const
{
    return (subsNotifyEventList);
}
 
///
bool SipMsg::compareSubsNotifyEvent(const SipMsg& src) const
{
    int srcSubsNotifyEventsize = src.getNumSubsNotifyEvent();

    if ( srcSubsNotifyEventsize != getNumSubsNotifyEvent())
    {
        return false;
    }
    else
    {
	for (int i = 0; i < srcSubsNotifyEventsize; i++)
	{
	    if ( !(getSubsNotifyEvent(i) == src.getSubsNotifyEvent(i)) )
	    {
	        return false;
	    }
	}
	return true;
    }

}
#endif


// ----------------- CSeq Header Methods ------------------
/// Get the current CSeq header 
const SipCSeq&
SipMsg::getCSeq() const
{
    Sptr<SipCSeq> cseq;
    myHeaderList.getParsedHeader(cseq, SIP_CSEQ_HDR);
    return *cseq;
}
    

/// Set the CSeq header 
void SipMsg::setCSeq( const SipCSeq& newcseq)
{
    Sptr<SipCSeq> cseq;
    myHeaderList.getParsedHeader(cseq, SIP_CSEQ_HDR);
    *cseq = newcseq;
}

    
void SipMsg::setCSeq( const Data& newcseq)
{
    Sptr<SipCSeq> cseq;
    myHeaderList.getParsedHeader(cseq, SIP_CSEQ_HDR);
    cseq->decode(newcseq);
}    
    

// ----------------- CallId Header Methods ------------------
/// Get the current CallId header 
const SipCallId& 
SipMsg::getCallId() const
{
    Sptr<SipCallId> callId;
    myHeaderList.getParsedHeader(callId, SIP_CALLID_HDR);
    return *callId;
}

/// Set the CallId header 
void SipMsg::setCallId( const SipCallId& newcallId)
{
    Sptr<SipCallId> callId;
    myHeaderList.getParsedHeader(callId, SIP_CALLID_HDR);
    *callId = newcallId;
}

    
void SipMsg::setCallId( const Data& data)
{
    Sptr<SipCallId> callId;
    myHeaderList.getParsedHeader(callId, SIP_CALLID_HDR);
    callId->decode(data);
}
    
// ----------------- Also Header Methods ------------------
/// Get the current Also header 
const SipAlso& 
SipMsg::getAlso() const
{
    Sptr<SipAlso> also;
    myHeaderList.getParsedHeader(also, SIP_ALSO_HDR);
    return *also;
}

/// Set the Also header 
void SipMsg::setAlso( const SipAlso& newalso)
{
    Sptr<SipAlso> also;
    myHeaderList.getParsedHeader(also, SIP_ALSO_HDR);
    *also = newalso;
}
    

/// Set the Also header 
void SipMsg::setAlso( const Data& newalso)
{
    Sptr<SipAlso> also;
    myHeaderList.getParsedHeader(also, SIP_ALSO_HDR);
    also->decode(newalso);
}

    
// ----------------- CallLeg Header Methods ------------------
/// Get the current CallLeg header 
SipCallLeg
SipMsg::computeCallLeg() const
{
    SipCallLeg callLeg(getFrom(), getTo(), getCallId());
    return callLeg;
//form callleg from from, to , callid and return.
}


int SipMsg::getNumContact() const
{
    return myHeaderList.getNumHeaders(SIP_CONTACT_HDR);
}


const SipContact&
SipMsg::getContact( int i /*default argument*/) const
{
    Sptr<SipContact> x;
    myHeaderList.getParsedHeader(x, SIP_CONTACT_HDR, i);
    return *x;
}


void SipMsg::setContact(SipContact item, int index /*default argument*/ )
{
    Sptr<SipContact> x = new SipContact(item);
    myHeaderList.appendHeader(SIP_CONTACT_HDR, x, index);
}

void SipMsg::setContact(const Data& item, int index /*default argument*/ )
{
    Sptr<SipContact> x = new SipContact(item);
    myHeaderList.appendHeader(SIP_CONTACT_HDR, x, index);
}
    

void SipMsg::setNumContact(int index)
{
    myHeaderList.setNumHeaders(SIP_CONTACT_HDR, index);
}


SipContactList SipMsg::getContactList() const
{
    SipContactList tmp;
    myHeaderList.getHeaderList(&tmp, SIP_CONTACT_HDR);
    return tmp;
}
    

#if 0


bool SipMsg::compareContact(const SipMsg& src) const
{
    int srcContactsize = src.getNumContact();
   

    if ( srcContactsize != getNumContact())
    {
        return false;
    }
    else
    {
	for (int i = 0; i < srcContactsize; i++)
	{
	    if ( !(getContact(i) == src.getContact(i)) )
	    {
	        return false;
	    }
	}
	return true;
    }
}

// ----------------- ContentEncoding Header Methods  -----------------
/// get the number of ContentEncoding items 
int SipMsg::getNumContentEncoding() const
{
    return contentencodingList.size();
}

/// Get the i'th ContentEncoding item. If i is -1, it gets the last one 
const SipContentEncoding&
SipMsg::getContentEncoding( int i /*default argument*/) const
{
    int size = contentencodingList.size();

    if ( ( i < -1 ) || ( i > size) )
    {
        throw SipHeaderException(OUT_OF_RANGE);
    }
    else
    {
        if (i == -1)
        {
            
            return *(contentencodingList[size -1]);
        }
        else
        {
            return *(contentencodingList[i]);
        }
    }

}
    
    
/** set or add another ContentEncoding itme, if the index is -1, it is appended 
      to the current list */
void SipMsg::setContentEncoding(SipContentEncoding item, int index)
{
    int size = contentencodingList.size();

    if ( (index >= -1) && (index <= size) )
    {
          SipContentEncoding* contentencoding = new SipContentEncoding(item);
          assert(contentencoding);

          if (index == -1)
          {
                contentencodingList.push_back(contentencoding);
          }
          else if ( index <= size)
          {
               vector<SipContentEncoding*>::iterator iter;

               iter = contentencodingList.begin();
               iter+=index;
               //insert it at index.
               contentencodingList.insert(iter, contentencoding);
          }
    }
    else
    {
          cpLog(LOG_DEBUG, "index to insert ContentEncoding item not within scope");
    }
}

    
/** set or add another ContentEncoding itme, if the index is -1, it is appended 
      to the current list */
void SipMsg::setContentEncoding(const Data& item, int index)
{
    int size = contentencodingList.size();

    if ( (index >= -1) && (index <= size) )
    {
          SipContentEncoding* contentencoding = new SipContentEncoding(item);
          assert(contentencoding);

          if (index == -1)
          {
                contentencodingList.push_back(contentencoding);
          }
          else if ( index <= size)
          {
               vector<SipContentEncoding*>::iterator iter;

               iter = contentencodingList.begin();
               iter+=index;
               //insert it at index.
               contentencodingList.insert(iter, contentencoding);
          }
    }
    else
    {
          cpLog(LOG_DEBUG, "index to insert ContentEncoding item not within scope");
    }
}

    
    
/** Set number of ContentEncoding items.
If i is less than current number then the extras are deleted. */
void SipMsg::setNumContentEncoding(int index)
{
    if ( (index < 0) || (index > ( static_cast<int>(contentencodingList.size())) ) )
    {
        throw SipHeaderException(OUT_OF_RANGE);
    }
    else
    {
        vector<SipContentEncoding*>::iterator iter;
        iter = contentencodingList.begin();

        //increment to index
        iter+=index;

        //delete from iter
        while (iter != contentencodingList.end())
        {
            assert(*iter);
            delete (*iter);

            ++iter;
        }
    }

}
    

   void SipMsg::copyContentEncoding( const SipMsg& srcMsg) 
{
    
    vector<SipContentEncoding*> newcontentencodingList = srcMsg.getContentEncodingList();

    if (newcontentencodingList.size() > 0)
    {
        vector <SipContentEncoding*>::iterator iter;
        iter = newcontentencodingList.begin();
        
        if (contentencodingList.size() > 0)
        {
            flushContentEncodingList();
        }

        //copy
        while (iter != newcontentencodingList.end())
        {
            if (*iter)
            {
                SipContentEncoding* sipcontentencoding = new SipContentEncoding;
                

                //copy the object
                *sipcontentencoding = *(*iter);

                //assign to the vector
                addContentEncoding(sipcontentencoding);
            }
            ++iter;
        }
    }
    else
    {
        if (contentencodingList.size() > 0)
        {
            flushContentEncodingList();
        }
    }
}


void SipMsg::flushContentEncodingList()
{
    vector<SipContentEncoding*>::iterator iter = contentencodingList.begin();
   
    while (iter != contentencodingList.end())
    {
        if (*iter)
        {
	    delete (*iter);
        }
        ++iter;
    }
    contentencodingList.clear();
}

const vector<SipContentEncoding*>& SipMsg::getContentEncodingList() const
{
    return (contentencodingList);
}

void SipMsg::addContentEncoding(SipContentEncoding* sipcontentencoding)
{
    contentencodingList.push_back(sipcontentencoding);
}



Data 
SipMsg::encodeContentEncodingList() const
{
 //get each element in the ContentEncodingList. Call encode.
    vector<SipContentEncoding*>::const_iterator iter;
    Data encContentEncodingList;

    if (contentencodingList.size() > 0)
    {
        encContentEncodingList = CONTENT_ENCODING;
        encContentEncodingList+= SP;
        iter = contentencodingList.begin();


        while (iter != contentencodingList.end())
        {
            //get the object.
            if  (*iter)
            {
                //this is a contentencoding item.
                encContentEncodingList+= (*iter)->encode();
                //this should call encode of SipContentEncoding.
            }
            ++iter;
            if (iter != contentencodingList.end())
            {   
	      encContentEncodingList+= COMMA;
	       
            }
        }
        encContentEncodingList+= CRLF;
    }
    return encContentEncodingList;
} 
// end of content encoding functions.

bool SipMsg::compareContentEncoding (const SipMsg& src) const
{
    int srcContentEncodingsize = src.getNumContentEncoding();

    if ( srcContentEncodingsize != getNumContentEncoding())
    {
        return false;
    }
    else
    {
        for (int i = 0; i < srcContentEncodingsize; i++)
	{
	    if ( !(getContentEncoding(i) == src.getContentEncoding(i)) )
	    {
	        return false;
	    }
	}
	return true;
    }
}

#endif


// ----------------- Date Header Methods ------------------
/// Get the current Date header 
const SipDate&
SipMsg::getDate() const
{
    Sptr<SipDate> date;
    myHeaderList.getParsedHeader(date, SIP_DATE_HDR);
    return *date;
}
    
    
/// Set the Date header 
void SipMsg::setDate( const SipDate& newdate)
{
    Sptr<SipDate> date;
    myHeaderList.getParsedHeader(date, SIP_DATE_HDR);
    *date = newdate;
}


/// Set the Date header 
void SipMsg::setDate( const Data& newdate)
{
    Sptr<SipDate> date;
    myHeaderList.getParsedHeader(date, SIP_DATE_HDR);
    date->decode(newdate);
}

// ----------------- Encryption Header Methods ------------------
/// Get the current Encryption header 
const SipEncryption&
SipMsg::getEncryption() const
{
    Sptr<SipEncryption> encryption;
    myHeaderList.getParsedHeader(encryption, SIP_ENCRYPTION_HDR);
    return *encryption;
}
/// Set the Encryption header 
void  SipMsg::setEncryption(const SipEncryption&  newencryption)
{
    Sptr<SipEncryption> encryption;
    myHeaderList.getParsedHeader(encryption, SIP_ENCRYPTION_HDR);
    *encryption = newencryption;
}
    
/// Set the Encryption header 
void  SipMsg::setEncryption(const Data&  newencryption)
{
    Sptr<SipEncryption> encryption;
    myHeaderList.getParsedHeader(encryption, SIP_ENCRYPTION_HDR);
    encryption->decode(newencryption);
}  
    

// ----------------- Expires Header Methods ------------------
/// Get the current Expires header 
const SipExpires& 
SipMsg::getExpires() const
{
    Sptr<SipExpires> expires;
    myHeaderList.getParsedHeader(expires, SIP_EXPIRES_HDR);
    return *expires;
}

    
/// Set the Expires header 
void SipMsg::setExpires( const SipExpires& newexpires)
{
    Sptr<SipExpires> expires;
    myHeaderList.getParsedHeader(expires, SIP_EXPIRES_HDR);
    *expires = newexpires;
}
    
    
/// Set the Expires header 
void SipMsg::setExpires( const Data& newexpires)
{
    Sptr<SipExpires> expires;
    myHeaderList.getParsedHeader(expires, SIP_EXPIRES_HDR);
    expires->decode(newexpires);
}  
    

// ----------------- From Header Methods ------------------
/// Get the current From header 
const SipFrom&
SipMsg::getFrom() const
{
    Sptr<SipFrom> from;
    myHeaderList.getParsedHeader(from, SIP_FROM_HDR);
    return *from;
}
    
    
const SipMimeVersion&
SipMsg::getMimeVersion() const
{
    Sptr<SipMimeVersion> mimeVersion;
    myHeaderList.getParsedHeader(mimeVersion, SIP_MIME_VERSION_HDR);
    return *mimeVersion;
}
    
    
void SipMsg::setMimeVersion( const SipMimeVersion& newmv)
{
    Sptr<SipMimeVersion> mimeVersion;
    myHeaderList.getParsedHeader(mimeVersion, SIP_MIME_VERSION_HDR);
    *mimeVersion = newmv;
}   
    
    
void SipMsg::setMimeVersion( const Data& newmv)
{
    Sptr<SipMimeVersion> mimeVersion;
    myHeaderList.getParsedHeader(mimeVersion, SIP_MIME_VERSION_HDR);
    mimeVersion->decode(newmv);
}


/// Set the From header 
void SipMsg::setFrom( const SipFrom& newfrom)
{
    Sptr<SipFrom> from;
    myHeaderList.getParsedHeader(from, SIP_FROM_HDR);
    *from = newfrom;
}
    
    
void SipMsg::setFrom( const Data& formData)
{
    Sptr<SipFrom> from;
    myHeaderList.getParsedHeader(from, SIP_FROM_HDR);
    from->decode(formData);
}


// ----------------- RecordRoute Header Methods  -----------------


void SipMsg::copyRecordRoute( const SipMsg& srcMsg) 
{
    Sptr <SipRawHeader> hdr = srcMsg.myHeaderList.getHeader(SIP_RECORD_ROUTE_HDR);
    Sptr <SipRawHeader> copy;
    if(hdr != 0)
    {
        copy = hdr->duplicate();
    }
    myHeaderList.replaceHeader(SIP_RECORD_ROUTE_HDR, copy);
}


void SipMsg::flushrecordrouteList()
{
    myHeaderList.setNumHeaders(SIP_RECORD_ROUTE_HDR, 0);
}


SipRecordRouteList SipMsg::getrecordrouteList() const
{
    SipRecordRouteList tmp;
    myHeaderList.getHeaderList(&tmp, SIP_RECORD_ROUTE_HDR);
    return tmp;
}


#if 0


void SipMsg::addRecordRoute(SipRecordRoute* siprecordroute)
{
    recordrouteList.push_back(siprecordroute);
}



Data 
SipMsg::encodeRecordRouteList() const
{
 //get each element in the RecordRouteList. Call encode.
    vector<SipRecordRoute*>::const_iterator iter;
    Data encRecordRouteList;

    if (recordrouteList.size() > 0)
    {
        encRecordRouteList = RECORD_ROUTE;
        encRecordRouteList+= SP;
        iter = recordrouteList.begin();


        while (iter != recordrouteList.end())
        {
            //get the object.
            if  (*iter)
            {
                //this is a RecordRoute item.
                encRecordRouteList+= (*iter)->encode();
                //this should call encode of SipRecordRoute.
            }
            ++iter;
            if (iter != recordrouteList.end())
            {   
	      encRecordRouteList+= COMMA;
            }
        }
        encRecordRouteList+= CRLF;
    }
    return encRecordRouteList;
}

#endif


int SipMsg::getNumRecordRoute() const
{
    return myHeaderList.getNumHeaders(SIP_RECORD_ROUTE_HDR);
}


const SipRecordRoute&
SipMsg::getRecordRoute( int i /*default Arguments*/) const
{
    Sptr<SipRecordRoute> x;
    myHeaderList.getParsedHeader(x, SIP_RECORD_ROUTE_HDR, i);
    return *x;
}


void SipMsg::setRecordRoute(SipRecordRoute item, int index /*Default Arguments*/)
{
    Sptr<SipRecordRoute> x = new SipRecordRoute(item);
    myHeaderList.appendHeader(SIP_RECORD_ROUTE_HDR, x, index);
}

void SipMsg::setRecordRoute(const Data& item, int index /*Default Arguments*/)
{
    Sptr<SipRecordRoute> x = new SipRecordRoute(item);
    myHeaderList.appendHeader(SIP_RECORD_ROUTE_HDR, x, index);
}


void SipMsg::setNumRecordRoute(int index)
{
    myHeaderList.setNumHeaders(SIP_RECORD_ROUTE_HDR, index);
}


bool SipMsg::compareRecordRoute(const SipMsg& src) const
{
    int srcRecordRoutesize = src.getNumRecordRoute();

    if ( srcRecordRoutesize != getNumRecordRoute())
    {
        return false;
    }
    else
    {
        for (int i = 0; i < srcRecordRoutesize; i++)
	{
	    if ( !(getRecordRoute(i) == src.getRecordRoute(i)) )
	    {
	        return false;
	    }
	}
	return true;
    }
}

#if 0

///---------------------Supported Header Methods--------------//

void SipMsg::copySupported( const SipMsg& srcMsg) 
{
    
    vector<SipSupported*> newsupportedList = srcMsg.getsupportedList();

    if (newsupportedList.size() > 0)
    {
        vector <SipSupported*>::iterator iter;
        iter = newsupportedList.begin();
        
        if (supportedList.size() > 0)
        {
            flushsupportedList();
        }

        //copy
        while (iter != newsupportedList.end())
        {
            if (*iter)
            {
                SipSupported* sipsupported = new SipSupported;
                

                //copy the object
                *sipsupported = *(*iter);

                //assign to the vector
                addSupported(sipsupported);
            }
            ++iter;
        }
    }
    else
    {
        if (supportedList.size() > 0)
        {
            flushsupportedList();
        }
    }
}


void SipMsg::flushsupportedList()
{
    vector<SipSupported*>::iterator iter = supportedList.begin();;
    
    while (iter != supportedList.end())
    {
        if (*iter)
        {
	    delete (*iter);
        }
        ++iter;
    }
    supportedList.clear();

}

const vector<SipSupported*>& SipMsg::getsupportedList() const
{
    return (supportedList);
}

void SipMsg::addSupported(SipSupported* sipsupported)
{
    supportedList.push_back(sipsupported);
}



Data 
SipMsg::encodeSupportedList() const
{
 //get each element in the SupportedList. Call encode.
    vector<SipSupported*>::const_iterator iter;
    Data encSupportedList;

    if (supportedList.size() > 0)
    {
        encSupportedList = SUPPORTED;
        encSupportedList+= SP;
        iter = supportedList.begin();


        while (iter != supportedList.end())
        {
            //get the object.
            if  (*iter)
            {
                //this is a Supported item.
                encSupportedList+= (*iter)->encode();
                //this should call encode of SipSupported.
            }
            ++iter;
            if (iter != supportedList.end())
            {   
	      encSupportedList+= COMMA;
            }
        }
    }
    if (encSupportedList.length())
    {
	encSupportedList+= CRLF;
    }
    return encSupportedList;
}

/// get the number of Supported items 
int SipMsg::getNumSupported() const
{
   return supportedList.size();

}
/// Get the i'th Supported item. If i is -1, it gets the last one 
const SipSupported&
SipMsg::getSupported( int i /*default Arguments*/) const
{
    int size = supportedList.size(); 

    if ( ( i < -1 ) || ( i > size) )
    {
        throw SipHeaderException(OUT_OF_RANGE);
    }
    else
    {
        if (i == -1)
        {
           
            return *(supportedList[size -1]);
        }
        else
        {
            return *(supportedList[i]);
        }
    }
}



void SipMsg::setSupported(SipSupported item, int index )
{

    if (index >= -1)
    {
        SipSupported* newSupported = new SipSupported(item);
	

        if (index == -1)
        {
            supportedList.push_back(newSupported);
        }
        else if ( index <= static_cast<int>(supportedList.size()) )
        {
            vector<SipSupported*>::iterator iter;
            
            iter = supportedList.begin();
            iter+=index;
            //insert it at index. 
            supportedList.insert(iter , newSupported);
        }
    }
}
    
    
void SipMsg::setSupported(const Data&item, int index )
{

    if (index >= -1)
    {
        SipSupported* newSupported = new SipSupported(item);
	

        if (index == -1)
        {
            supportedList.push_back(newSupported);
        }
        else if ( index <= static_cast<int>(supportedList.size()) )
        {
            vector<SipSupported*>::iterator iter;
            
            iter = supportedList.begin();
            iter+=index;
            //insert it at index. 
            supportedList.insert(iter , newSupported);
        }
    }
}
    

/** Set number of Supported items.
If i is less than current number then the extras are deleted. */
void SipMsg::setNumSupported(int index)
{
    if ( (index < 0) || (index > (static_cast<int>(supportedList.size())) ) )
    {
        throw SipHeaderException(OUT_OF_RANGE);
    }
    else
    {
        vector<SipSupported*>::iterator iter;
        iter = supportedList.begin();

        //increment to index
        iter+=index;

        //delete from iter
        while (iter != supportedList.end())
        {
            assert(*iter);
            delete (*iter);

            ++iter;
        }
    }

}


bool SipMsg::compareSupported(const SipMsg& src) const
{
    int srcSupportedsize = src.getNumSupported();
    
    if ( srcSupportedsize != getNumSupported() )
    {
        return false;
    }
    else
    {
        for (int i = 0; i < srcSupportedsize; i++)
	{
	    if ( !(getSupported(i) == src.getSupported(i)) )
	    {
		return false;
	    }
	}
        return true;   
    }
}

///--------------------------Content Language Methods -----------------------


void SipMsg::copyContentLanguage( const SipMsg& srcMsg) 
{
    
    vector<SipContentLanguage*> newContentLanguageList = srcMsg.getContentLanguageList();

    if (newContentLanguageList.size() > 0)
    {
        vector <SipContentLanguage*>::iterator iter;
        iter = newContentLanguageList.begin();
        
        if (contentlanguageList.size() > 0)
        {
            flushContentLanguageList();
        }

        //copy
        while (iter != newContentLanguageList.end())
        {
            if (*iter)
            {
                SipContentLanguage* sipContentLanguage = new SipContentLanguage;
                

                //copy the object
                *sipContentLanguage = *(*iter);

                //assign to the vector
                addContentLanguage(sipContentLanguage);
            }
            ++iter;
        }
    }
    else
    {
        if (contentlanguageList.size() > 0)
        {
            flushContentLanguageList();
        }
    }
}


void SipMsg::flushContentLanguageList()
{
    vector<SipContentLanguage*>::iterator iter = contentlanguageList.begin();;
    
    while (iter != contentlanguageList.end())
    {
        if (*iter)
        {
	    delete (*iter);
        }
        ++iter;
    }
    contentlanguageList.clear();

}

const vector<SipContentLanguage*>& SipMsg::getContentLanguageList() const
{
    return (contentlanguageList);
}

void SipMsg::addContentLanguage(SipContentLanguage* sipContentLanguage)
{
    contentlanguageList.push_back(sipContentLanguage);
}



Data 
SipMsg::encodeContentLanguageList() const
{
 //get each element in the ContentLanguageList. Call encode.
    vector<SipContentLanguage*>::const_iterator iter;
    Data encContentLanguageList;

    if (contentlanguageList.size() > 0)
    {
        encContentLanguageList = CONTENTLANGUAGE;
        encContentLanguageList+= SP;
        iter = contentlanguageList.begin();


        while (iter != contentlanguageList.end())
        {
            //get the object.
            if  (*iter)
            {
                //this is a ContentLanguage item.
                encContentLanguageList+= (*iter)->encode();
                //this should call encode of SipContentLanguage.
            }
            ++iter;
            if (iter != contentlanguageList.end())
            {   
	      encContentLanguageList+= COMMA;
            }
        }
        encContentLanguageList+= CRLF;
    }
    return encContentLanguageList;
}

/// get the number of ContentLanguage items 
int SipMsg::getNumContentLanguage() const
{
   return contentlanguageList.size();

}
/// Get the i'th ContentLanguage item. If i is -1, it gets the last one 
const SipContentLanguage&
SipMsg::getContentLanguage( int i /*default Arguments*/) const
{
    int size = contentlanguageList.size();

    if ( ( i < -1 ) || ( i > size) )
    {
        throw SipHeaderException(OUT_OF_RANGE);
    }
    else
    {
        if (i == -1)
        {
            
            return *(contentlanguageList[size -1]);
        }
        else
        {
            return *(contentlanguageList[i]);
        }
    }
}

/** set or add another ContentLanguage itme, if the index is -1, it is appended 
    to the current list */
void SipMsg::setContentLanguage(SipContentLanguage item, int index /*Default Arguments*/)
{

    if (index >= -1)
    {
        SipContentLanguage* newcL = new SipContentLanguage(item);
	// assert(newvia);

        if (index == -1)
        {
            contentlanguageList.push_back(newcL);
        }
        else if ( index <= static_cast<int>(contentlanguageList.size()) )
        {
            vector<SipContentLanguage*>::iterator iter;
            
            iter = contentlanguageList.begin();
            iter+=index;
            //insert it at index. 
            contentlanguageList.insert(iter , newcL);
        }
    }
}

    
/** set or add another ContentLanguage itme, if the index is -1, it is appended 
    to the current list */
void SipMsg::setContentLanguage(const Data& item, int index /*Default Arguments*/)
{

    if (index >= -1)
    {
        SipContentLanguage* newcL = new SipContentLanguage(item);
	// assert(newvia);

        if (index == -1)
        {
            contentlanguageList.push_back(newcL);
        }
        else if ( index <= static_cast<int>(contentlanguageList.size()) )
        {
            vector<SipContentLanguage*>::iterator iter;
            
            iter = contentlanguageList.begin();
            iter+=index;
            //insert it at index. 
            contentlanguageList.insert(iter , newcL);
        }
    }
}
    

/** Set number of ContentLanguage items.
If i is less than current number then the extras are deleted. */
void SipMsg::setNumContentLanguage(int index)
{
    if ( (index < 0) || (index > (static_cast<int>(contentlanguageList.size())) ) )
    {
        throw SipHeaderException(OUT_OF_RANGE);
    }
    else
    {
        vector<SipContentLanguage*>::iterator iter;
        iter = contentlanguageList.begin();

        //increment to index
        iter+=index;

        //delete from iter
        while (iter != contentlanguageList.end())
        {
            assert(*iter);
            delete (*iter);

            ++iter;
        }
    }

}


bool SipMsg::compareContentLanguage(const SipMsg& src) const
{
    int srcContentLanguagesize = src.getNumContentLanguage();
    
    if ( srcContentLanguagesize != getNumContentLanguage() )
    {
        return false;
    }
    else
    {
        for (int i = 0; i < srcContentLanguagesize; i++)
	{
	    if ( !(getContentLanguage(i) == src.getContentLanguage(i)) )
	    {
		return false;
	    }
	}
        return true;   
    }
}
#endif





///-----------------------------------------------------------------------

// ----------------- Timestamp Header Methods ------------------
/// Get the current Timestamp header 
const SipTimestamp&
SipMsg::getTimestamp() const
{
    Sptr<SipTimestamp> timestamp;
    myHeaderList.getParsedHeader(timestamp, SIP_TIMESTAMP_HDR);
    return *timestamp;    
    
}

/// Set the Timestamp header 
void SipMsg::setTimestamp( const SipTimestamp& newtimestamp)
{
    Sptr<SipTimestamp> timestamp;
    myHeaderList.getParsedHeader(timestamp, SIP_TIMESTAMP_HDR);
    *timestamp = newtimestamp;
    
}

    
/// Set the Timestamp header 
void SipMsg::setTimestamp( const Data& newtimestamp)
{
    Sptr<SipTimestamp> timestamp;
    myHeaderList.getParsedHeader(timestamp, SIP_TIMESTAMP_HDR);
    timestamp->decode(newtimestamp);
}



// ----------------- To Header Methods ------------------
/// Get the current To header 
const SipTo&
SipMsg::getTo() const
{
    Sptr<SipTo> to;
    myHeaderList.getParsedHeader(to, SIP_TO_HDR);
    return *to;
}


/// Set the To header 
void SipMsg::setTo( const SipTo& newto)
{
    Sptr<SipTo> to;
    myHeaderList.getParsedHeader(to, SIP_TO_HDR);
    *to = newto;
}


void SipMsg::setTo( const Data& toData)
{
    Sptr<SipTo> to;
    myHeaderList.getParsedHeader(to, SIP_TO_HDR);
    to->decode(toData);
}

// ----------------- Via Header Methods  -----------------
/// get the number of Via items 
int SipMsg::getNumVia() const
{
    return myHeaderList.getNumHeaders(SIP_VIA_HDR);
}

/// Get the i'th Via item. If i is -1, it gets the last one 
const SipVia&
SipMsg::getVia( int i /*default Arguments*/) const
{
    Sptr<SipVia> x;
    myHeaderList.getParsedHeader(x, SIP_VIA_HDR, i);
    return *x;
}

void SipMsg::removeVia(int index /*deafult Arguments*/)
{
    myHeaderList.removeHeader(SIP_VIA_HDR, index);
}


/** set or add another Via item, if the index is -1, it is appended 
    to the current list */

void SipMsg::setVia(SipVia item, int index /* Default Arguments*/)
{
    Sptr<SipVia> x = new SipVia(item);
    myHeaderList.appendHeader(SIP_VIA_HDR, x, index);
}

void SipMsg::setVia(const Data& item, int index /* Default Arguments*/)
{
    Sptr<SipVia> x = new SipVia(item);
    myHeaderList.appendHeader(SIP_VIA_HDR, x, index);
}


/** Set number of Via items.
If i is less than current number then the extras are deleted. */
void SipMsg::setNumVia(int index)
{
    myHeaderList.setNumHeaders(SIP_VIA_HDR, index);
}


void SipMsg::flushViaList()
{
    myHeaderList.setNumHeaders(SIP_VIA_HDR, 0);
}


void SipMsg::copyViaList( const SipMsg& srcMsg) 
{
    Sptr <SipRawHeader> hdr = srcMsg.myHeaderList.getHeader(SIP_VIA_HDR);
    Sptr <SipRawHeader> copy;
    if(hdr != 0)
    {
        copy = hdr->duplicate();
    }
    myHeaderList.replaceHeader(SIP_VIA_HDR, copy);
}


SipViaList SipMsg::getViaList() const
{
    SipViaList tmp;
    myHeaderList.getHeaderList(&tmp, SIP_VIA_HDR);
    return tmp;
}


#if 0
void SipMsg::addVia(SipVia* sipvia)
{
    viaList.push_back(sipvia);
}


Data SipMsg::encodeViaList() const
{
    Data encViaList;

    //get each element in the viaList. Call encode.
    vector<SipVia*>::const_iterator iter;
    
    iter = viaList.begin();
    while (iter != viaList.end())
    {
        //get the object.
        if  (*iter)
        {
            encViaList += (*iter)->encode();
        }
        ++iter;
    }
    return encViaList;
}


bool SipMsg::ltViaList(const SipMsg& src) const
{
    int srcViasize = src.getNumVia();
    int selfViasize = getNumVia();
    
    if ( srcViasize < selfViasize)
    {
        return true;
    }
    else if ( srcViasize > selfViasize )
    {
	return false;
    }
    else
    {
	// sizes are equal, so compare each
        for (int i = 0; i < srcViasize; i++)
	{
	    if ( (getVia(i) < src.getVia(i) ) )
	    {
		return true;
	    }
	    else if ( (src.getVia(i) < getVia(i) ))
	    {
		return false;
	    }
	}
        return false;
    }
}


bool SipMsg::compareViaList(const SipMsg& src) const
{
    int srcViasize = src.getNumVia();
    
    if ( srcViasize != getNumVia() )
    {
        return false;
    }
    else
    {
        for (int i = 0; i < srcViasize; i++)
	{
	    if ( !(getVia(i) == src.getVia(i)) )
	    {
		return false;
	    }
	}
        return true;   
    }
    else
    {
        vector<SipVia*>::iterator iter;
        iter = viaList.begin();

        //increment to index
        iter+=index;

        //delete from iter
        while (iter != viaList.end())
        {
            assert(*iter);
            delete (*iter);

            ++iter;
        }
	viaList.erase ( viaList.begin() + index, viaList.end() );	   
    }
}
#endif


    
void SipMsg::removeContact(int index /*default Arguments*/)
{
    myHeaderList.removeHeader(SIP_CONTACT_HDR, index);
}
    
    

/*-----------------------------------------------------------------------*/
//Diversion header details
//  getNumDiversion
//  removeDiversion
//  addDiversion
//  copyDiversionList
//  getDiversionList
//  flushDiversionList
//  encodeDiversionList
//  ltDiversionList
//  compareDiversionList
//  setDiversion
//  setNumDiversion
    

int SipMsg::getNumDiversion() const
{
    return myHeaderList.getNumHeaders(SIP_DIVERSION_HDR);
}


void SipMsg::removeDiversion(int index /* Default Arguement */)
{
    myHeaderList.removeHeader(SIP_DIVERSION_HDR, index);
    
#if 0
    if (index >= -1)
    {
        if (index == -1)
        {
            SipDiversion* diversion = diversionList.back();
            if(diversion) delete diversion;
            diversionList.pop_back();
        }
        else if  ( index <= static_cast<int>(diversionList.size()) )
        {
            vector<SipDiversion*>::iterator iter;
	    
            iter = diversionList.begin();
            iter+=index;
	    if(iter != diversionList.end())
	    {
		assert(*iter);
		diversionList.erase(iter);
		delete *iter;
		
	    }
        }
        if (diversionList.size() == 0)
        {
	    diversionList.clear();
        }
    }
#endif
}

#if 0
void SipMsg::addDiversion(SipDiversion* sipdiversion)
{
   
    diversionList.push_back(sipdiversion);
}

#endif


SipDiversionList SipMsg::getDiversionList() const
{
    SipDiversionList tmp;
    myHeaderList.getHeaderList(&tmp, SIP_DIVERSION_HDR);
    return tmp;
} 


void SipMsg::copyDiversionList( const SipMsg& srcMsg) 
{
    myHeaderList.replaceHeader(SIP_DIVERSION_HDR, 
                               srcMsg.myHeaderList.getHeader(SIP_DIVERSION_HDR));
}


void SipMsg::flushDiversionList()
{
    myHeaderList.setNumHeaders(SIP_DIVERSION_HDR, 0);
}


#if 0
bool SipMsg::ltDiversionList(const SipMsg& src) const
{
    int srcDiversionsize = src.getNumDiversion();
    int selfDiversionsize = getNumDiversion();
    
    if ( srcDiversionsize < selfDiversionsize )
    {
        return true;
    }
    else if ( srcDiversionsize > selfDiversionsize )
    {
	return false;
    }
    else
    {
	// sizes are equal, so compare each
        for (int i = 0; i < srcDiversionsize; i++)
	{
	    if ( (getDiversion(i) < src.getDiversion(i) ) )
	    {
		return true;
	    }
	    else if ( (src.getDiversion(i) < getDiversion(i) ))
	    {
		return false;
	    }
	}
        return false;
    }
} 
#endif

const SipDiversion&
SipMsg::getDiversion( int i /*Default Arguments*/) const
{
    Sptr<SipDiversion> x;
    myHeaderList.getParsedHeader(x, SIP_DIVERSION_HDR, i);
    return *x;
#if 0
    if ( ( i < -1 ) || ( i >= (static_cast<int>(diversionList.size())) ) 
	 || (diversionList.size() == 0) )
    {
        throw SipHeaderException(OUT_OF_RANGE);
    }
    else
    { 
	if (i == -1)
	{
	    int size = diversionList.size();
	    return *(diversionList[size -1]);
        } 
        else
        { 
	    return *(diversionList[i]);
        } 
    } 
#endif
} 

#if 0
bool SipMsg::compareDiversionList(const SipMsg& src) const
{
    int srcDiversionsize = src.getNumDiversion();
    
    if ( srcDiversionsize != getNumDiversion() )
    {
        return false;
    }
    else
    {
        for (int i = 0; i < srcDiversionsize; i++)
	{
	    if ( !(getDiversion(i) == src.getDiversion(i)) )
	    {
		return false;
	    }
	}
        return true;   
    }
}
#endif

void SipMsg::setDiversion(SipDiversion item, int index /*Default Arguments*/)
{
     Sptr<SipDiversion> x = new SipDiversion(item);
     myHeaderList.appendHeader(SIP_DIVERSION_HDR, x, index);
#if 0
    if ( (index >= -1) && (index <= static_cast<int>(diversionList.size()) ) )
    {
        SipDiversion* newdiversion = new SipDiversion(item);
        assert(newdiversion);
	
        if (index == -1)
        {
            diversionList.push_back(newdiversion);
        }
        else if ( index <= static_cast<int>(diversionList.size()) )
        {
            vector<SipDiversion*>::iterator iter;
            
            iter = diversionList.begin();
            iter+=index;
            //insert it at index. 
            diversionList.insert(iter , newdiversion);
        }
    }
    else
    {
	cpLog(LOG_DEBUG_STACK, "index to insert Diversion item not within scope");
    }
#endif
}

void SipMsg::setNumDiversion(int index)
{
    myHeaderList.setNumHeaders(SIP_DIVERSION_HDR, index);

#if 0
    if ( (index < 0) || (index > (static_cast<int>(diversionList.size())) ) )
    {
        throw SipHeaderException(OUT_OF_RANGE);
    }
    else
    {
        vector<SipDiversion*>::iterator iter;
        iter = diversionList.begin();
	
        //increment to index
        iter+=index;
	
        //delete from iter
        while (iter != diversionList.end())
        {
            assert(*iter);
            delete (*iter);
	    
            ++iter;
        }
	diversionList.erase ( diversionList.begin() + index, diversionList.end() );	   
    }
#endif
}  

#if 0
Data SipMsg::encodeDiversionList() const
{
    Data encDiversionList;
    
    //get each element in the viaList. Call encode.
    vector<SipDiversion*>::const_iterator iter;
    
    iter = diversionList.begin();
    while (iter != diversionList.end())
    {
        //get the object.
        if  (*iter)
        {
            encDiversionList += (*iter)->encode();
	   
        }
        ++iter;
    }
    return encDiversionList;
}   

#endif 
    
void SipMsg::setVersion(Data ver)
{
    version = ver;
}

Data SipMsg::getVersion() const 
{
    return version;
}

bool SipMsg::isRetry() const
{
    return retry;
}
    
const SipRetryAfter&
SipMsg::getRetryAfter() const
{
    Sptr<SipRetryAfter> retryAfter;
    myHeaderList.getParsedHeader(retryAfter, SIP_RETRY_AFTER_HDR);
    return *retryAfter;
}	

    
void SipMsg::setRetryAfter( const SipRetryAfter& newretry)
{
    Sptr<SipRetryAfter> retryAfter;
    myHeaderList.getParsedHeader(retryAfter, SIP_RETRY_AFTER_HDR);
    *retryAfter  = newretry;
    retry = true;
}
    
    
void SipMsg::setRetryAfter( const Data& newretry)
{
    Sptr<SipRetryAfter> retryAfter;
    myHeaderList.getParsedHeader(retryAfter, SIP_RETRY_AFTER_HDR);
    retryAfter->decode(newretry);
    retry = true;
}
    

///for authentication
Data SipMsg::getReceivedIPName()
{
    return recvdIPName;
}

void SipMsg::setReceivedIPName(Data ipName)
{
    if (ipName.length())
    {
        recvdIPName = ipName;
    }
}

///
Data  SipMsg::getReceivedIPPort()
{
    return  recvdIPPort;
}
    
///
void  SipMsg::setReceivedIPPort(Data ipPort)
{
    recvdIPPort = ipPort;
}

//transfer functions.
const SipTransferTo&
SipMsg::getTransferTo() const
{
    Sptr<SipTransferTo> transferTo;
    myHeaderList.getParsedHeader(transferTo, SIP_TRANSFER_TO_HDR);
    return *transferTo;
}

void SipMsg::setTransferTo(const SipTransferTo& transTo)
{
    Sptr<SipTransferTo> transferTo;
    myHeaderList.getParsedHeader(transferTo, SIP_TRANSFER_TO_HDR);
    *transferTo = transTo;
}
    
void SipMsg::setTransferTo(const Data& transTo)
{
    Sptr<SipTransferTo> transferTo;
    myHeaderList.getParsedHeader(transferTo, SIP_TRANSFER_TO_HDR);
    transferTo->decode(transTo);
} 


void SipMsg::setReferredBy(const SipReferredBy& newreferredBy)
{
    Sptr<SipReferredBy> referredBy;
    myHeaderList.getParsedHeader(referredBy, SIP_REFERRED_BY_HDR);
    *referredBy = newreferredBy;
}

    
void SipMsg::setReferredBy(const Data& newreferredBy)
{
    Sptr<SipReferredBy> referredBy;
    myHeaderList.getParsedHeader(referredBy, SIP_REFERRED_BY_HDR);
    referredBy->decode(newreferredBy);
}
    
    
const SipReferredBy&
SipMsg::getReferredBy() const
{
    Sptr<SipReferredBy> referredBy;
    myHeaderList.getParsedHeader(referredBy, SIP_REFERRED_BY_HDR);
    return *referredBy;
}



void SipMsg::setReferTo(const SipReferTo& newreferTo)
{
    Sptr<SipReferTo> referTo;
    myHeaderList.getParsedHeader(referTo, SIP_REFER_TO_HDR);
    *referTo = newreferTo;
}
    

void SipMsg::setReferTo(const Data& newreferTo)
{
    Sptr<SipReferTo> referTo;
    myHeaderList.getParsedHeader(referTo, SIP_REFER_TO_HDR);
    referTo->decode(newreferTo);
}   
   
    
//transfer functions.
const SipReferTo&
SipMsg::getReferTo() const
{
    Sptr<SipReferTo> referTo;
    myHeaderList.getParsedHeader(referTo, SIP_REFER_TO_HDR);
    return *referTo;
}


/// Get the current UserAgent header
const SipUserAgent&
SipMsg::getUserAgent() const
{
    Sptr<SipUserAgent> userAgent;
    myHeaderList.getParsedHeader(userAgent, SIP_USER_AGENT_HDR);
    return *userAgent;
}

    
/// Set the UserAgent header
void SipMsg::setUserAgent( const SipUserAgent& newUserAgent)
{
    Sptr<SipUserAgent> userAgent;
    myHeaderList.getParsedHeader(userAgent, SIP_USER_AGENT_HDR);
    *userAgent = newUserAgent;
}

    
/// Set the UserAgent header
void SipMsg::setUserAgent( const Data& newUserAgent)
{
    Sptr<SipUserAgent> userAgent;
    myHeaderList.getParsedHeader(userAgent, SIP_USER_AGENT_HDR);
    userAgent->decode(newUserAgent);
}

    

    
void SipMsg::flushAllowList()
{
    myHeaderList.setNumHeaders(SIP_ALLOW_HDR, 0);
    
}

#if 0
const vector < SipAllow* > & SipMsg::getAllowList() const
{
    return (allowList);
}

#endif

void SipMsg::addAllow(SipAllow* sipallow)
{
    assert(0); // this will not work, but is needed!
}
    
void SipMsg::removeAllow(int index /*deafult Arguments*/)
{
    myHeaderList.removeHeader(SIP_ALLOW_HDR, index);
}
    
#if 0
void SipMsg::copyAllowList( const SipMsg& srcMsg)
{

    vector < SipAllow* > newallowList = srcMsg.getAllowList();

    if (newallowList.size() > 0)
    {
        vector < SipAllow* > ::iterator iter;
        iter = newallowList.begin();

        if (allowList.size() > 0)
        {
            flushAllowList();
        }

        //copy
        while (iter != newallowList.end())
        {
            if (*iter)
            {
                SipAllow* sipallow = new SipAllow;
                assert(sipallow);
                //copy the object
                *sipallow = *(*iter);

                //assign to the vector
                addAllow(sipallow);
            }
            ++iter;
        }
    }
    else
    {
        if (allowList.size() > 0)
        {
            flushAllowList();
        }
    }
}
#endif

const SipAllow&
SipMsg::getAllow( int i /* Default Arguments */) const
{
    Sptr<SipAllow> x;
    myHeaderList.getParsedHeader(x, SIP_ALLOW_HDR, i);
    return *x;
    
#if 0
    int allowsize = static_cast < int > (allowList.size());
    if ( ( i < -1 ) || ( i > allowsize ) )
    {
        throw SipHeaderException(OUT_OF_RANGE);
    }
    else
    {
        if (i == -1)
        {
            int size = allowList.size();
            return *(allowList[size - 1]);
        }
        else
        {
            return *(allowList[i]);
        }
    }
#endif
}


void SipMsg::setAllow(SipAllow item, int index /* Default Arguments */ )
{
    Sptr<SipAllow> x = new SipAllow(item);
    myHeaderList.appendHeader(SIP_ALLOW_HDR, x, index);
}

    
void SipMsg::setAllow(const Data&item, int index /* Default Arguments */ )
{
    Sptr<SipAllow> x = new SipAllow(item);
    myHeaderList.appendHeader(SIP_ALLOW_HDR, x, index);
}


int SipMsg::getNumAllow() const
{
     return myHeaderList.getNumHeaders(SIP_ALLOW_HDR);
}


void SipMsg::setNumAllow(int index)
{
    myHeaderList.setNumHeaders(SIP_ALLOW_HDR, index);
}

    
#if 0
bool SipMsg::compareAllow (const SipMsg& src) const
{
    int srcAllowsize = src.getNumAllow();

    if ( srcAllowsize != getNumAllow())
    {
        return false;
    }
    else
    {
        for (int i = 0; i < srcAllowsize; i++)
	{
	    if ( !(getAllow(i) == src.getAllow(i)) )
	    {
	        return false;
	    }
	}
	return true;
    }
}


Data SipMsg::encodeAllowList() const
{
    //run through allowList, and call encode()
    //add CRLF, after each line.
    //put in encodedAllow and return it.

    Data encAllowList;

    //get each element in the allowList. Call encode.
    vector < SipAllow* > ::const_iterator iter;

    iter = allowList.begin();
    if (iter != allowList.end())
    {
        encAllowList = ALLOW;
        encAllowList+= SP;
    }

    while (iter != allowList.end())
    {
        //get the object.
        if (*iter)
        {
            encAllowList += (*iter)->encode();
        }
        ++iter;
	if (iter != allowList.end())
	{
	    //add comma here.
	    encAllowList+= COMMA;
	}
    }
   if (encAllowList.length())
    {
      encAllowList+= CRLF;
    }

    return encAllowList;
}

    
void SipMsg::removeSpaces(Data* data)
{
    Data beforeSpace;
    do
    {
        beforeSpace=EMPTY;
        data->match(SP, &beforeSpace, false);
        if (beforeSpace == SP)
        {
	    //replace with nothing
            data->match(SP, &beforeSpace, true);
        }
        
    }while (beforeSpace == SP);
}

#endif


Method
SipMsg::getMethod(const Data& method)
{
    Method type = SIP_UNKNOWN;

    if (method == INVITE_METHOD)
    {
	type = SIP_INVITE;
    }
    else if (method == CANCEL_METHOD)
    {
	type = SIP_CANCEL;
    }
     else if (method == REFER_METHOD)
    {
	type = SIP_REFER;
    }
    else if (method == ACK_METHOD)
    {
	type = SIP_ACK;
    }
    else if (method == BYE_METHOD)
    {
	type = SIP_BYE;
    }
    else if (method == INFO_METHOD)
    {
	type = SIP_INFO;
    }
    else if (method == OPTIONS_METHOD)
    {
	type = SIP_OPTIONS;
    }
    else if (method == REGISTER_METHOD)
    {
	type = SIP_REGISTER;
    }
    else if (method == TRANSFER_METHOD)
    {
        type = SIP_TRANSFER;
    }
    else if (method == SUBSCRIBE_METHOD)
    {
	type = SIP_SUBSCRIBE;
    }
    else if (method == NOTIFY_METHOD)
    {
	type = SIP_NOTIFY;
    }
    else if (method == "SIP/2.0")
    {
        type = SIP_STATUS;
    }
    
    return type;
}


bool
SipMsg::parse(Data msg)
{
    bool retVal = false;
    retVal = parseHeaders(msg);

    // at this point, msg should only contain the body, if any
    parseMime(msg);
    return retVal;
}
    
    
bool 
SipMsg::parseHeaders(Data& msg)
{
    return myHeaderList.decode(&msg);
}
    

void
SipMsg::parseMime(Data msg)
{
    //need to pass contentlength, contenttype, contentdisposition.
    Sptr < SipContentLength> contlen = 
                               new SipContentLength(getContentLength());
    Sptr <SipContentType> contType = 
                               new SipContentType(getContentType());
    Sptr <SipContentDisposition> contDisp = 
                               new SipContentDisposition(getContentDisposition());
    
    myMimeList.decode(msg,contlen, contType, contDisp);
}


SipMsg*
SipMsg::decode(Data& data)
{
    // this code needs to be rewritten to do the right thing... ?

    SipMsg* sipMsg = NULL;
    cpLog(LOG_DEBUG_STACK,"Going to decode a sip Msg");
    
    data.removeLWS();
    //new parser hooks.
    data.removeSpaces();
    
    //call expand.
    
    int sipMethod;

    Data tmp = data;
    Data startline;

    bool getLineFailed = false;

    startline = tmp.getLine(&getLineFailed);

    if(getLineFailed)
    {
	// this is a bad message
	cpLog(LOG_DEBUG_STACK, "Parse failed.  no newlines found in message");
	return 0;
    }

    char matchedChar = '\0';

    Data method = startline.matchChar(" \t", &matchedChar);

    sipMethod  = SipMsg::getMethod(method);

    if(sipMethod == SIP_UNKNOWN)
    {
        cpLog(LOG_DEBUG_STACK, "failed to match method '%s'", method.getData());
    }


#if 0
    ret = data.match(CRLF, &startline, false);
    if  (ret == FOUND)
    {
	sipMethod  = SipMsg::getMethod(startline);
    }
    else if (ret == FIRST)
    {
	cpLog(LOG_DEBUG_STACK, "wrong syntax");
    }
    else if (ret == NOT_FOUND)
    {
	cpLog(LOG_DEBUG_STACK, "no CRLF, may have LF only");
	int ret1 = data.match(LF, &startline,false);
	if (ret1 == FOUND)
	{
	    sipMethod  = SipMsg::getMethod(startline);
	}
	else  if (ret1 == FIRST)
	{
	    cpLog(LOG_DEBUG_STACK, "wrong syntax");
	}
	else if (ret1 == NOT_FOUND)
	{
	    cpLog(LOG_DEBUG_STACK, "no CRLF or LF alone, may have CR only ");
	    int ret2 = data.match(CR, &startline,false);
	    if (ret2 == FOUND)
	    {
	    	cpLog(LOG_DEBUG_STACK, "Failed in creating the Refer msg");
		sipMethod  = SipMsg::getMethod(startline);
	    }
	    else if (ret2 == FIRST)
	    {
		sipMsg = new NotifyMsg(data);
		cpLog(LOG_DEBUG_STACK, "wrong syntax");
	    }
	    else if (ret2 == NOT_FOUND)
	    {
		cpLog(LOG_ERR, "wrong syntax But Parsing Without CRLF, or LF or CR");
	    }
	      
	}
    } 
#endif

    switch(sipMethod)
    {
    case SIP_INVITE:
	try
	{
	    sipMsg = new InviteMsg(data);
	}
	catch (SipParserException& e)
	{
	    cpLog(LOG_DEBUG_STACK, "Failed in creating the invite msg");
	}
	break;
    case SIP_BYE:
	try
	{
	    sipMsg = new ByeMsg(data);
	}
	catch(SipParserException& e)
	{
	    cpLog(LOG_DEBUG_STACK, "Failed in creating the bye msg");
	}
	break;
    case SIP_CANCEL:
	try
	{
	    sipMsg = new CancelMsg(data);
	}
	catch(SipParserException& e)
	{
	    cpLog(LOG_DEBUG_STACK, "Failed in creating the cancel msg");
	}
	break;
    case SIP_ACK:
	try
	{
	    sipMsg = new AckMsg(data);
	}
	catch(SipParserException& e)
	{
	    cpLog(LOG_DEBUG_STACK, "Failed in creating the Ack msg");	
	}
	break;
    case SIP_STATUS:
	try
	{
	    sipMsg = new StatusMsg(data);
	}
	catch(SipParserException& e)
	{
	    cpLog(LOG_DEBUG_STACK, "Failed in creating the status msg");
	}
	break;
    case SIP_INFO:
	try
	{
	    sipMsg = new InfoMsg(data);
	}
	catch(SipParserException& e)
	{
	    cpLog(LOG_DEBUG_STACK, "Failed in creating the info msg");	
	}
	break;
    case SIP_OPTIONS:
	try
	{
	    sipMsg = new OptionsMsg(data);
	}
	catch(SipParserException& e)
	{
	    cpLog(LOG_DEBUG_STACK, "Failed in creating the options msg");
	}
	break;
    case SIP_REGISTER:
	try
	{
	    sipMsg = new RegisterMsg(data);
	}
	catch(SipParserException& e)
	{
	    cpLog(LOG_DEBUG_STACK, "Failed in creating the Register msg");
	}
	break;
    case SIP_TRANSFER:
	try
	{
	    sipMsg = new TransferMsg(data);
	}
	catch(SipParserException& e)
	{
	    cpLog(LOG_DEBUG_STACK, "Failed in creating the Transfer msg");
	}
	break;
    case SIP_SUBSCRIBE:
	try
	{
	    sipMsg = new SubscribeMsg(data);
	}
	catch(SipParserException& e)
	{
	    cpLog(LOG_DEBUG_STACK, "Failed in creating the Subscribe msg");
	}
	break;
    case SIP_REFER:
	try
	{
	    sipMsg = new ReferMsg(data);
	}
	catch(SipParserException& e)
	{
	    cpLog(LOG_DEBUG_STACK, "Failed in creating the Refer msg");
	}
	break;
    case SIP_NOTIFY:
	try
	{
	    sipMsg = new NotifyMsg(data);
	}
	catch(SipParserException& e)
	{
	    cpLog(LOG_DEBUG_STACK, "Failed in creating the Notify msg");
	}
	break;
    case SIP_UNKNOWN:
	try
	{
	    sipMsg = new UnknownExtensionMsg(data);
	}
	catch(SipParserException& e)
	{
	    cpLog(LOG_DEBUG_STACK, "Failed in creating the UnknownExtension msg");
	}
    default:
        cpLog(LOG_DEBUG_STACK, "NOT A KNOWN SIP MESSAGE");
        cpLog(LOG_INFO, "NOT A KNOWN SIP MESSAGE");

    }//end switch
    
    //print it out here.
    if(sipMsg != 0)
    {
	cpLog(LOG_DEBUG_STACK, 
	      " In sipmsg:     after decoding: %s", 
	      sipMsg->encode().getData());
    }
    
    return sipMsg;
}

/** set or add another ContentLength item, if the index is -1, it is appended 
       to the current list */
void 
SipMsg::setContentLength(const SipContentLength& item ) 
{
    Sptr<SipContentLength> contentLength;
    myHeaderList.getParsedHeader(contentLength, SIP_CONTENT_LENGTH_HDR);
    *contentLength = item;
}

void
SipMsg::encodeBody(Data* msg) const
{
    // need to encode body first to do calculations of length
    Data body;
    myMimeList.encode(&body);


    Sptr <SipContentLength> contLen = myMimeList.getContentLength();
    if (contLen != 0)
    {
        Sptr<SipContentLength> contentLength;
        myHeaderList.getParsedHeader(contentLength, SIP_CONTENT_LENGTH_HDR);
        *contentLength = *contLen;
    }
   
    Sptr <SipContentType> contType = myMimeList.getContentType();
    if (contType != 0)
    {
        Sptr<SipContentType> contentType;
        myHeaderList.getParsedHeader(contentType, SIP_CONTENT_TYPE_HDR);
        *contentType = *(contType);
    }
    
    Sptr <SipContentDisposition> contDisp = myMimeList.getContentDisposition();
    if (contDisp != 0)
    {
        Sptr<SipContentDisposition> contentDisposition;
        myHeaderList.getParsedHeader(contentDisposition, SIP_CONTENT_DISPOSITION_HDR);
        *contentDisposition = *(contDisp);
    }
    
    // encode the body of the message

    myHeaderList.encode(msg);
    *msg += body;
}
    



#if 0

#if USE_HASH_MAP
size_t SipMsg::hashfn() const
{
    return callId.hashfn() ^ cseq.hashfn() ^ getType();
}
#endif
// hash function for a SipMsg:
#if USE_HASH_MAP
size_t hash<SipMsg>::operator()(const SipMsg& key) const
{
    return key.hashfn();
}
#endif


#endif


Sptr<SipMsg> copyPtrtoSptr(SipMsg* sipMsg)
{
    assert(sipMsg);
    Sptr<SipMsg> sptrsipMsg;
    
    switch(sipMsg->getType())
    {
        case SIP_INVITE:
        {
            InviteMsg* inviteMsg = dynamic_cast<InviteMsg*>(sipMsg);
            if (inviteMsg)
            {
                sptrsipMsg = new InviteMsg(*inviteMsg);
            }
            break;
	}

        case SIP_ACK:
        {
            AckMsg* ackMsg = dynamic_cast<AckMsg*>(sipMsg);
            if (ackMsg)
            {
                sptrsipMsg = new AckMsg(*ackMsg);
            }
            break;
	}

        case SIP_BYE:
        {
            ByeMsg* byeMsg = dynamic_cast<ByeMsg*>(sipMsg);
            if (byeMsg)
            {
                sptrsipMsg = new ByeMsg (*byeMsg);
            }
            break;
	}

        case SIP_CANCEL:
        {
            CancelMsg* cancelMsg = dynamic_cast<CancelMsg*>(sipMsg);
            if (cancelMsg)
            {
                sptrsipMsg = new CancelMsg (*cancelMsg);
            }
            break;
	}

        case SIP_STATUS:
        {
            StatusMsg* statusMsg = dynamic_cast<StatusMsg*>(sipMsg);
            if (statusMsg)
            {
                sptrsipMsg = new StatusMsg (*statusMsg);
            }
            break;
	}

	case SIP_REFER:
        {
            ReferMsg* referMsg  = dynamic_cast<ReferMsg*>(sipMsg);
            if (referMsg)
            {
                sptrsipMsg = new ReferMsg (*referMsg);
            }
            break;
	}
        case SIP_REGISTER:
        {
            RegisterMsg* registerMsg  = dynamic_cast<RegisterMsg*>(sipMsg);
            if (registerMsg)
            {
                sptrsipMsg = new RegisterMsg (*registerMsg);
            }
            break;
	}

        case SIP_OPTIONS:
        {
            OptionsMsg* optionsMsg  = dynamic_cast<OptionsMsg*>(sipMsg);
            if (optionsMsg)
            {
                sptrsipMsg = new OptionsMsg (*optionsMsg);
            }
            break;
	}

        case SIP_INFO:
        {
            InfoMsg* infoMsg  = dynamic_cast<InfoMsg*>(sipMsg);
            if (infoMsg)
            {
                sptrsipMsg = new InfoMsg (*infoMsg);
            }
            break;
	}

        case SIP_TRANSFER:
        {
            TransferMsg* transferMsg  = dynamic_cast<TransferMsg*>(sipMsg);
            if (transferMsg)
            {
                sptrsipMsg = new TransferMsg (*transferMsg);
            }
            break;
        }
        case SIP_SUBSCRIBE:
        {
            SubscribeMsg* subsMsg  = dynamic_cast<SubscribeMsg*>(sipMsg);
            if (subsMsg)
            {
                sptrsipMsg = new SubscribeMsg (*subsMsg);
            }
            break;
	}
         case SIP_NOTIFY:
        {
            NotifyMsg* notifyMsg  = dynamic_cast<NotifyMsg*>(sipMsg);
            if (notifyMsg)
            {
                sptrsipMsg = new NotifyMsg (*notifyMsg);
            }
            break;
	}

        default:
        {
	  UnknownExtensionMsg* unknownExtensionMsg = dynamic_cast<UnknownExtensionMsg*>(sipMsg);
	  if (unknownExtensionMsg)
            {
	      sptrsipMsg = new UnknownExtensionMsg (*unknownExtensionMsg);
            }
	  break;
	  
	   
	}
    }

    assert (sptrsipMsg != 0);
    return sptrsipMsg;
}


#if 0


Data SipMsg::encodeContentDataList() const
{
    Data encContentDataList;
    //get each element in the contentLengthList. Call encode.
    vector<SipContentData*>::const_iterator iter;

    iter = contentDataList.begin();
    //assert(iter);
    int datalist = getNumContentData();
   
    if (datalist > 1)
    {
	encContentDataList += getContentType().encode();
	encContentDataList += CRLF;

	while (iter != contentDataList.end())
	{
	    encContentDataList += Data("--");
	    Data bounds = contentType.getAttributeValue(Data("boundary"));
	    bounds.removeSpaces();
	    encContentDataList += bounds;
	    encContentDataList += CRLF;
	    //get the object.
	    if  (*iter)
	    {
		//this is a SipContentData item.
		encContentDataList+= (*iter)->encode();
		//this should call encode of SipSdp.
	    }
	    
	    //if there is another SDP data then add CRLF.
	    iter++;
	} 	
		encContentDataList += "--";
	 encContentDataList +=  contentType.getAttributeValue(Data("boundary"));
	  encContentDataList += "--";
	encContentDataList += CRLF;  //    TODO check if this is correct
    }
    else
    {
      Data tempcon;
	while (iter != contentDataList.end())
	{
	    //get the object.
	    if  (*iter)
	    {
		
		//this is a SipContentData item.
		encContentDataList+= (*iter)->encode();
		
		//this should call encode of SipSdp.
	    }
	    ++iter;
	 
	} 		
    }
     
    return encContentDataList;

}
#endif
// ----------------- ContentLength Header Methods  -----------------
/// get the number of ContentLength items 

/// Get the i'th ContentLength item. If i is -1, it gets the last one 
const  SipContentLength&
SipMsg::getContentLength( ) const
{
    Sptr<SipContentLength> contentLength;
    myHeaderList.getParsedHeader(contentLength, SIP_CONTENT_LENGTH_HDR);
    return *contentLength;
}

    

    
    
    
//content disposition functions.   
const  SipContentDisposition&
SipMsg::getContentDisposition( ) const
{
    Sptr<SipContentDisposition> contentDisposition;
    myHeaderList.getParsedHeader(contentDisposition, SIP_CONTENT_DISPOSITION_HDR);
    return *contentDisposition;
}
    

void SipMsg::setContentDisposition(const SipContentDisposition& item /* Default Arguments*/)
{
    Sptr<SipContentDisposition> contentDisposition;
    myHeaderList.getParsedHeader(contentDisposition, SIP_CONTENT_DISPOSITION_HDR);
    *contentDisposition = item;
}
    
    
 void SipMsg::setContentDisposition(const Data& item /* Default Arguments*/)
{
    Sptr<SipContentDisposition> contentDisposition;
    myHeaderList.getParsedHeader(contentDisposition, SIP_CONTENT_DISPOSITION_HDR);
    contentDisposition->decode(item);
}   
    

/** set or add another ContentLength item, if the index is -1, it is appended 
       to the current list */
void SipMsg::setContentLength(const Data& item /* Default Arguments*/)
{
    Sptr<SipContentLength> contentLength;
    myHeaderList.getParsedHeader(contentLength, SIP_CONTENT_LENGTH_HDR);
    contentLength->decode(item);
}



// ----------------- ContentType Header Methods  -----------------

const  SipContentType&
SipMsg::getContentType( ) const
{
    
    Sptr<SipContentType> contentType;
    myHeaderList.getParsedHeader(contentType, SIP_CONTENT_TYPE_HDR);
    return *contentType;
    
}


void SipMsg::setContentType(const SipContentType& item /*default arguments*/ )
{
    Sptr<SipContentType> contentType;
    myHeaderList.getParsedHeader(contentType, SIP_CONTENT_TYPE_HDR);
    *contentType = item;
}  
    

void SipMsg::setContentType(const Data& item /*default arguments*/ )
{
    Sptr<SipContentType> contentType;
    myHeaderList.getParsedHeader(contentType, SIP_CONTENT_TYPE_HDR);
    contentType->decode(item);
}  

// ----------------- ContentData Header Methods  -----------------
/// get the number of ContentData items 
int SipMsg::getNumContentData() const
{
    return myHeaderList.getNumHeaders(SIP_CONTENT_DATA_HDR);
}

/// Get the i'th ContentData item. If i is -1, it gets the last one 
Sptr<SipContentData>
SipMsg::getContentData( int i/*default arguments*/) const
{
    
    return myMimeList.getContentData(i);
    
#if 0

    Sptr<SipContentData> x;
    myHeaderList.getParsedHeader(x, SIP_CONTENT_DATA_HDR, i);
    return *x;


    if ( ( i < -1 ) || ( i > (static_cast<int>(contentDataList.size())) ) 
	 || (contentDataList.size() == 0) )
    {
	throw SipHeaderException(OUT_OF_RANGE);
    }
    else
    {   
	if (i == -1)
	{
	    int size = contentDataList.size();
	    return *(contentDataList[size -1]);
	}
	else
	{
	    return *(contentDataList[i]);
	}
    }
#endif
}


    

/** set or add another ContentData itme, if the index is -1, it is appended 
    to the current list */

void SipMsg::setContentData(SipContentData* item, int index /*default Arguments*/)
{
    assert(item != 0);
    
    myMimeList.setContentData( item->duplicate(), index);
    
#if 0
    Sptr<SipContentData> x = new SipContentData(item);
    myHeaderList.appendHeader(SIP_CONTENT_DATA_HDR, x, index);

   
    SipContentData* newdata = NULL;

    SipSdp* sdp = dynamic_cast<SipSdp*>(item);
    SipIsup* isup = dynamic_cast<SipIsup*>(item);
    SipOsp* osp = dynamic_cast<SipOsp*>(item);
    
   

    if (sdp)
    {
        SipSdp* newsdpdata = new SipSdp(*sdp);
	
        assert(newsdpdata);
        newdata = dynamic_cast<SipContentData*>(newsdpdata);
        //contenttype->setType("application");
        //contenttype->setSubType("sdp");

    }
     else if (isup)
    {
        SipIsup* newisupdata = new SipIsup(*isup);
		
        assert(newisupdata);
        newdata = dynamic_cast<SipContentData*>(newisupdata);
        
    }
    else if (osp)
    {
        SipOsp* newospdata = new SipOsp(*osp);

        assert(newospdata);
        newdata = dynamic_cast<SipContentData*>(newospdata);
	
    }
    if (newdata)
    {
	// check if 1 entry 
	int numb = getNumContentData();
	if ( numb == 0)
	{
	   SipContentType contype = getContentType();
	  if ( contype.getType().length() == 0) {
	    setContentType( newdata->getCtype());
	  }
	}
	else if ( numb == 1)
	{
	  
	    SipContentType contype = getContentType();
	   
	    if ( contype.getType().length() == 0)
	      {
		contype.setType( Data("multipart"));
		contype.setSubType(Data("mixed"));
		contype.setTokenDetails(Data("boundary"),Data("next"));
		setContentType(contype);
		    }
	  
	}
	//     set contenttype to multipart
	// check if 0 entry
	//     set the same contenttype
        if ( (index >= -1) && (index <= static_cast<int>(contentDataList.size()) ) )
        {
           if (index == -1) 
           {
                //add it to the end.
                contentDataList.push_back(newdata);
           }
           else if (index <= static_cast<int>(contentDataList.size()) )
           {
	        if (sdp )
                {
		  vector<SipContentData*>::iterator iter;

		  iter = contentDataList.begin();
		  iter+=index;
		  // insert it at index
		  contentDataList.insert(iter, newdata);
    
		  
		}
           }
	   }
        else
        {
            cpLog(LOG_DEBUG, "index to insert SipContentData item not within scope");
        }
    }
    else
    {
        cpLog(LOG_INFO, " UNKNOWN SDP DATA");
        //need to implement this.
    }//

    //needed, but need to revamp
    setContentLengthValue();
#endif
}



/** Set number of ContentData items.
If i is less than current number then the extras are deleted. */
void SipMsg::setNumContentData(int index)
{
    myHeaderList.setNumHeaders(SIP_CONTENT_DATA_HDR, index);
#if 0
    
    if ( (index < 0) || (index > (static_cast<int>(contentDataList.size())) ) )
    {
        throw SipHeaderException(OUT_OF_RANGE);
    }
    else
    {
        vector<SipContentData*>::iterator iter;
        iter = contentDataList.begin();
     
        //increment to index
        iter+=index;
        
        //delete from iter
        while (iter != contentDataList.end())
        {
            assert(*iter);
            delete (*iter);

            ++iter;
        }
    }
#endif
}


#if 0
   
///
void SipMsg::copyContentData(const SipMsg& src) 
{
    vector<SipContentData*> newItem = src.getContentDataList();

    if (newItem.size() > 0)
    {
        vector <SipContentData*>::iterator iter;
        iter = newItem.begin();

        if (contentDataList.size() > 0)
        {
            flushContentData();
        }

        //copy
        while (iter != newItem.end())
        {
            if (*iter)
            {
                SipSdp* sdp = dynamic_cast<SipSdp*>(*iter);
		SipIsup* isup = dynamic_cast<SipIsup*>(*iter);
                SipOsp* ospContent = dynamic_cast<SipOsp*>(*iter);
    
                if (sdp)
                {
                    SipSdp* newObject = new SipSdp;
                    assert(newObject);        
                    //copy the object
                    *newObject = *sdp; 
                    //cpLog(LOG_DEBUG_STACK, "should be calling  = operator of SipSdp.");
                    contentDataList.push_back(newObject);
                }
		else if (isup)
                {
                    SipIsup* newObject = new SipIsup;
                    assert(newObject);

                    //copy the object
                    *newObject = *isup;
                    //cpLog(LOG_DEBUG_STACK, "should be calling  = operator of SipSdp.");
                    contentDataList.push_back(newObject);

                }
    
                else if (ospContent)
                {
                     SipOsp* newObject = new SipOsp;
                    assert(newObject);
 
                    //copy the object
                    *newObject = *ospContent;
                    //cpLog(LOG_DEBUG_STACK, "should be calling  = operator of SipSdp.");
                    contentDataList.push_back(newObject);            
                }
                else
                {
                    //need to implement this.
		    cpLog(LOG_ERR, "Needs to implement the content type");
                    assert(0);
                }
            }
            ++iter;
        }
    }
    else
    {
        if (contentDataList.size() > 0)
        {
            flushContentData();
        }
    }
}

#endif
    
void SipMsg::removeContentData(int index /*Default Agruments*/)
{
    
    
    myMimeList.removeContentData(index);
#if 0
    myHeaderList.setNumHeaders(SIP_CONTENT_DATA_HDR, 0);


    if (index >= -1)
    {
        if (index == -1)
        {
            SipContentData* contentData = contentDataList.back();
            if (contentData) 
	    {
		delete contentData;
	    }
            contentDataList.pop_back();
        }
        else if  ( index <= static_cast<int>(contentDataList.size()) )
        {
            vector<SipContentData*>::iterator iter;

            iter = contentDataList.begin();
            iter+=index;
	    if(iter != contentDataList.end())
	    {
		delete *iter;
		contentDataList.erase(iter);
	    }
        }
        if (contentDataList.size() == 0)
        {
            contentDataList.clear();
        }
    }
#endif    
}
    

///
void SipMsg::flushContentData()
{
#if 0
    myHeaderList.setNumHeaders(SIP_CONTENT_DATA_HDR, 0);
#endif

    vector<SipContentData*>::iterator iter = contentDataList.begin();
    
    while (iter != contentDataList.end())
    {
        if (*iter)
        {
            delete (*iter);
        }
        ++iter;
    }
    contentDataList.clear();

}
    
#if 0
void SipMsg::setContentLengthValue()
{
    Data encContentDataList;
    //get each element in the contentLengthList. Call encode.
    vector<SipContentData*>::iterator iter;
    iter = contentDataList.begin();
    int datalist = getNumContentData();
    Data total = 0;
    if (datalist > 1)
    {
	while (iter != contentDataList.end())
	{ 
	    //get the object.
	    if  (*iter)
	    { Data temp;
		//this is a SipContentData item.
	        temp = (*iter)->getContentLength().getLength();
		total += static_cast<int>(temp);
		//this should call encode of SipSdp.
	    }
	    
	    //if there is another SDP data then add CRLF.
	    iter++;
	}

	contentLength.setLength(total);
	
	//    TODO check if this is correct
    }
    else if ( datalist == 1 )
    {
      Data tempcon;
	while (iter != contentDataList.end())
	{
	    //get the object.
	    if  (*iter)
	    {
		Data temp;
		//this is a SipContentData item.
		temp = (*iter)->getContentLength().getLength();
	        total += static_cast<int>(temp);
	      
	    }
	    ++iter;
	    
	}	contentLength.setLength(total);
    }		
    else 
      {
	contentLength.setLength(0); 
      }

   
}
    
      
const vector<SipContentData*>& SipMsg::getContentDataList() const
{
    
    return (contentDataList);
}



// function 1 brought here

Data SipMsg::getMultipartBoundary(Data data)
{
    //here, the content type is obtained, and the boundary is obtained
    //and returned.
    Data beforecontdata;
    
    int beforecontvalue = data.match(CONTENT_TYPE1, &beforecontdata, true);
    
    Data multipartboundary;
    
    if (beforecontvalue == FOUND)
    {
        Data contdata;
        //find CR, LF or CRLF in data to isolate contenttype.
        int contvalue = data.match(CR, &contdata, true);
        if (contvalue == FOUND)
        {
            SipContentType contType(contdata);
            
            multipartboundary = contType.getAttributeValue(Data("boundary"));
            if ( multipartboundary.length() != 0)
            {
                setContentType( contType);
            }
        }
        else if (contvalue == NOT_FOUND)
        {
            contvalue = data.match(LF, &contdata, true);
            if (contvalue == FOUND)
            {
                SipContentType contType(contdata);
                multipartboundary = contType.getAttributeValue(Data("boundary"));
                if ( multipartboundary.length() != 0)
                {
                    setContentType( contType);
                }
                
            }
        }
    }
    
    return multipartboundary;
}


//This function accepts the content type and data in arguements 
//and sets the contentdata in the contentdatalist

void SipMsg::singleMimeDecode(Data singleMimeData, Data contTypeData)
{
  Data isupfound;
  Data tempcondata = contTypeData;
  int isuphack = tempcondata.match( "ISUP", &isupfound,true);
  Data subtype;
  if (isuphack == FOUND)
    {
      subtype = "ISUP";
    }
  else
    {
      SipContentType contType(contTypeData);
      subtype = contType.getSubType();
     
    }
  if (isEqualNoCase(subtype, "SDP")) 
    {
      
      int matchval;
      Data vValue;
      matchval = singleMimeData.match("v=0", &vValue, false);
      if (matchval == FIRST)
	{
	  Data sessionData = singleMimeData;
	  SdpSession sessionInfo;
	  SipContentType contype(contTypeData);
	  if ( sessionInfo.decode(sessionData) )
	    {				
	      SipSdp sdp;
	      sdp.setCtype(contype);
	      sdp.setSdpDescriptor(sessionInfo);
	      setContentData(&sdp);
	    }
	}// if (matchval == FIRST)  
    }// if (isEqualNoCase(subtype, "SDP")) 
  if (isEqualNoCase(subtype, "ISUP")) 
    {
      // SipIsup and SipOsp needs to change to take only the ISUP and OSP data respectively instead of ContentType and data. --- Amit C.
      
      Data hackData = contTypeData;
      hackData+= CRLF2;
      
      hackData+= singleMimeData;
      hackData += CRLF;
      
      SipIsup isup(hackData);
      setContentData(&isup);
      
    } 
  if (isEqualNoCase(subtype, "OSP")) 
    {
      // SipIsup and SipOsp needs to change to take only the ISUP and OSP data respectively instead of ContentType and data.--- Amit C.
      // Data hackData = contTypeData;
      // hackData+= CRLF2;
      // hackData+= singleMimeData;
      // hackData += CRLF;
      
      SipOsp osp( singleMimeData);
      setContentData(&osp);
    } 
}


void 
SipMsg::sdpdecode(Data sData, Data multipartboundary)
{
  //sData will contain MIME info after the first unique boundary.

  // if no multipart then contenttype + CRLF + data
  if ( multipartboundary.length() == 0)
  {
	if ( contentType.getType().length() == 0)
	  {
	    //SP SipContentType cont;
	    Data chekdata = sData;
	    Data tempchek;
	    int chek = chekdata.match("v=0", &tempchek, true);
	    if ( chek == FIRST )
	      {
		//SP cont.setType(Data("applicaton"));
		//SP cont.setSubType(Data("SDP"));
                SipContentType cont("applicaton", "SDP");
		singleMimeDecode(sData, cont.encode());
		setContentType(cont);
	      } else
		{
		  //SP cont.setType(Data("applicaton"));
		  //SP cont.setSubType(Data("octet-stream"));
                  SipContentType cont("applicaton", "octet-stream");
		  singleMimeDecode(sData, cont.encode());
		  setContentType(cont);
		}
	  }
	else
	  {
	    //SP Data contTypeData = contype.encode();
	    //SP singleMimeDecode(sData, contype.encode());
	    singleMimeDecode(sData, contentType.encode());
	  }
    } //multipartboundary.length() == 0
    else
    {
        // if multipart then 
        bool boundary = true;
	
	Data boundCRLF(Data("--"));
       boundCRLF +=  multipartboundary;
	Data boundendCRLF = boundCRLF;
	boundendCRLF += "--";
	boundendCRLF += CRLF;
       boundCRLF += CRLF;
      
	while (boundary == true)
        {
	    Data mimedata;

	    //since all boundaries MUST be followd by CRLF.

	    
	   
	    int mimefound = sData.match( boundCRLF, &mimedata, true);
	    if ( mimefound == FOUND)
	    {
	     
	        //send to function
	      	mimeDecode(mimedata);
	       
	    }
	    else if ( mimefound == FIRST)
	    {
	      //throw exception
	    }
	    else if ( mimefound == NOT_FOUND)
	    {
	      Data fmimedata;
	      int mimenotfound = sData.match( boundendCRLF, &fmimedata, true);
	      if ( mimenotfound == FOUND)
		{
		  	mimeDecode(fmimedata);
		}
	      else
		{
	      boundary = false;
		}
	    }
	}//while boundary==true
    }//if boundary == true
}



void SipMsg::mimeDecode( const Data& mimeData )
{

    Data tempData = mimeData;

    //match CRLF2, CRLF3 or CRLF4
    
        
    Data contTypeData;
   
    
    int contTypevalue = tempData.match( CRLF2, &contTypeData, true);
    if (contTypevalue == FIRST)
      {
	//throw exception, or do nothing. No content type for multi mime type.
      } 
    else if (contTypevalue == NOT_FOUND)
      {
	  //find CRLF3
	  contTypevalue = tempData.match(CRLF3, &contTypeData, true);
	  if ( contTypevalue == FIRST)
          {
	      //throw exception, or do nothing. No content type for multi mime type.
          }
          else if (contTypevalue == NOT_FOUND)
	  {
	      //find CRLF4
	      contTypevalue = tempData.match(CRLF4, &contTypeData, true);
	      if (contTypevalue == FIRST)
	      {
		  //throw exception, or do nothing. No content type for multi mime type.
	      }
	      else if (contTypevalue == NOT_FOUND)
	      {
		  //throw exception, or do nothing. No content type for multi mime type.
	      }
	      else if (contTypevalue == FOUND)//fond CRCRLF
	      {
	          singleMimeDecode(tempData, contTypeData);
		  
		  
	      }
	  }
	  else if (contTypevalue == FOUND)//found LFCRLF
          {
	         singleMimeDecode(tempData, contTypeData);
          }
    }
    else if (contTypevalue == FOUND) //found CRLFCRLF
    {
        singleMimeDecode(tempData, contTypeData);
   }
}

#endif

#ifdef USE_BIND
    
Sptr<DnsResolver> SipMsg::getDnsResolver()
{
    return dnsResolver;
}
        
     
void SipMsg::setDnsResolver( Sptr<DnsResolver> dnsresolver ) 
{
    dnsResolver = dnsresolver;
}
    

#endif
    
void
SipMsg::forceParse()
{
    myHeaderList.forceParse();
}    


/* Local Variables: */
/* c-file-style: "stroustrup" */
/* indent-tabs-mode: nil */
/* c-file-offsets: ((access-label . -) (inclass . ++)) */
/* c-basic-offset: 4 */
/* End: */
