#ifndef  OSPServerData_hxx
#define  OSPServerData_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const OSPServerData_hxx_Version =
    "$Id: OSPServerData.hxx,v 1.6 2000/12/18 23:42:51 bko Exp $";



#include <string>
#include <list>

#include "VXmlParser.hxx"
#include "VFile.hxx"
#include "cpLog.h"


static const int LOCAL_VALIDATION = 1;
static const int DEF_SSL_LIFETIME_IN_SECS = 300;  /*  seconds, Do not change */
static const int DEF_HTTP_MAXCONN = 5;
static const int DEF_HTTP_PERSIST_IN_SECS = 60000;   /* seconds, Do not change */
static const int DEF_HTTP_RETRYDELAY = 0;
static const int DEF_HTTP_RETRYLIMIT = 0;
static const int DEF_HTTP_TIMEOUT_IN_MS = 60000;     /* ms, Do not change */
static const int DEF_TIME_LIMIT = 4;
static const long DEF_CUST_ID = 1000L;
static const long DEF_DEVICE_ID = 1000L;


typedef enum CustomExtension
{
    NO_EXTENSION = 0,
    TRANSNEXUS,
    GRIC,
    MAX_CUSTOM_EXTENSION
} CUSTOM_EXTENSION;


/** This class contains data required to maintain connections
  * with the OSP Service Point.  It does not contain certificate
  * authroity information
 */
class OSPServerData
{

    public:
        ///
        OSPServerData(void)
        {
            _numServicePoints = 0;
            _localValidation = 0;
            _sslLifeTime = DEF_SSL_LIFETIME_IN_SECS;   // 300 sec.
            _httpMaxConnections = DEF_HTTP_MAXCONN;    // 5
            _httpPersistence = DEF_HTTP_PERSIST_IN_SECS;  // 60000 sec
            _httpRetryDelay = DEF_HTTP_RETRYDELAY;    // 0
            _httpRetryLimit = DEF_HTTP_RETRYLIMIT;     // 0
            _httpTimeout = DEF_HTTP_TIMEOUT_IN_MS;   // 60000 ms
            _customExt = NO_EXTENSION;
            _customerID = "noCustomerId";
            _deviceID = "noDeviceId";
            _caEnrollmentServer = "noCaEnrollServer";
            _caEnrollmentPort = "noCaEnrollPort";
            _auditURL = "noAuditURL";
        }

        ///
        OSPServerData(const VFile& xmlFile)
        {
            parse(xmlFile);
        }

        ///  called by PolicyCfg:
        OSPServerData( VXmlParser& parser)
        {
            parse(parser);
        }

        ///
        OSPServerData(const string& path, const string& fileName);

        ///
        OSPServerData& operator=(const OSPServerData& other);

        ///
        friend ostream& operator<<(ostream& os, const OSPServerData& data);

        ///
        int getNumServicePoints( void ) const
        {
            return _numServicePoints;
        }
        ///
        const list < string > & getServicePoint( void ) const
        {
            return _servicePoint;
        }
        ///
        bool getLocalValidation( void ) const
        {
            return _localValidation;
        }
        ///
        int getSslLifeTime( void ) const
        {
            return _sslLifeTime;
        }
        ///
        int getHttpMaxConnections( void ) const
        {
            return _httpMaxConnections;
        }
        ///
        int getHttpPersistence( void ) const
        {
            return _httpPersistence;
        }
        ///
        int getHttpRetryDelay( void ) const
        {
            return _httpRetryDelay;
        }
        ///
        int getHttpRetryLimit( void ) const
        {
            return _httpRetryLimit;
        }
        ///
        int getHttpTimeout( void ) const
        {
            return _httpTimeout;
        }

        /// Clearinghouse solutions provider customization:
        enum CustomExtension getCustomExt( void ) const
        {
            return _customExt;
        }
        ///
        const string& getCustomerID( void ) const
        {
            return _customerID;
        }

        const string& getDeviceID(void ) const
        {
            return _deviceID;
        }

        ///  Certificate Authority Enrollment information:
        const string& getCaEnrollmentServer( void ) const
        {
            return _caEnrollmentServer;
        }
        ///
        const string& getCaEnrollmentPort( void ) const
        {
            return _caEnrollmentPort;
        }

        /// Audit Control information: (not used for now)
        const string& getAuditURL( void ) const
        {
            return _auditURL;
        }

        ///
        void parse( VXmlParser& parser);

        ///
        void parse(const VFile& vf)
        {
            VXmlParser parser( vf );
            parse( parser );
        }
    private:
        int _numServicePoints;
        list < string > _servicePoint;         // Service Point URLs
        bool _localValidation;
        int _sslLifeTime;          // xml data in ms, store as sec.
        int _httpMaxConnections;   // max. # simulaneous connections
        int _httpPersistence;      // xml data in ms, here, in sec.
        int _httpRetryDelay;       // ditto
        int _httpRetryLimit;       // max # of connection retries
        int _httpTimeout;          // ms. to wait for response from
        // the OSP server
        CustomExtension _customExt;
        string _customerID;
        string _deviceID;

        /// for certificate enrollment:
        string _caEnrollmentServer;
        string _caEnrollmentPort;

        /// for Audit control:
        string _auditURL;
};

#endif
