#ifndef  PolicyCfg_hxx
#define  PolicyCfg_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const PolicyCfg_hxx_Version =
    "$Id: PolicyCfg.hxx,v 1.15 2001/01/12 01:35:50 wbreu Exp $";


#include <assert.h>
#include "global.h"
#include <stdio.h>
#include <string>

#include "VXmlParser.hxx"
#include "VFile.hxx"

#include "VOspServerData.hxx"

///  Interface object used to obtain policyServer config data from Provisioning
class PolicyCfg
{
    public:
        /// PolicyCfg singleton object accessor
        static PolicyCfg& Instance( const char* psHost = 0, int psPort = 0,
                                    const char* altHost = 0, int altPort = 0 )
        {
            if ( _instance == 0 )
            {
                _instance = new PolicyCfg(psHost, psPort, altHost, altPort);
                assert( _instance );
            }
            return (*_instance);
        }

	/// clean up on exit()
        static void destroy( void );

        /// assignment operator
        PolicyCfg& operator=(const PolicyCfg& other);

        /// output operator
        friend ostream& operator<<(ostream& os, const PolicyCfg& data);

        /// 
        const VOspServerData& getOSPServerData(void) const
        {
            return _ospServerData;
        }

        ///
        bool isOspDataChanged()
        {
            return ( _ospDataChanged);
        }

        ///
        bool isPdpDataChanged()
        {
            return ( _pdpDataChanged);
        }

        /// flag that OSP Server related data has been changed
        void ospDataChanged(bool chg = true)
        {
            _ospDataChanged = chg;
        };

        /// flag that PDP Agent related data has been changed
        void pdpDataChanged(bool chg = true)
        {
            _pdpDataChanged = chg;
        };

    protected:
        /// constructor
        PolicyCfg(const char* psHost, int psPort,
                  const char* altHost, int altPort );

        ///
        static void notifyOspUpdate(const string& data,
                                    const string& fileName, bool delFlg);
        ///
        static void notifyPdpUpdate(const string& data,
                                    const string& fileName, bool delFlg);

        ///
        void setOspServerData(const VOspServerData& data)
        {
            _ospServerData = data;
        };
    private:
        ///
        VOspServerData _ospServerData;

        ///
        static PolicyCfg* _instance;

        ///
        bool _ospDataChanged;
        ///
        bool _pdpDataChanged;
};

#endif
