#ifndef COPSClientOpenMsg_hxx
#define COPSClientOpenMsg_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const COPSClientOpenMsg_hxx_Version =
    "$Id: COPSClientOpenMsg.hxx,v 1.11 2000/12/18 23:42:53 bko Exp $";

#include "COPSMsg.hxx"

#include "COPSHeader.hxx"
#include "COPSPepId.hxx"
#include "COPSIpv4LastPdpAddr.hxx"
#include "COPSIpv6LastPdpAddr.hxx"
#include "COPSClientSI.hxx"
#include "COPSIntegrity.hxx"

///COPSClientOpenMsg

class COPSClientOpenMsg : public COPSMsg
{
    public:
        ///
        COPSClientOpenMsg();

        ///
        virtual ~COPSClientOpenMsg();

        ///
        COPSClientOpenMsg& operator=(const COPSClientOpenMsg& other);

        ///
        COPSClientOpenMsg(const COPSClientOpenMsg& other);


        /** Checks the sanity of COPS message and throw an
          * COPSBadDataException when data is bad.
          */
        void checkSanity() const throw (COPSBadDataException&);

        ///Add message header
        COPSClientOpenMsg& operator<<(const COPSHeader& obj);

        ///Add PEPID object to the message
        COPSClientOpenMsg& operator<<(const COPSPepId& obj);

        ///Add clientSI objects
        COPSClientOpenMsg& operator<<(const COPSClientSI& obj);

        ///Add Last PDP address objects
        COPSClientOpenMsg& operator<<(const COPSPdpAddress& obj);

        ///Add Integrity objects
        COPSClientOpenMsg& operator<<(const COPSIntegrity& obj);

        ///Dumps the whole object to a output stream
        void dump(ostream& os) const;

        ///
        string className() const
        {
            return "COPSClientOpenMsg";
        };

        ///
        const COPSPepId& getPepId() const
        {
            return *_pepId;
        };

        ///Returns true if it has ClientSI object
        bool hasClientSI() const
        {
            return (_clientSI.getPtr() != 0);
        };

        ///Should check hasClientSI() before calling
        const Sptr < COPSClientSI > & getClientSI() const
        {
            return (_clientSI);
        }

        ///Returns true if it has PDPAddress object
        bool hasPdpAddress() const
        {
            return (_pdpAddress.getPtr() != 0);
        };

        ///Should check hasPdpAddress() before calling
        const Sptr < COPSPdpAddress > & getLastPdpAddress() const
        {
            return (_pdpAddress);
        }

        ///Returns true if it has Integrity object
        bool hasIntegrity() const
        {
            return (_integrity.getPtr() != 0);
        };

        ///Should check hasIntegrity() before calling
        const Sptr < COPSIntegrity > & getIntegrity() const
        {
            return (_integrity);
        }

        ///Writes data to given network socket
        void writeData(int id) const throw(VNetworkException&);

    private:
        ///
        void copyRhs(const COPSClientOpenMsg& other);

        friend class COPSMsgParser;
        ///
        COPSClientOpenMsg(const char* data);

        ///
        void parse(const char* data) throw (COPSBadDataException&);

        ///
        void parse(const COPSHeader&hdr, const char* data)
        throw (COPSBadDataException&);

        ///Set the message length, base on the set of objects it contains
        void setMsgLength();

        /* COPSHeader coming from base class */
        Sptr < COPSPepId > _pepId;
        Sptr < COPSClientSI > _clientSI;
        Sptr < COPSPdpAddress > _pdpAddress;
        Sptr < COPSIntegrity > _integrity;
};

#endif
