#ifndef COPSContext_hxx
#define COPSContext_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const COPSContext_hxx_Version =
    "$Id: COPSContext.hxx,v 1.8 2000/12/18 23:42:53 bko Exp $";

#include <sys/types.h>
#include <string>

#include "COPSObjBase.hxx"
#include "COPSObjHeader.hxx"
#include "Sptr.hxx"

///COPSContext

class COPSContext : public COPSObjBase
{
    public:
        enum ReqType {
            IN_ADMIN = 0x01,
            RES_ALLOC = 0x02,
            OUT = 0x04,
            CONFIG = 0x08
    };

        ///
        COPSContext(u_int16_t rType, u_int16_t mType );


        ///
        COPSContext& operator=(const COPSContext& other);

        ///
        bool operator==(const COPSContext& other) const;

        ///
        friend bool operator<(const Sptr < COPSContext > & left, const Sptr < COPSContext > & right);

        ///
        bool operator<(const COPSContext& other) const;

        ///Write object in network byte order to a given network socket
        void writeData(int id) const throw(VNetworkException&);


        ///Returns size in number of octects, including header
        u_int16_t getDataLength() const;

        void dump(ostream& os)const;

        ///Returns the detail description of the request type
        string getDescription() const;

        ///
        string className() const
        {
            return "COPSContext";
        };

        ///
        bool isIncomingMessage() const
        {
            return (_dataRep._rType & IN_ADMIN);
        };
        ///
        bool isAdminControl() const
        {
            return (_dataRep._rType & IN_ADMIN);
        };
        ///
        bool isResourceAllocationReq() const
        {
            return (_dataRep._rType & RES_ALLOC);
        };
        ///
        bool isOutgoingMessage() const
        {
            return (_dataRep._rType & OUT);
        };
        ///
        bool isConfigRequest() const
        {
            return (_dataRep._rType & CONFIG);
        };

        ///
        u_int16_t getMessageType() const
        {
            return (_dataRep._mType);
        };

        ///
        u_int16_t getRequestType() const
        {
            return (_dataRep._rType);
        };

        ///
        bool isContext() const
        {
            return true;
        };
    private:
        friend class COPSReqMsg;
        friend class COPSDecisionMsg;
        ///Parse the data and create a Context object
        COPSContext(const char* dataPtr);

        COPSObjHeader _objHdr;
        struct
        {
            u_int16_t _rType;
            u_int16_t _mType;
        }
        _dataRep;
};

inline bool operator<(const Sptr < COPSContext > & left, const Sptr < COPSContext > & right)
{
    if (!left || !right) return false;
    if (left->_objHdr < right->_objHdr)
    {
        if (left->_dataRep._rType < right->_dataRep._rType)
        {
            if (left->_dataRep._mType < right->_dataRep._mType)
            {
                return true;
            }
        }
    }
    return false;
}

#endif
