
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const COPSIntegrity_cxx_Version =
    "$Id: COPSIntegrity.cxx,v 1.10 2001/05/13 07:26:01 icahoon Exp $";


#include "VocalCommon.hxx"
#include <netinet/in.h>


#include "COPSIntegrity.hxx"

COPSIntegrity::COPSIntegrity(const char* dataPtr)
{
    _objHdr.parse(dataPtr);
    _objHdr.checkDataLength();

    u_int32_t keyId, seqNum;
    memcpy(&keyId, dataPtr + 4, 4);
    memcpy(&seqNum, dataPtr + 8, 4);

    _keyId = ntohl(keyId);
    _seqNum = ntohl(seqNum);
    u_int16_t dLen = _objHdr.getDataLength() - 12;
    COPSData d(dataPtr + 12, dLen);
    setKeyDigest(d);
}

void
COPSIntegrity::setKeyDigest(const COPSData& keyDigest)
{
    _keyDigest = keyDigest;
    if (_keyDigest.length() % 4)
    {
        int padLen = 4 - _keyDigest.length() % 4;
        _padding = getPadding(padLen);
    }
    _objHdr.setDataLength(sizeof(u_int32_t)
                          + sizeof(u_int32_t) + _keyDigest.length());
}

void
COPSIntegrity::dump(ostream& os) const
{
    os << _objHdr << endl;
    os.width(25);
    os << "Key Id: " << _keyId << endl;
    os.width(25);
    os << "Sequence: " << _seqNum << endl;
    os.width(25);
    os << "Key digest: " << _keyDigest << endl;
}

COPSIntegrity&
COPSIntegrity::operator=(const COPSIntegrity& other)
{
    if (this != &other)
    {
        _objHdr = other._objHdr;
        _keyId = other._keyId;
        _seqNum = other._seqNum;
        _keyDigest = other._keyDigest;
        _padding = other._padding;
    }
    return (*this);
}

u_int16_t
COPSIntegrity::getDataLength()const
{
    //Add the size of the header also
    return (_objHdr.getDataLength() + _padding.length());
}


void
COPSIntegrity::writeData(int id) const throw(VNetworkException&)
{
    cpLog(LOG_DEBUG, "COPSIntegrity::writeData");
    _objHdr.writeData(id);
    u_int32_t keyId = htonl(_keyId);
    u_int32_t seqNum = htonl(_seqNum);

    COPSUtil::writeData(id, &keyId, sizeof(u_int32_t));
    COPSUtil::writeData(id, &seqNum, sizeof(u_int32_t));

    COPSUtil::writeData(id, _keyDigest.getData(), _keyDigest.length());
    if (_padding.length())
    {
        COPSUtil::writeData(id, _padding.getData(), _padding.length());
    }
}
