#ifndef COPSIpv4Interface_hxx
#define COPSIpv4Interface_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const COPSIpv4Interface_hxx_Version =
    "$Id: COPSIpv4Interface.hxx,v 1.6 2000/12/18 23:42:54 bko Exp $";

#include <sys/types.h>
#include <string>

#include "COPSInterface.hxx"
#include "COPSObjHeader.hxx"
#include "COPSIpv4Address.hxx"

///COPSIpv4Interface

class COPSIpv4Interface : public COPSInterface
{
    public:
        ///
        bool isIpv4Address() const
        {
            return true;
        };

        ///
        void setIpAddress(const string& hostName);

        ///
        string getIpName() const;

        ///
        u_int32_t getIpAddress() const;

        ///Returns size in number of octects, including header
        u_int16_t getDataLength() const;

        ///
        COPSIpv4Interface& operator=(const COPSIpv4Interface& other);

    protected:
        COPSIpv4Interface()
        {
            _objHdr.setCType(1);
            _objHdr.setDataLength(_dataRep._addr.getDataLength() + sizeof(u_int32_t));
        }

        COPSIpv4Interface(const char* dataPtr)
        {
            _objHdr.parse(dataPtr);
            _objHdr.checkDataLength();
            _dataRep._addr.parse(dataPtr + 4);
            memcpy(&_dataRep._ifindex, dataPtr + 8, 4);
        }

        COPSObjHeader _objHdr;
        struct
        {
            COPSIpv4Address _addr;
            u_int32_t _ifindex;
        }
        _dataRep;
};

#endif
