
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const COPSIpv4PdpAddress_cxx_Version =
    "$Id: COPSIpv4PdpAddress.cxx,v 1.8 2001/05/13 07:26:01 icahoon Exp $";
#include <netdb.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <arpa/inet.h>


#include "COPSIpv4PdpAddress.hxx"

COPSIpv4PdpAddress::COPSIpv4PdpAddress()
{
    _objHdr.setCType(1);
    _objHdr.setDataLength(_dataRep._addr.getDataLength() + sizeof(u_int32_t));
}

COPSIpv4PdpAddress::COPSIpv4PdpAddress(const char* dataPtr)
{
    _objHdr.parse(dataPtr);
    _objHdr.checkDataLength();
    _dataRep._addr.parse(dataPtr + 4);

    u_int16_t reserved;
    u_int16_t tcpPort;
    memcpy(&reserved, dataPtr + 8, 2);
    memcpy(&tcpPort, dataPtr + 10, 2);

    _dataRep._reserved = ntohs(reserved);
    _dataRep._tcpPort = ntohs(tcpPort);

    _objHdr.setDataLength(_dataRep._addr.getDataLength() + sizeof(u_int32_t));
}

void
COPSIpv4PdpAddress::dump(ostream& os) const
{
    os << _objHdr << endl;
    os.width(25);
    os << "Ipv4PdpAddress" << endl;
    os.width(25);
    os << "Address: " << _dataRep._addr.getIpName() << endl;
    os.width(25);
    os << "Port: " << _dataRep._tcpPort << endl;
}


u_int16_t
COPSIpv4PdpAddress::getDataLength()const
{
    //Add the size of the header also
    return (_objHdr.getDataLength());
}

void
COPSIpv4PdpAddress::setIpAddress(const string& hostName)
{
    _dataRep._addr.setIpAddress(hostName);
}

string
COPSIpv4PdpAddress::getIpName() const
{
    return (_dataRep._addr.getIpName());
}

u_int32_t
COPSIpv4PdpAddress::getIpAddress() const
{
    return (_dataRep._addr.getIpAddress());
}

COPSIpv4PdpAddress&
COPSIpv4PdpAddress::operator=(const COPSIpv4PdpAddress& other)
{
    if (this != &other)
    {
        _objHdr = other._objHdr;
        _dataRep._addr = other._dataRep._addr;
        _dataRep._reserved = other._dataRep._reserved;
        _dataRep._tcpPort = other._dataRep._tcpPort;
    }
    return (*this);
}

COPSIpv4PdpAddress::COPSIpv4PdpAddress(const COPSIpv4PdpAddress& other)
{
    _objHdr = other._objHdr;
    _dataRep._addr = other._dataRep._addr;
    _dataRep._reserved = other._dataRep._reserved;
    _dataRep._tcpPort = other._dataRep._tcpPort;
}

void
COPSIpv4PdpAddress::writeData(int id) const throw (VNetworkException&)
{
    cpLog(LOG_DEBUG, "COPSIpv4PdpAddress::writeData");
    _objHdr.writeData(id);
    _dataRep._addr.writeData(id);
    u_int16_t reserved = htons(_dataRep._reserved);
    u_int16_t tcpPort = htons(_dataRep._tcpPort);
    COPSUtil::writeData(id, &reserved, sizeof(u_int16_t));
    COPSUtil::writeData(id, &tcpPort, sizeof(u_int16_t));
}
