#ifndef COPSPolElementHeader_hxx
#define COPSPolElementHeader_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const COPSPolElementHeader_hxx_Version =
    "$Id: COPSPolElementHeader.hxx,v 1.4 2000/12/26 19:35:21 veer Exp $";

#include <sys/types.h>
#include "global.h"
#include "iostream.h"
#include "COPSBadDataException.hxx"
#include "VNetworkException.hxx"
#include "vtypes.h"
typedef enum {
    COPS_POL_EL_RSVP_PROXY_SUPPORT = 1,
    COPS_POL_EL_POLICING_SUPPORT,
    COPS_POL_EL_MARKING_SUPPORT,
    COPS_POL_EL_IN_ROLE_COMB,
    COPS_POL_EL_OUT_ROLE_COMB
} COPSp_type;

typedef struct PolDataHeader_t
{
    u_int16_t   _len;
    u_int16_t   _pType;
} PolDataHeader;

///COPSPolElementHeader

class COPSPolElementHeader 
{
    public:
        ///
        COPSPolElementHeader(COPSp_type ptype)
        {
            _dataRep._len = 4;
            _dataRep._pType = ptype;
        }

        ///
        COPSPolElementHeader(const COPSPolElementHeader& other);

        ///
        string getStrPType() const;

        ///
        bool operator==(const COPSPolElementHeader& other) const
        {
            if (_dataRep._len == other._dataRep._len)
            {
                if (_dataRep._pType == other._dataRep._pType)
                {
                    return true;
                }
            }
            return false;
        }

        ///
        bool operator<(const COPSPolElementHeader& other) const
        {
            if (_dataRep._len < other._dataRep._len)
            {
                if (_dataRep._pType < other._dataRep._pType)
                {
                    return true;
                }
            }
            return false;
        }


        ///Accessor

        ///Get the data length in number of octets
        u_int16_t getDataLength() const
        {
            return _dataRep._len;
        };

        ///Get the class information identifier cNum
        COPSp_type getPType() const
        {
            return static_cast < COPSp_type > (_dataRep._pType);
        };

        ///Mutator

        /**Set the obj length, the length is the length of the data following
          * the object header.The length of the object header (4 bytes) is added
          * to the length passed.
          */
        void setDataLength(u_int32_t len);

        ///
        friend ostream& operator<<(ostream& os,const COPSPolElementHeader& obj);

        ///
        COPSPolElementHeader& operator=(const COPSPolElementHeader& other);

        ///Writes data to a given network socket
        void writeData(int id) const throw (VNetworkException&);

    private:
        ///
        friend class COPSReqMsg;
        ///
        friend class COPSClientOpenMsg;

        ///
        void parse(const char* data);

        ///
        COPSPolElementHeader()
        {
            _dataRep._len = 4;
            _dataRep._pType = COPS_POL_EL_RSVP_PROXY_SUPPORT;
        }
        ///
        COPSPolElementHeader(const char* data)
        {
            parse(data);
        }

        ///
        PolDataHeader _dataRep;

        ///
        friend class COPSRsvpObjBase;
};

inline ostream& operator<<(ostream& os,const COPSPolElementHeader& obj)
{
    os << "**PolElement**" << endl;
    os.width(25);
    os << "Length: " << obj._dataRep._len << endl;
    os.width(25);
    os << "P-type: " << obj.getStrPType().c_str() << endl;
    return os;
}

#endif
