
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const COPSPolicyElement_cxx_Version =
    "$Id: COPSPolicyElement.cxx,v 1.5 2001/05/13 07:26:01 icahoon Exp $";

#include "VocalCommon.hxx"
#include <netinet/in.h>

#include "COPSPolicyElement.hxx"
#include "COPSBadDataException.hxx"
#include "COPSUtil.hxx"
#include "cpLog.h"

COPSPolicyElement::COPSPolicyElement(COPSp_type pType, u_int16_t flgs)
    : COPSRsvpObjBase(pType)
{
    _dataRep._flags = flgs;
    _objHdr.setDataLength(8);
}

///
COPSPolicyElement::COPSPolicyElement(const char* dataPtr)
    : COPSRsvpObjBase(dataPtr)
{
    u_int16_t flags;
    memcpy(&flags, dataPtr + 4, sizeof(u_int16_t));
    _dataRep._flags = ntohs(flags);
    _objHdr.setDataLength(8);
}

COPSPolicyElement::COPSPolicyElement(const COPSPolicyElement& other)
{
    copyObj(other);
}

COPSPolicyElement&
COPSPolicyElement::operator=(const COPSPolicyElement& other)
{
    if (this != &other)
    {
        copyObj(other);
    }
    return (*this);
}

void
COPSPolicyElement::copyObj(const COPSPolicyElement& other)
{
    _objHdr = other._objHdr;
    _dataRep._flags = other._dataRep._flags;
    _dataRep._reserved = other._dataRep._reserved;
}

void
COPSPolicyElement::writeData(int id) const throw(VNetworkException&)
{
    cpLog(LOG_DEBUG, "COPSPolicyElement::writeData");
    COPSRsvpObjBase::writeData(id);
    struct shipIt
    {
        u_int16_t flags;
        u_int16_t res;
    } sIt;
    sIt.flags = htons(_dataRep._flags);
    sIt.res = htons(_dataRep._reserved);
    COPSUtil::writeData(id, &sIt, sizeof(struct shipIt));
}
