
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const COPSPrObjBase_cxx_Version =
    "$Id: COPSPrObjBase.cxx,v 1.5 2000/12/18 23:42:55 bko Exp $";

#include "vin.h"
#include "COPSPrObjBase.hxx"

COPSPrObjBase::COPSPrObjBase(const char* data)
{
    _dataRep = 0;
    u_int16_t a;
    memcpy(&a, data, 2);
    a = ntohs(a);
    memcpy(&_len, &a, 2);
    memcpy(&_sNum, data + 2, 2);
    //Get the length of data following the obj header
    u_int16_t dLen = _len - 4;
    COPSData d(data + 4, dLen);
    setData(d);
}

void
COPSPrObjBase::setData(const COPSData& data)
{
    _data = data;
    if (_data.length() % 4)
    {
        int padLen = 4 - _data.length() % 4;
        _padding = getPadding(padLen);
    }
    _len = _data.length() + 4;
}

COPSPrObjBase&
COPSPrObjBase::operator=(const COPSPrObjBase& other)
{
    if (this != &other)
    {
        _len = other._len;
        _sNum = other._sNum;
        _sType = other._sType;
        _data = other._data;
        _padding = other._padding;
    }
    return (*this);
}

u_int16_t
COPSPrObjBase::getDataLength()const
{
    //Add the size of the header also
    //Header length contains the header+data length
    return (_len + _padding.length());
}


void
COPSPrObjBase::writeData(int id) const throw(VNetworkException&)
{
    u_int16_t len = htons(_len);
    COPSUtil::writeData(id, &len, 2);
    COPSUtil::writeData(id, &_sNum, 2);
    COPSUtil::writeData(id, _data.getData(), _data.length());
    if (_padding.length())
    {
        COPSUtil::writeData(id, _padding.getData(), _padding.length());
    }
}

string
COPSPrObjBase::getStrSNum() const
{
    switch (getSNum())
    {
        case PR_PRID:
        return ("PRID");
        break;
        case PR_PPRID:
        return ("PRID Prefix");
        break;
        case PR_EPD:
        return ("EPD");
        break;
        case PR_GPERR:
        return ("GPERR");
        break;
        case PR_CPERR:
        return ("CPERR");
        case PR_IDERR:
        return ("IDERR");
        default:
        return ("Unknown");
    }
}

void
COPSPrObjBase::dump(ostream& os)const
{
    os.width(25);
    os << "**" << getStrSNum() << "**" << endl;
    os.width(25);
    os << dec << endl;
    os << "Length: " << _len << endl;
    os.width(25);
    os << "S-num: " << static_cast < int > (_sNum) << endl;
    os.width(25);
    os << "S-type: " << static_cast < int > (_sType) << endl;
}

void*
COPSPrObjBase::getDataRep()
{
    delete _dataRep;
    _dataRep = new char[getDataLength()];

    u_int16_t len = htons(_len);
    memcpy(_dataRep, &len, 2);
    memcpy(_dataRep + 2, &_sNum, 2);
    memcpy(_dataRep + 4, _data.getData(), _data.length());
    if (_padding.length())
    {
        memcpy(_dataRep + 4 + _data.length(), _padding.getData(), _padding.length());
    }
    return (void*)_dataRep;
}
