#ifndef COPSPrObjBase_hxx
#define COPSPrObjBase_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const COPSPrObjBase_hxx_Version =
    "$Id: COPSPrObjBase.hxx,v 1.7 2001/05/13 07:26:02 icahoon Exp $";

#include "VocalCommon.hxx"
#include <cassert>
#include <netinet/in.h>
#include <iostream>
#include <string>
#include "COPSUtil.hxx"
#include "COPSData.hxx"
#include "COPSObjHeader.hxx"
#include "cpLog.h"
#include "VNetworkException.hxx"

///COPSPrObjBase
class COPSPrObjBase
{
    public:
        enum PrSNum
        {
            PR_PRID = 1,
            PR_PPRID = 2,
            PR_EPD = 3,
            PR_GPERR = 4,
            PR_CPERR = 5,
            PR_IDERR = 6
    };
        enum PrSType
        {
            PR_BER = 1
    };

        /**Add padding in the data, if the Provisioning data does
         * not fall on 32-bit boundary
         */
        COPSData getPadding(int len)
        {
            char* padBuf = new char[len];
            memset(padBuf, '0', len);
            COPSData d(padBuf, len);
            delete [] padBuf;
            return d;
        }

        ///
        virtual string className()const
        {
            return "COPSPrObjBase";
        };

        ///
        friend ostream& operator<<(ostream& os, const COPSPrObjBase&);

        ///
        virtual bool isPRID() const
        {
            return false;
        };
        ///
        virtual bool isPRIDPrefix() const
        {
            return false;
        };
        ///
        virtual bool isEncodedInstanceData() const
        {
            return false;
        };
        ///
        virtual bool isGlobalPrError() const
        {
            return false;
        };
        ///
        virtual bool isPRCClassError() const
        {
            return false;
        };
        ///
        virtual bool isErrorPRID() const
        {
            return false;
        };

        ///
        virtual void setData(const COPSData& data);

        ///
        COPSPrObjBase& operator=(const COPSPrObjBase& other);

        ///Get the class information identifier cNum
        COPSPrObjBase::PrSNum getSNum() const
        {
            return static_cast < COPSPrObjBase::PrSNum > (_sNum);
        };

        ///Get the type per sNum
        COPSPrObjBase::PrSType getSType() const
        {
            return static_cast < COPSPrObjBase::PrSType > (_sType);
        };


        ///Get stringified CNum
        string getStrSNum() const;

        ///Returns size in number of octects, including header
        virtual u_int16_t getDataLength() const;

        ///
        const COPSData& getData() const
        {
            return _data;
        };

        ///
        virtual void dump(ostream& os)const;

        ///Write data on a given network socket
        virtual void writeData(int id) const throw(VNetworkException&);

        ///Get the binary data contained in the object
        virtual void* getDataRep();

        ///
        virtual ~COPSPrObjBase()
        { }
        ;

    protected:
        friend class COPSPrObjParser;
        friend class COPSReqMsg;
        friend class COPSReportMsg;

        ///
        COPSPrObjBase(): _dataRep(0)
        { }
        ;

        ///
        COPSPrObjBase(const char* dataPtr);

        u_int16_t _len;
        u_int8_t _sNum;
        u_int8_t _sType;
        COPSData _data;
        COPSData _padding;

        char* _dataRep;
};

inline ostream& operator<<(ostream& os, const COPSPrObjBase& obj)
{
    obj.dump(os);
    return os;
}

#endif
