
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const COPSReason_cxx_Version =
    "$Id: COPSReason.cxx,v 1.8 2001/05/13 07:26:02 icahoon Exp $";


#include "VocalCommon.hxx"
#include <netinet/in.h>

#include "COPSReason.hxx"

static const char* G_msgArray[] =
    {
        "Unknown.",
        "Unspecified.",
        "Management.",
        "Preempted (Another request state takes precedence).",
        "Tear (Used to communicate a signaled state removal).",
        "Timeout ( Local state has timed-out).",
        "Route change (Change invalidates request state).",
        "Insufficient Resources.",
        "PDP's Directive.",
        "Unsupported decision.",
        "Synchronize handle unknown.",
        "Transient handle.",
        "Malformed decision.",
        "Unknown COPS object from PDP.",
    };

COPSReason::COPSReason(u_int16_t reasonCode, u_int16_t subCode)
{
    _dataRep._reasonCode = reasonCode;
    _dataRep._reasonSubCode = subCode;
    _objHdr.setCNum(COPS_REASON_CODE);
    _objHdr.setCType(1);
    _objHdr.setDataLength(sizeof(u_int32_t));
}

COPSReason::COPSReason(const char* dataPtr)
{
    _objHdr.parse(dataPtr);
    _objHdr.checkDataLength();
    u_int16_t rCode;
    u_int16_t rSubCode;
    memcpy(&rCode, dataPtr + 4, sizeof(u_int16_t));
    memcpy(&rSubCode, dataPtr + 6, sizeof(u_int16_t));

    _dataRep._reasonCode = ntohs(rCode);
    _dataRep._reasonSubCode = ntohs(rSubCode);
    _objHdr.setDataLength(sizeof(u_int32_t));
}

COPSReason::COPSReason(const COPSReason& other)
{
    _objHdr = other._objHdr;
    _dataRep._reasonCode = other._dataRep._reasonCode;
    _dataRep._reasonSubCode = other._dataRep._reasonSubCode;
}

string
COPSReason::getDescription() const
{
    string reasonStr1;
    string reasonStr2;

    ///Get the details from the error code
    reasonStr1 = G_msgArray[_dataRep._reasonCode];
    //TODO - defind reason sub-codes
    reasonStr2 = "";
    return (reasonStr1 + ":", reasonStr2);
}
void
COPSReason::dump(ostream& os) const
{
    os << _objHdr << endl;
    os.width(25);
    os << "Reason Code: " << _dataRep._reasonCode << endl;
    os.width(25);
    os << "Reason Sub Code: " << _dataRep._reasonSubCode << endl;
}

COPSReason&
COPSReason::operator=(const COPSReason& other)
{
    if (this != &other)
    {
        _objHdr = other._objHdr;
        _dataRep._reasonCode = other._dataRep._reasonCode;
        _dataRep._reasonSubCode = other._dataRep._reasonSubCode;
    }
    return (*this);
}


void
COPSReason::writeData(int id) const throw(VNetworkException&)
{
    cpLog(LOG_DEBUG, "COPSContext::writeData");
    _objHdr.writeData(id);
    u_int16_t rCode = htons(_dataRep._reasonCode);
    u_int16_t rSubCode = htons(_dataRep._reasonSubCode);
    COPSUtil::writeData(id, &rCode, 2);
    COPSUtil::writeData(id, &rSubCode, 2);
}
