
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const COPSReportType_cxx_Version =
    "$Id: COPSReportType.cxx,v 1.9 2001/05/13 07:26:02 icahoon Exp $";

#include "VocalCommon.hxx"
#include <netinet/in.h>


#include "COPSReportType.hxx"

static const char* msgMap[] = {
    "Unknown",
    "Success",
    "Failure",
    "Accounting",
};

COPSReportType::COPSReportType(RepType rType)
{
    _objHdr.setCNum(COPS_RPT);
    _objHdr.setCType(1);
    _dataRep._rType = rType;
    _objHdr.setDataLength(sizeof(u_int32_t));
}

///
COPSReportType::COPSReportType(const char* dataPtr)
{
    _objHdr.parse(dataPtr);
    _objHdr.checkDataLength();
    u_int16_t rType;
    u_int16_t reserved;
    memcpy(&rType, dataPtr + 4, 2);
    memcpy(&reserved, dataPtr + 6, 2);

    _dataRep._rType = ntohs(rType);
    _dataRep._reserved = ntohs(reserved);

    _objHdr.setDataLength(sizeof(u_int32_t));
}

COPSReportType::COPSReportType(const COPSReportType& other)
{
    _objHdr = other._objHdr;
    _dataRep._rType = other._dataRep._rType;
    _dataRep._reserved = other._dataRep._reserved;
}

void
COPSReportType::dump(ostream& os) const
{

    os << _objHdr << endl;
    os.width(25);
    os << "Report: " << msgMap[_dataRep._rType] << endl;
}


COPSReportType&
COPSReportType::operator=(const COPSReportType& other)
{
    if (this != &other)
    {
        _objHdr = other._objHdr;
        _dataRep._rType = other._dataRep._rType;
        _dataRep._reserved = other._dataRep._reserved;
    }
    return (*this);
}

u_int16_t
COPSReportType::getDataLength()const
{
    //Add the size of the header also
    return (_objHdr.getDataLength());
}

void
COPSReportType::writeData(int id) const throw(VNetworkException&)
{
    cpLog(LOG_DEBUG, "COPSContext::writeData");
    _objHdr.writeData(id);
    u_int16_t rType = htons(_dataRep._rType);
    u_int16_t reserved = htons(_dataRep._reserved);
    COPSUtil::writeData(id, &rType, 2);
    COPSUtil::writeData(id, &reserved, 2);
}
