#ifndef COPSReqMsg_hxx
#define COPSReqMsg_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const COPSReqMsg_hxx_Version =
    "$Id: COPSReqMsg.hxx,v 1.10 2000/12/18 23:42:55 bko Exp $";

#include <vector>
#include <map>

#include "Sptr.hxx"
#include "COPSMsg.hxx"


#include  "COPSHandle.hxx"
#include  "COPSContext.hxx"
#include  "COPSInterface.hxx"
#include  "COPSClientSI.hxx"
#include  "COPSLPDPDecision.hxx"
#include  "COPSIntegrity.hxx"
#include  "VNetworkException.hxx"

///COPSReqMsg

class COPSReqMsg : public COPSMsg
{
    public:
        ///
        typedef map < Sptr < COPSContext > , vector < COPSLPDPDecision > , less < Sptr < COPSContext > > > DecisionMap;
        ///
        COPSReqMsg();

        ///
        virtual ~COPSReqMsg();


        /** Checks the sanity of COPS message and throw an
          * COPSBadDataException when data is bad.
          */
        void checkSanity() const throw (COPSBadDataException&);

        ///Add header to the message
        COPSReqMsg& operator<<(const COPSHeader& obj);

        ///Add Context object to the message
        COPSReqMsg& operator<<(const COPSContext& obj);

        ///Add client handle to the message
        COPSReqMsg& operator<<(const COPSHandle& obj);

        ///Add an IN or OUT interface object
        COPSReqMsg& operator<<(const COPSInterface& obj);

        ///Add one or more clientSI objects
        COPSReqMsg& operator<<(const COPSClientSI& obj);

        /**Add one or more local decision object for a given decision context
         * the context is optional, if null all decision object are tided to
         * message context
         */
        void addLocalDecision(const COPSLPDPDecision& obj, Sptr < COPSContext > cntxt = 0);

        ///Add integrity object
        COPSReqMsg& operator<<(const COPSIntegrity& obj);


        ///Dumps the whole object to a output stream
        void dump(ostream& os) const;

        ///Writes data to given file descriptor
        void writeData(int id) const throw (VNetworkException&);

        ///
        string className() const
        {
            return "COPSReqMsg";
        };


        ///Accessor
        const COPSHeader& getHeader() const
        {
            return *_hdr;
        };
        ///
        const COPSHandle& getClientHandle() const
        {
            return *_clientHandle;
        };
        ///
        const COPSContext& getContext() const
        {
            return *_context;
        };

        ///Returns true if it has In Interface
        bool hasInInterface() const
        {
            return (_inInterface.getPtr() != 0);
        };

        ///Should check hasInInterface() before calling
        const Sptr < COPSInterface > & getInInterface() const
        {
            return _inInterface;
        };

        ///Returns true if it has Out interface
        bool hasOutInterface() const
        {
            return (_outInterface.getPtr() != 0);
        };

        ///Should check hasOutInterface() before calling
        const Sptr < COPSInterface > & getOutInterface() const
        {
            return _outInterface;
        };

        ///Returns a vector if ClientSI objects
        const vector < COPSClientSI > & getClientSI() const
        {
            return _clientSI;
        };

        ///Returns a Map of any local decisions
        const DecisionMap& getLpdpDecisions() const
        {
            return _decisions;
        };

        ///Returns true if it has Integrity object
        bool hasIntegrity() const
        {
            return (_integrity.getPtr() != 0);
        };

        ///Should check hasIntegrity() becfore calling
        const Sptr < COPSIntegrity > & getIntegrity() const
        {
            return _integrity;
        };

    private:
        ///
        friend class COPSMsgParser;

        ///
        COPSReqMsg(const char* data);

        ///Parses the data and fills COPSReqMsg with its constituents
        void parse(const char* data) throw (COPSBadDataException&);

        ///Parses the data and fills that follows the header hdr and fills COPSReqMsg
        void parse(const COPSHeader&hdr, const char* data) throw (COPSBadDataException&);

        ///
        void setMsgLength();

        /* COPSHeader coming from base class */
        Sptr < COPSHandle > _clientHandle;
        Sptr < COPSContext > _context;
        Sptr < COPSInterface > _inInterface;
        Sptr < COPSInterface > _outInterface;
        vector < COPSClientSI > _clientSI;
        DecisionMap _decisions;
        Sptr < COPSIntegrity > _integrity;
        Sptr < COPSContext > _lpdpContext;
};

#endif
