#ifndef COPSRsvpObjBase_hxx
#define COPSRsvpObjBase_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const COPSRsvpObjBase_hxx_Version =
    "$Id: COPSRsvpObjBase.hxx,v 1.3 2001/05/13 07:26:02 icahoon Exp $";

#include "VocalCommon.hxx"
#include <cassert>
#include <netinet/in.h>
#include <iostream>
#include <string>
#include "COPSData.hxx"
#include "COPSPolElementHeader.hxx"

///COPSRsvpObjBase
class COPSRsvpObjBase
{
    public:
        /**Add padding in the data, if the Provisioning data does
         * not fall on 32-bit boundary
         */
        COPSData getPadding(int len)
        {
            char* padBuf = new char[len];
            memset(padBuf, '0', len);
            COPSData d(padBuf, len);
            delete [] padBuf;
            return d;
        }

        ///Returns size in number of octects
        u_int16_t getDataLength() const
        {
            return (_objHdr.getDataLength());
        };


        ///
        virtual string className()const
        {
            return "COPSRsvpObjBase";
        };

        ///Writes object to given network socket in network byte order
        virtual void writeData(int id) const throw(VNetworkException&)
        {
            _objHdr.writeData(id);
        }

        ///
        virtual bool isRsvpProxySupport() const
        {
            return (_objHdr.getPType() == COPS_POL_EL_RSVP_PROXY_SUPPORT);
        };

        ///
        virtual bool isPolicingSupport() const
        {
            return (_objHdr.getPType() == COPS_POL_EL_POLICING_SUPPORT);
        };
        ///
        virtual bool isMarkingSupport() const
        {
            return (_objHdr.getPType() == COPS_POL_EL_MARKING_SUPPORT);
        };
        ///
        virtual bool isInRoleCombination() const
        {
            return (_objHdr.getPType() == COPS_POL_EL_IN_ROLE_COMB);
        };
        ///
        virtual bool isOutRoleCombination() const
        {
            return (_objHdr.getPType() == COPS_POL_EL_OUT_ROLE_COMB);
        };

        ///
        virtual ~COPSRsvpObjBase() { };

        ///
        friend ostream& operator<<(ostream& os, const COPSRsvpObjBase& obj);

        ///
        virtual void dump(ostream& os) const = 0;

    protected:
        ///
        friend class COPSRsvpObjParser;
        ///
        friend class COPSReqMsg;
        ///
        friend class COPSOpenMsg;

        ///
        COPSRsvpObjBase() { };

        ///
        COPSRsvpObjBase(COPSp_type pType)
        {
            _objHdr = COPSPolElementHeader(pType);
        };

        COPSRsvpObjBase(const char* dataPtr)
        {
            _objHdr.parse(dataPtr);
        }

        ///
        COPSPolElementHeader _objHdr;
};

inline ostream& operator<<(ostream& os, const COPSRsvpObjBase& obj)
{
    os << obj._objHdr << endl;
    obj.dump(os);
    return os;
}


#endif
