
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const OSPAuthInfo_cxx_Version =
    "$Id: OSPAuthInfo.cxx,v 1.8 2001/01/20 01:33:06 wbreu Exp $";
#include "cpLog.h"
#include "OSPAuthInfo.hxx"

OSPAuthInfo::OSPAuthInfo()
{
    cpLog(LOG_DEBUG, "OSPAuthInfo ctor invoked");
    _ospAuthInfo._numCallIds = 1;
    _ospAuthInfo._numDest = 1;
    _ospAuthInfo._preferDests = 0;
    _ospAuthInfo._callIds = 0;
}

OSPAuthInfo::~OSPAuthInfo()
{
    cpLog(LOG_DEBUG, "OSPAuthInfo dtor invoked ");
    // For Future: need to delete list of callIds and preferred destinations
    //      for now, use 1 callId and no preferred destinations.
    if (_ospAuthInfo._callIds)
    {
        OSPCallIdList *callId = _ospAuthInfo._callIds;
        OSPPCallIdDelete(&callId);
    }
}

const char * OSPAuthInfo::getSource( void ) const
{
    return ( _ospAuthInfo._source.c_str() );
}

int OSPAuthInfo::setSource( const string& source )
{
    if (source.length() > 0)
    {
        _ospAuthInfo._source = source;
        return 0;
    }
    else return -1;
}

int OSPAuthInfo::setCallId( int idx, int len, const char * val )
{
    assert(val);
    if (_ospAuthInfo._callIds)  // already has a CallId
    {
        cpLog(LOG_DEBUG,
              "_ospAuthInfo._callIds already initialized, cur callId=%s ",
              _ospAuthInfo._callIds->ospmCallIdVal);
        return -1;
    }
    // Future enhancements: to handle an array of CallIds
    _ospAuthInfo._callIds = OSPPCallIdNew( len, reinterpret_cast < const unsigned char* > (val) );
    assert(_ospAuthInfo._callIds);
    return 0;
}


void OSPAuthInfo::dump( void )
{
    cpLog( LOG_DEBUG, "source=%s, calling#=%s", _ospAuthInfo._source.c_str(),
           _ospAuthInfo._callingNum.c_str() );
}

// end of file
