#ifndef OSPVALINFO_HXX
#define OSPVALINFO_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const OSPValInfo_hxx_Version =
    "$Id$";

#include <sys/types.h>
#include <string>
#include "OSPDefines.hxx"


/// Information class needed for validating authorization request
class OSPValInfo
{
    public:
        /// constructor
        OSPValInfo( void );
        /// destructor
        virtual ~OSPValInfo();

        ///  get _source data member accessor
        const char * getSource( void );

	/// set source data
        int setSource( const string& source );

        /// _destination data member accessor
        const char * getDestination( void );

	/// set destination data
        int setDestination( const string& dest);

        /// sourceDevice accessor
        const char * getSourceDevice(void)
        {
            return (_ospValInfo._sourceDevice.c_str());
        }

	/// set source device info
        void setSourceDevice( const char * s )
        {
            assert( s );
            _ospValInfo._sourceDevice = s;
        }

        /// destDevice accessor
        const char * getDestDevice(void)
        {
            return (_ospValInfo._destDevice.c_str());
        }

	/// set destDevice info
        void setDestDevice( const char * s )
        {
            assert( s );
            _ospValInfo._destDevice = s;
        }

        /// _callingNum accessor
        const char * getCallingNum(void)
        {
            return (_ospValInfo._callingNum.c_str());
        }

	/// set calling number
        void setCallingNum( const char * cn )
        {
            assert( cn );
            _ospValInfo._callingNum = cn;
        }

        /// _calledNum accessor
        const char * getCalledNum(void)
        {
            return (_ospValInfo._calledNum.c_str());
        }

	/// set callee's number
        void setCalledNum( const char * cn )
        {
            assert( cn );
            _ospValInfo._calledNum = cn;
        }

        /// _token accessor
        const char * getToken(void)
        {
            return (_ospValInfo._token.c_str());
        }

	/// get size of token in bytes
        const unsigned int getTokenSize( void ) const
        {
            return ( _ospValInfo._token.length() );
        }

	/// store token given a C-style string and its size
        void setToken( const char *t, unsigned int size)
        {
            assert( t );
            _ospValInfo._token.assign( t, size );
        }

	/// store token given a string object
        void setToken( string& t )
        {
            _ospValInfo._token = t;
        }

        // _callId accessor
        const char * getCallId( void )
        {
            return ( _ospValInfo._callId.c_str());
        }

	/// get size of call id
        unsigned int getCallIdSize( void )
        {
            return ( _ospValInfo._callId.length() );
        }

	/// store call id
        void setCallId( const char * val )
        {
            assert( val );
            _ospValInfo._callId = val;
        }

        /// _authFlag accessor
        unsigned int getAuthFlag( void ) const
        {
            return ( _ospValInfo._authFlag);
        }

	/// store value of authorization flag
        void setAuthFlag( unsigned int sts )
        {
            _ospValInfo._authFlag = sts;
        }

        /// _timeLimit accessor
        unsigned int getTimeLimit( void ) const
        {
            return ( _ospValInfo._timeLimit );
        }

	/// store time limit
        void setTimeLimit( unsigned int t )
        {
            _ospValInfo._timeLimit = t;
        }

        /// debug only
        void dump(void );

    private:
        /// equal operator - not in use
        bool operator==( const OSPValInfo& rhs );
        /// copy constructor - not in use
        OSPValInfo( const OSPValInfo& rhs );
        /// assignment operator - not in use
        OSPValInfo& operator=( const OSPValInfo& rhs );

        /// structure wrapping information needed for validating the auth token
        struct
        {
            string _source;        // "[192.82.1.101]:112" or "www.carrier.com"
            string _destination;   // terminating end of a call
            string _sourceDevice;  // optional, OK to be ""
            string _destDevice;    // optional, OK to be ""
            string _callingNum;    // "14083831000"
            string _calledNum;     // "14087673676"
            string _callId;        // call identifier
            string _token;         // token

            /// OSP Server returns validate authorization status and time limit
            unsigned int _authFlag;    // non-zero value = call is authorized
            // 0 = authorization failure
            unsigned int _timeLimit;   // # of seconds for which call is initially
            // authorized
        }

        _ospValInfo;

};

// end of File
#endif
