
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const PolicyTransactionVersion = "$Id: PolicyTransaction.cxx,v 1.16 2001/01/20 01:33:06 wbreu Exp $";



#include "stdio.h"    // needed for sprintf()
#include "cpLog.h"
#include "OSPTime.hxx"
#include "PolicyTransaction.hxx"
// uncomment for debug: #include "ospb64.h"


///
PolicyTransaction::PolicyTransaction()
{
    cpLog(LOG_DEBUG, "PolicyTransaction ctor invoked" );
    _Id = -1;         // OSPInvalidTransHandle;
    _status = -1;     // default to failure
    _duration = 0;
    _tokenSize = 0;
    _valSts = 0;      // authorization token validation fails
                      // used for incoming call validation
    _validAfterTs = "UNLIMITED";
    _validUntilTs = "UNLIMITED";
    _token = "Uninitialized";
}

///
PolicyTransaction::~PolicyTransaction()
{
    cpLog(LOG_DEBUG, "PolicyTransaction dtor invoked" );
}

///
string PolicyTransaction::encode( void ) const
{
    char str1[512];
    string str2;
    memset( str1, 0, sizeof( str1 ));
    //  encoding scheme:
    //  <id>|<status>|<duration>|<tokensize>|<callid>|<validafter>|<validuntil>|<valStatus>|<token>
    sprintf(str1, "%d|%d|%d|%d|%s|%s|%s|%u|",
            _Id, _status, _duration, _tokenSize,
            _callId.c_str(), _validAfterTs.c_str(),
            _validUntilTs.c_str(), _valSts);
    str2 = str1;
    str2 += _token.c_str();     // let token be the last encoded piece

    cpLog( LOG_DEBUG, "encoded string: |%s|", str2.c_str());
    return str2;

}   // encode()

///
void PolicyTransaction::decode( const string & str ) throw
                        (PolicyBadDataException&)
{
    if (str.length() == 0)
    {
        cpLog(LOG_ALERT, "cannot decode a nil string");

        throw PolicyBadDataException( "string to be decoded is nil",
                                      __FILE__, __LINE__, 0 );
    }

    char * code = new char[ str.length() + 1 ];
    char * tok;
    memset( code, 0, str.length() + 1);

    strncpy(code, str.c_str(), str.length());

    char * s = code;
    int i = 0;
    tok = strtok( s, "|");

    // coded string is contains fields delimited by "|":
    //  <id>|<status>|<duration>|<tokensize>|<callid>|<validafter>|<validuntil>|<valStatus>|<token>
    // extract each field and update this object's data members: 
    while (tok)   // in order the data encoded above:
    {
        i += 1;
        switch (i)
        {
            case 1:
            _Id = atoi( tok );
            break;
            case 2:
            _status = atoi( tok );
            break;
            case 3:
            _duration = atoi( tok );
            break;
            case 4:
            _tokenSize = atoi( tok );
            break;
            case 5:
            _callId = tok;
            break;
            case 6:
            _validAfterTs = tok;
            break;
            case 7:
            _validUntilTs = tok;
            break;
            case 8:
            _valSts = atoi(tok);
            break;
            case 9:
            _token = tok;
            break;
            default:   // ignore additional token encountered
            cpLog(LOG_ALERT, "decode() ignoring additional field, %s",
                  tok);
            break;
        }
        tok = strtok( 0, "|");
    }
    delete [] code;
    print();


}   // decode()

///
time_t PolicyTransaction::getTm( const char* ts ) const
{
    time_t t = 0;
    if ( ts != 0)
    {
        t = OSPTime::timeStringToTime_t( ts );
    }


    return ( t );

}

/// Debug only:
void PolicyTransaction::print( void )
{
    char output[256];
    output[0] = '\0';
    // uncomment for debug:
    // int outsize = 255;
    // OSPPBase64Encode(
    //     reinterpret_cast<const unsigned char*>(getMd5CallId()),
    //     16, output, &outsize);
    // output[outsize] = '\0';
    cpLog( LOG_DEBUG, "Transaction id: %d\n"
           "Call Id: |%s|  Status: %d\n"
           //    "Md5 Call Id: |%s| (in base 64 encoding)\n"
           "Duration: %d   Token size: %d  Validation status: %u\n"
           "Valid After: |%s|  Valid Until: |%s|\n"
           "Validation Status: %d\n",
           // "Token: |%s|\n",
           // getID(), getCallId(), getStatus(), output,
           getID(), getCallId(), getStatus(),
           getDuration(), getTokenSize(), getValidationSts(),
           getValidAfterTs(), getValidUntilTs(), getValidationSts() );
    // getValidAfterTs(), getValidUntilTs(), getToken() );

}

// end of file
