/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const PolRequest_cxx_Version =
    "$Id: PolRequest.cxx,v 1.28.22.1 2002/05/15 01:24:05 sprajpat Exp $";

#include <stdio.h>
#include <strstream.h>

#include "PepAgent.hxx"
#include "VThreadPool.hxx"
#include "PolRequest.hxx"
#include "ReqStateComplete.hxx"
#include "ReqStateFail.hxx"
#include "ReqStateImplFinish.hxx"
#include "ReqStateImplStart.hxx"
#include "ReqStatePending.hxx"
#include "ReqStateReceived.hxx"
#include "ReqStateSent.hxx"
#include "ReqStateTimeout.hxx"
#include "ReqStateUninitialized.hxx"
#include "ReqStateDelete.hxx"
#include "OSPTime.hxx"
#include "COPSReportMsg.hxx"
#include "COPSHandle.hxx"
#include "Data.hxx"

static const int BUFSIZE = 256;
static const int BIG_BUFSIZE = 4096;
static const int USAGE_RETRIES = 3;


PolRequest::PolRequest(const string& calledNumber,
                       const string& callingNumber,
                       const string& callId,
                       COPS_msg_type mType):
        _state(new ReqStateUninitialized),
        _clientHandle(""),
        _copsMsg(0),
        _polTransaction(0) ,
        _startTime(0),
        _endTime(0),
        _status(PolRequest::STATUS_OK),
        _usageRetry(0),
        _sendRetry(0)
{
    _callId = callId;

    _clientHandle = callId;

    //Create COPS Message

    COPSHeader hdr(COPS_OP_REQ, C_SIP);
    COPSContext cntxt(COPSContext::OUT, M_OSP);

    COPSClientSI cSi(COPSClientSI::CSI_NAMED);
    COPSData csiData(COPS_key::callingNumber + "|" + callingNumber);
    cSi.setData(csiData);

    COPSClientSI cSi2(COPSClientSI::CSI_NAMED);
    COPSData csi2Data(COPS_key::calledNumber + "|" + calledNumber);
    cSi2.setData(csi2Data);

    COPSClientSI cSi3(COPSClientSI::CSI_NAMED);
    COPSData csi3Data(COPS_key::user + "|" + "Vovida");
    cSi3.setData(csi3Data);

    COPSClientSI cSi4(COPSClientSI::CSI_NAMED);
    COPSData csi4Data(COPS_key::source + "|" + PepAgent::instance().getDomainName());
    cSi4.setData(csi4Data);

    COPSClientSI cSi5(COPSClientSI::CSI_NAMED);
    COPSData csi5Data(COPS_key::sourceDev + "|" + "PolicyServer");
    cSi5.setData(csi5Data);

    COPSClientSI cSi6(COPSClientSI::CSI_NAMED);
    COPSData csi6Data(COPS_key::callId + "|" + callId);
    cSi6.setData(csi6Data);

    COPSHandle handle;
    handle.setId( _clientHandle );

    COPSReqMsg* msg = new COPSReqMsg;
    *msg << hdr ;
    *msg << handle ;
    *msg << cntxt ;
    *msg << cSi ;
    *msg << cSi2 ;
    *msg << cSi3 ;
    *msg << cSi4 ;
    *msg << cSi5 ;
    *msg << cSi6 ;

    _copsMsg = msg;
}

PolRequest::PolRequest(const string& calledNumber,
                       const string& callingNumber,
                       const string& callId,
                       const COPSData& token,
                       COPS_msg_type mType):
        _state(new ReqStateUninitialized),
        _clientHandle(""),
        _copsMsg(0),
        _polTransaction(0) ,
        _startTime(0),
        _endTime(0),
        _status(PolRequest::STATUS_OK),
        _usageRetry(0),
        _sendRetry(0)
{
    _callId = callId;
    
    _clientHandle = callId;

    //Create COPS Message

    COPSHeader hdr(COPS_OP_REQ, C_SIP);
    COPSContext cntxt(COPSContext::IN_ADMIN, M_OSP);

    COPSClientSI cSi(COPSClientSI::CSI_NAMED);
    COPSData csiData(COPS_key::callingNumber + "|" + callingNumber);
    cSi.setData(csiData);

    COPSClientSI cSi2(COPSClientSI::CSI_NAMED);
    COPSData csi2Data(COPS_key::calledNumber + "|" + calledNumber);
    cSi2.setData(csi2Data);

    COPSClientSI cSi3(COPSClientSI::CSI_NAMED);
    COPSData csi3Data(COPS_key::user + "|" + "Vovida");
    cSi3.setData(csi3Data);

    COPSClientSI cSi4(COPSClientSI::CSI_NAMED);
    COPSData csi4Data(COPS_key::source + "|" + PepAgent::instance().getDomainName());
    cSi4.setData(csi4Data);

    COPSClientSI cSi5(COPSClientSI::CSI_NAMED);
    COPSData csi5Data(COPS_key::sourceDev + "|" + "PolicyServer");
    cSi5.setData(csi5Data);

    COPSClientSI cSi6(COPSClientSI::CSI_NAMED);
    COPSData csi6Data(COPS_key::callId + "|" + callId);
    cSi6.setData(csi6Data);


    COPSClientSI cSi7(COPSClientSI::CSI_NAMED);
    COPSData csi7Data(COPS_key::destination + "|" + PepAgent::instance().getHostName());
    cSi7.setData(csi7Data);

    COPSClientSI cSi8(COPSClientSI::CSI_NAMED);
    COPSData csi8Data(token);
    cSi8.setData(csi8Data);

    COPSHandle handle;
    handle.setId( _clientHandle );

    COPSReqMsg* msg = new COPSReqMsg;
    *msg << hdr ;
    *msg << handle ;
    *msg << cntxt ;
    *msg << cSi ;
    *msg << cSi2 ;
    *msg << cSi3 ;
    *msg << cSi4 ;
    *msg << cSi5 ;
    *msg << cSi6 ;
    *msg << cSi7 ;
    *msg << cSi8 ;

    _copsMsg = msg;
}


PolRequest::PolRequest(const string& callId, COPS_msg_type mType):
        _state(new ReqStateUninitialized),
        _clientHandle(""),
        _copsMsg(0),
        _polTransaction(0) ,
        _startTime(0),
        _endTime(0),
        _status(PolRequest::STATUS_OK)
{

    //Create COPS Message

    _callId = callId;
    _clientHandle = COPSData(_callId);

    COPSHeader hdr(COPS_OP_REQ, C_SIP);

    COPSHandle handle;
    handle.setId(_clientHandle);

    COPSContext cntxt(COPSContext::RES_ALLOC, M_QOS);

    COPSReqMsg* msg = new COPSReqMsg;
    *msg << hdr ;
    *msg << handle ;
    *msg << cntxt ;

    _copsMsg = msg;
}

void
PolRequest::enableQoS(const string& sourceIp, int sourcePort,
                      const string& destIp, int destPort)
{
    char buf[BIG_BUFSIZE];
    char buf_src[BUFSIZE], buf_dest[BUFSIZE];

    sprintf(buf_src, "%s_%d", sourceIp.c_str(), sourcePort);
    sprintf(buf_dest, "%s_%d", destIp.c_str(), destPort);

    //Create COPS Message

    COPSClientSI cSi1(COPSClientSI::CSI_NAMED);
    COPSData csi1Data(ENABLE_QOS);
    cSi1.setData(csi1Data);

    COPSClientSI cSi2(COPSClientSI::CSI_NAMED);
    COPSData csi2Data(_callId.c_str());
    cSi2.setData(csi2Data);

    COPSClientSI cSi3(COPSClientSI::CSI_NAMED);
    COPSData csi3Data(buf_src);
    cSi3.setData(csi3Data);

    COPSClientSI cSi4(COPSClientSI::CSI_NAMED);
    COPSData csi4Data(buf_dest);
    cSi4.setData(csi4Data);

    COPSReqMsg* msg = dynamic_cast<COPSReqMsg*>(_copsMsg.getPtr());
    *msg << cSi1 << cSi2 << cSi3 << cSi4;

    ostrstream ostr(buf, BIG_BUFSIZE-1);
    ostr << *msg << endl << ends;
    cpLog(LOG_DEBUG, "Enabling QoS %s", ostr.str());

    if (PepAgent::instance().sendSyncWithRetries(*this) == -1)
    {
        cpLog(LOG_DEBUG, "Failed to enable QoS for %s", _callId.c_str());
    }
}

void
PolRequest::disableQoS()
{
    char buf[BIG_BUFSIZE];

    COPSClientSI cSi1(COPSClientSI::CSI_NAMED);
    COPSData csi1Data(DISABLE_QOS);
    cSi1.setData(csi1Data);

    COPSClientSI cSi2(COPSClientSI::CSI_NAMED);
    COPSData csi2Data(_callId.c_str());
    cSi2.setData(csi2Data);

    COPSReqMsg* msg = dynamic_cast<COPSReqMsg*>(_copsMsg.getPtr());
    *msg << cSi1 << cSi2;

    ostrstream ostr(buf, BIG_BUFSIZE-1);
    ostr << *msg << endl << ends;
    cpLog(LOG_DEBUG, "Disabling QoS %s", ostr.str());

    if (PepAgent::instance().sendSyncWithRetries(*this) == -1)
    {
        cpLog(LOG_DEBUG, "Failed to disable QoS for %s", _callId.c_str());
    }
}

void
PolRequest::init()
{
    _state->init(*this);
}

void
PolRequest::auth()
{
    init();
}

void
PolRequest::validate()
{
    init();
}

void
PolRequest::sendCOPSMsg(Connection& conn) throw (VNetworkException&)
{
    assert(_copsMsg != 0);
    try
    {
        _copsMsg->writeData(conn.getConnId());
        _state->send(*this);
        _sendRetry = 0;
    }
    catch (VNetworkException& e)
    {
        cpLog(LOG_ALERT, "Failed to send COPS req, reason:%s", e.getDescription().c_str());
        conn.close();
        reSend();
        throw VNetworkException(e.getDescription(), __FILE__, __LINE__, 0);
    }
}

void
PolRequest::timeOut()
{
    _status = STATUS_TIMEDOUT;
    _state->timeOut(*this);
}

void
PolRequest::fail()
{
    _status = STATUS_FAILED;
    _state->fail(*this);
}

void
PolRequest::received()
{
    _state->received(*this);
}

void
PolRequest::implStart()
{
    _startTime = OSPTime::getCurrentTime();
    _state->implStart(*this);
}

void
PolRequest::implAbort()
{
    _endTime = OSPTime::getCurrentTime();
    _state->fail(*this);
}

void
PolRequest::implFinish()
{
    _endTime = OSPTime::getCurrentTime();
    _state->implFinish(*this);
}

void
PolRequest::createUsageReport(COPSReportType::RepType rType, bool noUsage)
{
    COPSHeader hdr(COPS_OP_RPT, C_SIP);

    COPSHandle handle ;
    COPSData id(_clientHandle);
    handle.setId(id);

    COPSReportType report(rType);

    if (_polTransaction == 0)
    {
        cpLog(LOG_ALERT, "Invalid Transaction for callId %s", _callId.c_str());
        return ;
    }

    if (noUsage)
    {
        _polTransaction->setDuration(0);
    }
    else
    {
        _polTransaction->setDuration(_endTime - _startTime);
    }

    COPSClientSI cSi(COPSClientSI::CSI_NAMED);
    cSi.setData(_polTransaction->encode());

    COPSReportMsg* msg = new COPSReportMsg;
    *msg << hdr ;
    *msg << handle ;
    *msg << report ;
    *msg << cSi ;
    _copsMsg = msg;
}

void
PolRequest::sendUsage(Connection& conn) throw (VNetworkException&)
{
    try
    {
        cpLog(LOG_DEBUG, "Thread (%d) sending usage for handle:(%s)",
              pthread_self(), getClientHandle().str().c_str());
        if(_copsMsg != 0)
        {
           _copsMsg->writeData(conn.getConnId());
        }
        _state->finish(*this);
        cpLog(LOG_DEBUG, "***Policy Request for callId [%s] marked for deletion",
              getCallId().c_str());
        _usageRetry = 0;
    }
    catch (VNetworkException& e)
    {
        cpLog(LOG_ALERT, "Failed to send usage, reason:%s", e.getDescription().c_str());
        conn.close();
        resendUsage();
    }
}

void
PolRequest::resendUsage()
{
    _usageRetry++;
    if (_usageRetry == USAGE_RETRIES) fail();
    else
    {
        //Queue it back for the usage
        PepAgent::instance().queueForUsage(*this);
    }
}

void
PolRequest::reSend()
{
    _sendRetry++;
    if (_sendRetry == USAGE_RETRIES) fail();
    else
    {
        //Queue it back for the send
        PepAgent::instance().queueForSend(*this);
    }
}

void
PolRequest::finish()
{}


void
PolRequest::remove()
{}



void
PolRequest::createNewHandle()
{
    char buf[BUFSIZE];
    sprintf(buf, "%s", _callId.c_str());
    _clientHandle = COPSData(buf);
}


void
PolRequest::setPolicyTransaction(PolicyTransaction* polT)
{
    _polTransaction = polT;
}

PolRequest::PolReqStatus
PolRequest::getStatus() const
{
    return _status;
}


string
PolRequest::getToken() const
{
    assert(_polTransaction != 0);
    return _polTransaction->getToken();
}

void
PolRequest::checkIfTimedOut()
{
    _state->checkIfTimedOut(*this);
}


const COPSContext&
PolRequest::getContext() const
{
    const COPSHeader& header = _copsMsg->getHeader();
    assert(header.isARequest());
    COPSReqMsg* rMsg = dynamic_cast < COPSReqMsg* > (_copsMsg.getPtr());
    return rMsg->getContext();
}

bool
PolRequest::isIncoming() const
{
    return (getContext().isIncomingMessage());
}

bool
PolRequest::toBeDeleted() const
{
    return (_state->toBeDeleted());
}

bool
PolRequest::hasFailed() const
{
    return (_state->hasFailed());
}
