
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const testNamedParser_cxx_Version =
    "$Id: testNamedParser.cxx,v 1.6.22.1 2002/05/15 01:24:06 sprajpat Exp $";
#include <stdio.h>
#include <iostream.h>
#include <fstream.h>

#include "COPSReqMsg.hxx"
#include "VMissingDataException.hxx"
#include "NamedDataParser.hxx"

int
main(int argc, char** argv)
{
    cpLogSetPriority(LOG_DEBUG);
    COPSHeader hdr(COPS_OP_REQ);
    COPSHandle handle;
    COPSContext cntxt(COPSContext::IN_ADMIN | COPSContext::OUT, 0);

    COPSClientSI cSi(COPSClientSI::CSI_NAMED);
    COPSData csiData("Called number|cullen@vovida.com");
    cSi.setData(csiData);

    COPSClientSI cSi2(COPSClientSI::CSI_NAMED);
    COPSData csi2Data("Calling number|luan@vovida.com");
    cSi2.setData(csi2Data);

    COPSClientSI cSi3(COPSClientSI::CSI_NAMED);
    COPSData csi3Data("Source|private.vovida.com");
    cSi3.setData(csi3Data);

    COPSClientSI cSi4(COPSClientSI::CSI_NAMED);
    COPSData csi4Data("User|luan@vovida.com");
    cSi4.setData(csi4Data);

    COPSData id("12345@domain.com");
    handle.setId(id);

    COPSLPDPDecision dec1;
    dec1.setCmdCode(COPSDecision::DEC_INSTALL);

    COPSLPDPDecision dec2(DEC_STATELESS);
    dec2.setData(COPSData("INSTALL CONfig."));

    COPSIntegrity intr;
    intr.setKeyId(19);
    intr.setSeqNum(9);
    intr.setKeyDigest(COPSData("KEY DIGEST"));

    COPSReqMsg msg;
    msg << hdr ;
    msg << handle ;
    msg << cntxt ;
    msg << cSi ;
    msg << cSi2 ;
    msg << cSi3 ;
    msg << cSi4 ;

    msg.addLocalDecision(dec1, NULL);
    msg.addLocalDecision(dec2, NULL);
    msg << intr;

    cerr << msg << endl;

    NamedDataParser parser(msg.getClientSI());
    cerr << "**DATA**" << endl;
    cerr << parser << endl;

}
