
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const testDecisionMsg_cxx_Version =
    "$Id: testPrDecisionMsg.cxx,v 1.4 2001/06/29 21:02:41 bko Exp $";
#include <stdio.h>
#include <iostream.h>
#include <fstream.h>

#include "COPSDecisionMsg.hxx"
#include "Tcp_ClientSocket.hxx"
#include "NetworkAddress.h"
#include "COPSPrID.hxx"
#include "COPSPrObjParser.hxx"

int
main(int argc, char** argv)
{
    if (argc != 2)
    {
        cerr << "Usage:" << argv[0] << " <policyServer:port>" << endl;
        exit( -1);
    }

    cpLogSetPriority(LOG_DEBUG);
    COPSHeader hdr(COPS_OP_DEC);
    COPSHandle handle;
    COPSData id("12345@domain.com");
    handle.setId(id);

    COPSDecision dec1;
    dec1.setCmdCode(COPSDecision::DEC_INSTALL);

    COPSDecision dec2(DEC_NAMED);
    COPSPrID prid;
    prid.setData(COPSData("0000-PRID-0101"));
    dec2.setData(COPSData((char*)prid.getDataRep(), prid.getDataLength()));

    COPSDecision dec3(DEC_NAMED);
    COPSPrefixPrID prefixId;
    prefixId.setData(COPSData("Prefix-0101"));
    dec3.setData(COPSData((char*)prefixId.getDataRep(), prefixId.getDataLength()));

    COPSDecision dec4(DEC_NAMED);
    COPSPrEPD epd;
    epd.setData(COPSData("Encoded-data"));
    dec4.setData(COPSData((char*)epd.getDataRep(), epd.getDataLength()));

    COPSDecision dec5(DEC_NAMED);
    COPSPrClassError cError(8, 0);
    dec5.setData(COPSData((char*)cError.getDataRep(), cError.getDataLength()));

    COPSContext cntxt(COPSContext::CONFIG, 0);

    COPSIntegrity intr;
    intr.setKeyId(19);
    intr.setSeqNum(9);
    intr.setKeyDigest(COPSData("KEY DIGEST"));

    COPSDecisionMsg msg;
    msg << hdr ;
    msg << handle ;
    msg.addDecision(dec1, cntxt);
    msg.addDecision(dec2, cntxt);
    msg.addDecision(dec3, cntxt);
    msg.addDecision(dec4, cntxt);
    msg.addDecision(dec5, cntxt);
    msg << intr;

    cerr << msg << endl;

    try
    {
        NetworkAddress addr(argv[1]);
        TcpClientSocket cSock(addr);
        Connection conn;
        cSock.connect();
        msg.writeData(cSock.getConn().getConnId());
    }
    catch (VException& e)
    {
        cerr << "Caught exception:" << e.getDescription() << endl;
        exit( -1);
    }
}
