
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const testReportMsg_cxx_Version =
    "$Id: testReportMsg.cxx,v 1.7 2001/06/29 21:02:41 bko Exp $";
#include <stdio.h>
#include <iostream.h>
#include <fstream.h>

#include "COPSReportMsg.hxx"
#include "Tcp_ClientSocket.hxx"
#include "NetworkAddress.h"

int
main(int argc, char** argv)
{
    if (argc != 2)
    {
        cerr << "Usage:" << argv[0] << " <policyServer:port>" << endl;
        exit( -1);
    }

    cpLogSetPriority(LOG_DEBUG);
    COPSHeader hdr(COPS_OP_RPT);

    COPSHandle handle ;
    COPSData id("12345@domain.com");
    handle.setId(id);

    COPSReportType report(COPSReportType::SUCCESS);

    COPSClientSI cSi(COPSClientSI::CSI_NAMED);
    COPSData csiData("Called number:To cullen@vovida.com");
    cSi.setData(csiData);

    COPSClientSI cSi2(COPSClientSI::CSI_NAMED);
    COPSData csi2Data("Calling number:From luan@vovida.com");
    cSi2.setData(csi2Data);


    COPSIntegrity intr;
    intr.setKeyId(19);
    intr.setSeqNum(9);
    intr.setKeyDigest(COPSData("KEY DIGEST"));

    COPSReportMsg msg;
    msg << hdr ;
    msg << handle ;
    msg << report ;
    msg << cSi ;
    msg << cSi2 ;
    msg << intr;

    cerr << msg << endl;

    try
    {
        NetworkAddress addr(argv[1]);
        TcpClientSocket cSock(addr);
        Connection conn;
        cSock.connect();
        msg.writeData(cSock.getConn().getConnId());
    }
    catch (VException& e)
    {
        cerr << "Caught exception:" << e.getDescription() << endl;
        exit( -1);
    }
}
