/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const ServerTable_cxx_Version = "";
#include "ServerTable.hxx"

tableElement::tableElement(serverInfoT tmpServer)
{
    element.index = tmpServer.index;
    strncpy(element.hostName, tmpServer.hostName, 255);
    element.port  = tmpServer.port ;
    element.type  = tmpServer.type ;
    element.status  = tmpServer.status ;
}

ServerTable::ServerTable()
{
}

ServerTable::~ServerTable(){}

serverInfoT 
ServerTable::getServerFromIndex(unsigned long index)
{
    tableElement tmp(index);
    serverInfoT aServer;

    tableLock.lock();
    apI = serverCache.find(tmp);
    if (apI == serverCache.end())
	aServer.index = 0;
    else 
    {
	aServer.index = apI->element.index;
	strncpy(aServer.hostName, apI->element.hostName, 255);
	aServer.port  = apI->element.port ;
	aServer.type  = apI->element.type ;
	aServer.status  = apI->element.status ;
    }
    tableLock.unlock();
    return (aServer);
}

unsigned long 
ServerTable::getNextServerIndex(unsigned long index)
{
    int retIndex = 0;

    tableLock.lock();
    if(index == 0){
        apI = serverCache.begin();
    } else {
        tableElement tmp(index);
        apI = serverCache.find(tmp);
        apI++;
    }
    if (apI != serverCache.end())
	retIndex =  apI->element.index;
    tableLock.unlock();

    return retIndex;
}

void 
ServerTable::clearServerCache()
{
    serverCache.clear();
    return ;
}

void 
ServerTable::insertServer(serverInfoT aServer)
{
    tableElement tmp(aServer);

    tableLock.lock();
    serverCache.insert(tmp);
    tableLock.unlock();

    return ;
}

