
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const TrapAgent_cxx_Version =
    "$Id: TrapAgent.cxx,v 1.19 2001/01/26 04:05:18 maneri Exp $";
#include "TrapAgent.hxx"
#include "cpLog.h"


applInfoT *
TrapAgent::GetApplicationList()
{

    applInfoT *applList = 0, *lastOne = 0, *tmpNode = 0;

    for (int i = 1; i <= applTable.getNumApps(); i++)
    {
        tmpNode = new applInfoT;
        memset (tmpNode, 0, sizeof(tmpNode));

        ApplInfo ai = applTable.getAppl(i);
        if (ai.index == 0)
        {
            cpLog(LOG_ERR, "recieved inval application info from applTable");
            return 0;
        }
        tmpNode->index = ai.index;
        strncpy(tmpNode->name, ai.name.c_str(), min(sizeof(tmpNode->name), ai.name.length()));
        tmpNode->stackPort = ai.stackPort;
        tmpNode->type = ai.type;
        tmpNode->applPort = ai.applPort;
        tmpNode->status = (operStatusT)ai.status;
        tmpNode->startTime = ai.startTime;
        tmpNode->lastUpdateTime = ai.lastUpdateTime;

        if (!lastOne)
        {
            lastOne = tmpNode;
            applList = tmpNode;
        }
        else
        {
            lastOne->next = tmpNode;
            lastOne = tmpNode;
        }
    }
    return applList;
}


applInfoT
TrapAgent::GetApplication(int index)
{
    applInfoT tmpNode;
    memset(&tmpNode, 0, sizeof(tmpNode));

    ApplInfo ai = applTable.getAppl(index);
    if (ai.index == 0)
    {
        cpLog(LOG_ERR, "recieved inval application info from applTable");
    }
    else
    {
        tmpNode.index = ai.index;
        strncpy(tmpNode.name, ai.name.c_str(), min(sizeof(tmpNode.name), ai.name.length()));
        tmpNode.stackPort = ai.stackPort;
        tmpNode.type = ai.type;
        tmpNode.applPort = ai.applPort;
        tmpNode.status = (operStatusT)ai.status;
        tmpNode.startTime = ai.startTime;
        tmpNode.lastUpdateTime = ai.lastUpdateTime;
    }
    return tmpNode;
}


voReturnStatus
TrapAgent::processMessage(ipcMessage *message, NetworkAddress *sender)
{

    // hard code for now. todo.  read from config file and cache
    oid serverUp_oid[] = {1, 3, 6, 1, 4, 1, 5738, 1, 1, 6, 1};
    long serverUpLen = 11;
    oid serverDown_oid[] = {1, 3, 6, 1, 4, 1, 5738, 1, 1, 6, 2};
    long serverDownLen = 11;
    int *p1;

    cpLogSetPriority( LOG_DEBUG );
    cpLogSetLabel( "TestAgent" );

    struct variable_list *vp;
    if (!message)
	return voFailure;
    switch (message->action)
    {
        case Trap :
        {
            vp = (struct variable_list *) calloc (1, sizeof(struct variable_list));
            vp->next_variable = 0;
            switch (message->mibVariable)
            {
                case ServerUp :
                cpLog( LOG_DEBUG, "Received trap message: %s Server Up:", message->parm1);
                vp->name = (oid *)malloc(sizeof(oid) * serverUpLen);
                memcpy(vp->name, &serverUp_oid, sizeof(oid) * serverUpLen);
                vp->name_length = serverUpLen;
                vp->type = ASN_OCTET_STR;
                if (message->parm1)
                {
                    vp->val.string = (u_char *)strdup(message->parm1);
                    vp->val_len = strlen(message->parm1);
                }
                // send_enterprise_trap_vars  will free vp list in agent_trap.c
                send_enterprise_trap_vars( -1, -1, serverUp_oid, serverUpLen, vp);
                // todo free the message.  create a member function that does it.
                // call it here.
                break;
                case ServerDown :
                cpLog( LOG_DEBUG, "Received trap message: %s Server Down:", message->parm1);
                vp->name = (oid *)malloc(sizeof(oid) * serverDownLen);
                memcpy(vp->name, &serverDown_oid, sizeof(oid) * serverDownLen);
                vp->name_length = serverDownLen;
                vp->type = ASN_OCTET_STR;
                if (message->parm1)
                {
                    vp->val.string = (u_char *)strdup(message->parm1);
                    vp->val_len = strlen(message->parm1);
                }
                send_enterprise_trap_vars( -1, -1, serverDown_oid, serverDownLen, vp);
                break;
                default :
                return voFailure;
            }
            return voSuccess;
        }
        case Get :
        case Set :
        case Response :
        {
            return voFailure;
        }
        case Register :
        {
            // parm1 = ServerType (int), port (int)
            // parm2 = applType (string)
            p1 = (int *)message->parm1;
            cpLog( LOG_DEBUG, "Received Register message: %s:%d", message->parm2, p1[1]);
            return applTable.addOrUpdateAppl((ServerType)p1[0], p1[1], string( message->parm2));
        }
        default :
        {
            return voFailure;
        }
    }

}
