#ifndef TrapAgent_H
#define TrapAgent_H

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const TrapAgent_hxx_Version =
    "$Id: TrapAgent.hxx,v 1.14 2001/10/19 20:46:54 icahoon Exp $";

#ifdef __cplusplus
#include <AgentApi.hxx>
#include <asn1.h>
#include <snmp_api.h>
#include <agent_trap.h>
#include "ApplTable.hxx"
#include "ServerTable.hxx"
#endif // __cplusplus
#include <Common.h>

// there is a duplicate in ApplTable.hxx  This is needed
// here to allow for use in c files.
typedef enum {
    up = 1,
    down,
    halted,
    congested,
    restarting,
    quiescing
} operStatusT;

typedef struct applInfo
{
    int index;
    char name[256];
    int stackPort;
    int applPort;
    enum ServerType type;
    operStatusT status;
    struct timeval startTime;
    struct timeval lastUpdateTime;
    struct applInfo *next;
}
applInfoT;

#ifdef __cplusplus
/**
 * TrapAgent: Compiled into the snmp agent (snmpd) to receive server up/down messages from the
 * co-located heartbeat server then create and send snmp trap messages to management stations who have
 * registered to receive vovida traps. (vovidamib.localGrp).  It also receives the register messages from 
 * the co-located applications and puts that information into the applTable.
 */
class TrapAgent: public AgentApi
{
    public:
        ///
        TrapAgent() : AgentApi( SERVER_Agent, "NetMgntAgent")
        {}
        ;
        ///
        ~TrapAgent()
        {}
        ;

        /**
         * this function queries the application table and returns a list of
         * its entries.
         * @returns a vector of application descritpors.
         */
        applInfoT *
        GetApplicationList();

        /**
         * this function returns the details of a given application with application table
         * based on the index of the networkApplicationTable
         * @param index. index within the networkapplicationtable we want info on.
         * @returns detailed info on a given application
         */
        applInfoT
        GetApplication(int index);
        ///
        int
        GetStackPortFromIndex(int index)
        {
            return applTable.getStackPort(index);
        }
        ///
        int
        GetApplPortFromIndex(int index)
        {
            return applTable.getApplPort(index);
        }
        ///
        int
        GetNumOfApps()
        {
            return applTable.getNumApps();
        }
        ///
        int
        GetNextIndex(int index, enum ServerType type)
        {
            return applTable.getNextIndex(index, type);
        }

        ///  The following functions are for the server cache (serverTable class)
        ///
        serverInfoT 
	GetServerFromIndex(unsigned long index)
	{
	    return serverTable.getServerFromIndex(index);
	}

        ///
        unsigned long
	GetNextServerIndex(unsigned long index)
	{
	    return serverTable.getNextServerIndex(index);
	}

        ///
        void
	ClearServerCache()
	{
	    return serverTable.clearServerCache();
	}
    
        ///
        void
	InsertServer(serverInfoT aServer)
	{
	    return serverTable.insertServer(aServer);
	}

    protected:

        /**
         * This function will process the messages received by the snmpd.  There are
         * two types of messages 
         * trap and register.
         * @message the message we received from the other agentapi
         * @sender who sent the message
         * @returns voReturnStatus
         */
        virtual voReturnStatus processMessage(ipcMessage *message, NetworkAddress *sender);

    private:
        ApplTable applTable;
        ServerTable serverTable;

};

#endif // __cplusplus
#endif //TrapAgent_H
