
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const VovidaSnmpUtil_c_Version =
    "$Id: VovidaSnmpUtil.cxx,v 1.2 2001/10/19 20:46:54 icahoon Exp $";
#include <sys/types.h>
#include <sys/socket.h>
#include <stdio.h>
#include <string.h>
#include <sys/time.h>
#include <unistd.h>
#include <sys/param.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <netdb.h>
#include <netinet/in.h>
#include "VovidaSnmpUtil.h"

// todo:  get rid of perror and printf and replace them with real error messages.

extern int voSockFd;
extern unsigned long localTransactionNumber;
extern struct sockaddr_in txDestAddr;

// places local address in "inAddr"
// returns 0 on failure 1 on success
// assumes that inAddr has already been allocated memory.
int
GetMyAddress(struct sockaddr_in * inAddr)
{
    char hostname[MAXHOSTNAMELEN];
    struct hostent * tmphostent;
    if (!inAddr)
        return 0;

    if (gethostname(hostname, MAXHOSTNAMELEN) < 0)
    {
        perror("VovidaSnmpUtil:GetMyAddress:gethostname");
        return 0;
    }

    memset(inAddr, 0, sizeof(struct sockaddr_in) / sizeof(int));
    // needs uint  Ip address in network byte order
    tmphostent = gethostbyname(hostname);
    if (!tmphostent)
    {
        perror("VovidaSnmpUtil:GetMyAddress:gethostbyname");
        return 0;
    }
    // todo:  handle case for IPV4 and IPv6.

    memcpy(&inAddr->sin_addr.s_addr,
           tmphostent->h_addr,
           tmphostent->h_length);

    return 1;
}


// returns 0 on failure, a non-blocking socket on success
// if inPort = 0 we let the OS assign a port
int
GetNonblockingSocket (int inPort)
{
    int sockFd, flags;
    struct sockaddr_in mySockAddr;
#ifdef __linux__
    int buf1 = 1;
    int len1 = sizeof(buf1);
    struct protoent * protoent;
#endif

    if ((sockFd = socket(PF_INET, SOCK_DGRAM, IPPROTO_UDP)) == -1)
    {
        perror("VovidaSnmpUtil:GetNonblockingSocket:socket 1");
        return 0;
    }

    // set the socket to behave like bsd
#ifdef __linux__
    protoent = getprotobyname("icmp");

    if (!protoent)
    {
        fprintf(stderr, "VovidaSnmpUtil:GetNonblockingSocket:Cannot get icmp protocol\n");
        return 0;
    }
    else
    {
        if (setsockopt(sockFd, protoent->p_proto, SO_BSDCOMPAT,
                       (char*)&buf1, len1)
                == -1)
        {
            fprintf(stderr, "VovidaSnmpUtil:GetNonblockingSocket:setsockopt error SO_BSDCOMPAT :%s",
                    strerror(errno));
            return 0;
        }
    }
#endif

    memset(&mySockAddr, 0, sizeof(struct sockaddr) / sizeof(int));
    mySockAddr.sin_family = AF_INET;
    mySockAddr.sin_addr.s_addr = htonl(INADDR_ANY);
    if (inPort)
        mySockAddr.sin_port = inPort;
    if (bind(sockFd, reinterpret_cast<const sockaddr*> (&mySockAddr), sizeof(struct sockaddr)) == -1)
    {
        perror("VovidaSnmpUtil:GetNonblockingSocket:bind");
        return 0;
    }

    // set socket to non-blocking then return it back after receive call
    if ((flags = fcntl(sockFd, F_GETFL, 0)) < 0)
    {
        perror("VovidaSnmpUtil:GetNonblockingSocket:fcntl 1");
        return 0;
    }
    flags |= O_NONBLOCK;
    if (fcntl(sockFd, F_SETFL, flags) < 0)
    {
        perror("VovidaSnmpUtil:GetNonblockingSocket:fcntl 2");
        return 0;
    }
    return sockFd;
}

/*
 *  This is a generic interprocess communication request specificaly made for
 *  snmp.
 *  action:        IN  request type (GET, SET...)
 *  mibVariable    IN  mib variable for request.  does not include INDEX if needed.
 *                     This is not alwayse a mib variable, but actually an action code (bla..bla..bla..)
 *  toPort         IN  port we are requesting to
 *  rxMessage      OUT pointer to allocated space where the response will be placed.
 *
 */
enum voReturnStatus ipcRequest(actionT action,
                               AgentApiMibVarT mibVariable,
                               ushort toPort,
                               void *rxMessage)
{
    return ipcRequestFull(action, mibVariable, toPort, rxMessage, 0, 0);
}

enum voReturnStatus
ipcRequestFull(actionT action,
               AgentApiMibVarT mibVariable,
               ushort toPort,
               void *rxMessage,
               void *setValue,
               int setLen)
{
    int retVal, numBytesRx;
    struct sockaddr rxSourceAddr;
    int sockaddrSizeBytes = sizeof(struct sockaddr);
    struct timeval tv, txTv, rxTv;
    struct timezone tz;
    fd_set rset;
    ipcMessage txMessage, *rxMessageP;

    if (!rxMessage)
        return voInvalidInParm;

    // get the send time.  We will try to receive
    // valid messages through the entire RSAGENTIPCTIMEOUT
    // period or till the correct response comes in.
    // we will continue on gettimeof day failure, but set the tv=0
    // so that we will only recieve 1 message
    tz.tz_minuteswest = 0;
    tz.tz_dsttime = 0;
    timerclear(&txTv);
    if (gettimeofday(&txTv, &tz) > 0 )
    {
        perror("VovidaSnmpUtil:gettimeofday failed:");
        timerclear(&txTv);
    }

    // select will return upon timeout, error or received message
    FD_ZERO(&rset);
    FD_SET(voSockFd, &rset);
    tv.tv_sec = 0;
    tv.tv_usec = RSAGENTIPCTIMEOUT;

    txMessage.action = action;
    txMessage.mibVariable = mibVariable;
    localTransactionNumber ++;
    txMessage.transactionNumber = localTransactionNumber;
    txDestAddr.sin_port = htons(toPort);
    if (action == Set && setValue)
        memcpy(txMessage.parm1, setValue, setLen);
#ifndef __sparc
    if (sendto(voSockFd, &txMessage, sizeof(ipcMessage), 0, (struct sockaddr *)&txDestAddr, sockaddrSizeBytes) == -1)
#else
    if (sendto(voSockFd, (void *)&txMessage, sizeof(ipcMessage), 0, (struct sockaddr *)&txDestAddr, sockaddrSizeBytes) == -1)
#endif
        perror("VovidaSnmpUtil:IpcRequest:sendto");
    while (1)
    {
        retVal = select(voSockFd + 1, &rset, NULL, NULL, &tv);
        switch (retVal)
        {
            case ( -1) :
                        perror("VovidaSnmpUtil:IpcRequest:select");
            return voFailure;
            case 0 :
            printf("VovidaSnmpUtil:IpcRequest:receive timeout\n");
            return voTimeout;
            default :
            if ((numBytesRx = recvfrom( voSockFd,
                                        rxMessage,
                                        sizeof(ipcMessage),
                                        0 /*flags */ ,
                                        (struct sockaddr*) & rxSourceAddr,
#ifndef __sparc
                                        (socklen_t *) & sockaddrSizeBytes)) == -1)
#else
(int *) & sockaddrSizeBytes)) == -1)
#endif
    {
                perror("VovidaSnmpUtil:IpcRequest:recvfrom");
                return voFailure;
            }
            else
            {
                rxMessageP = reinterpret_cast<ipcMessage*>(rxMessage);
                if (rxMessageP->transactionNumber == localTransactionNumber)
                {
                    return voSuccess;
                }
                else
                {
                    tz.tz_minuteswest = 0;
                    tz.tz_dsttime = 0;
                    timerclear(&rxTv);
                    if (gettimeofday(&rxTv, &tz) > 0 )
                    {
                        perror("VovidaSnmpUtil:gettimeofday 2 failed:");
                        return voFailure;
                    }
                    if (rxTv.tv_sec - txTv.tv_sec > (RSAGENTIPCTIMEOUT / 1000))
                    {
                        printf("VovidaSnmpUtil:IpcRequest:complete request timeout\n");
                        return voFailure;
                    }
                }
            }
        }
    }
}

int min(int a, int b)
{
    if (a < b) return a;
    else return b;
}
int max(int a, int b)
{
    if (a > b) return a;
    else return b;
}

