#ifndef CPLUrlData_hxx
#define CPLUrlData_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */





static const char* const CPLUrlDataVersion =
    "$Id: CPLUrlData.hxx,v 1.10 2002/03/19 21:39:39 jason Exp $";



#include <vector>
#include "SipUrl.hxx"
#include <Data.hxx>

namespace Vocal
{

/**
  *  Holds a set of URL to be used by CPLOpRedirect or CPLOpRSLookup
  *  @see CPLOpRedirect
  *  @see CPLOpRSLookup
  */
class CPLUrlData
{
    public:
        /**@name Initialization & Deallocation Functions */
        //@{
        ///Constructor
        CPLUrlData();

        ///Destructor
        ~CPLUrlData();

        ///Assignment operator
        CPLUrlData& operator=(const CPLUrlData& rhs);
        //@}

        /**@name location set and display name vectors manipulation Functions */
        //@{
        /// Returns TRUE if vector has 0 elements.
        bool empty() const;

        /// Returns the number of URLs in the location set
        unsigned int size() const;

        /// Clears the location set and the display name set
        int clearAll();

        /// Get a URL element in the location set
        bool getURL( SipUrl& returnValue, int index);

        /// Get a display name element in the display name vector
        bool getDispName( Data& returnValue, int index);

        /// Clear the location/display sets and add a URL & display name
        int clearAndPush(const Sptr<BaseUrl> url, const Data& dispName = "");

        /// Add a url location & display name to the bottom of the stack
        int push(const Sptr<BaseUrl> url, const Data& dispName = "");

        /// Get a url location/a display name from the top of the stack
        bool top( SipUrl& url, Data& dispName );

        /// Get a url location from the top of the stack
        bool top( SipUrl& url );
        //@}

        /**@name Debugging Functions */
        //@{
        ///
        void logData() const;
        //@}
    private:
        /**@name Data members */
        //@{
        ///
        vector < SipUrl > myLocation;
        ///
        vector < Data > myDispName;
        //@}

}
;
 
}

#endif
