#ifndef FsProvisioning_hxx
#define FsProvisioning_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */





static const char* const FsProvisioningVersion =
    "$Id: FsProvisioning.hxx,v 1.16 2002/03/19 21:39:40 jason Exp $";



#include <string>
#include <Sptr.hxx>
#include "ProvisionInterface.hxx"
#include "FsFeature.hxx"
#include "FsUserData.hxx"

namespace Vocal
{

//Callback routines (required by Provisioning Interface)
void notifyAddDel( const string& data, const string& fileName, bool );

/**
  *  Interface to contact Provisioning Server for data 
  * 
  */

class FsProvisioning
{
    public:
        ///
        typedef enum Features { FeaturesIncoming, FeaturesOutgoing };

        ///
        static FsProvisioning* Instance();

        /// Destructor
        ~FsProvisioning();

        /// Retrieve all the user cpl scripts for the current feature server
        void initialize();

        /**Called when the feature information has been changed in provisioning:
           The state machine is rebuilt accordingly
         */
        void updateUserAddDel( const string& data, const string& filename );

        ///  Only used for testing
        void testUpdate();

        /// Add or update the user data with the feature created from the cpl script
        bool addFeature( const string& userFileName,
                         Sptr < FsFeature > newFeature,
                         const int& featureType);

        /// Parse the script and create the feature state machine
        bool createFeatureFromCPL( const string& cplScript,
                                   const string& userFilename );

        /// Get the cpl script from the user cpl file name through provisioning
        bool getProvisionedUserFeature( const int scriptType,
                                        const string& userFilename,
                                        string& cplScript );
        /// Gets the user name (everything before the "@") from the cpl file name
        bool getUserName( const string& fileName, string& user );

        /// Checks if the passed file ends with ".cpl"
        bool isValidCPLFilename( const string& filename ) const;

        /// Retrieve all the cpl scripts associated with the current feature server
        void retrieveFeatureUsers( int scriptType );

        /// Get the user cpl script and create the state machine for the feature
        void retrieveUserCPLFeatures( const char* userFile );

        /// Get global data from provisioning
        void retrieveGlobalData();

    private:
        /// Constructor
        FsProvisioning();

        ///
        static FsProvisioning* myInstance;

        /// Pointer to the provisioning interface (see provisioning)
        ProvisionInterface* myPif ;

        /// IP address of the current feature server
        string myFeatureServerName;

};

}

#endif
