
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const FsStateMachnData_cxx_Version =
    "$Id: FsStateMachnData.cxx,v 1.11 2002/03/19 21:39:40 jason Exp $";
#include "FsStateMachnData.hxx"

using namespace Vocal;

///
FsStateMachnData::FsStateMachnData()
{
    myBranchNum = 100;
    myProxyStatus = FsStateMachnData::NoProxy;
    myLocationBkup = 0;
    myInRequest = 0;
    myBestResponse = 0;
    myOutRequests = 0;
    myUrlCollector = new CPLUrlData;

#if 0  //Change to 1 for debugging
    cpLog( LOG_DEBUG, "new [%x]", this );
#endif
} //Constructor


///
FsStateMachnData::FsStateMachnData( const FsStateMachnData& rhs )
{
    *this = rhs;
} //Copy Constructor


///
FsStateMachnData::~FsStateMachnData()
{
#if 0  //Change to 1 for debugging
    cpLog( LOG_DEBUG, "~[%x]", this );
#endif
} //Destructor


///
FsStateMachnData&
FsStateMachnData::operator=( const FsStateMachnData& rhs )
{
    myUrlCollector = rhs.myUrlCollector;
    myProxyStatus  = rhs.myProxyStatus;
    myBranchNum    = rhs.myBranchNum;
    myLocationBkup = rhs.myLocationBkup;
    myOutRequests  = rhs.myOutRequests;
    myInRequest    = rhs.myInRequest;
    myBestResponse = rhs.myBestResponse;
    myOrigFrom     = rhs.myOrigFrom;
    myPrefixCIDBlk = rhs.myPrefixCIDBlk;
    myPrefixTo     = rhs.myPrefixTo;
    myNoCIDBlk     = rhs.myNoCIDBlk;
    myFeatureCode  = rhs.myFeatureCode; 
    
    return *this;
} //operator=


const char* const
FsStateMachnData::name() const
{
    return "FsStateMachnData";
} //name


///
Sptr < CPLUrlData >
FsStateMachnData::getLocations() const
{
    return myUrlCollector;
} //getLocations

///
Data
FsStateMachnData::getBranch() const
{
    return Data(myBranchNum);
}  //getBranch


#if 0
///
void
FsStateMachnData::setBranch( const int branch )
{
    myBranchNum = branch;
} //setBranch
#endif


///
Data
FsStateMachnData::getNextBranch()
{
    myBranchNum += 1;
    return Data(myBranchNum);
}  //getNextBranch


///
void
FsStateMachnData::setProxyStatus( const int proxyStatus )
{
    if ((proxyStatus >= FsStateMachnData::NoProxy) &&
            (proxyStatus <= FsStateMachnData::Unknown) )
        myProxyStatus = proxyStatus;
    else
        myProxyStatus = FsStateMachnData::Unknown;

}  //setProxyStatus


///
int
FsStateMachnData::getProxyStatus() const
{
    return myProxyStatus;
} //getProxyStatus


///
Sptr < CPLUrlData >
FsStateMachnData::getLocationBkup() const
{
    return myLocationBkup;
} //getLocationBkup


///
void
FsStateMachnData::setLocationBkup( Sptr < CPLUrlData > locationBkup )
{
    myLocationBkup = locationBkup;
}  //setLocationBkup


///
Sptr < vector < Sptr < InviteMsg > > >
FsStateMachnData::getOutRequests() const
{
    return myOutRequests;
}  //getOutRequests


///
void
FsStateMachnData::setOutRequests( Sptr < vector < Sptr < InviteMsg > > > outRequests )
{
    myOutRequests = outRequests;
} //setOutRequests


///
Sptr < InviteMsg >
FsStateMachnData::getInRequest() const
{
    return myInRequest;
}  //getInRequest


///
void
FsStateMachnData::setInRequest( Sptr < InviteMsg > inRequest )
{
    myInRequest = inRequest;
}  //setInRequest


///
Sptr < StatusMsg >
FsStateMachnData::getBestResponse() const
{
    return myBestResponse;
}  //getBestResponse


///
void
FsStateMachnData::setBestResponse( Sptr < StatusMsg > bestResponse )
{
    myBestResponse = bestResponse;
}  //setBestResponse


void
FsStateMachnData::setOrigFrom( const SipFrom from )
{
    myOrigFrom = from;
}

const SipFrom
FsStateMachnData::getOrigFrom() const
{
    return myOrigFrom;
}

void
FsStateMachnData::setPrefixCIDBlk( const Data prefix)
{
    myPrefixCIDBlk = prefix;
}

const Data
FsStateMachnData::getPrefixCIDBlk() const
{
    return myPrefixCIDBlk;
}

void
FsStateMachnData::setPrefixTo( const SipTo toWithPrefix)
{
    myPrefixTo = toWithPrefix;
}

const SipTo
FsStateMachnData::getPrefixTo() const
{
    return myPrefixTo;
}

void
FsStateMachnData::setNoCIDBlk( const bool noCidBlk)
{
    myNoCIDBlk = noCidBlk;
}

const bool
FsStateMachnData::getNoCIDBlk() const
{
    return myNoCIDBlk;
}

void
FsStateMachnData::setFeatureCode( const string theCode)
{
    myFeatureCode = theCode;
}

const string
FsStateMachnData::getFeatureCode() const
{
    return myFeatureCode;
}


//End of file
