
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const FsUserData_cxx_Version =
    "$Id: FsUserData.cxx,v 1.11 2002/03/19 21:39:40 jason Exp $";
#include "FsUserData.hxx"

using namespace Vocal;

///
FsUserData::FsUserData()
{
    myCalledFeature = 0;
    myCalleeFeature = 0;
    myDispName = "";
    myUserFilename = "";

}    // FsUserData::FsUserData


///
FsUserData::~FsUserData()
{}    // FsUserData::FsUserData


///

Sptr < Feature >
FsUserData::getCallingFeature() const
{
    return myCalleeFeature;
}    // FsUserData::getCallingFeature


///
Sptr < Feature >
FsUserData::getCalledFeature() const
{
    return myCalledFeature;
}    // FsUserData::getCalledFeature


///
bool
FsUserData::replaceCalleeFeature( Sptr < Feature > feature )
{
    myCalleeFeature = feature;
    return true;
}    //replaceCalleeFeature


///
bool
FsUserData::replaceCalledFeature( Sptr < Feature > feature )
{
    myCalledFeature = feature;
    return true;
}    //replacedCalledFeature


///
bool
FsUserData::addCalledFeature( Sptr < Feature > feature )
{
    bool rc = true;

    if ( myCalledFeature == 0 )
    {
        myCalledFeature = feature;
    }
    else
    {
        cpLog( LOG_ERR, "Cannot add %s feature: User already has %s feature",
               feature->name(), myCalledFeature->name() );
        rc = false;
    }
    return rc;
}    // FsUserData::addCalledFeature


///
bool
FsUserData::addCalleeFeature( Sptr < Feature > feature )
{
    bool rc = true;

    if ( myCalleeFeature == 0 )
    {
        myCalleeFeature = feature;
    }
    else
    {
        cpLog( LOG_ERR, "Cannot add %s feature: User already has %s feature",
               feature->name(), myCalleeFeature->name() );
        rc = false;
    }
    return rc;
}    // FsUserData::addCalleeFeature


///
void
FsUserData::showUserData() const
{
    if ( myCalleeFeature != 0 )
        cpLog( LOG_INFO, "Outgoing ftr: %s", myCalleeFeature->name() );

    if ( myCalledFeature != 0 )
        cpLog( LOG_INFO, "Incoming ftr: %s", myCalledFeature->name() );

}    // FsUserData::showUserData


///
void
FsUserData::setUserFileName( const char* userFilename )
{
    myUserFilename = userFilename;
} //setUserFileName



///
bool
FsUserData::getUserData( string& str ) const
{
    if (( myCalleeFeature == 0 ) && ( myCalledFeature == 0 ))
    {
        //Features haven't been downloaded yet, but
        //FS knows what file to call when the feature
        //is activated for the first time.
        if ( myUserFilename.length() == 0 )
            return false;
        else
        {
            LocalScopeAllocator lo;
            str.erase( str.begin(), str.end() );
            str += "Pending ftr: ";
            str += myUserFilename.getData(lo);
            return true;
        } /*else*/
    } /*if*/

    str.erase( str.begin(), str.end() );

    if ( myCalleeFeature != 0 )
    {
        str += "Outgoing ftr: ";
        str += myCalleeFeature->name();
        str += "\t";
    }

    if ( myCalledFeature != 0 )
    {
        str += "Incoming ftr: ";
        str += myCalledFeature->name();
    }

    return true;
}  //getUserData


///
Data
FsUserData::getDispName() const
{
    return myDispName;
} /*getDispName*/


///
SipUrl
FsUserData::getLastCaller() const
{
    return myLastCaller;
} /*getLastCaller*/


///
void
FsUserData::setDispName( const Data& theDispName )
{
    myDispName = theDispName;
} /*setDispName*/


///
void
FsUserData::setLastCaller( const SipUrl& theLastCaller )
{
    myLastCaller = theLastCaller;
} /*setLastCaller*/


const Data&
FsUserData::getUserFileName() const
{
     return ( myUserFilename );
}  //getUserFileName



//End of file
