
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const cplTest001_cxx_Version =
    "$Id: cplTest001.cxx,v 1.8 2000/12/18 23:43:47 bko Exp $";
#include <stdio.h>
#include <cassert>
#include <cstring>
#include <iostream.h>  //khom
#include <fstream.h>   //ifstream
#include "CPLFeatureBuilder.hxx"
#include "CPLState.hxx"
#include "CPLStartStateHolder.hxx"
#include "State.hxx"
#include "cpLog.h"

//////////////////////////////////////////////////////
/////////// PROTOTYPES ///////////////////////////////
//////////////////////////////////////////////////////
void runFeature( Sptr < State > startState );
Sptr < CPLStartStateHolder > parseXML ( char* fname );

//////////////////////////////////////////////////////
/////////// FUNCTIONS ////////////////////////////////
//////////////////////////////////////////////////////
///
Sptr < CPLStartStateHolder >
parseXML ( char* fname )
{

    string script;
    char ch;

    //Read file to memory
    ifstream inFile( fname, ios::in );

    if (!inFile.is_open() )
    {
        cout << "Unknown file: " << fname << endl;
        exit( EXIT_FAILURE );
    }

    while (inFile.get( ch ) )
        script += ch;

    return (CPLFeatureBuilder::Instance()->create( script ));
}  /*parseXML*/

void runFeature( Sptr < CPLStartStateHolder > startState )
{
    if (startState->getIncoming() != 0 )
    {
        cout << endl << endl << ".. INCOMING FEATURE STATE MACHINE .." << endl;
        CPLState* cplStartState = (CPLState*) startState->getIncoming().getPtr();
        cplStartState->doCpLog();
        cout << ".. INCOMING FTR FINISHES .." << endl;
    } /*if*/

    if (startState->getOutgoing() != 0 )
    {
        cout << endl << endl << ".. OUTGOING FEATURE STATE MACHINE .." << endl;
        CPLState* cplStartState = (CPLState*) startState->getOutgoing().getPtr();
        cplStartState->doCpLog();
        cout << ".. OUTGOING FTR FINISHES .." << endl;
    } /*if*/
}  // runFeature

//////////////////////////////////////////////////////
/// TESTING FUNCTIONS ////////////////////////////////
//////////////////////////////////////////////////////
int
main(int argc, char* argv [])
{
    if (argc < 2)
    {
        cout << argv[0] << ": Missing parameters.  " << endl;
        cout << "Usage:  " << argv[0] << " XML_FILENAME" << endl;
        exit(EXIT_FAILURE);
    }  /*if*/
    else
    {
        cpLogSetPriority( LOG_DEBUG );
        Sptr < CPLStartStateHolder > s;
        
        //Parse all of command line
        for (int idx = 1; idx < argc; idx++ )
        {
            cpLog( LOG_DEBUG, "\n{{{{{{{{{{Parsing \"%s\".}}}}}}}}}}\n", 
                   argv[idx]  ); 
            s = parseXML( argv[idx] );
            if ( s != 0 )
            {
                runFeature( s );
            }
            else
            {
                s = 0;   //Force smart pointer to deallocate object 
                exit(EXIT_FAILURE);
            } /*else*/
        } /*for*/
    } /*else*/

}  // main


//End of File
