#ifndef CPLBuildStatus_Hxx
#define CPLBuildStatus_Hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */





static const char* const CPLBuildStatusVersion =
    "$Id: CPLBuildStatus.hxx,v 1.12 2002/03/19 21:39:40 jason Exp $";



#include <vector>
#include "Sptr.hxx"
#include "CPLListElement.hxx"

namespace Vocal
{

/**
  *  Used to control how operators and states are added to the 
  *  state machine. 
  */
class CPLBuildStatus
{
    public:
        /// instantiates the singleton instance
        static CPLBuildStatus* Instance();

        /// destructor
        ~CPLBuildStatus();

        /**processes a node before processing its siblings in tree
         *@param op reference to the state m/c operator corresponding to this node
         *@param node the ID of this node
         *@return bool result if processed correctly
         */
        bool nodeEntry( Sptr < CPLOperator > op, int node );

        /**processes a node after processing its siblings in tree
         *@param op reference to the state m/c operator corresponding to this node
         *@param node the ID of this node
         *@return bool result if processed correctly
         */
        bool nodeExit( int node );

        /**get the reference to the subaction being referred
         *@param id the name of the subaction referred
         *@return reference to the state m/c operator corresponding to subaction
         */
        Sptr < CPLOperator > getSubAction( const string& id ) const;

        /**get the reference to the timezone being referred
         *@param id the name of the timezone referred
         *@return reference to the state m/c operator corresponding to timezone
         */
        Sptr < CPLOperator > getTimeZone( const string& id ) const;

        /// test if the state m/c was built cleanly
        bool isBuildClean() const;

        ///Used for debugging member variables
        void showData() const;

        /**@name Constants */
        //@{
        /** SimpleOp (500) - Add new operator to last simple operator,
         * CompoundOp (501) - New Operator is saved as a Compound Operator 
         * CompoundOpChild (502) - New Operator is added to the last Compound Operator
         * NoOpChild (503) - No operator needs to be added;
         */
        enum _addMethods { SimpleOp = 500, CompoundOp, CompoundOpChild, NoOpChild };
        ///
        typedef _addMethods AddMethods;
        //@}

    protected:
        /// protected constructor for singleton implementation
        CPLBuildStatus();

    private:
        /// reference to the singleton instance
        static CPLBuildStatus* myInstance;

        /// list of embedded compound operators (switches)
        vector < CPLListElement > myCompoundOperatorList;

        /// list of sub action definitions
        vector < CPLListElement > mySubActionList;

        /// list of timezone definitions
        vector < CPLListElement > myTimeZoneList;

        /// reference to the last inserted operator in state m/c
        Sptr < CPLOperator > myLastOp;

        /// ID of the last processed node
        int myLastNode;
        /**method to add an operator to the state m/c
         *@param op reference to the operator
         *@param currentNode ID of the node corresponding to the operator
         */
        void appendToStateMachine( Sptr < CPLOperator > op, int currentNode );
        /// initialize the state m/c build status
        void resetBuildStatus();
};
 
}

#endif

