#ifndef CPLInterpreter_Hxx
#define CPLInterpreter_Hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */





static const char* const CPLInterpreterVersion =
    "$Id: CPLInterpreter.hxx,v 1.18 2002/03/19 21:39:40 jason Exp $";



#include <parser.h>
#include <vector>
#include "Sptr.hxx"
#include "CPLBuildStatus.hxx"
#include "CPLState.hxx"
#include "CPLOperator.hxx"
#include "CPLStartStateHolder.hxx"
#include "CPLListElement.hxx"

namespace Vocal{/**
  *  Follows the LIBXML DOM tree and creates operator(s) & 
  *  states based on the type of CPL tag. 
  */
class CPLInterpreter
{
    public:
        /// iterator to traverse the vector
        typedef vector < CPLListElement > ::iterator listIterator;

        /// singleton implementation
        static CPLInterpreter* Instance();

        /**method to create state machine based on a CPL script
         *@param rootPtr the root node in parsed CPL script
         *@return reference to the start state of the state machine
         */
        Sptr < CPLStartStateHolder > createFeatureFromCPL( xmlNodePtr rootPtr );

        /**method to process attributes of a CPL tag
         *@param node reference to the current state machine operator
         *@param ptr the current node in CPL script
         *@param attrTable reference to table of attributes for the current node
         *@param tableSize count of the entries in the table
         */
        void processNodeAttributes( Sptr < CPLOperator > node,
                                    xmlNodePtr ptr,
                                    const char* attrTable[],
                                    const unsigned int tableSize );

        /**method to traverse the DOM tree starting at given node
         *@param ptr the given node in the tree
         *@return bool result if the tree was traversed correctly
         */
        bool traverse( xmlNodePtr ptr );

        /**method to process a node at the start of traversal
         *@param nodeFound ID of the current node
         *@param ptr the current node
         *@return bool to indicate if processed correctly
         */
        bool nodeStart( int nodeFound, xmlNodePtr ptr );

        /**method to process a node at the end of traversal
         *@param nodeFound ID of the node
         *@param ptr the node
         *@return bool result if processed correctly
         */
        bool nodeExit( int nodeFound, xmlNodePtr ptr );

        /**method to search for a node in the list of nodes
         *@param ptr starting node in the list
         *@param nodeName the name of node to be searched
         *@return bool result of the search
         */
        bool searchNodes( xmlNodePtr ptr, const char* nodeName );

        /// method to process the CPL tag
        bool processCpl( xmlNodePtr ptr );

        /// method to process the LOG tag
        bool processLog( xmlNodePtr ptr );

        /// method to process the SUB tag
        bool processSub( xmlNodePtr ptr );

        /// method to process the BUSY tag
        bool processBusy( xmlNodePtr ptr );

        /// method to process the MAIL tag
        bool processMail( xmlNodePtr ptr );

        /// mehtod to process the TIME tag
        bool processTime( xmlNodePtr ptr );

        /// method to process the PROXY tag
        bool processProxy( xmlNodePtr ptr );

        /// method to process the LOOKUP tag
        bool processLookup( xmlNodePtr ptr );

        /// method to process the REJECT tag
        bool processReject( xmlNodePtr ptr );

        /// method to process the STRING tag
        bool processString( xmlNodePtr ptr );

        /// method to process the TIMEZONE tag
        bool processTimeZone( xmlNodePtr ptr );

        /// method to process the DAYLIGHT tag
        bool processDaylight( xmlNodePtr ptr );

        /// method to process the STANDARD tag
        bool processStandard( xmlNodePtr ptr );

        /// method to process the ADDRESS tag
        bool processAddress( xmlNodePtr ptr );

        /// method to process the FAILURE tag
        bool processFailure( xmlNodePtr ptr );

        /// method to process the SUCCESS tag
        bool processSuccess( xmlNodePtr ptr );

        /// method to process the INCOMING tag
        bool processIncoming( xmlNodePtr ptr );

        /// method to process the LOCATION tag
        bool processLocation( xmlNodePtr ptr );

        /// method to process the NOANSWER tag
        bool processNoAnswer( xmlNodePtr ptr );

        /// method to process the NOTFOUND tag
        bool processNotFound( xmlNodePtr ptr );

        /// method to process the OUTGOING tag
        bool processOutgoing( xmlNodePtr ptr );

        /// method to process the PRIORITY tag
        bool processPriority( xmlNodePtr ptr );

        /// method to process the REDIRECT tag
        bool processRedirect( xmlNodePtr ptr );

        /// method to process the VMREDIRECT tag
        bool processVmRedirect( xmlNodePtr ptr );

        /// method to process the OTHERWISE tag
        bool processOtherwise( xmlNodePtr ptr );

        /// method to process the SUBACTION tag
        bool processSubAction( xmlNodePtr ptr );

        /// method to process the NOTPRESENT tag
        bool processNotPresent( xmlNodePtr ptr );

        /// method to proces the TIMESWITCH tag
        bool processTimeSwitch( xmlNodePtr ptr );

        /// method to process the STRINGSWITCH tag
        bool processStringSwitch( xmlNodePtr ptr );

        /// method to process the ADDRESSSWITCH tag
        bool processAddressSwitch( xmlNodePtr ptr );

        /// method to process the PRIORITYSWITCH tag
        bool processPrioritySwitch( xmlNodePtr ptr );

        /// method to process the REMOVELOCATION tag
        bool processRemoveLocation( xmlNodePtr ptr );

        /// method to add default action for a switch
        bool processDefaultAction( xmlNodePtr ptr );

        /// method to add default action for a switch in Outgoing features
        bool processDefaultOutgoing( xmlNodePtr ptr );

        /// method to add default action for a switch in Incoming features
        bool processDefaultIncoming( xmlNodePtr ptr );

        /// method to add <default> (proxy's output)
        bool processDefault( xmlNodePtr ptr );
                 
        /// method to add <redirection> (proxy's output)
        bool processRedirection( xmlNodePtr ptr );

    protected:
        /// protected constructor for singleton implementation
        CPLInterpreter();

    private:
        /**Converts CPL node strings to integer
         *@param str name of the node
         *@param strTable table of the names of all CPL nodes
         *@param tableSize size of the table
         *@return an integer corresponding to node's name
         */
        int nodeToI( const char* str, const char* strTable[], const int tableSize);

        /** Processes the &lt;proxy&gt when its 'ordering' attribute is equal 
         * to first-only.  This differs from processProxyFork() in
         * that there is 1 operator that proxies the INVITE, 1 state
         * that awaits for the response, and another operator that
         * processes the response or returns to the waiting state.
         * There is no loop to do another proxy (send another INVITE).  
         **/
        bool processProxyFirstOnly( xmlNodePtr ptr );

        /** Processes the &lt;proxy&gt when its 'ordering' attribute is
         * equal to sequential, parallel, or hunt.   This differs
         * from processProxyFirstOnly() in that 1 state and 1 Fork
         * operator are created, and these 2 are looped until the 
         * best response is determined.  
         **/
        bool processProxyFork( xmlNodePtr ptr );

        /// reference to singleton instance
        static CPLInterpreter* myInstance;

        /// reference to the state machine build status
        Sptr < CPLBuildStatus > myCplBuildStatus;

        /// list of time-zones (peripheral information)
        vector < CPLListElement > myTimeZoneList;

        /// reference to the start state
        Sptr < CPLStartStateHolder > myStartState;

        /// the default proxy state
        Sptr < CPLState > myProxyState;

        /// the default proxy operator
        Sptr < CPLOperator > myProxyOp;

        /// the end operator
        Sptr < CPLOperator > myEndOp;

        /// the end state
        Sptr < CPLState > myEndState;
};
 
} // namespace Vocal
 
#endif




