
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const CPLOpLocation_cxx_Version =
    "$Id: CPLOpLocation.cxx,v 1.23 2002/03/19 21:39:41 jason Exp $";
#include "CPLOpLocation.hxx"
#include <iostream.h>
#include <SipUrl.hxx>
#include <SipEvent.hxx>
#include "FsUsers.hxx"
#include "FsUserData.hxx"

using namespace Vocal;

///
CPLOpLocation::CPLOpLocation():
    myClearLocationSet( false  )
{
#ifdef MEM_TEST
    cpLog(LOG_DEBUG, "CPLOpLocation[%x]", this );
#endif
}  //CPLOpLocation


///
CPLOpLocation::~CPLOpLocation()
{
#ifdef MEM_TEST
    cpLog(LOG_DEBUG, "~CPLOpLocation[%x]", this );
#endif
}  //~CPLOpLocation


///
const Sptr < State >
CPLOpLocation::process( const Sptr < SipProxyEvent > event )
{
    if (myURL == "callreturn")
    {
        retrieveLastCaller(event);
    } /*if*/
    else
    {
        Sptr<BaseUrl> baseUrl = new SipUrl( myURL ); 

        if (myClearLocationSet)
            getStateMachineData(event)->getLocations()->clearAndPush( baseUrl );
        else
            getStateMachineData(event)->getLocations()->push( baseUrl );

        //Clear smart pointer 
        baseUrl = 0; 

    } /*else*/

    return CPLOperator::process( event );

}  //process


///

const char* const
CPLOpLocation::name() const
{
    return "CPLOpLocation";
} //name


///
bool
CPLOpLocation::setNextOperator( Sptr < Operator > op, int )
{
    addOperator( op );
    return true;
} //setNextOperator


///
bool
CPLOpLocation::setAttributes( const char* attrName, const char* attrValue)
{
    if ( strcmp( attrName, "url" ) == 0)
    {
        if ( attrValue != 0 )
        {
            myURL = attrValue;

            //Special case for 'callreturn' keyword. 
            if ( myURL.find( "callreturn" ) != string::npos )
                return true; 

            //validateUrl checks for unimplemented URLs.
            //Also, it will add a hostname to the URL if only a
            //user is defined.
            if ( validateUrl( myURL ) == false )
                myURL.erase( myURL.begin(), myURL.end() );

        } /*if*/
    } /*if url*/
    else if ( strcmp( attrName, "clear" ) == 0)
    {
        if ( strcmp( attrValue, "yes" ) == 0)
            myClearLocationSet = true;
        else
            myClearLocationSet = false;
    } /*if clear*/

    return true;
}  //setAttributes


///
void
CPLOpLocation::logData() const
{
    if (myClearLocationSet)
        cpLog( LOG_DEBUG, "%s-CLEAR(yes) myURL(%s)", name(), myURL.c_str() );
    else
        cpLog( LOG_DEBUG, "%s-CLEAR(no) myURL(%s)", name(), myURL.c_str() );

}  //logData

void
CPLOpLocation::retrieveLastCaller(Sptr < SipProxyEvent > event )
{
    cpLog(LOG_DEBUG, "Call Return:  Retrieve");

    //Retrieve the SIP message from the event 
    Sptr < SipEvent > sipEvent;
    sipEvent.dynamicCast( event );
    assert (sipEvent != 0 );

    Sptr < SipMsg > sipMsg;
    sipMsg = sipEvent->getSipMsg();
    assert (sipMsg != 0 );

    //Retrieve the From's and To's URLs.
    Sptr<BaseUrl> from = sipMsg->getFrom().getUrl();
    Sptr<BaseUrl> to = sipMsg->getTo().getUrl();

    //Print information for debugging 
    cpLog(LOG_DEBUG, "From=[%s]", from->getUserValue().logData());
    cpLog(LOG_DEBUG, "To=[%s]", to->getUserValue().logData());
    cpLog(LOG_DEBUG, "Request[%s]",
          sipEvent->getInvite()->getRequestLine().getUrl()->getUserValue().logData());
    LocalScopeAllocator lo;
    string userFrom( from->getUserValue().getData(lo) );

    //Look for data for the user in the "From" field. 
    Sptr<FsUserData> userData = FsUsers::instance()->getUserData(userFrom);
    if ( userData != 0 )
    {
        //Look for saved caller information 
        Sptr<BaseUrl> baseUrl =  new SipUrl( userData->getLastCaller() );
        Data dspName = userData->getDispName();
        if (baseUrl->getUserValue().length() > 0)
        {
            //Put last caller's URL and DisplayName into the locations set.
            getStateMachineData(event)->getLocations()->clearAndPush( baseUrl,
                                                                      dspName );
            return; 
        } /*if length() > 0*/
    } /*if (userData != 0)*/

    cpLog(LOG_DEBUG, "Failed to find last caller.");
} //retrieveLastCaller


/********************************************
 *  Non-Class Member Functions
 *******************************************/
///This function takes a URL and modifies it, if necessary, to make it a
///valid URL
bool
Vocal::validateUrl( string& url )
{
    //Place a fake host divider if the URL is missing host.
    if ( url.find("@") == string::npos )
    {
        unsigned int idx = url.find(";");
        if ( idx != string::npos )
            url.insert( idx, "@fakeHost.com" );
        else
        {
            cpLog( LOG_ERR, "URL(%s) missing parameters.", url.c_str() );
            url.append( "@fakeHost.com");
            return true;
        }  /*else*/
    } /*if*/

    return true;
} /*validateUrl*/


//End of File

