/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const CPLOpLog_cxx_Version =
    "$Id$";

#include "CPLOpLog.hxx"
#include "CPLOperator.hxx"
#include "CPLTimeRange.hxx"
#include <string.h>
#include "CallProcessingLanguage.hxx"
#include "Operator.hxx"
#include "State.hxx"
#include "Sptr.hxx"
#include "Data.hxx"
#include "ProvisionInterface.hxx"
#include <stdlib.h>
#include "FsFeature.hxx"
#include "SipEvent.hxx"
#include "TimerEvent.hxx"
#include "CallInfo.hxx"
#include "FsUsers.hxx"

using namespace Vocal;

CPLOpLog::CPLOpLog()
{
    myComment = "";     // initial comments ...
    try
    {
        myPif = &(ProvisionInterface::instance());
    } /*try*/
    catch (VException& e)
    {
        cpLog( LOG_ERR, "Failed to contact Provisioning: %s",
                        e.getDescription().c_str() ); 
    } /*catch*/
}


CPLOpLog::~CPLOpLog()
{
    /// no op...
}


bool CPLOpLog::setAttributes(const char* type, const char* value)
{
    if ( strcmp( type, "name" ) == 0 )
        myName = value;
    else if ( strcmp( type, "comment" ) == 0 )
        myComment = value;
    else  //Unknown attribute name
        assert(0);

    return true;
}


const Sptr < State > CPLOpLog::process(const Sptr < SipProxyEvent > anEvent)
{
    if (myName == "lastcaller")
    {
        updateLastCaller(anEvent);
    }
    else
    {
        log(anEvent);
    }
    return CPLOperator::process(anEvent);
}


bool CPLOpLog::setNextOperator(Sptr < Operator > anOperator, int nodeId /*default value*/)
{
    addOperator(anOperator);
    return true;
}


const char* const CPLOpLog::name() const
{
    return "CPLOpLog";
}


void CPLOpLog::logData(void) const
{
    cpLog(LOG_DEBUG, "[%x] log file \"%s\" <%s> ...",
          this, myName.c_str(), myComment.c_str());
}


void CPLOpLog :: log(const Sptr < SipProxyEvent > anEvent)
{
    //If there was trouble accessing Provisioning Server, 
    //cease to continue. 
    if (myPif == 0) return; 

    /**
     * calculate the "log data" and log into the log file...
     */

    // step 1. creat log file name...
    if (myName.size() == 0)
    {
        Sptr < SipEvent > sipEvent;
        Sptr < TimerEvent > timerEvent;
        Sptr < FsFeature > fsPtr;
        sipEvent.dynamicCast(anEvent);
        if (sipEvent != 0)
        {
            fsPtr.dynamicCast(sipEvent->getCallInfo()->getFeature());
        }
        else
        {
            timerEvent.dynamicCast(anEvent);
            if (timerEvent != 0)
            {
                fsPtr.dynamicCast(timerEvent->getCallInfo()->getFeature());
            }
        }
        if (fsPtr != 0)
        {
            //Retrieve the script filename and strip out
            //the user name.  
            string tmpStr = fsPtr->getFeatureScript();
            char *ptr = (char *)tmpStr.c_str();
            char *ptr2 = ptr;
            while (*ptr)
            {
                if (*ptr == '.') ptr2 = ptr;
                ptr++;
            }
            *ptr2 = 0;
            myName = myName + tmpStr.c_str();
        }
    }
    if (myName.size() == 0)
    {
        cpLog(LOG_ERR, "could not get script name !!!");
        myName = "DEFAULT";
    }
    cpLog(LOG_DEBUG, "logging on file \"%s\"...", myName.c_str());


    // step 2. Get previous contents...
    string contents;
    try
    {
        contents = myPif ->readItem("log", myName);
    }
    catch (VException &ve)
    {
        contents = "NEW LOG START...\r\n";
    }

    // step 3. append new contents...
    contents = contents + "Log On " + MyTime().toString() + "COMMENTS:\r\n";
    contents = contents + myComment;

    // step 4. write the data...
    try
    {
        myPif ->writeItem("log", myName, contents);
    }
    catch (VException &ve)
    {
        cpLog(LOG_ERR, "ERR: Could'nt Log (%s)!!!", ve.getDescription().c_str());
        return ;
    }
    cpLog(LOG_DEBUG, "Logged on \"%s\" following...\r\n%s", myName.c_str(), contents.c_str());
}

void
CPLOpLog::updateLastCaller(Sptr < SipProxyEvent > anEvent)
{
    //Find the SIP message 
    Sptr < SipEvent > sipEvent;
    Sptr < SipMsg > sipMsg;
    sipEvent.dynamicCast( anEvent );
    assert (sipEvent != 0 );
    sipMsg = sipEvent->getSipMsg();
    assert (sipMsg != 0);

    //Get the Contact's URL. 
    Sptr<SipUrl> theLastCaller;
    theLastCaller.dynamicCast( sipMsg->getContact().getUrl() );
       
    ///Do not save number for Call Return if Caller ID was
    ///blocked by a previous feature server. 
    if ( sipMsg->getFrom().getUrl()->getUserValue() == "-" )
    {
        cpLog( LOG_DEBUG, "Caller ID blocked: Do not save [%s@%s:%s] ",
                          theLastCaller->getUserValue().logData(),
                          theLastCaller->getHost().logData(),
                          theLastCaller->getPort().logData() 
             ); 
           
        //Erase last caller already stored in FsUserData
        SipUrl emptyUrl;
        LocalScopeAllocator lo;
        string callee( sipMsg->getTo().getUrl()->getUserValue().getData(lo) ); 
        FsUsers::instance()->getUserData(callee)->setLastCaller(emptyUrl);
          
        return; 
    } /*if*/

    //Get the Request URL. 
    Sptr<SipUrl> theCallee;
    theCallee.dynamicCast( sipEvent->getInvite()->getRequestLine().getUrl() );

    Data name = sipMsg->getFrom().getDisplayName();

    cpLog(LOG_DEBUG, "Call Return: Saving-To[%s], From[%s], displayNme[%s]", 
                     theLastCaller->getUserValue().logData(),
                     theCallee->getUserValue().logData(), 
                     name.logData());
    LocalScopeAllocator lo;
    string callee(theCallee->getUserValue().getData(lo));
    FsUsers::instance()->getUserData(callee)->setLastCaller(*theLastCaller);
    FsUsers::instance()->getUserData(callee)->setDispName(name);
}

