
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const CPLOpNonAnswer_cxx_Version =
    "$Id: CPLOpNonAnswer.cxx,v 1.33 2002/04/18 16:17:14 rkripa Exp $";
#include "CPLOpNonAnswer.hxx"
#include <iostream.h>
#include <SipEvent.hxx>
#include <TimerEvent.hxx>
#include <CallInfo.hxx>
#include <CancelMsg.hxx>
#include <ByeMsg.hxx>
#include "StatusMsg.hxx"
#include "AckMsg.hxx"
#include "InviteMsg.hxx"

using namespace Vocal;

///
CPLOpNonAnswer::CPLOpNonAnswer()
{
#ifdef MEM_TEST
    cpLog(LOG_DEBUG, "CPLOpNonAnswer[%x]", this );
#endif
}  //CPLOpNonAnswer


///
CPLOpNonAnswer::~CPLOpNonAnswer()
{
#ifdef MEM_TEST
    cpLog(LOG_DEBUG, "~CPLOpNonAnswer[%x]", this );
#endif
}  //~CPLOpNonAnswer


///
const Sptr < State >
CPLOpNonAnswer::process( const Sptr < SipProxyEvent > event )
{
    cpLog(LOG_DEBUG, "process()" );

    if ( isTimeout( event) == true )
        return (processTimerEvent( event ) );
    else
        return (processSipEvent( event ) );

}  // process


///
Sptr < State >
CPLOpNonAnswer::processTimerEvent( const Sptr < SipProxyEvent > event )
{
    //Determine if event is a timeout 
    Sptr < TimerEvent > timeoutEvent;
    timeoutEvent.dynamicCast( event );

    if (timeoutEvent != 0)
    {
        Sptr < SipCommand > sipCommand = getSipCommand( event );
        if ( sipCommand == 0 )
        {
            cpLog( LOG_ERR, "No SIP cmd found!" );
            assert( sipCommand != 0 );
        }  /*if*/

        CancelMsg cancelMsg( *sipCommand );
        sendAsync( event, cancelMsg );
        return ( handleResponse( DestinationNoAnswer, event ) );

    } /*if timeoutEvent != 0*/
    else
    {
        cpLog( LOG_ERR, "Not a timer event." );
        return 0;
    }
} //processTimerEvent


///
Sptr < State >
CPLOpNonAnswer::processSipEvent( const Sptr < SipProxyEvent > event )
{
    Sptr < SipEvent > sipEvent;
    sipEvent.dynamicCast(event);

    if ( sipEvent == 0 )
    {
        cpLog( LOG_DEBUG, "Not a SIP event." );
        return 0;
    } /*if*/

    Sptr < SipMsg > sipMsg = sipEvent->getSipMsg();

    //If this is a CANCEL, cancel timer & exit function
    Sptr < CancelMsg > cancelMsg;
    cancelMsg.dynamicCast( sipMsg );
    if ( cancelMsg != 0 )
    {
        cancelTimer( event );
        return 0;
    }  /* if cancelMsg != 0 */

    //If this is a BYE message that needs to be treated like
    //a CANCEL (e.g. Pingtel's requirements), cancel the timer 
    //and exit this function.
    Sptr < ByeMsg > byeMsg;
    byeMsg.dynamicCast( sipMsg );
    if ( byeMsg != 0 )
    {
        cancelTimer( event );
        return 0;
    }  /* if byeMsg != 0 */

    //Try to process event as a Status message
    Sptr < StatusMsg > statusMsg;
    statusMsg.dynamicCast( sipMsg );
    if ( statusMsg == 0)
    {
        cpLog( LOG_ERR, "Unexpecting message.  Cancelling timer.");
        cancelTimer( event );
        return 0;
    } /*if statusMsg != 0 */

    int statusCode = statusMsg->getStatusLine().getStatusCode();
    cpLog( LOG_DEBUG, "Received %d", statusCode );

    switch (statusCode)
    {
        case 200 :  //Successful OK 
        {
            //If the 200 is a reply for something else other than an
            //INVITE, let CPLOpProxy handle it.
            if ( statusMsg->getCSeq().getMethod() != "INVITE" )
            {
                return 0; 
            } /*if*/

            cancelTimer( event );
            getStateMachineData( event )->setProxyStatus( FsStateMachnData::Answered );
            return (handleResponse( DestinationMisc, event ));
        } //200
        break;
        case 300:  //Redirection 3xx
        case 301:
        case 302:
        case 305:
        {
            cancelTimer( event );
            return ( handleResponse( DestinationRedirection, event ) ); 
        } /*3xx*/
        break; 
        case 486 :  //Busy Received
        case 600 :
        {
            cancelTimer( event );
            return (handleResponse( DestinationBusy, event ));
        }
        break;
        default:  //All other messages 
        {
            //Informational messages should be handled by another
            //operator.
            if ( ( statusCode >= 100 ) && ( statusCode < 200 ) )
                return 0;

            //Most of 4xx,5xx, and 6xx can be treated as a failure.
            if (( statusCode >= 400 ) && ( statusCode < 700 ))
            {
                cancelTimer( event );
                return (handleResponse( DestinationFailure, event ));
            } /*if*/

            cpLog( LOG_ERR, "Unexpected message.  Cancelling timer." );
            cancelTimer( event ); 
        }  /*default*/
    } // end of switch (statusCode)

    assert(0);  // control reaches end of non-void function
    return 0;

} //processSipEvent


///

bool
CPLOpNonAnswer::isTimeout( Sptr < SipProxyEvent > event )
{
    bool returnValue = false;

    Sptr < TimerEvent > timeoutEvent;
    timeoutEvent.dynamicCast( event );

    if (timeoutEvent != 0)
        returnValue = true;

    return returnValue;
} //isTimeout



///
Sptr < State >
CPLOpNonAnswer::handleResponse( Destination_Response callResponse,
                                Sptr < SipProxyEvent > event )
{
    Sptr < State > returnState;

    switch ( callResponse )
    {
        case DestinationBusy:
        {
            cpLog( LOG_DEBUG, "Busy arrived." );
            getStateMachineData( event )->setProxyStatus( FsStateMachnData::Busy );
            if ( myBusyReceived != 0 )
            {
                ackBasedOnInvite( event );
                returnState = myBusyReceived->process( event );
            }
            else if ( myUndefinedOutput != 0 )
            {
                //Do the <default> since <busy> was undefined.
                ackBasedOnInvite( event );
                returnState = myUndefinedOutput->process( event );
            } /*else*/
        }
        break;
        case DestinationNoAnswer:
        {
            cpLog( LOG_DEBUG, "NoAnswer arrived." );
            getStateMachineData( event )->setProxyStatus( FsStateMachnData::NoAnswer );
            if ( myNoAnswerReceived != 0 )
                returnState = myNoAnswerReceived->process( event );
            else if ( myUndefinedOutput != 0 )
            {
                //Do the <default> since <noanswer> was undefined.
                ackBasedOnInvite( event );
                returnState = myUndefinedOutput->process( event );
            } /*else*/
        }
        break;
        case DestinationFailure:
        {
            cpLog( LOG_DEBUG, "Failure arrived." );
            getStateMachineData( event )->setProxyStatus( FsStateMachnData::Failure );
            if ( myFailedReceived != 0 )
                returnState = myFailedReceived->process( event );
            else if ( myUndefinedOutput != 0 )
            {
                //Do the <default> since <failure> was undefined.
                ackBasedOnInvite( event );
                returnState = myUndefinedOutput->process( event );
            } /*else*/
        }
        break;
        case DestinationRedirection:
        {
            //Process only if <redirection> output was defined. 
            if ( my3xxReceived != 0)
            {
                //Reply to sender with ACK. 
                ackStatusMsg( event );

                //Save Contact info in The locations set. 
                Sptr<BaseUrl> contactUrl = getContactUrl( event );
                getStateMachineData(event)->getLocations()->push(contactUrl);

                returnState = my3xxReceived->process( event );
            } /*if*/
            else if ( myUndefinedOutput != 0 )
            {
                //Do the <default> since <redirection> was undefined.
                returnState = myUndefinedOutput->process( event );
            } /*else*/
        } /*case*/
        break;
        case DestinationMisc: 
        default:
        ;   //Do nothing but wait to do default operators in CPLOperator.
    } /*switch*/

    //If an operator was executed, return the next state

    if (returnState != 0 ) return returnState;

    //If operator returned 0 or no operator was executed, do the
    //default operators
    cpLog(LOG_DEBUG, "Doing default operators." );
    return ( CPLOperator::process( event ) );

}  //handleResponse


///
void
CPLOpNonAnswer::ackBasedOnInvite( Sptr < SipProxyEvent > event )
{
    Sptr < SipEvent > sipEvent;
    Sptr < StatusMsg > statusMsg;

    sipEvent.dynamicCast(event);

    if ( sipEvent != 0 )
    {
        Sptr < SipMsg > sipMsg = sipEvent->getSipMsg();
        statusMsg.dynamicCast( sipMsg );
    }

    assert( statusMsg != 0 );

    // retrieve INVITE associated with sipEvent
    Sptr < InviteMsg > savedInvite = sipEvent->getInvite();
    if ( savedInvite == 0 )
    {
        // error, no corresponding INVITE, return without sending ACK
        cpLog( LOG_ERR, "No INVITE associated with STATUS" );
        return ;
    }

    // send ACK to receiver of corresponding INVITE by setting Request URI
    // of ACK to that of the INVITE
    AckMsg ack( *statusMsg );
    SipRequestLine ackRequestLine( SIP_ACK,
                                   savedInvite->getRequestLine().getUrl() );
    ack.setRequestLine( ackRequestLine );
    sipEvent->getSipStack()->sendAsync( ack );

} ///ackBasedOnInvite

///
const char* const
CPLOpNonAnswer::name() const
{
    return "CPLOpNonAnswer";
} //name


///
bool
CPLOpNonAnswer::setNextOperator( Sptr < Operator > op, int prevNode)
{
    switch (prevNode)
    {
        case CPLNodeBusy:
        {
            myBusyReceived = op;
            return true;
        }
        break;
        case CPLNodeNoanswer:
        {
            myNoAnswerReceived = op;
            return true;
        }
        break;
        case CPLNodeFailure:
        {
            myFailedReceived = op;
            return true;
        }
        break;
        case CPLNodeRedirection:
        {
            my3xxReceived = op;
            return true;
        }
        break;
        case CPLNodeDefault:
        {
            myUndefinedOutput= op;
            return true;
        }
        break;
        default:
        return false;
    }
} //setNextOperator


///
void
CPLOpNonAnswer::doCpLog() const
{
    logData();

    if (myBusyReceived != 0)
    {
        cpLog(LOG_DEBUG, "++BUSY++" );
        myBusyReceived->doCpLog();
        cpLog( LOG_DEBUG, "++++++++" );
    }

    if (myNoAnswerReceived != 0)
    {
        cpLog( LOG_DEBUG, "++NO ANSWER++" );
        myNoAnswerReceived->doCpLog();
        cpLog( LOG_DEBUG, "+++++++++++++" );
    }

    if (myFailedReceived != 0)
    {
        cpLog( LOG_DEBUG, "++FAILURE++" );
        myFailedReceived->doCpLog();
        cpLog( LOG_DEBUG, "+++++++++++" );
    }

    if (my3xxReceived != 0)
    {
        cpLog( LOG_DEBUG, "++REDIRECTION++" );
        my3xxReceived->doCpLog();
        cpLog( LOG_DEBUG, "+++++++++++" );
    }

    if (myUndefinedOutput != 0)
    {
        cpLog( LOG_DEBUG, "++DEFAULT++" );
        myUndefinedOutput->doCpLog();
        cpLog( LOG_DEBUG, "+++++++++++" );
    }

    if ( myOperators.size() > 0 )
    {
        cpLog( LOG_DEBUG, ",+,NONE OF THE ABOVE,+," );
        myOpConstIterator it;
        myOpConstIterator endOfList = myOperators.end();
        for (it = myOperators.begin(); it != endOfList; ++it )
        {
            if ( strcmp( (*it)->name(), "CPLOpProxy" ) == 0 )
                cpLog( LOG_DEBUG, (*it)->name() );
            else
                (*it)->doCpLog();
        }  /*for*/
        cpLog( LOG_DEBUG, "+,+,+,+,+,+,+," );
    }  /*ANSWERED*/


} //doCpLog

///
bool
CPLOpNonAnswer::setAttributes( const char* attrName, const char* attrValue)
{
    //This is not a CPL tag;  there should never be attributes
    return true;
}  //setData


///
void
CPLOpNonAnswer::logData() const
{
    cpLog( LOG_DEBUG, "CPLOpNonAnswer" );
}  //debugStatus


//End of File

