#ifndef _CPL_OP_PRIORITY__HXX
#define _CPL_OP_PRIORITY__HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */





static const char* const CPLOpPriority_hxx_Version =
    "$Id$";





#include "CPLOperator.hxx"
#include <list>
#include "CallProcessingLanguage.hxx"
#include "Operator.hxx"
#include "State.hxx"
#include "Sptr.hxx"
#include "Data.hxx"

namespace Vocal
{


/**
 * Class implementing the o/p of CPL Priority switch
 */

class CPLOpPriority : public CPLOperator
{
    public:
        /// default constructor
        CPLOpPriority();

        /// destructor
        ~CPLOpPriority();

        /**
         *@name CPLOperator implementation methods...
         */
        //@{
        ///
        bool setAttributes(const char*, const char*);
        ///
        const Sptr < State > process(const Sptr < SipProxyEvent > );
        ///
        bool setNextOperator(Sptr < Operator > , int = CPLNodeEndofNodes);
        ///
        const char* const name() const;
    private:
        ///
        void logData(void) const;
        //@}

    public:
        /**
         *@name CPLOpPriority specific typedef
         */
        //@{
        enum _Field {FieldInvalid = 0, FieldLess, FieldGreater, FieldEqual};
        /// Field = {FieldInvalid, FieldLess, FieldGreater, FieldEqual};
        typedef enum _Field Field;
        //@}

        /**
         * method to test if this instance of o/p is applicable
         * in the CPLOpProiritySwitch
         *@param data reference to the data item containing priority of the call
         *@return bool result of the test
         */
        bool compare(Data& data);

    private:
        /**
         *@name CPLOpPriority attributes
         */
        //@{
        /// stores the type of comparison
        Field myField;

        /// stores the priority to match
        string myStr;
        //@}
};
 
} // namespace Vocal

#endif

