
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const CPLOpTimeZone_cxx_Version =
    "$Id$";
#include "CPLOpTimeZone.hxx"
#include "CPLOffset.hxx"
#include "MyTime.hxx"

using namespace Vocal;

///
CPLOpTimeZone::CPLOpTimeZone()
{}  //Constructor


///
CPLOpTimeZone::~CPLOpTimeZone()
{}  //Destructor


///
bool
CPLOpTimeZone::setAttributes(const char* type, const char* value)
{
    //check attribute type
    if (strcmp(type, "name") == 0)
    {
        myName = value;
        return true;
    }

    //illegal attribute type
    assert(0);

    return false;  
}  //setAttributes


///

const Sptr < State >
CPLOpTimeZone::process(const Sptr < SipProxyEvent > anEvent)
{
    /// there is nothing to process, this method will never be called 'coz
    /// timezone is not an operator, its only using the interface

    return 0;

}  //process


///
bool
CPLOpTimeZone::setNextOperator(Sptr < Operator > anOperator,
                               int nodeId /*def value*/)
{
    addOperator(anOperator);
    return true;
}  //setNextOperator


///
const char* const
CPLOpTimeZone::name() const
{
    // as name is an attrib of timezone, hence this attrib should be returned
    //    return "CPLOpTimeZone";

    return myName.c_str();
}  //name


///
void
CPLOpTimeZone::doCpLog(void) const
{
    logData();

    CPLOperator::myOpConstIterator curr = myOperators.begin();
    CPLOperator::myOpConstIterator end = myOperators.end();

    while (curr != end)
    {
        cpLog(LOG_DEBUG, "offset %s[%x] follows",
              (*curr)->name(), curr->getPtr());
        (*curr)->doCpLog();
        curr++;
    }
}  //doCpLog


///
void
CPLOpTimeZone::logData(void) const
{
    cpLog(LOG_DEBUG, "[%x]: time zone is %s.",
          this, myName.c_str());
}  //logData


///
MyTime
CPLOpTimeZone::getTime() const
{
    MyTime myTime;

    cpLog(LOG_DEBUG, ":getTime()");

    CPLOperator::myOpConstIterator curr = myOperators.begin();
    CPLOperator::myOpConstIterator end = myOperators.end();

    while (curr != end)
    {
        CPLOffset *offset;
        if ((offset = dynamic_cast < CPLOffset* > (curr->getPtr())) != 0)
        {
            if (offset->test() == true)
            {   // current offset is applicable
                cpLog(LOG_DEBUG, "%s offset is applicable", offset->name());
                myTime = offset->getTime();
                break;
            }
            else
                cpLog(LOG_DEBUG, "%s offset is not applicable",
                      offset->name());
        }
        else
        {
            cpLog(LOG_DEBUG, "ERR: cannot cast %s into offset",
                  (*curr)->name());
        }
        curr++;
    }
    return myTime;
}  //getTime


///
CPLOpTimeZoneLocal::CPLOpTimeZoneLocal()
{
    myName = "local";
}  //Copy Constructor


///
MyTime
CPLOpTimeZoneLocal::getTime() const
{
    return MyTime();
}  //getTime


///
CPLOpTimeZoneUTC::CPLOpTimeZoneUTC()
{
    myName = "UTC";
}  //Constructor


MyTime
CPLOpTimeZoneUTC::getTime() const
{
    time_t junk;
    time(&junk);
    return MyTime(*gmtime(&junk));
}  //getTime


//End of file


