#ifndef CPLOpVmRedirect_Hxx
#define CPLOpVmRedirect_Hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const CPLOpVmRedirectVersion =
    "$Id: CPLOpVmRedirect.hxx,v 1.15 2002/03/19 21:39:41 jason Exp $";





#include "CPLOperator.hxx"
#include "CPLUrlData.hxx"
#include <string>
#include "InviteMsg.hxx"
#include "SipEvent.hxx"
#include "TimerEvent.hxx"
#include "VmPortContainer.hxx"

namespace Vocal
{

/**
  *  Redirects a call to an available uavm. Upon reception of an Invite, 
  *  this operator gets an available (i.e. configured and not already used)
  *  uavm from the VmPortContainer. It then checks if this port is active: if
  *  it is, the call is proxied to this uavm for further processing.
  */
class CPLOpVmRedirect: public CPLOperator
{
    public:
        ///Constructor
        CPLOpVmRedirect();

        ///Destructor
        ~CPLOpVmRedirect();

        /**Execute the operator for the state machine
          *@param event Event from the Sip stack 
          *@return Pointer to next state or 0 if the operator did not need to
          *process the event. 
          */
        const Sptr < State > process( const Sptr < SipProxyEvent > event );

        ///Returns the name of the class.
        const char* const name() const;

        /** Save CPL tag attributes
          *@param attrName Attribute's name
          *@param attrValue Value of the attribute 
          *@return True if successful setting attributes */
        bool setAttributes( const char* attrName, const char* attrValue );

        /**Add special operators
           @param op Pointer to the operator to add
           @param prevNode ID of the last CPL tag.
           @return true if function did not have any errors setting next
            operator. 
        */
        bool setNextOperator( Sptr < Operator > op, int prevNode );

    protected:

        ///Prints debug information
        void logData() const;
        /**Check for available uavm port and redirect to an available uavm.
          *@param event Event from the Sip stack 
          *@param inviteMsg The Invite Msg from the Event
          */
        void processInvite( Sptr < SipEvent > sipEvent,
                            Sptr < InviteMsg > inviteMsg );

        /**Determines if a specified uavm (with port) is down or not.
          *@param vmPort Host and Port of a uavm 
          *@return True if uavm is down;  otherwise False is returned.
          */
        bool isUavmDown(const vmPort& uavmPort);

        /** Determines what the mailbox number is for the user in the "To" 
          *@param userTo The user in the "To"
          *@return mailbox number. If can't access provisioning, return userTo
          */
        const string getMailboxFromProvisioning( const string userTo );
};

 
} // namespace Vocal

#endif

//End of File

