
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const MailThread_cxx_Version =
    "$Id$";
#include "MailThread.hxx"

using namespace Vocal;


MailThread::MailThread()
{
    myDoneFlag = false;
}  //Constructor


///
MailThread::~MailThread()
{
    myDoneFlag = true;

    //Do a join() to block this class destruction from
    //finishing until the actual thread has terminated
    //because they share data. 
    join();

}  //Destructor


///
void
MailThread::mail(const Mail& aMail)
{
    Sptr < Mail > mailPtr = new Mail(aMail);
    cpLog(LOG_DEBUG, "going to enqueue message...");
    myMailQ.add(mailPtr);
    cpLog(LOG_DEBUG, "done.");
}  //mail


///
void
MailThread::thread()
{
    MailMan mailMan;
    Sptr < Mail > mail;

    cpLog(LOG_DEBUG, "Mail thread running ...");

    while (myDoneFlag != true)
    {
        if (myMailQ.size())
        {
            mail = myMailQ.getNext();
            if (mailMan.mail(mail->mailTo, mail->msg) == true)
            {
                cpLog(LOG_DEBUG, "mailed to <%s>...", mail->mailTo.c_str());
            }
            else
            {
                cpLog(LOG_DEBUG, "ERR while mailto <%s>...",
                      mail->mailTo.c_str());
            }
        }
    }
    while (myMailQ.size())
        myMailQ.getNext();  // just discard ...
    cpLog(LOG_DEBUG, "Mail thread done its job ...");
}  //thread



Mail::Mail()
{
    //no op!!!
}  //Mail()


///

Mail::Mail(const Mail& aMail)
{
    *this = aMail;
}  //Copy Constructor


///
Mail&
Mail::operator = (const Mail& rhs)
{
    mailTo = rhs.mailTo;
    msg = rhs.msg;
    return *this;
}  //Mail::operator=

//End of file
