
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const Range_cxx_Version =
    "$Id$";
#include "Range.hxx"
#include <strstream>


///
template < class T > Pair < T > ::Pair()
{
    hi = lo = 0;
}


///
template < class T > Pair < T > ::Pair(const Pair < T > & aPair)
{
    this->hi = aPair.hi;
    this->lo = aPair.lo;
}


///
template < class T > istrstream& operator>>(istrstream& ist, Pair < T > & aPair)
{
    char junk;

    //Read in the lower value of the range
    ist >> aPair.lo >> junk;
    if (junk == '-')
    {
        //Read in the higher value of the range
        ist >> aPair.hi >> junk;
    } /*if*/
    else 
    {
        aPair.hi = aPair.lo;
    } /*else*/

    return ist;
}




///
template < class T > ostrstream& operator<<(ostrstream& ost, Pair < T > & aPair)
{

    //Write a range (low value - high value) to the ostrstream.  
    ost << aPair.lo;
    if (aPair.lo != aPair.hi)
    {
        ost << '-' << aPair.hi;
    }
    return ost;
}


///
template < class T > Range < T > ::~Range()
{
    flush();
}


///
template < class T > void Range < T > ::flush()
{
    vector < Pair < T > * > ::iterator curr = myRangeList.begin();
    vector < Pair < T > * > ::iterator last = myRangeList.end();

    while (curr != last)
    {
        delete (*curr);
        curr++;
    }
}


///
template < class T > Range < T > ::Range(const Range < T > & aRange)
{
    *this = aRange;
}


///
template < class T > Range < T > & Range < T > ::operator=(const Range < T > & aRange)
{
    flush();

    vector < Pair < T > * > ::const_iterator curr = aRange.myRangeList.begin();
    vector < Pair < T > * > ::const_iterator last = aRange.myRangeList.end();

    while(curr != last)
    {
        add (**curr);
        curr++;
    }
    return *this;
}


///
template < class T > Range < T > ::Range(const char *str)
{
    add(str);
}


///
template < class T > Range < T > ::Range(const Pair < T > & p)
{
    add(p);
}


///
template < class T > Range < T > & Range < T > ::add(const Pair < T > & p)
{
    Pair < T > * pPtr = new Pair < T > ();
    *pPtr = p;

    myRangeList.push_back(pPtr);
    return *this;
}


///
template < class T > Range < T > & Range < T > ::add(const char* str)
{
    istrstream ist(str);
    while (ist)
    {
        Pair < T > *pPtr = new Pair < T > ();
        ist >> *pPtr;
        myRangeList.push_back(pPtr);
    }
    return *this;
}




///
template < class T > char * const Range < T > ::toString()
{
    static char str[256];
    ostrstream ost(str, 256);

    vector < Pair < T > * > ::const_iterator curr = myRangeList.begin();
    vector < Pair < T > * > ::const_iterator last = myRangeList.end();

    while (curr != last)
    {
        ost << **curr;
        curr++;
        if (curr != last) ost << ", ";
    }

    //Put terminating character in str[].
    ost << '\0';  

    return str;
}


///
template < class T > bool Range < T > ::operator==(const T& value) const
{
    vector < Pair < T > * > ::const_iterator curr = myRangeList.begin();
    vector < Pair < T > * > ::const_iterator last = myRangeList.end();

    while (curr != last)
    {
        if ( (value >= (*curr)->lo) && (value <= (*curr)->hi) )
        {
            return true;
        } /*if*/
        curr++;
    }
    return false;
}

template < class T >
bool
operator==(const T& value, const Range < T > & aRange)
{
    return (aRange == value);
}


//End of file

