#ifndef VmPortContainer_hxx
#define VmPortContainer_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */





static const char* const VmPortContainerVersion =
    "$Id: VmPortContainer.hxx,v 1.15 2002/03/19 21:39:41 jason Exp $";



#include "global.h"
#include <iostream>
#include <deque>
#include "Data.hxx"

namespace Vocal
{


/** vmPorts can be on different servers: this structure contains uavm servers*/
struct vmPort
{
    string host;
    int port;
};

/**
  *  Holds a set of available Voice Mail front end ports.
  *  When the VM Feature Server receives an INVITE, it asks the 
  *  VmPortContainer for an available port.
  *  An available port is a uavm port configured in provisioning and not 
  *  involved in a call.
  *  @see CPLOpVmRedirect
  */
class VmPortContainer
{
    public:

        ///Retrieve singleton pointer
        static VmPortContainer* instance();

        ///Constructor
        VmPortContainer();
        
        ///Destructor
        ~VmPortContainer();

        ///Get total VM ports
        int getNumberOfVmPorts() const;

        ///Get the next available VM port
        bool getVmPort(vmPort& returnValue);

        ///Add a VM port to the deque
        int appendVmPort(vmPort voiceMailPort);

        ///Assign port total to member function
        void setNumberOfConfiguredPorts( const int port);

        ///Retrieve port total
        const int getNumberOfConfiguredPorts() const;

        ///Display debug information regarding the class' member variables.
        void logData();

        ///Remove all from the deque.
        int clearAll();

        ///Retrieve pointer to deque.
        deque < vmPort > getListOfVmPorts() const; 

    private:
        ///Singleton instance
        static VmPortContainer* myVmPortContainerInstance;

        ///Data structure for holding voice mail ports
        deque < vmPort > myPortContainer;

        ///
        int myNumberOfConfiguredPorts;
};
 
}

#endif


//End of File

