/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
package com.vovida.telephony;

import javax.telephony.*;
import javax.telephony.capabilities.*;
import javax.telephony.callcontrol.*;
import com.vovida.telephony.events.*;
import com.vovida.telephony.capabilities.*;
import com.vovida.telephony.*;
import com.vovida.telephony.callcontrol.*;
import com.vovida.singleton.*;
import java.util.*;


/**********************************************************/
/* VCCall is the Vocal Client Call Class which is part  */
/* of the JTapi implementation. It implements the Call   */
/* interface from the JTapi specification.               */
/*********************************************************/

/**
 * VCCall is the Vocal Client Call Class which is part
 * of the JTapi implementation. It implements the Call
 * interface from the JTapi specification.
 */

public class VCCall implements Call
{

    protected Vector connections;
    protected VCProvider provider;
    protected Vector callObservers;
    protected CallEventNotifier callEvtNotifier;
    protected Object[] otherData;
    protected String name;
    protected int state;
    private VCIdentity idCreator;
    // the following data can be used by CallControlCall
    protected Address callingAddress;
    protected Address calledAddress;
    protected Terminal callingTerminal;

    /**
     * Constructor
     */
    protected VCCall(VCProvider provider) throws InvalidStateException
    {
	if (provider.inService() == true)
	{
	    state = IDLE;
	}
	else
	{
	    throw new InvalidStateException(this,
					    InvalidStateException.CALL_OBJECT,
					    IDLE);
	}
	this.provider = provider;
	this.callingAddress = null;
	this.calledAddress = null;
	this.callingTerminal = null;
	this.connections = new Vector();
	this.callObservers = new Vector();
	this.callEvtNotifier = new CallEventNotifier();
        idCreator = VCProvider.getIdCreator();
        this.name = idCreator.getCallId();
    }

    /**
     * private default constructor
     */
    private VCCall() {}

/************************************************************/
/* JTapi Implementation                                     */
/************************************************************/

    // return an array of the connections associated with this call
    /**
     * Refer To JTapi Call Interface
     */
    public Connection[] getConnections()
    {
	Connection[] connectionsArray = null;
	if (connections.size() > 0)
	{
	    connectionsArray = new Connection[connections.size()];
	    connections.copyInto(connectionsArray);
	}
	return connectionsArray;
    }

    // return the provider associated with this call
    /**
     * Refer To JTapi Call Interface
     */
    public Provider getProvider()
    {
	return provider;
    }

    // return the state of this call
    /**
     * Refer To JTapi Call Interface
     */
    public int getState()
    {
	return state;
    }


    // connect this call by creating 2 connections in the call,
    // each connection representing the association of this call
    // with the originating address or the terminating address
    // if successful the state of the call is active.. this does
    // not ensure that both connections are yet in a CONNECTED state

    /**
     * Refer To JTapi Call Interface
     */
    public Connection[] connect(Terminal origTerm, Address origAddr,
                              String dialedDigits)
	throws ResourceUnavailableException, PrivilegeViolationException,
	    InvalidPartyException, InvalidArgumentException,
	    InvalidStateException, MethodNotSupportedException
    {
	isProviderOK();
 	Terminal callingTerminal = origTerm;
	Address termAddr = null;
	if (((VCTerminal)origTerm).validAddress(origAddr) == false)
	{
	    throw new InvalidPartyException(InvalidPartyException.ORIGINATING_PARTY,
		"Call Connect()");
	}
	try
	{
	    termAddr = provider.getAddress(dialedDigits);
	}
	catch (InvalidArgumentException e)
	{
	    termAddr = ((VCProvider)provider).getTheAddress(dialedDigits);
	}
	Connection[] callConnections = createConnections(origAddr, termAddr);
	
	System.out.println("Connect Call - Initiate request to Server");
	// Make a request to the JTAPI Server
	String origDigits = origAddr.getName();
    System.out.println("Call From " + origDigits + " To " + dialedDigits);
	if (VCRequestUtil.callConnect(origDigits, dialedDigits, this))
	{
	    setState(ACTIVE);
	    return callConnections;
	}
	else
	{
	    removeConnection(callConnections[0]);
	    removeConnection(callConnections[1]);
	    return null;
	}
    }

    // add an observer to this call
    /**
     * Refer To JTapi Call Interface
     */
    public void addObserver(CallObserver observer)
	throws ResourceUnavailableException, MethodNotSupportedException
    {
	callObservers.addElement(observer);
    }

    // return an array of the observers associated with this call
    /**
     * Refer To JTapi Call Interface
     */
    public CallObserver[] getObservers()
    {
	CallObserver[] observersArray = null;
	if (callObservers.size() > 0)
	{
	    observersArray = new CallObserver[callObservers.size()];
	    callObservers.copyInto(observersArray);
	}
	return observersArray;
    }

    // remove this observer from the call and inform all observers that
    // an observation has ended on the call
    /**
     * Refer To JTapi Call Interface
     */
    public void removeObserver(CallObserver observer)
    {
	callObservers.removeElement(observer);
	callEvtNotifier.notifyObserversOfEvent(
	    new VCCallObservationEndedEv(observer, this, 0, 0, false));
    }

    // return the dynamic capabilities of the call
    /**
     * Refer To JTapi Call Interface
     */
    public CallCapabilities getCapabilities(Terminal terminal, Address address)
        throws InvalidArgumentException
    {
	return new VCDynamicCallCapabilities(this, address, terminal);
    }

    // as above, however this function format is @deprecated
    /**
     * Refer To JTapi Call Interface
     */
    public CallCapabilities getCallCapabilities(Terminal term, Address addr)
        throws InvalidArgumentException, PlatformException
    {
	return null;
    }

/**************************************************************/
/* Additional Functionality                                   */
/**************************************************************/

    /**
     * set state of the call to the given state
     * the new state is checked against the old state 
     * to ensure the transition is a valid one.
     */

    protected void setState(int newState) throws InvalidStateException
    {
	if (newState == IDLE)
	{
	    ; //empty statement
	}
	else
	{
	    if (state == newState)
	        return;
	}
	if (newState == ACTIVE)
	{
	    if (state == IDLE)
	    {
	        callEvtNotifier.notifyObserversOfEvent(new VCCallActiveEv(this));
	    }
	    else
	    {
		throw (new InvalidStateException(this,
		        InvalidStateException.CALL_OBJECT, state));
	    }
	}
	else if (newState == INVALID)
	{
	    if (state == ACTIVE)
	    {
	        callEvtNotifier.notifyObserversOfEvent(new VCCallInvalidEv(this));
	    }
	    else
	    {
		throw (new InvalidStateException(this,
                        InvalidStateException.CALL_OBJECT, state));
	    }
	}
        state = newState;
    }


    /**
     * add the given connection to this call
     */
    protected void addConnection(Connection conn)
    {
	connections.addElement(conn);
    }


    /**
     * removes the given connection from this call
     */
    protected void removeConnection(Connection conn) throws InvalidStateException
    {
	connections.removeElement(conn);
	if (connections.isEmpty())
	{
	    setState(Call.INVALID);
	    ((VCProvider)provider).removeCallFromList(this);
	}
    }


    /**
     * check if the given address name corresponds to any of the connections
     */
    public boolean hasConnection(String addressName)
    {
	Connection[] conns = getConnections();
	if ((conns != null) && (addressName != null))
	{
	    for ( int i=0; i < conns.length; i++)
	    {
		if ((((VCConnection)conns[i]).getAddress().getName()).compareTo(addressName) == 0)
		{
	    	    return true;
		}
	    }
	    return false;
	}
	return false;
    }

    /**
     * check if the passed addresss name corresponds to the address
     * name associated with the first connection of this call
     * first connection - connections.elementAt(0)
     */

    public boolean isOrigConnection(String addressName)
    {
        Connection[] conns = getConnections();
        if ((conns == null) || (addressName == null))
        {
	    return false;
	}
	if (conns[0] == null)
	{
	    return false;
 	}
        if ((((VCConnection)conns[0]).getAddress().getName()).compareTo(addressName) == 0)
        {
            return true;
        }
        return false;
    }

    /**
     * find the connection in this call which associates this call
     * with the passed address name
     */

    public Connection findConnection(String addressName)
    {
	Connection[] conns = getConnections();
	if (conns == null)
	{
	    return null;
	}
	for ( int i=0; i < conns.length; i++)
	{
	    if ((((VCConnection)conns[i]).getAddress().getName()).compareTo(addressName) == 0)
	    {
		return conns[i];
	    }
	}
	return null;
    }

    /**
     * create 2 connections for the call which associate the call with
     * the passsed addresses
     */

    public Connection[] createConnections(Address addr1, Address addr2)
    {
	Connection[] callConnections = null;
	if ((addr1 != null) && (addr2 != null))
	{
            callConnections = new Connection[2];

            // The originating address connection
            callConnections[0] = new VCCallControlConnection(addr1, this, true);

            // Create connection for destination endpoint
            callConnections[1] = new VCCallControlConnection(addr2, this, false);
	    callingAddress = addr1;
	    calledAddress = addr2;

	}
	
	return callConnections;
    }

    /**
     * set all Connections to a DISCONNECTED state
     * then remove all connections from the connections Vector.
     */

    void closeConnections()
    {
	try
	{
	    Connection[] conns = getConnections();
	    if (conns != null)
	    {
		for (int i=0; i < conns.length; i++)
		{
		    ((VCConnection)conns[i]).setState(Connection.DISCONNECTED);
		}
	    }
	    connections.setSize(0);
	    setState(INVALID);
	    ((VCProvider)provider).removeCallFromList(this);
	}
	catch (InvalidStateException e)
	{
	    e.printStackTrace();
	}
    }

    /**
     * set the call id of the call
     */
    public void setName(String name)
    {
	this.name = name;
    }

    /**
     * return the call id of the call
     */
    public String getName()
    {
	return name;
    }

    /**
     * any additional data related to the call can be set using this function
     */
    public Object[] getData()
    {
	return otherData;
    }

    /**
     * retrieve any additional data associated with the call
     */
    public void setData(Object[] data)
    {
	otherData = data;
    }

    /**
     * return the CallEventNotifier object associated with the call
     */
    public CallEventNotifier getCallEvtNotifier()
    {
	return callEvtNotifier;
    }

    /**
     * return the call id of the call
     */
    public String getCallId()
    {
        return this.name;
    }

    /**
     * check that the provider is in a suitable state
     * if not, an InvalidStateException is thrown
     */
    protected void isProviderOK() throws InvalidStateException
    {
	if (this.getProvider().getState() != Provider.IN_SERVICE)
	{
	    throw (new InvalidStateException(this.getProvider(),
					InvalidStateException.PROVIDER_OBJECT,
					this.getProvider().getState()));
 	}
	return;
    }

}


