/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
package com.vovida.telephony;

import javax.telephony.*;
import javax.telephony.capabilities.*;
import com.vovida.telephony.events.*;
import com.vovida.telephony.capabilities.*;
import java.util.*;


/**********************************************************/
/* VCTerminalConnection is the Vocal Client              */
/* TerminalConnection class which is an implementation    */
/* of TerminalConnection interface from the JTapi Spec.   */
/**********************************************************/

    /**
     * VCTerminalConnection is the Vocal Client
     * TerminalConnection class which is an implementation
     * of TerminalConnection interface from the JTapi Spec
     */

public class VCTerminalConnection implements TerminalConnection
{
    protected int state;
    protected Terminal terminal;
    protected Connection connection;
    protected CallEventNotifier callEvtNotifier;
    protected VCDynamicTerminalConnectionCapabilities capabilities;

    /**
     * Constructor
     */
    protected VCTerminalConnection(Terminal term, Connection conn)
    {
	try
	{
	    terminal = term;
	    connection = conn;
	    Call call = ((VCConnection)connection).getCall();
	    callEvtNotifier = ((VCCall)call).getCallEvtNotifier();
	    setState(IDLE);
	    ((VCTerminal)terminal).addTerminalConnection(this);
	    ((VCConnection)connection).addTerminalConnection(this);
	    capabilities = new VCDynamicTerminalConnectionCapabilities(this);
	}
	catch (InvalidStateException e)
	{
	    e.printStackTrace();
	}
    }

    /**
     * Private default constructor
     */
    private VCTerminalConnection() {}

/****************************************************/
/* JTapi Implementation                             */
/****************************************************/

    // return the state for the terminalconnection
    /**
     * Refer To JTapi TerminalConnection Interface
     */
    public int getState()
    {
	return state;
    }

    // return the terminal associated with the terminalconnection
    /**
     * Refer To JTapi TerminalConnection Interface
     */
    public Terminal getTerminal()
    {
	return terminal;
    }

    // return the connection associated with the terminalconnection
    /**
     * Refer To JTapi TerminalConnection Interface
     */
    public Connection getConnection()
    {
	return connection;
    }


    // answer a call from the terminalconnection
    // the terminalconnection must be RINGING and the associated
    // connection be ALERTING - afterwards the connection should be
    // CONNECTED and the terminalconnection ACTIVE

    /**
     * Refer To JTapi TerminalConnection Interface
     */
    public void answer() throws PrivilegeViolationException,
	ResourceUnavailableException, MethodNotSupportedException,
	InvalidStateException
    {
        if (state != TerminalConnection.RINGING)
	{
	    throw (new InvalidStateException(this,
			    InvalidStateException.TERMINAL_CONNECTION_OBJECT, state));
	}
        if (getConnection().getState() != Connection.ALERTING)
        {
            throw (new InvalidStateException(getConnection(),
                	    InvalidStateException.CONNECTION_OBJECT,
                            getConnection().getState()));
        }
	VCRequestUtil.answer(getTerminal().getName(), getConnection());
	try
	{
	    ((VCConnection)getConnection()).setState(Connection.CONNECTED);
	    setState(ACTIVE);
	}
	catch (InvalidStateException e)
	{
	    e.printStackTrace();
	}
    }

   // return the dynamic capabilities of the terminalconnection
    /**
     * Refer To JTapi TerminalConnection Interface
     */
    public TerminalConnectionCapabilities getCapabilities()
    {
	return capabilities;
    }

    // as above, this form of the function is @deprecated
    /**
     * Refer To JTapi TerminalConnection Interface
     */
    public TerminalConnectionCapabilities getTerminalConnectionCapabilities(
	Terminal terminal, Address address)
    {
	return null;
    }

/************************************************************/
/* Additional Functionalities                               */
/************************************************************/

    // set the state of the terminalconnection
    // if the terminalconnection is not changing state then there
    // is no need to create an event so return from this function.
    // The new state is checked against the old one to ensure
    // the state transition is a valid one
    // Otherwise create a new event depending the new state and 
    // set the state accordingly.

    /**
     * set the state of the terminalconnection 
     * The new state is checked against the old one to ensure
     * the state transition is a valid one
     */

    public void setState(int newState) throws InvalidStateException
    {
	System.out.println("TermConn Set State");
	// EVT Handling
	if (newState != IDLE)
	{
	    if (state == newState)
		return;
	}
	System.out.println("Setting NewState From: " + state + " To " + newState);
	switch (newState)
	{
	    case IDLE :
		{
		    callEvtNotifier.notifyObserversOfEvent(
					new VCTermConnCreatedEv(this));
		}
		break;
	    case RINGING :
		{
		    if (state == IDLE)
		        callEvtNotifier.notifyObserversOfEvent(
					new VCTermConnRingingEv(this));
		    else
			throw (new InvalidStateException(this,
			    InvalidStateException.TERMINAL_CONNECTION_OBJECT,
			    state));
		}
		break;
	    case PASSIVE :
		{
		    if ((state == IDLE) || (state == ACTIVE))
		    {
		        callEvtNotifier.notifyObserversOfEvent(
					new VCTermConnPassiveEv(this));
		    }
		    else
		    {
		        throw (new InvalidStateException(this,
			    InvalidStateException.TERMINAL_CONNECTION_OBJECT,
			    state));
		    }
		}
		break;
	    case ACTIVE :
		{
		    if ((state == IDLE) || (state == PASSIVE) || (state == RINGING))
		    {
		        callEvtNotifier.notifyObserversOfEvent(
					new VCTermConnActiveEv(this));
		    }
		    else
		    {
			throw (new InvalidStateException(this,
			    InvalidStateException.TERMINAL_CONNECTION_OBJECT,
			    state));
		    }
		}
		break;
	    case DROPPED :
		{
                    ((VCTerminal)terminal).removeTerminalConnection(this);
                    ((VCConnection)connection).removeTerminalConnection(this);
		    if (state != UNKNOWN)
		    {
		        callEvtNotifier.notifyObserversOfEvent(
					new VCTermConnDroppedEv(this));
		    }
		    else
		    {
			throw (new InvalidStateException(this,
			    InvalidStateException.TERMINAL_CONNECTION_OBJECT,
			    state));
		    }
		}
		break;
	    case UNKNOWN :
		{
		    callEvtNotifier.notifyObserversOfEvent(
					new VCTermConnUnknownEv(this));
		}
		break;
	}
	state = newState;
    }


    /**
     * check that the provider is in a suitable state 
     * if not, an InvalidStateException is thrown
     */
    protected void isProviderOK() throws InvalidStateException
    {
	Provider prov = this.getConnection().getCall().getProvider();
        if (prov.getState() != Provider.IN_SERVICE)
        {
            throw (new InvalidStateException(prov,
                                        InvalidStateException.PROVIDER_OBJECT,
                                        prov.getState()));
        }
        return;
    }


}
