/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
package com.vovida.telephony.callcontrol;

import javax.telephony.*;
import javax.telephony.callcontrol.*;
import javax.telephony.capabilities.*;
import com.vovida.telephony.*;
import com.vovida.telephony.callcontrol.capabilities.*;


/************************************************************/
/* VCCallControlCall is the Vocal Client CallControl Call  */
/* Class which is part of the JTapi implementation.         */
/* It implements the CallControl Call                       */
/* interface from the JTapi specification.                  */
/* This class provides functionality such as conference     */
/* and transfer a call                                      */
/************************************************************/

/**
 * Implementation of the Jtapi CallControlCall Interface
 *
 */
public class VCCallControlCall extends VCCall implements CallControlCall
{
    Address lastRedirectedAddress;
    TerminalConnection conferenceController, transferController;
    boolean conference, transfer;


    /**
     * Constructor
     */
    public VCCallControlCall(VCProvider provider) throws InvalidStateException
    {
	super(provider);
  	lastRedirectedAddress = null;
	conferenceController = null;
	transferController = null;
	conference = false;
 	transfer = false;
	
    }

/************************************************************/
/* JTapi Implementation                                     */
/************************************************************/


    // return callingAddress - stored in superclass VCCall
    // callingAddress - address from which call is originally placed
    /**
     * Refer To JTapi CallControlCall Interface
     */
    public Address getCallingAddress()
    {
	return callingAddress;
    }

    // return callingTerminal - stored in superclass VCCall
    // callingTerminal - terminal from which call is originally placed
    /**
     * Refer To JTapi CallControlCall Interface
     */
    public Terminal getCallingTerminal()
    {
	return callingTerminal;
    }

    // return calledAddress - stored in superclass VCCall
    // calledAddress - address to which call is originally placed
    /**
     * Refer To JTapi CallControlCall Interface
     */
    public Address getCalledAddress()
    {
	return calledAddress;
    }

    // return lastRedirectedAddress
    /**
     * Refer To JTapi CallControlCall Interface
     */
    public Address getLastRedirectedAddress()
    {
	return lastRedirectedAddress;
    }


    // add a new party to this call - the new party is represented
    // by the name of the destination address

    /**
     * Refer To JTapi CallControlCall Interface
     */
    public Connection addParty(String newParty)
	throws InvalidStateException, InvalidPartyException,
	    MethodNotSupportedException, PrivilegeViolationException,
	    ResourceUnavailableException
    {
	isProviderOK();
	if (this.getState() != Call.ACTIVE)
	{
	    throw (new InvalidStateException(this,
					     InvalidStateException.CALL_OBJECT,
					     this.getState()));
	}
	Connection[] conns = this.getConnections();
	if (conns != null)
	{
	    if (conns.length >= 2)
	    {
		    int establishedCount = 0;
	    	int queuedOfferedAlertingCount = 0;
		    int state;
		    for (int i=0; i < conns.length; i++)
		    {
		        state = ((CallControlConnection)conns[i]).getCallControlState();
		        if (state == CallControlConnection.ESTABLISHED)
			        establishedCount++;
		        if ((state == CallControlConnection.QUEUED) ||
			        (state == CallControlConnection.OFFERED) ||
			        (state == CallControlConnection.ALERTING))
			        queuedOfferedAlertingCount++;
		    }
		    if ((establishedCount < 2) || (queuedOfferedAlertingCount > 1))
		    {
            	throw (new InvalidStateException(this,
                                InvalidStateException.CALL_OBJECT,
                                this.getState(),
					            "Invalid Connections"));

		    }
	    }
	    else
	    {
            throw (new InvalidStateException(this,
                                InvalidStateException.CALL_OBJECT,
                                this.getState(),
					            "Invalid Connections"));
	    }
	}
	try
	{
	    Address addr = ((VCProvider)provider).getTheAddress(newParty);
	    Connection conn = null;
	    conn = new VCCallControlConnection(addr, this, false);
	    this.addConnection(conn);
	}
	catch (InvalidArgumentException e)
	{
	    throw (new InvalidPartyException
				    (InvalidPartyException.DESTINATION_PARTY));
	}
	// Call the Util fn & return the connection
	// when done remove the NotSupported Exception
	MethodNotSupportedException e = new MethodNotSupportedException();
	throw e;
    }


    // drop all connections from this call
    /**
     * Refer To JTapi CallControlCall Interface
     */
    public void drop()
        throws InvalidStateException, MethodNotSupportedException,
	    PrivilegeViolationException, ResourceUnavailableException
    {
        isProviderOK();
        if (this.getState() != Call.ACTIVE)
        {
            throw (new InvalidStateException(this,
                                             InvalidStateException.CALL_OBJECT,
                                             this.getState()));
        }
	Connection[] conns = this.getConnections();
	if (conns != null)
	{
	    for (int i=0; i < conns.length; i++)
	    {
		conns[i].disconnect();
	    }
	}
    }


    // Offhook the terminal of a call so that digits can be
    // dialled manually
    /**
     * Refer To JTapi CallControlCall Interface
     */
    public Connection offHook(Address origaddress, Terminal origterminal)
	throws InvalidStateException, MethodNotSupportedException,
	    PrivilegeViolationException, ResourceUnavailableException
    {
        isProviderOK();
        if (this.getState() != Call.IDLE)
        {
            throw (new InvalidStateException(this,
                                             InvalidStateException.CALL_OBJECT,
                                             this.getState()));
        }
	// send an offhook message to server
	setState(Call.ACTIVE);
	Connection conn = new VCCallControlConnection(origaddress, this, false);
	((VCCallControlConnection)conn).setCallControlState
					    (CallControlConnection.INITIATED);
	TerminalConnection tc = new VCCallControlTerminalConnection
							(origterminal, conn);
	((VCCallControlTerminalConnection)tc).setCallControlState
					(CallControlTerminalConnection.TALKING);
	// return the connection & remove the NotSupportedException
	MethodNotSupportedException e = new MethodNotSupportedException();
        throw e;
    }


    // Conference the otherCall with this call by adding all
    // unique connections and terminalconnections from the other call
    // to this call

    /**
     * Refer To JTapi CallControlCall Interface
     */
    public void conference(Call otherCall)
	throws InvalidStateException, InvalidArgumentException,
	    MethodNotSupportedException, PrivilegeViolationException,
	    ResourceUnavailableException
    {
	isProviderOK();
        if (this.getState() != Call.ACTIVE)
        {
            throw (new InvalidStateException(this,
                                             InvalidStateException.CALL_OBJECT,
                                             this.getState()));
        }
	if (this == otherCall)
	{
	    throw (new InvalidArgumentException());
	}

	TerminalConnection tc1 = getConferenceController();
	TerminalConnection tc2 = ((CallControlCall)otherCall).getConferenceController();
	if ((tc1 == null) || (tc2 == null))
	{
	    throw (new InvalidArgumentException());
	}
	if (tc1.getTerminal() != tc2.getTerminal())
	{
	    throw (new InvalidArgumentException());
	}
	int state = ((CallControlTerminalConnection)tc1).getCallControlState();
	if (state != CallControlTerminalConnection.TALKING)
	{
	    throw (new InvalidStateException(tc1,
			    InvalidStateException.TERMINAL_CONNECTION_OBJECT,
			    state));
	}
	state = ((CallControlTerminalConnection)tc2).getCallControlState();
	if (state != CallControlTerminalConnection.HELD)
	{
	    throw (new InvalidStateException(tc1,
                            InvalidStateException.TERMINAL_CONNECTION_OBJECT,
                            state));
	}
	
	// OK all pre-conditions are met

	// send the request to server

	copyConnections(otherCall);
    }


    // transfer all parties from another call to this call and
    // remove the transferController from the call. The other call
    // becomes invalid and all connections and terminalconnections
    // are removed from the call.

    /**
     * Refer To JTapi CallControlCall Interface
     */
    public void transfer(Call otherCall)
	throws InvalidStateException, InvalidArgumentException,
	    InvalidPartyException, MethodNotSupportedException,
	    PrivilegeViolationException, ResourceUnavailableException
    {
	VCCallControlTerminalConnection tc1, tc2;
        isProviderOK();
        if (this.getState() != Call.ACTIVE)
        {
            throw (new InvalidStateException(this,
                                             InvalidStateException.CALL_OBJECT,
                                             this.getState()));
        }
	if (otherCall.getState() != Call.ACTIVE)
	{
	    throw (new InvalidArgumentException());
	}
        if (this == otherCall)
        {
            throw (new InvalidArgumentException());
        }
	tc1 = (VCCallControlTerminalConnection)getTransferController();
	tc2 = (VCCallControlTerminalConnection)((VCCallControlCall)otherCall).getTransferController();
        if ((tc1 == null) || (tc2 == null))
        {
            throw (new InvalidArgumentException());
        }
        if (tc1.getTerminal() != tc2.getTerminal())
        {
            throw (new InvalidArgumentException());
        }
	int state = tc1.getCallControlState();
	if ((state != CallControlTerminalConnection.HELD) && 
			    (state != CallControlTerminalConnection.TALKING))
	{
	    throw (new InvalidStateException(tc1,
			    InvalidStateException.TERMINAL_CONNECTION_OBJECT,
			    state));
	}
	state = tc2.getCallControlState();
	if ((state != CallControlTerminalConnection.HELD) && 
                            (state != CallControlTerminalConnection.TALKING))
        {
            throw (new InvalidStateException(tc1,
                            InvalidStateException.TERMINAL_CONNECTION_OBJECT,
                            state));
        }

        // Pre Conditions met
	// Send the request to the server

	copyConnections(otherCall);
	tc1.setCallControlState(CallControlTerminalConnection.DROPPED);
	tc2.setCallControlState(CallControlTerminalConnection.DROPPED);

    }


    // Transfer a call to a destination specified by the address name
    // This is a one step transfer whereby the call to the new
    // destination is placed and the transfer completed at the same
    // time. The transfer controller is the terminalconnection which
    // will not exist on the call when the transfer is complete.

    /**
     * Refer To JTapi CallControlCall Interface
     */
    public Connection transfer(String address)
	throws InvalidArgumentException, InvalidStateException,
	    InvalidPartyException, MethodNotSupportedException,
	    PrivilegeViolationException, ResourceUnavailableException
    {
	String destStr = address;
	Connection conn = null;
	isProviderOK();
	if (getState() != Call.ACTIVE)
	{
	    throw (new InvalidStateException(this,
		InvalidStateException.CALL_OBJECT, getState()));
	}
	CallControlTerminalConnection tc = (CallControlTerminalConnection)transferController;
	if (tc == null)
	{
	    throw (new InvalidArgumentException());
	}
	if (tc.getCallControlState() != CallControlTerminalConnection.TALKING)
	{
	    throw (new InvalidStateException(tc,
		InvalidStateException.TERMINAL_CONNECTION_OBJECT,
		tc.getCallControlState()));
	}
	if (VCRequestUtil.transfer(this, tc, destStr) == true)
	{
	    
	}
	else
	{
	    return null;
	}

	Connection tc_conn = tc.getConnection();
        if (tc_conn.getTerminalConnections().length == 1)
        {
	    VCCallControlConnection vcConn = (VCCallControlConnection)tc_conn;
            vcConn.setCallControlState(CallControlConnection.DISCONNECTED);
        }
        VCCallControlTerminalConnection vcTConn = (VCCallControlTerminalConnection)tc;
        vcTConn.setCallControlState(CallControlTerminalConnection.DROPPED);

	Address destAddr = null;
	try
	{
	    destAddr = provider.getAddress(destStr);
	}
	catch (InvalidArgumentException e)
	{
	    return null;
	}
	
	conn = new VCCallControlConnection(destAddr, this, false);
	
	return conn;
    }


    // set the terminalconnection which is the conference controller
    // for this call
    /**
     * Refer To JTapi CallControlCall Interface
     */
    public void setConferenceController(TerminalConnection tc)
 	throws InvalidArgumentException, InvalidStateException,
	    MethodNotSupportedException, ResourceUnavailableException
    {
	conferenceController = tc;
    }


    // retrieve the conference controller for this call
    /**
     * Refer To JTapi CallControlCall Interface
     */
    public TerminalConnection getConferenceController()
    {
	return conferenceController;
    }


    // set the terminalconnection which is the transfer controller 
    // for this call
    /**
     * Refer To JTapi CallControlCall Interface
     */
    public void setTransferController(TerminalConnection tc)
	throws InvalidArgumentException, InvalidStateException,
	    MethodNotSupportedException, ResourceUnavailableException
    {
	transferController = tc;
    }


    // retrieve the transfer controller for this call
    /**
     * Refer To JTapi CallControlCall Interface
     */
    public TerminalConnection getTransferController()
    {
	return transferController;
    }


    // indicate whether this call is capable of conferencing

    /**
     * Refer To JTapi CallControlCall Interface
     */
    public void setConferenceEnable(boolean enable)
	throws InvalidArgumentException, InvalidStateException,
	    MethodNotSupportedException, PrivilegeViolationException
    {
        isProviderOK();
	if (this.getState() != Call.IDLE)
	{
	    throw (new InvalidStateException(this,
					    InvalidStateException.CALL_OBJECT,
					    this.getState()));
	}
	conference = enable;
    }


    // retrieve indication whether this call is capable of conference

    /**
     * Refer To JTapi CallControlCall Interface
     */
    public boolean getConferenceEnable()
    {
	return conference;
    }


    // indicate whether this call is capable of transfer

    /**
     * Refer To JTapi CallControlCall Interface
     */
    public void setTransferEnable(boolean enable)
  	throws InvalidArgumentException, InvalidStateException,
	    MethodNotSupportedException, PrivilegeViolationException
    {
        isProviderOK();
        if (this.getState() != Call.IDLE)
        {
            throw (new InvalidStateException(this,
                                            InvalidStateException.CALL_OBJECT,
                                            this.getState()));
        }
	transfer = enable;
    }


    // retrieve indication of whether this call is capable of transfer

    /**
     * Refer To JTapi CallControlCall Interface
     */
    public boolean getTransferEnable()
    {
	return transfer;
    }


    
    /**
     * Refer To JTapi CallControlCall Interface
     */
    public Connection[] consult(TerminalConnection tc, String dialedDigits)
	throws InvalidStateException, InvalidArgumentException,
	    MethodNotSupportedException, ResourceUnavailableException,
	    PrivilegeViolationException, InvalidPartyException
    {
	throw (new MethodNotSupportedException());
    }

    /**
     * Refer To JTapi CallControlCall Interface
     */
    public Connection consult(TerminalConnection tc)
	throws InvalidStateException, InvalidArgumentException,
	    MethodNotSupportedException, ResourceUnavailableException,
	    PrivilegeViolationException
    {
	throw (new MethodNotSupportedException());
    }


    // return the dynamic capabilities of the call
    /**
     * Refer To JTapi CallControlCall Interface
     */
    public CallCapabilities getCapabilities(Terminal terminal, Address address)
        throws InvalidArgumentException
    {
        return new VCDynamicCallControlCallCapabilities(this, address, terminal);
    }

/**************************************************************/
/* Additional Functionality                                   */
/**************************************************************/

    /**
     * Set the Calling Address for Call
     */
    public void setCallingAddress(Address addr)
    {
	callingAddress = addr;
    }

    /**
     * Set the Calling Terminal for call
     */
    public void setCallingTerminal(Terminal term)
    {
	callingTerminal = term;
    }

    /**
     * Set the Called Address for Call
     */
    public void setCalledAddress(Address addr)
    {
	calledAddress = addr;
    }

    /**
     * Set the Last Redirected Address for the Call
     */
    public void setLastRedirectedAddress(Address addr)
    {
	lastRedirectedAddress = addr;
    }


    /**
     * Copy the connections from another call to this call
     */
    private void copyConnections(Call otherCall) throws
        InvalidStateException, InvalidArgumentException
    {
        VCCallControlConnection otherConn, conn, matchedConn;
        VCCallControlTerminalConnection tc, matchedTC, newTC;
        int termConnState, connState;
        Connection[] otherConns = otherCall.getConnections();
        Connection[] conns = this.getConnections();


        if ((otherConns != null) && (conns != null))
        {
            // check each connection on otherCall
            for (int i=0; i < otherConns.length; i++)
            {
                otherConn = (VCCallControlConnection)otherConns[i];
                Address addr = otherConns[i].getAddress();
                String addrName = addr.getName();
                matchedConn = null;
                for (int k=0; k < conns.length; k++)
                {
                    conn = (VCCallControlConnection)conns[k];
                    if (conn.getAddress().getName().compareTo(addrName) == 0)
                    {
                        matchedConn = conn;
                    }
                }
                // if the connection is not matched then a new connection
                // for this address needs to be created for this call.
                // Then each terminalconnection from the otherCall connection
                // is added to this connection
                if (matchedConn == null)
                {
                    Connection newConn = new VCCallControlConnection(addr, this, false);
                    connState = otherConn.getCallControlState();
                    ((VCCallControlConnection)newConn).setCallControlState(connState);
                    TerminalConnection[] otherTCs = otherConn.getTerminalConnections();
                    if (otherTCs != null)
                    {
                        for (int j =0; j < otherTCs.length; j++)
                        {
                            tc = (VCCallControlTerminalConnection)otherTCs[j];
                            Terminal term = otherTCs[j].getTerminal();
                            newTC = new VCCallControlTerminalConnection
                                                                (term, newConn);                            termConnState = tc.getCallControlState();
                            newTC.setCallControlState(termConnState);
                            tc.setCallControlState
					(CallControlTerminalConnection.DROPPED);
                        }
                    }
                }
                // matched the connection only add unique termconnections
                // no need to create a connection as it already exists on this
                // call for the matched address name - however for each
                // terminalconnection on the otherCall which does not exist on
                // this call - a new terminalconnection must be created.
                else
                {
                    TerminalConnection[] otherTCs = otherConn.getTerminalConnections();
                    TerminalConnection[] tcs = matchedConn.getTerminalConnections();
                    if (otherTCs != null)
                    {
                        for (int j =0; j < otherTCs.length; j++)
                        {
                            tc = (VCCallControlTerminalConnection)otherTCs[j];
                            Terminal term = otherTCs[j].getTerminal();
                            String termName = term.getName();
                            if (tcs != null)
                            {
                                matchedTC = null;
                                for (int l=0; l < tcs.length; l++)
                                {
                                    if (tcs[l].getTerminal().getName().compareTo(termName) == 0)
                                    {
                                        matchedTC = (VCCallControlTerminalConnection)tcs[l];
                                    }
                                }
                                // only add a TC to existing connection
                                // if a TC for that Terminal does not exist
                                if (matchedTC == null)
                                {
                                    newTC = new VCCallControlTerminalConnection
                                                                (term, matchedConn);
                                    termConnState = tc.getCallControlState();
                                    newTC.setCallControlState(termConnState);
                                }
                            }
                            tc.setCallControlState
                                        (CallControlTerminalConnection.DROPPED);
                        }
                    }
                }

                otherConn.setCallControlState(CallControlConnection.DISCONNECTED);
            } // for statement
        }
        else
        {
            if (otherConns == null)
            {
                throw (new InvalidArgumentException());
            }
            else if(conns == null)
            {
                throw (new InvalidStateException(this,
                                            InvalidStateException.CALL_OBJECT,
                                            this.getState(), "No Connections"));            }
        }
    }


}

