/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
package com.vovida.telephony.callcontrol;

import javax.telephony.*;
import javax.telephony.callcontrol.*;
import com.vovida.telephony.*;
import java.util.*;

/************************************************************/
/* VCCallControlTerminal is the Vocal Client CallControl   */
/* Terminal Class which is part of the JTapi implementation.*/
/* It implements the CallControl Terminal                   */
/* interface from the JTapi specification.                  */
/* This class provides functionality such Do Not Disturb    */
/* and call pickup                                          */
/************************************************************/

/**
 * Implementation of the Jtapi CallControlTerminal Interface
 *
 */
public class VCCallControlTerminal extends VCTerminal implements CallControlTerminal
{
    
    boolean doNotDisturb;


    /**
     * Constructor
     */
    public VCCallControlTerminal(Provider prov, boolean isLocal, String str)
    {
	super(prov, isLocal, str);
	doNotDisturb = false;
	
    }

/************************************************************/
/* JTapi Implementation                                     */
/************************************************************/

    // return the DoNotdisturb setting for this terminal
    /**
     * Refer To JTapi CallControlTerminal Interface
     */
    public boolean getDoNotDisturb() throws MethodNotSupportedException
    {
	return doNotDisturb;
    }

    // set the DoNotDisturb attribute for this terminal
    /**
     * Refer To JTapi CallControlTerminal Interface
     */
    public void setDoNotDisturb(boolean enable)
	throws MethodNotSupportedException, InvalidStateException
    {
	isProviderOK();
	doNotDisturb = enable;
    }


    // this terminal attempts to pick up a call at which is queued
    // or ringing at the pickupConnection
    // The address associated with the connection may or may not be
    // the address associated with this terminal which requires to
    // pick up the call
    /**
     * Refer To JTapi CallControlTerminal Interface
     */
    public TerminalConnection pickup(Connection pickupConnection,
	Address terminalAddress) throws InvalidArgumentException,
	    InvalidStateException, MethodNotSupportedException,
	    PrivilegeViolationException, ResourceUnavailableException
    {
	isProviderOK();
	if ((pickupConnection.getCall()).getState() != Call.ACTIVE)
	{
	    throw (new InvalidStateException(pickupConnection.getCall(),
				    InvalidStateException.CALL_OBJECT,
				    (pickupConnection.getCall()).getState()));
	}
	int pickupState = ((CallControlConnection)pickupConnection).getCallControlState();
	if ((pickupState != CallControlConnection.QUEUED) && 
			(pickupState != CallControlConnection.ALERTING))
	{
	    throw (new InvalidStateException(pickupConnection,
					InvalidStateException.CONNECTION_OBJECT,
					pickupState));
	}

	boolean matchedAddress = false;
	Address[] addrs = this.getAddresses();
	if (addrs != null)
	{
	    for (int i=0; i < addrs.length; i++)
	    {
		if (terminalAddress == addrs[i])
		{
		    matchedAddress = true;
		}
	    }
	}
   	if (matchedAddress == false)
 	{
	    throw (new InvalidArgumentException());
	}

        // Pre conditions met
        // send the request to the server



	if (pickupConnection.getAddress() == terminalAddress)
	{
	    TerminalConnection tc = new VCCallControlTerminalConnection(this, pickupConnection);
	    ((VCCallControlConnection)pickupConnection).setCallControlState
				(CallControlConnection.ESTABLISHED);
	    ((VCCallControlTerminalConnection)tc).setCallControlState
				(CallControlTerminalConnection.TALKING);
	}
	else
	{
	    Call call = pickupConnection.getCall();
	    Connection conn = new VCCallControlConnection(terminalAddress, call, true);
	    TerminalConnection tc = new VCCallControlTerminalConnection(this, conn);
	    ((VCCallControlConnection)conn).setCallControlState
				(CallControlConnection.ESTABLISHED);
	    ((VCCallControlTerminalConnection)tc).setCallControlState
                                (CallControlTerminalConnection.TALKING);
	    ((VCCallControlConnection)pickupConnection).setCallControlState
                                (CallControlConnection.DISCONNECTED);
	}
			
	// return tc;
	MethodNotSupportedException e = new MethodNotSupportedException();
	throw e;
    }



    // this terminal attempts to pick up a call from the terminal connection
    // pickTermConn. 
    // The address associated with the connection may or may not be
    // the address associated with this terminal which requires to
    // pick up the call

    /**
     * Refer To JTapi CallControlTerminal Interface
     */
    public TerminalConnection pickup(TerminalConnection pickTermConn,
	Address terminalAddress) throws InvalidArgumentException,
            InvalidStateException, MethodNotSupportedException,
            PrivilegeViolationException, ResourceUnavailableException
    {
        isProviderOK();
	Connection pickupConn = pickTermConn.getConnection();
        if ((pickupConn.getCall()).getState() != Call.ACTIVE)
        {
            throw (new InvalidStateException(pickupConn.getCall(),
                                    InvalidStateException.CALL_OBJECT,
                                    (pickupConn.getCall()).getState()));
        }
        int pickupState = ((CallControlConnection)pickupConn).getCallControlState();
        if (pickupState != CallControlConnection.ALERTING)
        {
            throw (new InvalidStateException(pickupConn,
                                        InvalidStateException.CONNECTION_OBJECT,                                        pickupState));
        }

        boolean matchedAddress = false;
        Address[] addrs = this.getAddresses();
        if (addrs != null)
        {
            for (int i=0; i < addrs.length; i++)
            {
                if (terminalAddress == addrs[i])
                {
                    matchedAddress = true;
                }
            }
        }
        if (matchedAddress == false)
        {
            throw (new InvalidArgumentException());
        }

   	// pre conditions met
	// send the request to the server

        if (pickupConn.getAddress() == terminalAddress)
        {
            TerminalConnection tc = new VCCallControlTerminalConnection(this, pickupConn);
            ((VCCallControlConnection)pickupConn).setCallControlState
                                (CallControlConnection.ESTABLISHED);
            ((VCCallControlTerminalConnection)tc).setCallControlState
                                (CallControlTerminalConnection.TALKING);
	    ((VCCallControlTerminalConnection)pickTermConn).setCallControlState
				(CallControlTerminalConnection.DROPPED);
        }
        else
        {
            Call call = pickupConn.getCall();
            Connection conn = new VCCallControlConnection(terminalAddress, call, true);
            TerminalConnection tc = new VCCallControlTerminalConnection(this, conn);
            ((VCCallControlConnection)conn).setCallControlState
                                (CallControlConnection.ESTABLISHED);
            ((VCCallControlTerminalConnection)tc).setCallControlState
                                (CallControlTerminalConnection.TALKING);
            ((VCCallControlConnection)pickupConn).setCallControlState
                                (CallControlConnection.DISCONNECTED);
        }

 	// return tc;

        MethodNotSupportedException e = new MethodNotSupportedException();
        throw e;
    }


    // this terminal attempts to pick up a call from the address
    // pickupAddress.
    // The address associated with the connection may or may not be
    // the address associated with this terminal which requires to
    // pick up the call

    /**
     * Refer To JTapi CallControlTerminal Interface
     */
    public TerminalConnection pickup(Address pickupAddress,
	Address terminalAddress) throws InvalidArgumentException,
            InvalidStateException, MethodNotSupportedException,
            PrivilegeViolationException, ResourceUnavailableException
    {
        isProviderOK();
	Connection[] conns = pickupAddress.getConnections();
	if (conns == null)
	{
	    throw (new InvalidArgumentException());
	}
	Connection pickupConn = null;

 	// find the first alerting connection associated with the address
	for (int i=0; i < conns.length; i++)
	{
	    int state = ((CallControlConnection)conns[i]).getCallControlState();
	    if (state == CallControlConnection.ALERTING)
	    {
		pickupConn = conns[i];
		break;
	    }
	}
	if (pickupConn == null)
	{
	    // no alerting connection is found - find the first queued conn
	    for (int i=0; i < conns.length; i++)
	    {
		int state = ((CallControlConnection)conns[i]).getCallControlState();
		if (state == CallControlConnection.QUEUED)
		{
		    pickupConn = conns[i];
		    break;
		}
	    }
	}
	// no connection in ALERTING or QUEUED state found
	if (pickupConn == null)
	{
	    throw (new InvalidArgumentException());
	}

        if ((pickupConn.getCall()).getState() != Call.ACTIVE)
        {
            throw (new InvalidStateException(pickupConn.getCall(),
                                    InvalidStateException.CALL_OBJECT,
                                    (pickupConn.getCall()).getState()));
        }
        int pickupState = ((CallControlConnection)pickupConn).getCallControlState();
        if (pickupState != CallControlConnection.ALERTING)
        {
            throw (new InvalidStateException(pickupConn,
                                        InvalidStateException.CONNECTION_OBJECT,                                        pickupState));
        }

        boolean matchedAddress = false;
        Address[] addrs = this.getAddresses();
        if (addrs != null)
        {
            for (int i=0; i < addrs.length; i++)
            {
                if (terminalAddress == addrs[i])
                {
                    matchedAddress = true;
                }
            }
        }
        if (matchedAddress == false)
        {
            throw (new InvalidArgumentException());
        }

  	// pre conditions met
	// send a request to the server

        if (pickupConn.getAddress() == terminalAddress)
        {
            TerminalConnection tc = new VCCallControlTerminalConnection(this, pickupConn);
            ((VCCallControlConnection)pickupConn).setCallControlState
                                (CallControlConnection.ESTABLISHED);
            ((VCCallControlTerminalConnection)tc).setCallControlState
                                (CallControlTerminalConnection.TALKING);
        }
        else
        {
            Call call = pickupConn.getCall();
            Connection conn = new VCCallControlConnection(terminalAddress, call, true);
            TerminalConnection tc = new VCCallControlTerminalConnection(this, conn);
            ((VCCallControlConnection)conn).setCallControlState
                                (CallControlConnection.ESTABLISHED);
            ((VCCallControlTerminalConnection)tc).setCallControlState
                                (CallControlTerminalConnection.TALKING);
            ((VCCallControlConnection)pickupConn).setCallControlState
                                (CallControlConnection.DISCONNECTED);
        }

	// return tc;
        MethodNotSupportedException e = new MethodNotSupportedException();
        throw e;
    }

    /**
     * Refer To JTapi CallControlTerminal Interface
     */
    public TerminalConnection pickupFromGroup(String pickupGroup,
	Address terminalAddress) throws InvalidArgumentException,
	    InvalidStateException, MethodNotSupportedException,
            PrivilegeViolationException, ResourceUnavailableException
    {
        MethodNotSupportedException e = new MethodNotSupportedException();
        throw e;
    }

    /**
     * Refer To JTapi CallControlTerminal Interface
     */
    public TerminalConnection pickupFromGroup(Address terminalAddress)
	throws InvalidArgumentException, InvalidStateException,
	    MethodNotSupportedException, PrivilegeViolationException,
	    ResourceUnavailableException
    {
	MethodNotSupportedException e = new MethodNotSupportedException();
        throw e;
    }

/**************************************************************/
/* Additional Functionality                                   */
/**************************************************************/


}

    
