
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const JSNotification_cxx_Version =
    "$Id$";
#include "cpLog.h"
#include "JSNotification.hxx"


///
JSNotification::JSNotification()
{}

///

JSNotification::JSNotification(const Sptr < UdpStack > udpStack,
                               const NetworkAddress& dest)
        : itsUdpStack(udpStack),
        itsUdpDest(dest)
{
    assert( itsUdpStack != 0 );
    //    assert( itsUdpDest != 0 );
    itsUdpMessage = "";
}

///
JSNotification::JSNotification(const JSNotification& rhs)
{
    if (this != &rhs )
        copyRhsToThis ( rhs );
}

///
void
JSNotification::copyRhsToThis ( const JSNotification& rhs )
{}

///

JSNotification::~JSNotification()
{}

///

void
JSNotification::setType( const string& type )
{
    itsUdpMessage += addPair( "type", type );
}

///
void
JSNotification::setId( const string& id )
{
    itsUdpMessage += addPair( "id", id );
}

///
void
JSNotification::setCall( const string& callId )
{
    itsUdpMessage += addPair( "call", callId );
}

///
void
JSNotification::setAddr( const string& addrName )
{
    itsUdpMessage += addPair( "addr", addrName );
}

///
void
JSNotification::setResult( const string& value )
{
    itsUdpMessage += addPair( "value", value );
}

///
void
JSNotification::send()
{
    //    assert(itsUdpMessage != 0);
    assert(itsUdpStack != 0);
    //    assert(itsUdpDest != 0);
    itsUdpMessage = addJtapi(itsUdpMessage);
    NetworkAddress* netAddressPtr = 0;
    netAddressPtr = &itsUdpDest;

    itsUdpStack->transmitTo( itsUdpMessage.c_str(), itsUdpMessage.length(),
                             netAddressPtr);
    itsUdpMessage = "";

}


// private

///
string
JSNotification::addPair( const string& name, const string& value )
{
    string str;
    str = "<" + name + ">" + value + "</" + name + ">";
    return str;
}

///
string
JSNotification::addJtapi(const string& str)
{
    string jtapiStr;
    jtapiStr = "<jtapi>" + str + "</jtapi>";
    return jtapiStr;
}
// End of File
