
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const JSOpCallCreated_cxx_Version =
    "$Id: JSOpCallCreated.cxx,v 1.19 2002/02/25 05:51:16 sprajpat Exp $";
#include "JSOpCallCreated.hxx"
#include "SipEvent.hxx"
#include "JSStateMachine.hxx"
#include "JSCall.hxx"
#include "InviteMsg.hxx"
#include "JSSipCallContainer.hxx"
#include "JSProvider.hxx"
#include "SipVia.hxx"
#include "SipProxyAuthorization.hxx"
#include "UdpStack.hxx"
#include "Sptr.hxx"
#include "JSCommon.hxx"
#include "JSNotification.hxx"


///
JSOpCallCreated::JSOpCallCreated()
{}


///

JSOpCallCreated::~JSOpCallCreated()
{}


///

const char* const
JSOpCallCreated::name() const
{
    return "JSOpCallCreated";
}


///
const Sptr < State >
JSOpCallCreated::process( const Sptr < SipProxyEvent > event )
{
    // 1. Receive CALLCREATERESP from client (need a way to identify)
    // 2. Set the CallId for the call with corresponding SIP
    // 3. Create new INVITE message and send it to RS
    // 4. Check if this is a controlled call
    // 5. Yes, change state to "in progress"
    // 6. No, change state to "called"

    cpLog( LOG_DEBUG, "JSOpCallCreated operation" );

    Sptr < JSReqEvent > reqEvent;
    reqEvent.dynamicCast( event );

    if ( reqEvent == 0 )
    {
        return 0;
    }

    Sptr < JSCall > call;
    call.dynamicCast( event->getCallInfo() );
    assert( call != 0 );
    JSProvider* provider = call->getProvider();
    JSRequest* request = call->getRequest();

    if ( request->getMsgType() != 250 /* call create resp */)
    {
        // Only handle call create resp
        return 0;
    }

    call->setRecvClientCallID( request->getClientCallID() );

    NetworkAddress netAddr;
    if ( call->getControlled() )
        netAddr = call->getSender();
    else
        netAddr = call->getReceiver();

    string ipName = netAddr.getIpName ();
    string callID = ipName + request->getClientCallID();

    call->setCallID( callID );
    LocalScopeAllocator lo;
    string fromStr( call->getFromUser().getData(lo) );
    call->setCallerAddr( fromStr );
    string toStr( call->getToUser().getData(lo) );
    call->setCalleeAddr( toStr );

    Sptr < UdpStack > clientUdp = provider->getClientUdpStack();
    JSNotification notification(clientUdp, netAddr);
    if (netAddr.getPort() == -1)
    {
        cpLog( LOG_DEBUG, "JSOpCallCreated : no notification sent" );
    }

    else
    {
        notification.setType(CONNCREATED);
        notification.setCall(call->getRecvClientCallID());
        notification.setAddr( (call->getFromUser()).getData(lo) );
        notification.setAddr( (call->getToUser()).getData(lo) );
        notification.send();
        cpLog(LOG_DEBUG, "Send CONNCREATED notif to client");
        cpLog(LOG_DEBUG, "%s", notification.getNotificationString().c_str());
    }


    // Create new invite message and sent it to RS
    // 1. Get the "to" url and SDP from received invite.
    // 2. Create new invite.
    // 3. Change requestline to RS.
    // 4. Change SDP.

    Sptr < InviteMsg > invite = call->getFromInvite();

    Sptr <BaseUrl> toBaseUrl = invite->getTo().getUrl();
    Sptr <BaseUrl> fromBaseUrl = invite->getFrom().getUrl();
    assert( toBaseUrl.getPtr() != 0 );
    assert( fromBaseUrl.getPtr() != 0 );
    assert( toBaseUrl->getType() == SIP_URL );
    assert( fromBaseUrl->getType() == SIP_URL );

    // Get remote SDP from message
    Sptr<SipSdp> sdp;
    sdp.dynamicCast ( invite->getContentData(0) );

    if(sdp != 0)
    {
	call->setFromSdp( new SipSdp( *sdp ) );
    }

    int port;
    if ( call->getControlled() )
        port = provider->getConfig()->getJSNSipPort().convertInt();
    else
        port = provider->getConfig()->getJSDSipPort().convertInt();

    InviteMsg newInvite( toBaseUrl, port, provider->getConfig()->getRtpPort() );

    // Set from
    SipFrom sipFrom( fromBaseUrl );
    newInvite.setFrom( sipFrom );

    /// Set SDP
    newInvite.flushContentData();
    // newInvite.flushContentType();
//    newInvite.setContentData((SipContentData*) sdp);
    newInvite.setContentData(&(*sdp));

    // Change Via
    newInvite.flushViaList();
    SipVia sipVia;
    sipVia.setprotoVersion( "2.0" );
    sipVia.setHost( Data( theSystem.gethostAddress() ) );
    if ( call->getControlled() )
        sipVia.setPort( provider->getConfig()->getJSNSipPort().convertInt() );
    else
        sipVia.setPort( provider->getConfig()->getJSDSipPort().convertInt() );
    sipVia.setTransport( Data("UDP") );
    if ( call->getControlled() )
        sipVia.setBranch( REDIRECT_JSN );
    else
        sipVia.setBranch( REDIRECT_JSD );
    newInvite.setVia( sipVia );

    // Add authorization
    //newInvite.setAuthBasic( provider->getConfig()->getProxyAuthKey() );
    newInvite.setProxyAuth( provider->getConfig()->getProxyAuthKey() );

    // SIP call id of both "from" and "to" legs to sipmap
    SipCallId newInviteId;
    newInviteId = newInvite.getCallId();

    // Save call ids to call object
    call->setToCallID( newInviteId );
    cpLog(LOG_DEBUG, "Call Id of to   leg: %s", newInviteId.getData().logData());

    Sptr < JSSipCallContainer > sipMap = provider->getSipCallContainer();
    sipMap->addCall( newInviteId, call );

    /// Change request line to RS
    SipRequestLine rsRequestLine = newInvite.getRequestLine();
    Sptr <BaseUrl> rsBaseUrl = rsRequestLine.getUrl();
    assert( rsBaseUrl.getPtr() != 0 );
    assert( rsBaseUrl->getType() == SIP_URL);
    Sptr <SipUrl> rsUrl;
    assert( rsUrl.dynamicCast( rsBaseUrl ) != 0 );

    rsUrl->setHost( provider->getConfig()->getRedirectHost() );
    rsUrl->setPort( provider->getConfig()->getRedirectPort() );
    rsUrl->setUserParam( "phone" );
    rsRequestLine.setUrl( rsUrl ); 
    newInvite.setRequestLine( rsRequestLine );
    // Add contact
    Sptr <SipUrl> myUrl = new SipUrl();
    myUrl->setUserValue( provider->getConfig()->getUserName() );
    myUrl->setHost( Data( theSystem.gethostAddress() ) );
    if ( call->getControlled() )
        myUrl->setPort( provider->getConfig()->getJSNSipPort() );
    else
        myUrl->setPort( provider->getConfig()->getJSDSipPort() );

    SipContact me;
    me.setUrl( myUrl );
    newInvite.setNumContact( 0 );    // Clear
    newInvite.setContact( me );

    call->setToInvite( new InviteMsg( newInvite ) );
    call->setRSInvite( new InviteMsg( newInvite ) );

    Sptr < SipTransceiverFilter > sipStack = call->getProvider()->getJSDSipStack();
    cpLog(LOG_DEBUG, "JSOpCallCreate: Invite sent:");
    cpLog(LOG_DEBUG, "***************************************");
    cpLog(LOG_DEBUG, newInvite.encode().logData());
    cpLog(LOG_DEBUG, "***************************************");

    sipStack->sendAsync( newInvite );

    Sptr < JSStateMachine > stateMachine;
    stateMachine.dynamicCast( event->getCallInfo()->getFeature() );
    assert( stateMachine != 0 );

    if ( call->getControlled() )
        return stateMachine->findState( "JSStateInProgress" );
    else
        return stateMachine->findState( "JSStateCalled" );
}
