
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const JSOpCalleeAlerted_cxx_Version =
    "$Id$";
#include "JSOpCalleeAlerted.hxx"
#include "SipEvent.hxx"
#include "SipVia.hxx"
#include "InviteMsg.hxx"
#include "StatusMsg.hxx"
#include "JSCall.hxx"
#include "UdpStack.hxx"
#include "Sptr.hxx"
#include "JSCommon.hxx"
#include "JSNotification.hxx"



///
JSOpCalleeAlerted::JSOpCalleeAlerted()
{}


///

JSOpCalleeAlerted::~JSOpCalleeAlerted()
{}


///

const char* const
JSOpCalleeAlerted::name() const
{
    return "JSOpCalleeAlerted";
}


///
const Sptr < State >
JSOpCalleeAlerted::process( const Sptr < SipProxyEvent > event )
{
    // 1. Receve 100 or 180
    // 2. For 100, drop it
    // 3. For 180, check received as JSN or JSD
    // 4. If as JSD, do nothing, just pass on the message
    // 5. If as JSN, create a new 180 for fromINVITE and send it back
    //		to the caller

    cpLog( LOG_DEBUG, "JSOpCalleeAlerted operation" );

    Sptr < SipEvent > sipEvent;
    sipEvent.dynamicCast( event );
    if ( sipEvent == 0 )
    {
        return 0;
    }
    Sptr < SipMsg > sipMsg = sipEvent->getSipMsg();
    assert( sipMsg != 0 );

    Sptr < StatusMsg > msg;
    msg.dynamicCast( sipMsg );
    if ( msg == 0 )
    {
        return 0;
    }

    if ( (msg->getStatusLine().getStatusCode() != 100) &&
            (msg->getStatusLine().getStatusCode() != 180) &&
            (msg->getStatusLine().getStatusCode() != 183) )
    {
        return 0;
    }

    // Get the port from the top via and compare it with JSN, JSD ports
    int viaPort = sipMsg->getVia(0).getPort();

    Sptr < JSCall > call;
    call.dynamicCast(event->getCallInfo());
    assert( call != 0 );

    JSProvider* provider = call->getProvider();

    // need the udpStack, dest Network Address and request
    // to send a notification to client
    Sptr < UdpStack > clientUdp = provider->getClientUdpStack();

    JSRequest* req = call->getRequest();
    NetworkAddress dest;
    string callId;
    if ( viaPort == provider->getConfig()->getJSDSipPort().convertInt() )
    {
        dest = call->getReceiver();
        callId = req->getRecvClientCallID();
        if (dest.getPort() == -1)
        {
            cpLog( LOG_ERR, "JSOpCalleeAlerted port not set - Callee NJS??");
        }
    }
    else if ( viaPort == provider->getConfig()->getJSNSipPort().convertInt() )
    {
        dest = call->getSender();
        callId = req->getClientCallID();
    }
    if (dest.getPort() == -1)
    {
        cpLog( LOG_ERR, "JSOpCalleeAlerted port not set - no notifications" );
    }
    else
    {
        JSNotification notification(clientUdp, dest);

        if ( msg->getStatusLine().getStatusCode() == 100 )
        {
            // send a conninprogress for Callee
            notification.setType(CONNINPROGRESS);
            notification.setCall(callId);
            notification.setAddr(req->getCalleeAddr());
            notification.send();
            return 0;
        }

        if ( msg->getStatusLine().getStatusCode() == 180 )
        {
            // send a connalerting for callee
            notification.setType(CONNALERTING);
            notification.setCall(callId);
            notification.setAddr(req->getCalleeAddr());
            notification.send();

            // send a termconnring for callee
            notification.setType(TERMCONNRINGING);
            notification.setCall(callId);
            notification.setAddr(req->getCalleeAddr());
            notification.send();
        }
    }

    if ( viaPort == provider->getConfig()->getJSDSipPort().convertInt() )	// JSD
    {
        cpLog( LOG_DEBUG, "JSOpCalleeAlerted: receive 180 or 183 as JSD" );
        cpLog( LOG_DEBUG, "JSOpCalleeAlerted: Forward to previous hop" );
        // Forward the message to previous hop
        msg->removeVia( 0 );
        cpLog(LOG_DEBUG, "JSOpCalleeAlerted: 180 forward to previous hop");
        cpLog(LOG_DEBUG, "***************************************");
        cpLog(LOG_DEBUG, "%s", msg->encode().logData());
        cpLog(LOG_DEBUG, "***************************************");

        sipEvent->getSipStack()->sendReply( *msg );
    }
    else if ( viaPort == provider->getConfig()->getJSNSipPort().convertInt() ) // JSN
    {
        // Create a new 180 and send it back to caller
        cpLog( LOG_DEBUG, "JSOpCalleeAlerted: receive 180 or 183 as JSN" );
        cpLog( LOG_DEBUG, "JSOpCalleeAlerted: send new 180 or 183 to caller" );
        Sptr < InviteMsg > invite = call->getFromInvite();
        StatusMsg status( *invite, 180/*ringing*/ );

        // 180 received may have SDP, then need to add it to the new 180
        Sptr<SipSdp> sdp;
	sdp.dynamicCast ( sipMsg->getContentData(0) );

        if ( sdp != 0 )
        {
            // Add the SDP to the new 180
            status.flushContentData();
            // status.flushContentType();
            status.setContentData(&(*sdp));
        }
        cpLog(LOG_DEBUG, "JSOpCalleeAlerted: new 180 sent to caller");
        cpLog(LOG_DEBUG, "***************************************");
        cpLog(LOG_DEBUG, "%s", status.encode().logData());
        cpLog(LOG_DEBUG, "***************************************");
        sipEvent->getSipStack()->sendReply( status );
    }
    return 0;
}
