
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const JSOpCalling_cxx_Version =
    "$Id$";
#include "JSOpCalling.hxx"
#include "SipEvent.hxx"
#include "InviteMsg.hxx"
#include "JSStateMachine.hxx"
#include "JSCall.hxx"
#include "SipFrom.hxx"
#include "UdpStack.hxx"
#include "Sptr.hxx"
#include "JSCommon.hxx"
#include "JSNotification.hxx"



////
JSOpCalling::JSOpCalling()
{}


////

JSOpCalling::~JSOpCalling()
{}


////

const char* const
JSOpCalling::name() const
{
    return "JSOpCalling";
}


////
const Sptr < State >
JSOpCalling::process( const Sptr < SipProxyEvent > event )
{
    // 1. Receive INVITE of "from" leg from caller on JSN
    // 2. Create new INVITE  for "to" leg
    // 3. Send new INVITE to RS as JSN
    // 4. Change state to "InProgress"

    cpLog( LOG_DEBUG, "JSOpCalling operation" );
    Sptr < SipEvent > sipEvent;
    sipEvent.dynamicCast( event );
    if ( sipEvent == 0 )
    {
        // Not SIP message
        return 0;
    }

    Sptr < SipMsg > sipMsg = sipEvent->getSipMsg();
    if (sipMsg == NULL)
    {
        assert(0);
    }

    Sptr < InviteMsg > fromInvite;
    fromInvite.dynamicCast(sipMsg);
    if ( fromInvite == 0 )
    {
        return 0;
    }

    Sptr < JSCall > call;
    call.dynamicCast( event->getCallInfo() );
    assert( call != 0 );

    JSProvider* provider = call->getProvider();

    // Check if received on JSN or JSD
    Sptr <BaseUrl> baseUrl;
    baseUrl = fromInvite->getRequestLine().getUrl();
    assert( baseUrl.getPtr() != 0 );
    assert( baseUrl->getType() == SIP_URL ); // JTAPI doesn't support TEL_URL

    Sptr <SipUrl> sipUrl;
    assert( sipUrl.dynamicCast( baseUrl ) != 0 );

    Data viaPort = sipUrl->getPort();
    if ( viaPort.length() == 0 )
    {
        // Default SIP port
        viaPort = 5060;
    }

    if ( (viaPort != provider->getConfig()->getJSDSipPort()) &&
            (viaPort != provider->getConfig()->getJSNSipPort()) )
    {
        cpLog(LOG_ERR, "JSOpCalled: INVITE received on non-configured port %s:", viaPort.logData());
        Sptr < JSStateMachine > stateMachine;
        stateMachine.dynamicCast( event->getCallInfo()->getFeature() );
        assert( stateMachine != 0 );

        return stateMachine->findState( "JSStateUnknown" );
    }

    if ( viaPort == provider->getConfig()->getJSDSipPort() ) // JSD
    {
        // We shouldn't receive INVITE on JSD is this state
        // Terminate the call
        Sptr < JSStateMachine > stateMachine;
        stateMachine.dynamicCast( event->getCallInfo()->getFeature() );
        assert( stateMachine != 0 );

        return stateMachine->findState( "JSStateUnknown" );
    }

    // Add INVITE to call object
    call->setFromInvite( new InviteMsg( *fromInvite ) );

    Sptr <BaseUrl> toBaseUrl;
    toBaseUrl = fromInvite->getTo().getUrl();
    assert( toBaseUrl.getPtr() != 0 );
    assert( toBaseUrl->getType() == SIP_URL );

    InviteMsg toInvite( toBaseUrl, provider->getConfig()->getJSNSipPort().convertInt() );

    // Add SIP call id of both "from" and "to" legs to sipmap
    SipCallId fromId, toId;
    fromId = fromInvite->getCallId();
    toId = toInvite.getCallId();
    // Add both call ids to the call object
    call->setFromCallID( fromId );
    call->setToCallID( toId );

    Sptr < JSSipCallContainer > sipMap = provider->getSipCallContainer();
    sipMap->addCall( fromId, call );
    sipMap->addCall( toId, call );

    // Change "from" header to the caller, so he'll pay the call
    SipFrom from = fromInvite->getFrom();
    toInvite.setFrom( from );

    // Change toInvite's requestline for RS
    SipRequestLine rsRequestLine = toInvite.getRequestLine();
    Sptr <BaseUrl> rsBaseUrl = rsRequestLine.getUrl();
    assert( rsBaseUrl.getPtr() != 0 );
    assert( rsBaseUrl->getType() == SIP_URL );

    Sptr <SipUrl> rsUrl;
    assert( rsUrl.dynamicCast( rsBaseUrl ) != 0 );

    rsUrl->setHost( provider->getConfig()->getRedirectHost() );
    rsUrl->setPort( provider->getConfig()->getRedirectPort() );
    rsUrl->setUserParam( "phone" );
    rsRequestLine.setUrl( rsUrl );
    toInvite.setRequestLine( rsRequestLine );

    // Add contact
    Sptr <SipUrl> myUrl = new SipUrl;
    myUrl->setUserValue( provider->getConfig()->getUserName() );
    myUrl->setHost( Data( theSystem.gethostAddress() ) );
    myUrl->setPort( provider->getConfig()->getJSNSipPort() );

    SipContact me;
    me.setUrl( myUrl );
    toInvite.setNumContact( 0 );    // Clear
    toInvite.setContact( me );

    // Add authorization
    //toInvite.setAuthBasic( provider->getConfig()->getProxyAuthKey() );
    toInvite.setProxyAuth( provider->getConfig()->getProxyAuthKey() );

    // Change Via
    toInvite.flushViaList();
    SipVia sipVia;
    sipVia.setprotoVersion( "2.0" );
    sipVia.setHost( Data( theSystem.gethostAddress() ) );
    sipVia.setPort( provider->getConfig()->getJSNSipPort().convertInt() );
    sipVia.setTransport( Data("UDP") );
    sipVia.setBranch( REDIRECT_JSN );
    toInvite.setVia( sipVia );

    // Add SDP from the "from" invite to this invite
    // Get remote SDP from message
    Sptr<SipSdp> sdp;
    sdp.dynamicCast ( sipMsg->getContentData(0) );

    if (sdp == 0)
    {
        cpLog( LOG_ERR, "No remote SDP or invalid SDP in 200" );
        StatusMsg errorStatusMsg ( *fromInvite, 404 );  // 404 Not Found
        errorStatusMsg.removeVia( 0 );
        sipEvent->getSipStack()->sendReply( errorStatusMsg );
        //  Change state to unknow to terminate this call
        Sptr < JSStateMachine > stateMachine;
        stateMachine.dynamicCast( event->getCallInfo()->getFeature() );
        assert( stateMachine != 0 );

        return stateMachine->findState( "JSStateUnknown" );
    }

    toInvite.flushContentData();
    // toInvite.flushContentType();
    toInvite.setContentData( &(*sdp));

    call->setToInvite( new InviteMsg( toInvite ) );
    call->setRSInvite( new InviteMsg( toInvite ) );
    // Since we still JSN here, use the sip stack in the event
    cpLog(LOG_DEBUG, "JSOpCalling: Send new INVITE to callee");
    cpLog(LOG_DEBUG, "***************************************");
    cpLog(LOG_DEBUG, "%s", toInvite.encode().logData());
    cpLog(LOG_DEBUG, "***************************************");
    sipEvent->getSipStack()->sendAsync( toInvite );

    // send a conncreated event to the client
    // need udpStack, dest NetworkAddress and request
    // to send the notification
    Sptr < UdpStack > clientUdp = provider->getClientUdpStack();
    NetworkAddress dest = call->getSender();
    JSRequest* req = call->getRequest();
    JSNotification notification(clientUdp, dest);

    notification.setType(CONNCREATED);
    notification.setCall(req->getClientCallID());
    notification.setAddr(req->getCallerAddr());
    notification.setAddr(req->getCalleeAddr());
    notification.send();

    // send a conninprogress for Caller

    notification.setType(CONNINPROGRESS);
    notification.setCall(req->getClientCallID());
    notification.setAddr(req->getCallerAddr());
    notification.send();

    // Change state to In Progress
    Sptr < JSStateMachine > stateMachine;
    stateMachine.dynamicCast( event->getCallInfo()->getFeature() );
    assert( stateMachine != 0 );

    return stateMachine->findState( "JSStateInProgress" );
}
