
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const JSOpDisconnectOk_cxx_Version =
    "$Id$";
#include "JSOpDisconnectOk.hxx"
#include "SipEvent.hxx"
#include "AckMsg.hxx"
#include "SipVia.hxx"
#include "JSCall.hxx"
#include "SipRequestLine.hxx"
#include "SipUrl.hxx"
#include "StatusMsg.hxx"
#include "JSStateUnknown.hxx"
#include "UdpStack.hxx"
#include "Sptr.hxx"
#include "JSCommon.hxx"
#include "JSNotification.hxx"


///
JSOpDisconnectOk::JSOpDisconnectOk()
{}


///

JSOpDisconnectOk::~JSOpDisconnectOk()
{}


///

const char* const
JSOpDisconnectOk::name() const
{
    return "JSOpDisconnectOk";
}


///
const Sptr < State >
JSOpDisconnectOk::process( const Sptr < SipProxyEvent > event )
{
    // 1. Receive 200 for BYE
    // 2. Change state to unknown to drop the call object
    //

    cpLog( LOG_DEBUG, "JSOpDisconnectOk operation" );

    Sptr < SipEvent > sipEvent;
    sipEvent.dynamicCast( event );
    if ( sipEvent == 0 )
    {
        // Not SIP message
        return 0;
    }

    Sptr < SipMsg > sipMsg = sipEvent->getSipMsg();
    assert( sipMsg != 0);

    Sptr < StatusMsg > status;
    status.dynamicCast(sipMsg);
    if ( status == 0 )
    {
        return 0;
    }

    Data method = sipMsg->getCSeq().getMethod();

    if ( method != "BYE" )
    {
        // Only handle 200 for BYE here
        return 0;
    }

    if ( status->getStatusLine().getStatusCode() != 200 )
    {
        return 0;
    }

    Sptr < JSCall > call;
    call.dynamicCast( event->getCallInfo() );
    JSProvider* provider = call->getProvider();

    // Send notification to the answering vocalpad
    Sptr < UdpStack > recvClientUdp = provider->getClientUdpStack();
    NetworkAddress recvDest = call->getReceiver();
    JSNotification recvNotification(recvClientUdp, recvDest);

    if (recvDest.getPort() == -1)
    {
        cpLog( LOG_DEBUG, "JSOpDisconnectOk : no notification sent" );
    }
    else
    {
        recvNotification.setType(CONNDISCONNECTED);
        recvNotification.setCall(call->getRecvClientCallID());
        recvNotification.setAddr(call->getCalleeAddr());
        recvNotification.send();

        recvNotification.setType(CONNDISCONNECTED);
        recvNotification.setCall(call->getRecvClientCallID());
        recvNotification.setAddr(call->getCallerAddr());
        recvNotification.send();
    }

    Sptr < UdpStack > clientUdp = provider->getClientUdpStack();
    JSRequest* req = call->getRequest();
    NetworkAddress dest = call->getSender();
    JSNotification notification(clientUdp, dest);

    if (dest.getPort() == -1)
    {
        cpLog( LOG_DEBUG, "JSOpDisconnectOk : no notification sent" );
    }
    else
    {
        notification.setType(CONNDISCONNECTED);
        notification.setCall(call->getSendClientCallID());
        notification.setAddr(call->getCalleeAddr());
        notification.send();

        notification.setType(CONNDISCONNECTED);
        notification.setCall(call->getSendClientCallID());
        notification.setAddr(call->getCallerAddr());
        notification.send();
    }

    if (req->getMsgType() == TYPEDISCONNECT)
    {
        dest = req->getSender();
        JSNotification reqNotification(clientUdp, dest);
        if (dest.getPort() == -1)
        {
            cpLog( LOG_DEBUG, "JSOpDisconnectOk : no Disconnect Resp sent" );
        }
        else
        {
            reqNotification.setType(DISCONNECTRESP);
            reqNotification.setId(req->getMsgID());
            reqNotification.setCall(req->getClientCallID());
            reqNotification.setAddr(call->getCalleeAddr());
            reqNotification.setResult(JS_SUCCESS);
            reqNotification.send();
        }
    }



    // Change state to unknown
    Sptr < JSStateMachine > stateMachine;
    stateMachine.dynamicCast( event->getCallInfo()->getFeature() );
    assert( stateMachine != 0 );

    return stateMachine->findState( "JSStateUnknown" );
}
