
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const JSOpJSDAckFromCaller_cxx_Version =
    "$Id$";
#include "JSOpJSDAckFromCaller.hxx"
#include "SipEvent.hxx"
#include "AckMsg.hxx"
#include "SipVia.hxx"
#include "JSCall.hxx"
#include "UdpStack.hxx"
#include "Sptr.hxx"
#include "JSCommon.hxx"
#include "JSNotification.hxx"
#include "SipProxyAuthorization.hxx"



///
JSOpJSDAckFromCaller::JSOpJSDAckFromCaller()
{}


///

JSOpJSDAckFromCaller::~JSOpJSDAckFromCaller()
{}


///

const char* const
JSOpJSDAckFromCaller::name() const
{
    return "JSOpJSDAckFromCaller";
}


///
const Sptr < State >
JSOpJSDAckFromCaller::process( const Sptr < SipProxyEvent > event )
{
    // 1. Receive ACK from caller
    // 2. Create ACK for toINVITE and send it to callee
    // 3. Change state to "Connected"

    cpLog( LOG_DEBUG, "JSOpJSDAckFromCaller operation" );

    Sptr < SipEvent > sipEvent;
    sipEvent.dynamicCast( event );
    if ( sipEvent == 0 )
    {
        // Not SIP message
        return 0;
    }

    Sptr < SipMsg > sipMsg = sipEvent->getSipMsg();
    assert( sipMsg != 0 );

    if ( SIP_ACK != sipMsg->getType() )
        return 0;

    Sptr < AckMsg > message;
    message.dynamicCast(sipMsg);
    assert( message != 0 );

    Sptr < JSCall > call;
    call.dynamicCast(event->getCallInfo());
    assert( call != 0 );

    JSProvider* provider = call->getProvider();

    // added notification stuff

    Sptr < UdpStack > clientUdp = provider->getClientUdpStack();
    NetworkAddress dest = call->getReceiver();
    JSRequest* req = call->getRequest();
    JSNotification notification(clientUdp, dest);

    if (dest.getPort() == -1)
    {
        cpLog( LOG_DEBUG, "JSOpJSDAckFromCaller : no notification sent" );
    }
    else
    {
        // send a termconnactive on behalf of both connections
        // send a termconnactive for caller
        notification.setType(CONNCONNECTED);
        notification.setCall(req->getClientCallID());
        notification.setAddr(req->getCallerAddr());
        notification.send();

        // send a termconnactive for callee
        notification.setType(CONNCONNECTED);
        notification.setCall(req->getClientCallID());
        notification.setAddr(req->getCalleeAddr());
        notification.send();
    }

    AckMsg newAck( call->getTo200() );

    //SipProxyAuthorization authorization("VovidaClassXswitch");
    //newAck.setAuthBasic( provider->getConfig()->getProxyAuthKey() );
    newAck.setProxyAuth( provider->getConfig()->getProxyAuthKey() );

    cpLog(LOG_DEBUG, "JSOpJSDAckFromCaller: Send new ACK to callee");
    cpLog(LOG_DEBUG, "***************************************");
    cpLog(LOG_DEBUG, "%s", newAck.encode().logData());
    cpLog(LOG_DEBUG, "***************************************");
    sipEvent->getSipStack()->sendAsync( newAck );

    // Change state to Connected
    Sptr < JSStateMachine > stateMachine;
    stateMachine.dynamicCast( event->getCallInfo()->getFeature() );
    assert( stateMachine != 0 );

    return stateMachine->findState( "JSStateConnected" );
}
