
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const JSOpJSDFwdInvite_cxx_Version =
    "$Id$";
#include "JSOpJSDFwdInvite.hxx"
#include "SipEvent.hxx"
#include "InviteMsg.hxx"
#include "JSCall.hxx"
#include "JSProvider.hxx"
#include "SipRequestLine.hxx"
#include "SipUrl.hxx"
#include "SipVia.hxx"
#include "Operator.hxx"
#include "UdpStack.hxx"
#include "Sptr.hxx"
#include "JSCommon.hxx"
#include "JSNotification.hxx"



///
JSOpJSDFwdInvite::JSOpJSDFwdInvite()
{}


///

JSOpJSDFwdInvite::~JSOpJSDFwdInvite()
{}


///

const char* const
JSOpJSDFwdInvite::name() const
{
    return "JSOpJSDFwdInvite";
}


///
const Sptr < State >
JSOpJSDFwdInvite::process( const Sptr < SipProxyEvent > event )
{
    // 1. Receive INVITE3 as JSD
    // 2. Do nothing, send INVITE3 to RS for next hop

    cpLog( LOG_DEBUG, "JSOpJSDFwdInvite operation" );

    Sptr < SipEvent > sipEvent;
    sipEvent.dynamicCast( event );
    if ( sipEvent == 0 )
    {
        // Not SIP message
        return 0;
    }

    Sptr < SipMsg > sipMsg = sipEvent->getSipMsg();
    if (sipMsg == NULL)
    {
        assert(0);
    }

    if ( SIP_INVITE != sipMsg->getType() )
        return 0;

    Sptr < InviteMsg > invite;
    invite.dynamicCast(sipMsg);
    assert( invite != 0 );

    Sptr < JSCall > call;
    call.dynamicCast( event->getCallInfo() );
    if ( call == 0 )
    {
        cpLog(LOG_ERR, "Can't get call object from event received");
        assert(0);
    }

    JSProvider* provider = call->getProvider();

    // Check if received on JSN or JSD
    Sptr <BaseUrl> baseUrl;
    baseUrl = invite->getRequestLine().getUrl();
    assert( baseUrl.getPtr() != 0 );
    assert( baseUrl->getType() == SIP_URL ); // JTAPI doesn't support TEL_URL

    Sptr <SipUrl> sipUrl;
    assert( sipUrl.dynamicCast( baseUrl ) != 0 );

    Data viaPort = sipUrl->getPort();
    if ( viaPort.length() == 0 )
    {
        // Default SIP port 5060
        viaPort = 5060;
    }

    if ( (viaPort != provider->getConfig()->getJSDSipPort()) &&
            (viaPort != provider->getConfig()->getJSNSipPort()) )
    {
        cpLog(LOG_ERR, "JSOpCalled: INVITE received on non-configured port %s:", viaPort.logData());
        Sptr < JSStateMachine > stateMachine;
        stateMachine.dynamicCast( event->getCallInfo()->getFeature() );
        assert( stateMachine != 0 );

        return stateMachine->findState( "JSStateUnknown" );
    }

    if ( viaPort == provider->getConfig()->getJSNSipPort() ) // JSN
    {
        // We shouldn't receive INVITE on JSN in this state
        // Terminate the call
        Sptr < JSStateMachine > stateMachine;
        stateMachine.dynamicCast( event->getCallInfo()->getFeature() );
        assert( stateMachine != 0 );

        return stateMachine->findState( "JSStateUnknown" );
    }

    // need to inform callee's app of call incoming
    Sptr < JSUserRegContainer > addrMap;
    addrMap.dynamicCast( provider->getUserRegContainer() );
    assert( addrMap != 0 );

    JSRequest* request = call->getRequest();
    Data toUser(request->getCalleeAddr());
    NetworkAddress* netAddr_ptr = addrMap->findAddr( toUser );
    Sptr < UdpStack > clientUdp = provider->getClientUdpStack();
    if (netAddr_ptr == 0)
    {
        // We can't notify callee's vocalpad, but the call goes on
        cpLog(LOG_WARNING, "Port not set - user must not be registered");
    }
    else
    {
        JSNotification notification(clientUdp, *netAddr_ptr);
        notification.setType(CALLCREATE);
        notification.setId("9999");
        notification.send();

        // set the callid of the callee app's call
        request->setRecvClientCallID("99999");
        call->setReceiver( *netAddr_ptr );
        call->setRecvClientCallID( "99999" );

        // send the conncreated notification to callee client
        notification.setType(CONNCREATED);
        notification.setCall("99999");
        notification.setAddr( request->getCallerAddr() );
        notification.setAddr( request->getCalleeAddr() );
        notification.send();
    }

    // Add via
    SipVia sipVia;
    sipVia.setprotoVersion( "2.0" );
    sipVia.setHost( Data( theSystem.gethostAddress() ) );
    sipVia.setPort( provider->getConfig()->getJSDSipPort().convertInt() );
    sipVia.setTransport( Data("UDP") );
    sipVia.setBranch( REDIRECT_JSD );
    invite->setVia( sipVia, 0 );

    // Set Record Route
    cpLog( LOG_DEBUG, "adding RecordRoute" );
    SipRecordRoute route;

    OpAddRecordRoute( route, *sipUrl,
                      provider->getConfig()->getJSDSipPort().convertInt() );
    invite->setRecordRoute( route, 0 );

    // Set request line
    SipRequestLine rsRequestLine = invite->getRequestLine();

    sipUrl->setHost( provider->getConfig()->getRedirectHost() );
    sipUrl->setPort( provider->getConfig()->getRedirectPort() );
    sipUrl->setUserParam( "phone" );
    rsRequestLine.setUrl( sipUrl );
    invite->setRequestLine( rsRequestLine );

    call->setRSInvite( invite );
    cpLog(LOG_DEBUG, "JSOpJSDFwdInvite: Forward INVITE3 to RS");
    cpLog(LOG_DEBUG, "***************************************");
    cpLog(LOG_DEBUG, invite->encode().logData());
    cpLog(LOG_DEBUG, "***************************************");

    sipEvent->getSipStack()->sendAsync( *invite );

    return 0;
}
