
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const JSOpUserAlerted_cxx_Version =
    "$Id$";
#include "JSOpUserAlerted.hxx"
#include "SipEvent.hxx"
#include "SipVia.hxx"
#include "StatusMsg.hxx"
#include "JSCall.hxx"
#include "UdpStack.hxx"
#include "Sptr.hxx"
#include "JSCommon.hxx"
#include "JSNotification.hxx"



///
JSOpUserAlerted::JSOpUserAlerted()
{}


///

JSOpUserAlerted::~JSOpUserAlerted()
{}


///

const char* const
JSOpUserAlerted::name() const
{
    return "JSOpUserAlerted";
}


///
const Sptr < State >
JSOpUserAlerted::process( const Sptr < SipProxyEvent > event )
{
    // 1. Receive 100 or 180
    // 2. For 100, do nothing, drop it
    // 3. For 180, make sure it is not from JSN, shouldn't happened
    // 4. Create new 180, send it back to caller

    cpLog( LOG_DEBUG, "JSOpUserAlerted operation" );

    Sptr < SipEvent > sipEvent;
    sipEvent.dynamicCast( event );
    if ( sipEvent == 0 )
    {
        return 0;
    }
    Sptr < SipMsg > sipMsg = sipEvent->getSipMsg();
    assert( sipMsg != 0 );

    Sptr < StatusMsg > msg;
    msg.dynamicCast( sipMsg );
    if ( msg == 0 )
    {
        return 0;
    }

    if ( (msg->getStatusLine().getStatusCode() != 100) &&
            (msg->getStatusLine().getStatusCode() != 180) )
    {
        return 0;
    }

    Sptr < JSCall > call;
    call.dynamicCast(event->getCallInfo());
    assert( call != 0 );

    JSProvider* provider = call->getProvider();

    Sptr < UdpStack > clientUdp = provider->getClientUdpStack();
    NetworkAddress dest = call->getReceiver();
    JSRequest* req = call->getRequest();
    JSNotification notification(clientUdp, dest);

    if (dest.getPort() == -1)
    {
        cpLog( LOG_DEBUG, "JSOpUserAlerted : no notification sent" );
    }
    else
    {
        if ( msg->getStatusLine().getStatusCode() == 100 )
        {
            // send a conninprogress on behalf of both connections
            // send a conninprogress for caller
            notification.setType(CONNINPROGRESS);
            notification.setCall(req->getClientCallID());
            notification.setAddr(req->getCallerAddr());
            notification.send();

            // send a conninprogress for callee
            notification.setType(CONNINPROGRESS);
            notification.setCall(req->getClientCallID());
            notification.setAddr(req->getCalleeAddr());
            notification.send();
            return 0;
        }

        if ( msg->getStatusLine().getStatusCode() == 180 )
        {
            // send a connalerting for callee
            notification.setType(CONNALERTING);
            notification.setCall(req->getClientCallID());
            notification.setAddr(req->getCalleeAddr());
            notification.send();

            notification.setType(TERMCONNRINGING);
            notification.setCall(req->getClientCallID());
            notification.setAddr(req->getCalleeAddr());
            notification.send();
        }
    }

    // Get the port from the top via and compare it with JSN, JSD ports
    Data viaPort = sipMsg->getVia(0).getPort();

    // Only handle 180 received as JSD
    if (viaPort != provider->getConfig()->getJSDSipPort() )
    {
        // Throw away
        cpLog(LOG_ERR, "receive 180 on port: %s", viaPort.logData() );
        // Change state to unknown to terminate this call
        Sptr < JSStateMachine > stateMachine;
        stateMachine.dynamicCast( event->getCallInfo()->getFeature() );
        assert( stateMachine != 0 );

        return stateMachine->findState( "JSStateUnknown" );
    }

    Sptr < InviteMsg > invite = call->getFromInvite();
    StatusMsg status( *invite, 180/*ringing*/ );

    cpLog(LOG_DEBUG, "JSOpUserAlerted: 180 sent to caller:");
    cpLog(LOG_DEBUG, "***************************************");
    cpLog(LOG_DEBUG, "%s", status.encode().logData());
    cpLog(LOG_DEBUG, "***************************************");

    sipEvent->getSipStack()->sendReply( status );

    return 0;
}
