
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const JSStateMachine_cxx_Version =
    "$Id$";
#include "JSStateMachine.hxx"
#include "JSCall.hxx"
#include "JSStateIdle.hxx"
#include "JSStateInitCaller.hxx"
#include "JSStateInProgress.hxx"
#include "JSStateCalleeBusy.hxx"
#include "JSStateConnected.hxx"
#include "JSStateDisconnected.hxx"
#include "JSStateDisconnectReqd.hxx"
#include "JSStateCalled.hxx"
#include "JSStateTransferCaller.hxx"
#include "JSStateUnknown.hxx"

///
JSStateMachine::JSStateMachine()
{
    cpLog( LOG_DEBUG, "JTAPI Server State Machine" );

    addState( new JSStateIdle );
    addState( new JSStateInitCaller );
    addState( new JSStateTransferCaller );
    addState( new JSStateInProgress );
    addState( new JSStateCalleeBusy );
    addState( new JSStateConnected );
    addState( new JSStateDisconnected );
    addState( new JSStateDisconnectReqd );
    addState( new JSStateCalled );
    addState( new JSStateUnknown );
}    // JSStateMachine::JSStateMachine


///

void
JSStateMachine::process( Sptr < SipProxyEvent > event )
{
    assert ( event != 0 );

    Sptr < JSCall > call;
    call.dynamicCast( event->getCallInfo() );
    assert( call != 0 );

    cpLog( LOG_DEBUG, "Current State is %s", call->getState()->name() );
    Sptr < State > nextState = call->getState()->process( event );
    //assert( nextState != 0 );
    //call->setState( nextState );
    //cpLog( LOG_DEBUG, "Next State is %s", nextState->name() );
}    // JSStateMachine::process


///
Sptr < State >
JSStateMachine::findState( const string stateName )
{
    StateIter iter = myStates.begin();
    while ( iter != myStates.end() )
    {
        if ( stateName == (*iter)->name() )
        {
            return *iter;
        }
        iter++;
    }
    return 0;
}


///
const char* const
JSStateMachine::name() const
{
    return "User Agent State Machine";
}    // JSStateMachine::name


///
JSStateMachine::~JSStateMachine()
{}    // JSStateMachine::~JSStateMachine
