#ifndef VMBASICPROXY_H
#define VMBASICPROXY_H

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */






#include <iostream.h>
#include <cassert>
#include "cpLog.h"
#include "Builder.hxx"
#include "VmCallContainer.hxx"
#include "BasicProxy.hxx"
#include "HeartbeatRxThread.hxx"

static const char* const VmBasicProxyVersion = "$Id: VmBasicProxy.hxx,v 1.13 2002/03/20 05:44:58 bko Exp $";

/** This class is derived from BasicProxy: 
 * - To overwrite the Call Container reference pointer. This way, a 
 * VmBasicProxy object contains a VmCallContainer instead of a regular 
 * CallContainer (HearthLessProxy parent class).
 * - Also, it creates the hearthbeat threads that will listen to the uavm
 * hearthbeat messages sent on 226.2.2.5:6000 multicast address.
 * The parent classes create a  callprocessing fifo, sipstack, worker and sip 
 * threads (and the hearthbeat threads). VmBasicProxy::run method starts the 
 * worker, sip and hearthbeat threads.
 *
 */
class VmBasicProxy: public Vocal::BasicProxy
{
    public:
        /// constructor with default arguments for super class
        VmBasicProxy( const Sptr < Vocal::Builder > builder, unsigned short defaultSipPort = 5060, Data applName = "unknown" );

        /// destructor
        virtual ~VmBasicProxy();

        //create and run the HeartbeatRxThread, HeartBeatWorkerThread for the Uavm heartbeat multicast group.
        void startUavmHeartbeatThread();

    protected:
        /// reference to the heartbeat receiver
        Sptr < HeartbeatRxThread > itsUavmHeartbeatThread;

        /// reference to the housekeeping thread 
        Sptr < HouseKeepingThread > itsUavmHouseKeepingThread;
};


#endif // VMBASICPROXY_H
