#ifndef _VM_BUILDER_HXX_
#define _VM_BUILDER_HXX_

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */





static const char* const VmBuilder_hxx_Version =
    "$Id: VmBuilder.hxx,v 1.13 2002/03/20 05:44:58 bko Exp $";



#include <cassert>
#include "Builder.hxx"

/**
 * This class handles the reception of sip messages for the voicemail
 * feature server. When a message is received, the call information is
 * retrieved if we are in the middle of a call or a new state machine is
 * created if it's a brand new call. 
 * This class is derived from the Builder class as the process() method 
 * needs to be overloaded: in the parent class, the proxy is assumed to
 * be stateless and the call information is removed after each message
 * is processed. Because the fsvm is stateful, this information is needed
 * for the life of the call.
 * Also the constructor has to call the voicemail feature server provisioning
 * code. 
 */
class VmBuilder : public Vocal::Builder
{
    public:
        /// default constructor. Calls the voicemail provisioning code. 
        VmBuilder();

        /// destructor
        virtual ~VmBuilder();

        /**This is identical to Builder::process() but removeCall() is not called at
         * the end as it is for other applications (the call Info is needed for the 
         * life of the call as the fsvm is stateful)
         *@param nextEvent reference to the new event to be used to trigger features
         */
        void process( const Sptr < Vocal::SipProxyEvent > nextEvent );
};

#endif //_VM_BUILDER_HXX_
