#ifndef VMCALLCONTAINER_H
#define VMCALLCONTAINER_H

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


#include "Sptr.hxx"
#include "VMutex.h"
#include "SipCallLeg.hxx"
#include "CallInfo.hxx"
#include "CallContainer.hxx"
#include "VmCallInfo.hxx"

static const char* const VmCallContainerVersion = "$Id: VmCallContainer.hxx,v 1.14 2002/03/20 05:44:58 bko Exp $";

/** A container of callinfos
 *  This class is derived so that getCall() and removeCall() can be overloaded.
 *  For getCall(), we need to instantiate VmCallInfo objects instead of 
 *  CallInfo objects. For removeCall, we need to put back the uavm port in the
 *  list of available ports when the call is cleared.
 */
class VmCallContainer: public Vocal::CallContainer
{
    public:
        /// default constructor
        VmCallContainer();
        /// destructor
        virtual ~VmCallContainer();
        /**method to retrieve call info corresponding to a call leg
         *@param newOrExistingCall reference to the call leg to identify the call
         *@return reference to the call info (for the existing call) or 0
         */
        Sptr < Vocal::CallInfo > getCall( Sptr < Vocal::SipCallLeg > newOrExistingCall );

        /**method to destroy the call info
         *@param existingCall reference to the call leg of the call
         */
        void removeCall ( Sptr < Vocal::SipCallLeg > existingCall );

    private:

};

#endif //VMCALLCONTAINER_H
