
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const StatusPoller_cxx_Version =
    "$Id: StatusPoller.cxx,v 1.41 2002/04/16 00:54:24 bko Exp $";
#include <unistd.h>
#include <stdio.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <sys/param.h>
#include <errno.h>

#include "StatusPoller.hxx"
#include "NetworkAddress.h"
#include "SystemConfig.hxx"
#include "UdpStack.hxx"
#include "ProvisionInterface.hxx"
#include "BaseCommandLine.hxx"
#include "CmuSnmp.hxx"

const ulong num_servers_oid[] =
    {
        1, 3, 6, 1, 4, 1, 5738, 1, 1, 5, 1, 0
    };
const int serverTableLen = 5;
const ulong server_table_oid[] =
    {
        1, 3, 6, 1, 4, 1, 5738, 1, 1, 3, 1, 1, 1, 0
    };
const ulong add_trap_oid[] =
    {
        1, 3, 6, 1, 4, 1, 5738, 1, 1, 1, 7, 0
    };
const ulong delete_trap_oid[] =
    {
        1, 3, 6, 1, 4, 1, 5738, 1, 1, 1, 8, 0
    };

StatusPoller::StatusPoller(int inPollInt, BaseCommandData& inData) : ThreadIf()
{
    pollingIntervalSec = inPollInt;
    pServerData = inData;

}

StatusPoller::~StatusPoller()
{
    hsList.clear();
}

void
StatusPoller::updatePollingInterval (int inPollInt)
{
    pollingIntervalSec = inPollInt;
}

int
freeSystemConfigList(configListT& inList)
{

    return 0;
}

void
StatusPoller::thread()
{
    Data myIpAddress;
    Data result;
    CmuSnmp::snmpTypeT resultType;
    vector < ulong > deleteTrapOid(delete_trap_oid, delete_trap_oid + (sizeof(delete_trap_oid) / sizeof(delete_trap_oid[0]))) ;
    vector < ulong > addTrapOid(add_trap_oid, add_trap_oid + (sizeof(add_trap_oid) / sizeof(add_trap_oid[0]))) ;

    configListT rxConfigList;
    char myHostname[MAXHOSTNAMELEN];

    hsListT hsList = getHsList();

    if (hsList.empty())
    {
        cpLog(LOG_ERR, "Heartbear Server provisioning is incorrect ");
        return ;
    }

    if (!pollingIntervalSec)
    {
        cpLog(LOG_ERR, "polling interval = 0");
        return ;
    }

    if (gethostname(myHostname, MAXHOSTNAMELEN) < 0)
    {
        cpLog(LOG_WARNING, "gethostname failed: %s.  This host will receive no traps", strerror(errno));
    }
    else
    {
        struct hostent tmphostent;
        char buffer[4096];
        int neterr;
        struct in_addr inAddr;

        if (NetworkAddress::getHostByName( myHostname,
                                           &tmphostent,
                                           buffer,
                                           4096,
                                           &neterr ) == 0 )
        {

            memcpy(&inAddr, tmphostent.h_addr_list[0], sizeof(ulong));
            myIpAddress = inet_ntoa(inAddr);

        }
        else
        {
            cpLog(LOG_WARNING, "getHostByName failed:  This host will receive no traps");
        }
    }

    // we will now register this manager to receive snmp traps from each of the HSs
    for ( hsListIter i = hsList.begin(); i != hsList.end(); ++i)
    {
        CmuSnmp snmp(i->getHostName().c_str(), community);
        // ignore the result for now.
        if (!snmp.set( deleteTrapOid, 'a', myIpAddress, resultType, result))
        {
            cpLog(LOG_ERR, "Delete Trap failed");
        }
        if (!snmp.set( addTrapOid, 'a', myIpAddress, resultType, result))
        {
            cpLog(LOG_ERR, "Add Trap failed");
        }
    }

    while (true)
    {
        // re-register to receive traps.  This is in the case where the snmpd goes down
        // but the netMgnt is still up
        // register to receive traps.  Delete old instances first
        for ( hsListIter i = hsList.begin(); i != hsList.end(); ++i)
        {
            CmuSnmp snmp(i->getHostName().c_str(), community);
            // ignore the result for now.
            if (!snmp.set( deleteTrapOid, 'a', myIpAddress, resultType, result))
            {
                cpLog(LOG_ERR, "Delete Trap failed");
            }
            if (!snmp.set( addTrapOid, 'a', myIpAddress, resultType, result))
            {
                cpLog(LOG_ERR, "Add Trap failed");
            }
        }

        // periodic poll of system status
        for ( hsListIter i = hsList.begin(); i != hsList.end(); ++i)
        {
            if ((rxConfigList = pollSystemStatus(*i)) != 0)
            {
                if (SystemConfig::instance()->ProcessStatusPoll(rxConfigList) == voSuccess)
                {
                    freeSystemConfigList(rxConfigList);
                    break;
                }
                else
                {
                    cpLog(LOG_ERR, "Process of status poll failed");
                    freeSystemConfigList(rxConfigList);
                }
            }
        }
        sleep(pollingIntervalSec);
    }
}

hsListT
StatusPoller::getHsList()
{
    NetworkAddress anHs;
    hsListT fullHsList;
    string tmpName;
    list < string > rxHSList;
    try
    {
        // get RSlist from Provisioning Server
        ProvisionInterface& pif = ProvisionInterface::instance(pServerData.pServer.c_str(), pServerData.pServerPort,
                                  pServerData.backupPServer.c_str(), pServerData.backupPServerPort);
        pif.getHeartbeatServers( rxHSList );

        for (list < string > ::iterator i = rxHSList.begin(); i != rxHSList.end(); ++i)
        {
            anHs.setHostName(*i);
            fullHsList.push_back(anHs);
        }
    }
    catch (VException& e)
    {
        cpLog(LOG_ERR, "Failed to connect to Provisioning Server, reason %s", e.getDescription().c_str());
        return fullHsList;
    }

    return fullHsList;
}

configListT
StatusPoller::pollSystemStatus(const NetworkAddress& reqAddr)
{

    int numberOfServers;
    vector < Data > serverTable;

    string hostName = reqAddr.getHostName();
    string currOid, curtpString, origtpString, origOid;
    CmuSnmp snmp (hostName.c_str(), community);
    CmuSnmp::snmpTypeT resultType;
    Data result;
    vector < ulong > nextOid, inOid;
    vector < ulong > numServersOid(num_servers_oid, num_servers_oid + (sizeof(num_servers_oid) / sizeof(num_servers_oid[0]))) ;
    vector < ulong > serverTableOid(server_table_oid, server_table_oid + (sizeof(server_table_oid) / sizeof(server_table_oid[0]))) ;

    while (true)
    {
        if (snmp.get( numServersOid, resultType, result ) )
        {
            numberOfServers = result.convertInt();
            break;
        }
        else
        {
            cpLog(LOG_ERR, "Failed to get number of servers");
            sleep(30);
        }
    }

    // since snmp is unreliable we will walk the table untill we get the entire thing.
    while ( (numberOfServers * serverTableLen) > (int)serverTable.size())
    {
        // Now clear the serverTable so that we don't fill this out again with the
        // existing values

        while ( serverTable.size() > 0)
        {
            serverTable.pop_back();
        }

        // loop through untill we get an error or the end of the table is found
        inOid = serverTableOid;
        nextOid.clear();
        while (inOid.size() > 0 && snmp.getNext(inOid, nextOid, resultType, result ))
        {
            // serverTableOid.size() includes the table index and row of table
            // so we choose (serverTableOid.size() - 2) as oidncmp len
            if (snmp.oidncmp(serverTableOid, nextOid, serverTableOid.size() - 2 ) != 0)
                break;
            serverTable.push_back(result);
            inOid = nextOid;
            nextOid.clear();
        }
    }

    // convert the serverTable data into the statusList format the caller is expecting
    try
    {

        char rcvName[256];
        char portdata[8];
        string portstring;
        string tempString;

        ConfigData tmpConfigData;
        configListT statusList(new vector < ConfigData > );

        for (int i = 0; i < (numberOfServers); i++)
        {

            if ((int)serverTable.size() > i + numberOfServers*4)
            {
		LocalScopeAllocator lo;

                tempString = serverTable[i + (numberOfServers) * 1].getData(lo);

                strcpy(rcvName, tempString.c_str());
                strncat(rcvName, ":", 1);

                sprintf(portdata , "%s", serverTable[i + (numberOfServers)*2].getData(lo));

                strcat(rcvName, portdata);

                tmpConfigData.SvrName = rcvName;
                tmpConfigData.servertype = (ServerType)(atoi(serverTable[i + (numberOfServers) * 3].getData(lo)));
                tmpConfigData.status = (ServerStatus)(atoi(serverTable[i + (numberOfServers) * 4].getData(lo)));
                statusList->push_back(tmpConfigData);
            }
        }
        return statusList;
    }

    catch (VException& e)
    {
        cpLog(LOG_ERR, "received badly formated Response on periodic status poll %s", e.getDescription().c_str());
        return 0;
    }
}
