#ifndef SYSTEMCONFIG_H
#define SYSTEMCONFIG_H

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const SystemConfig_hxx_Version =
    "$Id: SystemConfig.hxx,v 1.25 2001/02/06 03:28:41 maneri Exp $";

#include <string>
#include <map>
#include <vector>
#include <list>
#include "Common.h"
#include "Sptr.hxx"
#include "Data.hxx"
#include "VRWLock.hxx"

class ConfigData
{
    public :
        ServerType servertype;
        int status;
        Data SvrName;

        ConfigData()
        {}
        ~ConfigData()
        {}
        bool operator==(const ConfigData& x)
        {
            return SvrName == x.SvrName;
        }

};

typedef Sptr < vector < ConfigData > > configListT;

class SystemConfig
{
    public:

        static Sptr < SystemConfig > instance();

        virtual ~SystemConfig();

        /**
         * This function replaces the server list with the list contained in 
         * rxConfigList.  This is done after the periodic system config poll.
         * @param rxConfigList a vector of the system configuration
         * @return voReturnStatus
         */
        voReturnStatus ProcessStatusPoll(const configListT& rxConfigList);

        /**
         * This function updates the system Config Cache with the input information
         * This is called when the netMgnt receives a trap from the snmpd
         * @param name the server name whose status has changed
         * @param Type the type of server (ms/rs/etc)
         * @param newStatus up/down
         */
        voReturnStatus ProcessTrap(string Name, ServerType type, ServerStatus newStatus);

        /**
         * return the version number of the system config cache
         * @return the version number of the system config cache
         */
        int getVerNum();

        /**
         * this function output the system config cache in an xml format that the gui 
         * understatds
         * @return xml string of the system config cache
         */
        string getConfigHTML();

        /**
         * creates and returns a vector of the server names in the system config cache
         * @return  a vector of the server names in the system config cache
         */
        list < string > getServerNameList();

        /**
         * get the number of servers in the server cache
         * @return number of servers in the server cache
         */
        int getNumberOfServers();

    private :
        SystemConfig();

        static Sptr < SystemConfig > theSystemConfig;

        static string itsServerTag;

        /**
         * update the system config cache with the input date
         * this is a wrapper around stl map insert
         * @param key for the update
         * @param cfgdata data to insert
         */
        void updateCfg(const Data& key, const Sptr < ConfigData > cfgdata);

        /**
         * using the key, remove the Server object in the map
         * @param key cache key
         */
        void remove ( Data& key );

        /**
         * increment the version number of the system config cache
         */
        void incVerNum ();

        string itsCfgFileName;

        int CfgVersion;
        typedef map < Data, Sptr < ConfigData > > Table;
        typedef Table::iterator TableIter;

        Table systemCfgList;
        VRWLock tableLock;

};
#endif // SYSTEMCONFIG_H
