
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const mkMrtgCfg_cxx_Version =
    "$Id: mkMrtgCfg.cxx,v 1.16 2002/04/17 03:32:29 bko Exp $";


#include <errno.h>
#include <unistd.h>
#include <iostream.h>
#include <stdio.h>
#include <string>
#include <list>
#include "ThreadIf.hxx"
#include "Sptr.hxx"
#include "SystemConfig.hxx" 
#include "mkMrtgCfg.hxx"
#include "cpLog.h"
#include "vocalconfig.h"

#define NETMGNT_PATH VOCAL_INSTALL_PATH "netMgnt"

static const char* const REMOVELOCKFILE = "cd " NETMGNT_PATH "; rm -rf mrtg.cfg_*";
static const char* const STARTMRTG = "cd " NETMGNT_PATH "; ./mrtg mrtg.cfg";
static const char* const KILLMRTGDEAMON = "killall -9 mrtg";
static const char* const MRTGINFILE = NETMGNT_PATH "/mrtg.in";
static const char* const MRTGCFGFILE = NETMGNT_PATH "/mrtg.cfg";

Sptr < mkMrtgCfg > mkMrtgCfg::themkMrtgCfg = 0;

/////////////////////////////////////////////////////////////////////////////
//
//  The intent of this module is to genertae a config file that the MRTG 
//  software takes in as a config file and generates the various oid maps
//  indicating the statics.A sample of the cfg file generated is as follows:
//
//      # Add a WorkDir: /some/path line to this file
//      WorkDir: /usr/local/vocal/NetMgnt
//
//      # run as a deamon every 5 minute poll
//      RunAsDaemon:Yes
//      Interval:5
//      # add 14all.cg later
//      #UseRRDTool: Yes
//      # Ethernet Traffic
//
//      Suppress[_]: ym
//      MaxBytes[_]: 1250000
//
//      @ETHSTART
//
//      #-------------------------------------------
//      # SIP MEssages
//      Suppress[_]: ym
//      MaxBytes[_]: 1250000
//      Options[_]: dorelpercent noinfo perminute
//      YLegend[_]: Msgs / min
//      ShortLegend[_]: #/min
//      LegendI[_]:In Msgs
//      LegendO[_]:Out Msgs
//      Legend1[_]: All Incomming per minute
//      Legend2[_]: All Outgoing per minute
//      Legend3[_]: (In )/(Out ) * 100
//      Title[_]: SipMessages
//
//      #------------------------------------------
//      #
//      #sipSummaryInRequests sipSummaryOutRequests
//      #
//
//      NoSpaceChar: ~
//      Target[^]: enterprises.5738.3.1.2.2.1.1.1.1/applIndex:private@~
//      @%TARGET:req
//
//      PageTop[$]:
//      <TABLE>
//          <TR><TD>sipSummaryInRequests:</TD><TD>In Green</TD></TR>
//          <TR><TD>sipSummaryOutRequests:</TD><TD>Out Blue</TD></TR>
//          <TR><TD>Ratio in/out </TD><TD>Amber</TD></TR>
//      </TABLE>
//      @&PAGE:req
//
/////////////////////////////////////////////////////////////////////////////////



Sptr < mkMrtgCfg >
mkMrtgCfg::instance()
{
    if ( themkMrtgCfg == 0 )
    {
        themkMrtgCfg = new mkMrtgCfg;
    }
    return themkMrtgCfg;
}

mkMrtgCfg::mkMrtgCfg() : ThreadIf()
{}



mkMrtgCfg::~mkMrtgCfg()
{}



void
mkMrtgCfg::thread()
{
    int agentDead = 0;
    while ( SystemConfig::instance()->getNumberOfServers() == 0)
    {
        agentDead ++;
        if (agentDead > 50)
        {
            cpLog(LOG_ALERT, "timeout waiting for snmp agent to respond.  It could be dead");
            agentDead = 0;
        }
        sleep(2);
    }
    createListOfServers();
    if (parseSeed() == voFailure)
        cpLog(LOG_DEBUG,"Failed to create the mrtg config file");
    return ;
}


// createListOfServers will take the configList
// from the StatusPoller and create a list of
// only the servers from the configList.
// Input : ConfigList from the StatusPoller
// Output: List of Servers


void mkMrtgCfg::createListOfServers()
{
    list < string > templist;
    list < string > ::iterator i;

    templist = ( SystemConfig::instance())->getServerNameList();

    for (i = templist.begin(); i != templist.end(); i++)
    {
        int dot = (*i).find(".");
        serverList.push_back((*i).substr(0, dot));
    }
}

// parseSeed will parse the seed cfg file
// and generate the mrtg config file.
// Input : The list of servers that is
// obtained from the previous function
// Output : Returns the enum status voReturnSatus
//

voReturnStatus
mkMrtgCfg::parseSeed()
{
    char targ[256];
    char title[256];
    char pagetop[256];
    char System[256];
    char Interface[256];
    char ip[256];
    char head[10];
    char speed[256];
    char kbyte[256];
    char tab[10];
    char endtable[10];
    char wdir[50];
    char pgtop[100];
    char systemCommand[1024];
    char buf[513];
    char pagetoptemp[513];
    char targettemp[513];
    char newBuf[513];

    unsigned int m, n;
    int i, reVal;
    int listSize = serverList.size();
    char* targetval;

    string temp;
    string newline = "\n";

    FILE* cfgf;
    FILE* mrtgf;

    cfgf = fopen(MRTGINFILE, "r");
    if (!cfgf)
    {
        cpLog(LOG_ERR, "can't open input file %s, reason %s","mrtg.in", strerror(errno));
        return voFailure;
    }
    mrtgf = fopen(MRTGCFGFILE, "w");
    if (!mrtgf)
    {
        cpLog(LOG_ERR, "can't open output file %s, reason %s", "mrtg.cfg", strerror(errno));
        return voFailure;
    }

    while (!feof(cfgf))
    {
        memset(buf, 0, sizeof(buf));
        fgets(buf, 512, cfgf);

        if (buf[0] == '$')
        {
            // write the buf as itis into the
            // mrtg cfg file. This line will
            // not be changed.
            for (m = 1, n = 0; m <= strlen(buf); m++, n++)
                newBuf[n] = buf[m];
            newBuf[n] = '\0';
            fputs(newBuf, mrtgf);
        }
        if (buf[0] == '@')
        {
            // Read the following word
            // and depending on the word
            // enter the appropriate lines
            // to the mrtg cfg file.

            if (!memcmp(buf, "@WORKDIR", strlen("@WORKDIR")))
            {
                strcpy(wdir, "WorkDir: " NETMGNT_PATH);
                fputs(wdir, mrtgf);
                fputs("\n", mrtgf);
            }

            if (!memcmp(buf, "@ETHSTART", strlen("@ETHSTART")))
            {
                for ( i = 0; i < listSize ; i++)
                {

                    if (!strcmp(serverList[i].c_str(), ""))
                        break;
                    //   Target[serverName.eth0]: 2:private@serverName
                    strcpy(targ, "Target[");
                    strcat(targ, serverList[i].c_str());
                    strcat(targ, ".eth0]: 2:private@");
                    strcat(targ, serverList[i].c_str());

                    fputs(targ, mrtgf);
                    fputs("\n", mrtgf);

                    //   Title[serverName.eth0]: serverName.private.vovida.com (serverName.private.vovida.com): eth0
                    strcpy(title, "Title[");
                    strcat(title, serverList[i].c_str());
                    strcat(title, ".eth0]: ");
                    strcat(title, serverList[i].c_str());
                    strcat(title, ": eth0");

                    fputs(title, mrtgf);
                    fputs("\n", mrtgf);

                    //    PageTop[serverName.eth0]: <H1>Traffic Analysis for eth0
                    strcpy(pgtop, "PageTop[");
                    strcat(pgtop, serverList[i].c_str());
                    strcat(pgtop, ".eth0]: <H1>Traffic Analysis for eth0");

                    fputs(pgtop, mrtgf);
                    fputs("\n", mrtgf);

                    //    </H1>
                    //    <TABLE>
                    strcpy(head, " </H1>");
                    strcpy(tab, "  <TABLE>");
                    fputs(head, mrtgf);
                    fputs("\n", mrtgf);
                    fputs(tab, mrtgf);
                    fputs("\n", mrtgf);

                    //     <TR><TD>System:</TD><TD>serverName.private.vovida.com</TD></TR>
                    strcpy(System, "   <TR><TD>System:</TD><TD>");
                    strcat(System, serverList[i].c_str());
                    strcat(System, "</TD><TR>");
                    fputs(System, mrtgf);
                    fputs("\n", mrtgf);

                    //     <TR><TD>Interface:</TD><TD>eth0 (2)</TD></TR>
                    strcpy(Interface, "   <TR><TD>Interface:</TD><TD>eth0 (2)</TD></TR>");
                    fputs(Interface, mrtgf);
                    fputs("\n", mrtgf);

                    //     <TR><TD>IP:</TD><TD>serverName.private.vovida.com (192.168.5.140)</TD></TR>
                    strcpy(ip, "   <TR><TD>IP:</TD><TD>");
                    strcat(ip, serverList[i].c_str());
                    strcat(ip, "</TD><TR>");
                    fputs(ip, mrtgf);
                    fputs("\n", mrtgf);

                    //     <TR><TD>Max Speed:</TD>
                    strcpy(speed, "   <TR><TD>Max Speed:</TD>");
                    fputs(speed, mrtgf);
                    fputs("\n", mrtgf);

                    //     <TD>1250.0 kBytes/s (ethernetCsmacd)</TD></TR>
                    strcpy(kbyte, "   <TD>1250.0 kBytes/s (ethernetCsmacd)</TD></TR>");
                    fputs(kbyte, mrtgf);
                    fputs("\n", mrtgf);

                    //     </TABLE>
                    strcpy(endtable, " </TABLE>");
                    fputs(endtable, mrtgf);
                    fputs("\n\n", mrtgf);
                }
            }

            if (buf[1] == '%')
            {

                // get the sip oid for which the target is to be created
                memcpy(targettemp, buf, strlen(buf));
                targetval = strchr(targettemp, ':');
                targetval ++;
                if (targetval)
                {
                    temp.assign(targetval);
                    temp = temp.erase(temp.find(newline, 0), temp.size());
                    for ( i = 0; i < listSize ; i++)
                    {
                        char target[256];
                        strcpy(target, "Target[");
                        strcat(target, serverList[i].c_str());
                        strcat(target, ".");
                        strcat(target, temp.c_str());
                        strcat(target, "]: ");
                        strcat(target, serverList[i].c_str());
                        fputs(target, mrtgf);
                        fputs("\n", mrtgf);
                    }
                }
            }

            if (buf[1] == '&')
            {
                // get the sip oid for which the target is to be created
                memcpy(pagetoptemp, buf, strlen(buf));
                targetval = strchr(pagetoptemp, ':');
                targetval ++;
                if (targetval)
                {
                    temp.assign(targetval);
                    temp = temp.erase(temp.find(newline, 0), temp.size());
                    for ( i = 0; i < listSize ; i++)
                    {
                        strcpy(pagetop, "PageTop[");
                        strcat(pagetop, serverList[i].c_str());
                        strcat(pagetop, ".");
                        strcat(pagetop, temp.c_str());
                        strcat(pagetop, "]:<H1> ");
                        strcat(pagetop, serverList[i].c_str());  // this should have been in uppercase !!!
                        strcat(pagetop, "  </H1>");
                        fputs(pagetop, mrtgf);
                        fputs("\n", mrtgf);
                    }
                }
            }

        }
        if (buf[0] == '0')
        {
            return voFailure;
        }
    }
    fclose(cfgf);
    fclose(mrtgf);

    memset(systemCommand, 0, sizeof(systemCommand));

    sprintf(systemCommand, KILLMRTGDEAMON );
    reVal = system(systemCommand);
    if (reVal == -1 || reVal == 127)
    {
        cpLog(LOG_WARNING, "SYSTEM command failed    %s", systemCommand);
    }
    sprintf(systemCommand, REMOVELOCKFILE );
    reVal = system(systemCommand);
    if (reVal == -1 || reVal == 127)
    {
        cpLog(LOG_WARNING, "SYSTEM command failed    %s", systemCommand);
    }

    sprintf(systemCommand, STARTMRTG );
    reVal = system(systemCommand);
    if (reVal == -1 || reVal == 127)
    {
        cpLog(LOG_WARNING, "SYSTEM command failed    %s", systemCommand);
    }

    return voSuccess;
}
