
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const RSMirror_cxx_Version =
    "$Id$";
#include "cpLog.h"
#include "SipVia.hxx"
#include "RSMirror.hxx"
#include "RSServerContainer.hxx"
#include "RSProxy.hxx"
#include "SystemInfo.hxx"

using namespace Vocal;

/**
 * Constructor
 */
RSMirror::RSMirror()
{}


/**
 * Destructor
 */
RSMirror::~RSMirror()
{}

/**
 * Copy Constructor
 * @param rhs Object from which to copy
 */
RSMirror::RSMirror(const RSMirror& rhs)
{}


/**
 * Overload operator assignment =
 * @param  rhs  Object from which to copy
 * @return  Reference to this object with new data
 */
RSMirror& 
RSMirror::operator= (const RSMirror& rhs)
{
   if (this != &rhs)
   {
       copyRhsToThis(rhs);
   }

   return *this;
}

/**
 * Copy member data from given object into this one
 * @param  rhs  Object from which to copy
 */
void
RSMirror::copyRhsToThis( const RSMirror& rhs )
{}


/**
 * Get the first via from the sipRegister, get the list of RS 
 * from the server container to compare with the first via.
 * @param  sipRegister  Sip register message to mirror
 * @return True if via match with any rs, else false
 */
bool RSMirror::isFromRs(const Sptr < RegisterMsg > sipRegister)
{
    // get first via from the sip message
    SipVia firstVia = sipRegister->getVia(0);

    Data via, port(firstVia.getPort());
    via = firstVia.getHost() + ":" + port;

    cpLog(LOG_DEBUG, "RSMirror::isFromRs, First via = %s", via.logData());

    // get the list of rs from the server container
    Sptr<Server> serverPtr;
    if (RSServerContainer::instance().search(via, serverPtr))
    {
	if (serverPtr != 0 && 
	    (serverPtr->getType() == SERVER_RS))
	{
            cpLog (LOG_DEBUG, "The Register Msg is from a RS. Don't mirror it");
            return true; 
	}
    }    

    return false;

}

/**
 * Clone the sip register, add self to first via. For
 * each redirect server from server container, change
 * request line to each rs, call sipstack to send async.
 * @param  sipEvent  Event used to send the Register messages.
 * @param  sipRegister  Register message to be mirrored.
 */
void RSMirror::mirrorSipRegister(const Sptr < SipEvent > sipEvent,
                                 const Sptr < RegisterMsg > sipRegister )
{
    cpLog( LOG_DEBUG, "RSMirror::mirrorSipRegister.");
    string addr_port;

    RegisterMsg registerMsg = *sipRegister;
    SipRequestLine requestLine(registerMsg.getRequestLine());

    // get the list of rs from the server container
    RSServerContainer & serverContainer = RSServerContainer::instance();
    Sptr < vector < Data > > listOfRs = new vector < Data > ;
    serverContainer.getListofRS(listOfRs);

    Data selfHost(theSystem.gethostAddress());
    Data localPort(RSProxy::instance().getLocalSipPort());

    // add self into first via
    SipVia via;
    via.setHost( theSystem.gethostAddress() );
    via.setPort( RSProxy::instance().getLocalSipPort() );
    via.setBranch( registerMsg.computeBranch() );
    registerMsg.setVia( via, 0 );

    static int lbrNo = 0;
    char lViaBranch[56];
    for ( vector < Data > ::iterator i = listOfRs->begin();
            i != listOfRs->end();
            ++i )
    {
        cpLog( LOG_DEBUG, "RSMirror::mirrorSipRegister, From list of redirect servers = %s.",
               i->logData() );
        
        Data newHost, newPort;
        // seperate address and port
        LocalScopeAllocator lo;
        addr_port = i->getData(lo);
        string::size_type colonPos = addr_port.find(":");
        if (colonPos != string::npos)
        {
            newHost = addr_port.substr(0, colonPos);
            newPort = addr_port.substr(colonPos + 1, addr_port.size());
        }

        cpLog (LOG_DEBUG, "comparing against: %s:%s", newHost.logData(), 
	       newPort.logData());

        // don't send to self url
        // Note: it's not sure that the requestLine from sipRegister doesn't
        // indicate self's address.  Must avoid sending to self.
        if ((newHost == selfHost) && (newPort == localPort))
        {
            cpLog (LOG_DEBUG, "Don't send to self: %s:%s", selfHost.logData(),
                   localPort.logData());
            continue;
        }

        Sptr<SipUrl> newUrl = new SipUrl();
        newUrl->setHost(newHost);
        newUrl->setPort(newPort);

        cpLog( LOG_DEBUG, "Sending register to = %s:%s",
               newHost.logData(),
               newPort.logData() );

        requestLine.setUrl( newUrl );
        registerMsg.setRequestLine( requestLine );
        SipVia via = registerMsg.getVia(0);
        sprintf(lViaBranch, ".%d", lbrNo++);
        via.setBranch( registerMsg.computeBranch() + Data(lViaBranch));
        registerMsg.removeVia(0);
        registerMsg.setVia(via, 0);
        sipEvent->getSipStack()->sendAsync( registerMsg );
        
    }
}


//End of File
// Local Variables:
// c-file-style:"stroustrup"
// c-basic-offset:4
// c-file-offsets:((inclass . ++))
// indent-tabs-mode:nil
// End:
