#ifndef RSPROXY_HXX
#define RSPROXY_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const RSProxy_hxx_Version =
    "$Id: RSProxy.hxx,v 1.31 2002/03/19 21:39:49 jason Exp $";


#include "cpLog.h"
#include "BasicProxy.hxx"
#include "RSBuilder.hxx"
#include "RSServerContainer.hxx"
#include "RedundancyRxThread.hxx"
#include "RedundancyWorkerThread.hxx"
#include "RsAgent.hxx"
#include "RSCommon.h"


static const char* const RSProxyVersion = "$Id: RSProxy.hxx,v 1.31 2002/03/19 21:39:49 jason Exp $";

/**
 * RS Proxy.  This class creates a call container, callprocessing fifo, 
 * sipstack, udpstack, worker and sip and heartbeat threads.
 * RSProxy::run method starts the worker and sip and heartbeat threads.
 * A builder object and SIP port number and UDP port is given upon instantiation.
 */
class RSProxy : public Vocal::BasicProxy
{
    public:

        /**
         * Return the singleton RSProxy or create it if does not exist.
	 * @return reference singleton of RSProxy
	 */
        static RSProxy& instance(Sptr<RSBuilder> rsBuilder = 0, 
				 unsigned short defaultSipPort = 5060);

        /// Begin the RS and its related threads
        virtual void run();

        /// Handle the ending of processes
        virtual void join();

        /// Destructor
        virtual ~RSProxy();

        /**
         * Obtain the SIP stack
         * @return  The SIP stack
         */
        Sptr < Vocal::SipTransceiver > getSipStack() const;

        /**
         * Obtain the current state of the RS
         * @return  The current state of the RS
         */
        RSSTATE getRsState() const;

        /**
         * Store the state of the RS
         * @param  newState  New state of the RS to be stored
         */
        void setRsState(RSSTATE newState);

        /**
         * Obtain the port used for SIP messaging
         * @return  The SIP port
         */
        unsigned short getLocalSipPort() const;

        /**
         * Store the port to be used for SIP messaging
         * @param  sipPort  Port to be used for SIP messaging
         */
        void setLocalSipPort(unsigned short sipPort);

        /**
         *  The sync port has been changed. Update the redundancy threads.
         * @param  oldSyncPort  The old port used for synchronization
         * @param  newSyncPort  The new port used for synchronization
         */
        void updateRedundancyPort(unsigned short oldSyncPort, unsigned short newSyncPort );

    protected:

        /// Redundancy receiver thread
        Sptr < RedundancyRxThread > itsRedundancyRxThread;

        /// Redundancy thread to handle redundancy messages and timers
        Sptr < RedundancyWorkerThread > itsRedundancyWorkerThread;

        /// Agent object
        Sptr < RsAgent > itsRsAgent;

        /**
         * Constructor
         * @param  rsBuilder  Redirect server builder object
         * @param  defaultRedundancyPort  Port to be used for syncronization
         * @param  defaultSipPort  Port to be used for SIP messaging
         */
        RSProxy( Sptr < RSBuilder > rsBuilder, unsigned short defaultSipPort = 5060);

    private:

        /// State of the RS
        RSSTATE itsState;

        /// Port to be used for SIP  messaging
        unsigned short localSipPort;

        /// singleton of the class RSProxy
        static RSProxy * rsInstance;

};



#endif // RSPROXY_HXX
