#ifndef REDUNDANCYEVENT_HXX
#define REDUNDNACYEVENT_HXX
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


#include "RSCommon.h"
#include "Sptr.hxx"
#include "Data.hxx"
#include "SipProxyEvent.hxx"

static const char* const RedundancyEventVersion = "$Id$";

/** 
 *  RedundancyEvent is a SipProxy Event. This event is posted when a redundancy
 *  message is received from a server.   
 */

class RedundancyEvent : public Vocal::SipProxyEvent
{
    public:

        /**
         * Constructor
         * @param  sender  Address of server initiating the message.
         * @param  info  Contains message type
         * @param  eventId  Unique identifier
         * @param  outputFifo 
         */
        RedundancyEvent( const Sptr < Data > sender, 
                         const InfoType info,
                         const Vocal::FifoEventId eventId, 
                         const Sptr < Vocal::Fifo < Sptr < Vocal::SipProxyEvent > > > outputFifo );

        /// Destructor
        virtual ~RedundancyEvent();

        /**
         * Obtain the sender information.
         * @return  Smart pointer to this object's sender data.
         */
        const Sptr < Data > getSender() const;

        /**
         * Obtain the info data.
         * @return  The info portion of this object.
         */
        const InfoType getInfo() const;

        /**
         * Obtain the event id data
         * @return  This object's event id
         */
        Vocal::FifoEventId getEventId() const;

        /**
         * Store the sender data into this object.
         * @param  sender  Address of server initiating the message.
         */
        void setSender( const Sptr < Data > sender );

        /**
         * Store the info passed in into this object.
         * @param  info  Typically a message type.
         */
        void setInfo( const InfoType info );

        /**
         * Store the passed in event id.
         * @param  eventId  Unique identifier
         */
        void setEventId( const Vocal::FifoEventId eventId );

        /**
         * Method of identifying this class.
         * @return  This object's identifier
         */
        virtual const char* const name() const
        {
            return "RedundancyEvent";
        }

    private:

        /// Address of server causing the event.
        Sptr < Data > itsSender;

        /// Various information related to event. Typically a message type.
        InfoType itsInfo;

        /// Unique identifier
        Vocal::FifoEventId itsEventId;

};

#endif //REDUNDNACYEVENT_HXX
