
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const RsAgent_cxx_Version =
    "$Id: RsAgent.cxx,v 1.30 2002/02/25 05:51:20 sprajpat Exp $";

#include <netdb.h>
#include "RsAgent.hxx"
#include "RSServerContainer.hxx"
#include "cpLog.h"

Sptr < RsAgent > RsAgent::theRsAgent = 0;


/**
 * Return the one RsAgent or create it if it doesn't already exist. 
 * @param  applPort  SIP port to use in creation of agent.
 * @return  Pointer to the singleton agent.
 */
Sptr < RsAgent >
RsAgent::instance( const unsigned short applPort /* HAS DEFAULT */)
{
    if ( theRsAgent == 0)
    {
        if (applPort == 0)
            cpLog(LOG_ALERT, "Failed to get sip port from RS, use default 0");

        Data nameport;
        nameport += RSNAME;
        nameport += ":";
        nameport += applPort;
        LocalScopeAllocator lo;
        theRsAgent = new RsAgent( nameport.getData(lo) );
    }
    return theRsAgent;
}

/**
 * Constructor
 * @param  applNamePort  Name to be used to identify this object
 */
RsAgent::RsAgent( const string applNamePort ) : AgentApi(SERVER_RS, applNamePort)
{}


/**
 * Process the message passed in.
 * @param  message  Message to be processed
 * @param  sender  Address of the message sender
 * @return  The return status
 */
voReturnStatus
RsAgent::processMessage(ipcMessage *msg, NetworkAddress *sender)
{

    switch (msg->action)
    {
        case Get :
        switch (msg->mibVariable)
        {
            case SysStatsTotalSubscribers :
            return sendResponse(VO_NA_INT, sender);
            case SysStatsTotalProcessDeaths :
            return sendResponse(VO_NA_INT, sender);
            case SysStatsTotalProcessRestarts :
            return sendResponse(VO_NA_INT, sender);
            default :
            return voUnknown;
        }
        case Set :
        case Trap :
        case Response :
        case Register :
        default :
        return voFailure;
    }
    return voFailure;
}
