/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal\@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

#include <config.h>


/* minimal include directives */
#include "mibincl.h"
#include "util_funcs.h"

#include <AgentApi.hxx>
#include "VovidaSnmpUtil.h"
#include "softSwitchStats.h"
#include "TrapAgentC.hxx"
#include "TrapAgent.hxx"
#include "Common.h"

/* 
 * softSwitchStats_variables_oid:
 *   this is the top level oid that we want to register under.  This
 *   is essentially a prefix, with the suffix appearing in the
 *   variable below.
 */


oid softSwitchStats_variables_oid[] = { 1,3,6,1,4,1,5738,1,1,5 };


/* 
 * variable2 softSwitchStats_variables:
 *   this variable defines function callbacks and type return information 
 *   for the softSwitchStats mib section 
 */


#define   SYSSTATSTOTALSERVERS  1
#define   SYSSTATSTOTALSUBSCRIBERS  2
#define   SYSSTATSTOTALPROCESSDEATHS  3
#define   SYSSTATSTOTALPROCESSRESTARTS  4

struct variable2 softSwitchStats_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */
  { SYSSTATSTOTALSERVERS, ASN_INTEGER   , RONLY , var_softSwitchStats, 1, { 1 } },
  { SYSSTATSTOTALSUBSCRIBERS, ASN_INTEGER   , RONLY , var_softSwitchStats, 1, { 2 } },
  { SYSSTATSTOTALPROCESSDEATHS, ASN_COUNTER   , RONLY , var_softSwitchStats, 1, { 3 } },
  { SYSSTATSTOTALPROCESSRESTARTS, ASN_COUNTER   , RONLY , var_softSwitchStats, 1, { 4 } },

};
/*    (L = length of the oidsuffix) */


/*
 * init_softSwitchStats():
 *   Initialization routine.  This is called when the agent starts up.
 *   At a minimum, registration of your variables should take place here.
 */
void init_softSwitchStats(void) {


  /* register ourselves with the agent to handle our mib tree */
  REGISTER_MIB("softSwitchStats", softSwitchStats_variables, variable2,
               softSwitchStats_variables_oid);


  /* place any other initialization junk you need here */
}


/*
 * var_softSwitchStats():
 *   This function is called every time the agent gets a request for
 *   a scalar variable that might be found within your mib section
 *   registered above.  It is up to you to do the right thing and
 *   return the correct value.
 *     You should also correct the value of "var_len" if necessary.
 *
 *   Please see the documentation for more information about writing
 *   module extensions, and check out the examples in the examples
 *   and mibII directories.
 */
unsigned char *
var_softSwitchStats(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{


  /* variables we may use later */
  static long long_ret;
  static unsigned char string[SPRINT_MAX_LEN];
  static oid objid[MAX_OID_LEN];
  static struct counter64 c64;

  ipcMessage message;
  struct sockaddr_in rxSourceAddr;
  struct timeval tv;
  fd_set rset;
  int retVal, sockaddrSize = sizeof(struct sockaddr);
  int done;
  int rsPort = TrapAgent_GetApplPortFromIndex((void *)trapAgent, TrapAgent_GetNextIndex((void *)trapAgent, 0, SERVER_RS));
  int hsPort = TrapAgent_GetApplPortFromIndex((void *)trapAgent, TrapAgent_GetNextIndex((void *)trapAgent, 0, SERVER_HB));
     
  if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED )
    return 0;

  /* 
   * this is where we do the value assignments for the mib results.
   */
  switch(vp->magic) {

    case SYSSTATSTOTALSERVERS:
        
	if(!hsPort || ipcRequest((actionT)Get,
		       SysStatsTotalServers,
		       hsPort,
		       &message) != voSuccess ) {
	    return 0;
	} else {
	    memcpy(&long_ret, message.parm2, sizeof(int));
	}
	return (unsigned char *) &long_ret;

    case SYSSTATSTOTALSUBSCRIBERS:
        
	if(!rsPort || ipcRequest((actionT)Get,
		       SysStatsTotalSubscribers,
		       rsPort,
		       &message) != voSuccess) {
	    return 0;
	} else {
	    memcpy(&long_ret, message.parm2, sizeof(int));
	}
	return (unsigned char *) &long_ret;

    case SYSSTATSTOTALPROCESSDEATHS:
        
	if(!hsPort || ipcRequest((actionT)Get,
		       SysStatsTotalProcessDeaths,
		       hsPort,
		       &message) != voSuccess) {
	    return 0;
	} else {
	    memcpy(&long_ret, message.parm2, sizeof(int));
	}
        return (unsigned char *) &long_ret;

    case SYSSTATSTOTALPROCESSRESTARTS:
        
	if(!hsPort || ipcRequest((actionT)Get,
		       SysStatsTotalProcessRestarts,
		       hsPort,
		       &message) != voSuccess) {
	    return 0;
	} else {
	    memcpy(&long_ret, message.parm2, sizeof(int));
	}
        return (unsigned char *) &long_ret;

    default:
      ERROR_MSG("");
      return 0;      
  }
  return 0;
}







