/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const SM_CapManager_cxx_Version = 
"$Id: SM_CapManager.cxx,v 1.4 2001/05/24 21:58:33 vzubarev Exp $";

#include <iterator>

#include "SM_CapManager.hxx"
#include "XlatorException.hxx"
#include "EvtCapMgrRcvdTCS.hxx"
#include "EvtCapMgrRcvdTCS0.hxx"
#include "EvtCapMgrSetCC.hxx"
#include "EvtCapMgrRcvdSDP.hxx"
#include "EvtCapMgrRcvdSDP0.hxx"
#include "G711Capability.hxx"
#include "GSMCapability.hxx"
#include "G7231Capability.hxx"
#include "G722Capability.hxx"
#include "G728Capability.hxx"
#include "G729Capability.hxx"

using namespace Translator;

using namespace CapabilityTranslator;

using namespace CapabilityManager;


SM_CapManager::SM_CapManager(translator::CallTranslatorEM *em) : evtMgr(em),
                                                                 f_tcs(this),
								 f_sdp(this),
    							         capSetIntersect(capSetSip,
								                 capSetH323)
{
    const string    fn("SM_CapManager::SM_CapManager");
    VLog	    log(fn);
  
    capabilityMap[1] = new G711Capability(G711Capability::muLaw, G711Capability::At64k);
    
    VDEBUG(log) << fn << ":G711 Cap was intialized with packet size : " 
    	    	<< capabilityMap[1]->getPacketSizeFrames()  << VDEBUG_END(log);
    capabilityMap[3] = new GSMCapability(GSMCapability::FullRate);
    capabilityMap[4] = new G7231Capability;
    capabilityMap[8] = new G711Capability(G711Capability::ALaw, G711Capability::At64k);
    capabilityMap[9] = new G722Capability(G722Capability::At64k);
    capabilityMap[15] = new G728Capability;
    capabilityMap[18] = new G729Capability;
    capabilityMap[96] = new GSMCapability(GSMCapability::HalfRate);
    capabilityMap[97] = new GSMCapability(GSMCapability::EnhancedFullRate);
}


}

SM_CapManager::~SM_CapManager() 
{  
  const string  fn("SM_CapManager::~SM_CapManager");
  VLog	    	log(fn);
    CapMap::const_iterator p = capabilityMap.begin();
    
    while(p != capabilityMap.end())
    {
      delete p->second;
      p++;
    }
  const string  fn2("exit CapabilityMap::~CapabilityMap");
  VLog	    	log2(fn2);

}

void SM_CapManager::init() {}

FeatureTCSBase* SM_CapManager::getFeatureTCS() throw()
{
    assert(this);
    
    return &f_tcs;
}

FeatureTCS& SM_CapManager::getFeatureTCSImp() throw()
{
    assert(this);
    
    return f_tcs;
}

FeatureSDPBase* SM_CapManager::getFeatureSDP() throw()
{
    assert(this);
    
    return &f_sdp;
}

FeatureSDP& SM_CapManager::getFeatureSDPImp() throw()
{
    assert(this);
    
    return f_sdp;
}

void SM_CapManager::parseCapsTCS(const TCS &msgTCS) throw()
{
  const string  fn("CapabilityManager::parseCapsTCS");
  VLog	    	log(fn);
  
  int i = 0, j = 0, h = 0, vecSize = 0, descrSize = 0;
  int altSize = 0, tableSize = 0;
  
  VDEBUG(log) << fn << ": BEGIN -------------- " << VDEBUG_END(log);
  
  if(!msgTCS.HasOptionalField(H245_TerminalCapabilitySet::e_capabilityTable))
   if(!msgTCS.HasOptionalField(H245_TerminalCapabilitySet::e_capabilityDescriptors))
    if(!msgTCS.HasOptionalField(H245_TerminalCapabilitySet::e_multiplexCapability))
    {
         VDEBUG(log) << fn << ": TCS 0 received " << VDEBUG_END(log);
	 
	 // TODO:
	 // here we should send
	 // CallTranslatorEM::startZeroTCS()
	 // call-back event
	 
	 evtMgr->startZeroTCS();
	 
	 // and 
	 // EvtCapMgrRcvdTCS0
	 // local event
	 
	 EvtCapMgrRcvdTCS0 rcvdTCS0(this);
	 rcvdTCS0.start();
    }
   
  
  tableSize = msgTCS.m_capabilityTable.GetSize();
  if(!tableSize) { VWARN(log) << fn << ": Seems like capTable is empty..."  << VDEBUG_END(log); }
  
  int pTypeCap, tableN;
  
  CapMap::const_iterator cM_I;

  for(i = 0; i < tableSize; i++)
  {
    if(msgTCS.m_capabilityTable[i].HasOptionalField(H245_CapabilityTableEntry::e_capability))
    {
       try
       {	
	 pTypeCap = globalCapTable.findCapTableEntry(msgTCS.m_capabilityTable[i].m_capability);
       }
       catch (XlatorException &mS)
       {
         VDEBUG(log) << fn << ": Do not support this capability " 
	    	     <<  mS.getDescription() << " continue..." << VDEBUG_END(log);
	 continue;
       }
      
       cM_I = capabilityMap.find(pTypeCap);
       
       if(cM_I == capabilityMap.end())
       {
         VERR(log) << fn << ": Capability number initialized in capabilityMap & in GlobalCapTable..." << VERR_END(log);
	 VERR(log) << fn << ": ...pTypeCap : "<< pTypeCap << " ..." << VERR_END(log);
	 VERR(log) << fn << ": ... do not match - check CapabilityMap constructor & GlobalCapTable" << VERR_END(log);
	 throw XlatorException("Capability number initialized in capabilityMap & in GCTable do not match",
	                       __FILE__, __LINE__);
       }
       
       Capability *capability = capabilityMap[pTypeCap];
       
       Capability *copy = capability->clone();

       if(copy->setParamRecvMsg(msgTCS.m_capabilityTable[i].m_capability))
       {
	  copy->assignCapabilityNumber(pTypeCap);
          VDEBUG(log) << fn << ": Capability " << pTypeCap << " with packetSize " << copy->getPacketSizeFrames() << VDEBUG_END(log);

	  tableN = (int)msgTCS.m_capabilityTable[i].m_capabilityTableEntryNumber;
	  
	  mapTable[tableN] = copy->getCapabilityNumber();
	  capSetH323.h323CapTable.addCapTableEntry(copy);
	    
       } else delete copy;
    }
  }
	
  vecSize = msgTCS.m_capabilityDescriptors.GetSize();
  
  int y = 0, z = 0;
  
  VDEBUG(log) << fn << ": Filled out the capabilityTable..." << VDEBUG_END(log);

  int hTableSize = capSetH323.h323CapTable.getCapabilityTable().size();
    
  VDEBUG(log) << fn << ": Now the size of H323 capTable is : "<< hTableSize << VDEBUG_END(log);

  if(!vecSize)
  {
      VWARN(log) << fn << ": Seems like capDescriptors is empty..." << VWARN_END(log);
  }
  
  for(i = 0; i < vecSize; i++)
  {
    H245_CapabilityDescriptor &descr = msgTCS.m_capabilityDescriptors[i];
    if(descr.HasOptionalField(H245_CapabilityDescriptor::e_simultaneousCapabilities))
    {
    descrSize = descr.m_simultaneousCapabilities.GetSize();

     z = 0;
     for(j = 0; j < descrSize; j++)
     {
      H245_AlternativeCapabilitySet &alt = descr.m_simultaneousCapabilities[j];
      altSize = alt.GetSize();

      for(h = 0; h < altSize; h++)
      {
	if(mapTable.find(alt[h]) != mapTable.end())
	{
	  try
	  {
	    capSetH323.capRecv.addCapability(mapTable[alt[h]], y, z);
	  }
	  catch (XlatorException &mS)
	  {
	    VERR(log) << fn << ": Error adding to capability descriptor : " << mS.getDescription() << VERR_END(log);
	    VERR(log) << fn << ":  y  -  z : " << y << " - " << z << VERR_END(log);
	  }
	}
	else
	{
	  VDEBUG(log) << fn << ": The capability " << (int)alt[h] << " from cap descriptor is not in capTable..." << VDEBUG_END(log);
	}
      }
      
      if(capSetH323.capRecv.getAltCapSize(y, z) != 0)z++;
        
     }
     
     if(capSetH323.capRecv.getCapDescrSize(y) != 0)y++;
    }
  }

  VDEBUG(log) << fn << ": Filled out the descriptors..." << VDEBUG_END(log);
  //capabilitySetGetTestM(capSetH323);
  
  // TODO:
  // send EvtCapMgrRcvdTCS
  
  EvtCapMgrRcvdTCS rcvdTCS(this);
  rcvdTCS.start();
  
  VDEBUG(log) << fn << ": END ----------------" << VDEBUG_END(log);
}


void SM_CapManager::parseCapsSDP(const SDP &msgSDP) throw()
{
/*   const string     fn("CapabilityMap::handleSDP");
   VLog     	    log(fn);
   
   bool found = false, gFound = false, haveSesCon;
   int rtpPacketSize = 0, j = 0;
   int clockRate, pTypeCap = 0;
   std::string sdpAddress;
   Address address;
   GlobalCapTable::GlobalCapTable_p p;
   Capability::intSDP packetSize = 0;
   std::vector<int> pTypeCapList;
   std::vector<int> pTypeCapListBad;
   
   IPAddress ipAddress;
   
   VDEBUG(log) << fn << ": BEGIN ------------------ " << VDEBUG_END(log);
   
   vector<SdpMedia*> mediaList;
   mediaList = msgSDP.getMediaList();
   vector<SdpMedia*>::const_iterator mediaIterator = mediaList.begin();
   
   SdpConnection *sessionConnection = msgSDP.getConnection();
   haveSesCon = (sessionConnection != NULL);
   
   while(mediaIterator != mediaList.end())
   {

     SdpConnection *mediaConnection = (*mediaIterator)->getConnection();

     if(haveSesCon == false && mediaConnection == NULL)
       throw XlatorException("No connection parameters...", __FILE__, __LINE__);
     else
      if(mediaConnection == NULL)mediaConnection = sessionConnection;
      
     sdpAddress = mediaConnection->getUnicast();
     
     if(sdpAddress == "0.0.0.0")
     {
         // TODO
	 // received SDP with "c=0.0.0.0" event
	 //
	 
	 // TODO:
	 // here we should send
	 // CallTranslatorEM::startZeroSDP()
	 // call-back event
	 
	 sm_CapMgr.getEM()->startZeroSDP();
	 
	 
	 EvtCapMgrRcvdSDP0 rcvdSDP0(&sm_CapMgr);
	 rcvdSDP0.start();
	 return();
     }
     
     EvtCapMgrRcvdSDP rcvdSDP(&sm_CapMgr);
    
     if(mediaConnection->getAddressType() != AddressTypeIPV4)
     {
         VDEBUG(log) << fn << "Not a IPV4 address..." << VDEBUG_END(log);
         
	 // TODO
	 // received address not in a IPV4 form
	 // need to resolve it
	 
	 sm_CapMgr->getFeatureTCS->setAddress(sdpAddress);
	 rcvdSDP.start();
	 return();
     }
       
     VDEBUG(log) << fn << ": IP Address from SDP : " << sdpAddress << VDEBUG_END(log);
     
     ipAddress.setIP(sdpAddress.c_str());
     ipAddress.setPort((*mediaIterator)->getPort());
     
     address = ipAddress;
     
     try
     {
       // Setting RTP host/port
       sDescriptor.transportIDSip.setRTPAddr(address, 0, j);
     }
     catch (XlatorException)
     {
       VERR(log) << fn << ": Something wrong with setting RTP address..." << VERR_END(log);
     }
     
     ipAddress.setPort((*mediaIterator)->getPort()+1);
     
     address = ipAddress;
     
     try
     {
       // Setting RTCP host/port
       sDescriptor.transportIDSip.setRTCPAddr(address, 0, j);
     }
     catch (XlatorException)
     {
       VERR(log) << fn << ": Something wrong with setting RTCP address..." << VERR_END(log);
     }

    vector<int> *formatList = (*mediaIterator)->getFormatList();
    vector<int>::const_iterator fmtListIterator = formatList->begin();
   
    MediaAttributes *mediaAttrib;
    mediaAttrib = (*mediaIterator)->getMediaAttributes();
   
    rtpPacketSize = 0; 
    packetSize = 0;

    if(mediaAttrib != 0)
    {
      vector<ValueAttribute*> *valueAttribList = mediaAttrib->getValueAttributes();
      vector<ValueAttribute*>::const_iterator attribIterator = valueAttribList->begin();
     
      while(attribIterator != valueAttribList->end())
      {
        char *attribName = (*attribIterator)->getAttribute();
       
        if(strcmp(attribName, "ptime") == 0)
        {
          rtpPacketSize = Data((*attribIterator)->getValue());
	  break;
        }
        attribIterator++;
      }

      vector<SdpRtpMapAttribute*> *rtpMapAttribList = mediaAttrib->getmap();
      vector<SdpRtpMapAttribute*>::const_iterator rtpMapIterator = rtpMapAttribList->begin();
     
      found = false;

      if((rtpMapAttribList == 0)&&(formatList == 0))
      {
         VERR(log) << fn << ": No capabilities given in SDP message... " << VERR_END(log);
	 throw XlatorException("No capabilities given in SDP message...", __FILE__, __LINE__);
      }

      while(rtpMapIterator != rtpMapAttribList->end())
      {
       char *capName = (*rtpMapIterator)->getEncodingName();

       try
       {	
	 p = globalCapTable.getTableIter_ni(capName);
	 pTypeCap = p->ptypeCap;
       }
       catch (XlatorException)
       {
	 pTypeCapListBad.push_back((*rtpMapIterator)->getPayloadType());
         rtpMapIterator++;
	 continue;
       }
      
       if((*rtpMapIterator)->getClockRate()!=8000)
       {
         rtpMapIterator++;
	 VWARN(log) << fn << ": Clockrate for this capability is not 8000Hz, just skip it" << VWARN_END(log);
	 continue;
       }
       
       found = true;
       if(!gFound)gFound = true;
       
       Capability *capability = capabilityMap[pTypeCap];
       
       Capability *copy = capability->clone();
       clockRate = (*rtpMapIterator)->getClockRate()/1000;
       packetSize = rtpPacketSize*clockRate;

       if(rtpPacketSize != 0)
       {
         if(packetSize >= copy->getSamplesPerFrame())
	 {
	   copy->setParamRecvMsg(packetSize);
	 }
	 else 
	 {
	   VERR(log) << fn << "Packet Size in SDP is less than ms per Frame parameter "
	                   << "leaving this capability out..." << VERR_END(log);
	   fmtListIterator++;
	   continue;
	 }
       }
       
       VDEBUG(log) << fn << ": packetSize : " << packetSize << VDEBUG_END(log);
       VDEBUG(log) << fn << ": default pSize of copy : " << copy->getPacketSizeMilSec() << VDEBUG_END(log);
       VDEBUG(log) << fn << ": Here is the cap from rtpmap- to add : " << pTypeCap << VDEBUG_END(log);
       copy->assignCapabilityNumber(pTypeCap);
       capSetSip.h323CapTable.addCapTableEntry(copy);
       capSetSip.capRecv.addCapability(pTypeCap, 0, j);
       
       pTypeCapList.push_back(pTypeCap);
	    
       rtpMapIterator++;
      }
    } // if(mediaAttrib)
     
     VDEBUG(log) << fn << ": Out of rtpMapAtrribList... " << VDEBUG_END(log);
     
     while(fmtListIterator != formatList->end())
     {
        pTypeCap = *fmtListIterator;
	
	VDEBUG(log) << fn << ": pTypeCap from formatList : " << pTypeCap << VDEBUG_END(log);

	if(pTypeCap == 0)pTypeCap = 1;
	   
	try
        {	
	  p = globalCapTable.getTableIter_pt(pTypeCap);
        }
        catch (XlatorException)
        {
	  fmtListIterator++;
	  p++;
	  continue;
        }
        
	if((std::find(pTypeCapList.begin(), 
	        pTypeCapList.end(), pTypeCap) == pTypeCapList.end())&&
          (std::find(pTypeCapListBad.begin(), 
	        pTypeCapListBad.end(), pTypeCap) == pTypeCapListBad.end())&&
	  (p->nameCapH323!= -1))
	{
           found = true;

           if(!gFound)gFound = true;
       
           Capability *capability = capabilityMap[pTypeCap];
       
           Capability *copy = capability->clone();
           clockRate = 8;

           VDEBUG(log) << fn << ": Here is the packetSize : " << rtpPacketSize << VDEBUG_END(log);
           VDEBUG(log) << fn << ": default pSize of copy : " << copy->getPacketSizeMilSec() << VDEBUG_END(log);
	   
	   packetSize = rtpPacketSize*clockRate;
	   
	   if(rtpPacketSize != 0)
	   {
	     if(packetSize >= copy->getSamplesPerFrame())
	     {
	       copy->setParamRecvMsg(packetSize);
	     }
	     else 
	     {
	       VERR(log) << fn << "Packet Size in SDP is less than ms per Frame parameter "
	                   << "leaving this capability out..." << VERR_END(log);
	       fmtListIterator++;
	       continue;
	     }
	   }
	     
	     VDEBUG(log) << fn << ": Here is the cap from fmtList to add : " << pTypeCap << VDEBUG_END(log);
	   
	   copy->assignCapabilityNumber(pTypeCap);
	   capSetSip.h323CapTable.addCapTableEntry(copy);
           capSetSip.capRecv.addCapability(pTypeCap, 0, j);
	}

        fmtListIterator++;
     }

     VDEBUG(log) << fn << ": Out of formatList... " << VDEBUG_END(log);

    //if(found)
    //{


     j++;
    //} // if(found)
   
     VDEBUG(log) << fn << ": While mediaIterator... " << VDEBUG_END(log);
     
     mediaIterator++;
   } // while mediaIterator

   if(!gFound)throw XlatorException("No capability supported...", __FILE__, __LINE__);

   int sTableSize = capSetSip.h323CapTable.getCapabilityTable().size();
   VDEBUG(log) << fn << ": Now the size of Sip capTable is : " << sTableSize << VDEBUG_END(log);
   
   receivedSDP = true;

   capabilitySetGetTestM(capSetSip);
   
   rcvdSDP.start();

   VDEBUG(log) << fn << ": END ------------------ " << VDEBUG_END(log);

*/
}

void SM_CapManager::setControlChannel()
{
    // set Control Channel event
    
    EvtCapMgrSetCC setCC(this);
    setCC.start();
    
}
