/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const H323AdmissionEvent_cxx_Version = 
    "$Id: H323AdmissionEvent.cxx,v 1.11 2001/06/01 06:07:07 icahoon Exp $";


#include "H323AdmissionEvent.hxx"


using Vocal::H323Events::H323AdmissionEvent;
using Vocal::H323Events::H323CallEvent;
using Vocal::SipH323Info::EndpointInfo;
using Vocal::Transport::IPAddress;


H323AdmissionEvent::H323AdmissionEvent(
    FifoBase< Sptr<Event> > &	fifo
) 
    : 	H323CallEvent(fifo),
    	destCallSignalAddress_(0),
    	destAlias_(0),
    	destEndpointInfo_(0),
    	srcCallSignalAddress_(0),
    	srcAlias_(0),
    	srcEndpointInfo_(0),
        srcDisplay_("")
{
}


H323AdmissionEvent::H323AdmissionEvent(
    const CallIdentifier    	&   callId, 
    FifoBase< Sptr<Event> > 	&   fifo
)
    : 	H323CallEvent(callId, fifo),
    	destCallSignalAddress_(0),
    	destAlias_(0),
    	destEndpointInfo_(0),
    	srcCallSignalAddress_(0),
    	srcAlias_(0),
    	srcEndpointInfo_(0),
        srcDisplay_("")
{
}



H323AdmissionEvent::H323AdmissionEvent(const H323AdmissionEvent & src)
    :	H323CallEvent(src),
    	destCallSignalAddress_(src.destCallSignalAddress_),
    	destAlias_(src.destAlias_),
    	destEndpointInfo_(src.destEndpointInfo_),
    	srcCallSignalAddress_(src.srcCallSignalAddress_),
    	srcAlias_(src.srcAlias_),
    	srcEndpointInfo_(src.srcEndpointInfo_),
        srcDisplay_(src.srcDisplay_)
{
}


H323AdmissionEvent::~H323AdmissionEvent()
{
}


H323AdmissionEvent & 
H323AdmissionEvent::operator=(const H323AdmissionEvent & src)
{
    if ( this != & src )
    {
    	H323CallEvent::operator=(src);
    	destCallSignalAddress_ = src.destCallSignalAddress_;
    	destAlias_ = src.destAlias_;
    	destEndpointInfo_ = src.destEndpointInfo_;
    	srcCallSignalAddress_ = src.srcCallSignalAddress_;
    	srcAlias_ = src.srcAlias_;
    	srcEndpointInfo_ = src.srcEndpointInfo_;
        srcDisplay_ = src.srcDisplay_;
    }
    return ( *this );
}


void	    	    	    
H323AdmissionEvent::destCallSignalAddress(
    const IPAddress & destCallSignalAddress
)
{
    destCallSignalAddress_ = new IPAddress(destCallSignalAddress);
}


Sptr<IPAddress>
H323AdmissionEvent::destCallSignalAddress() const
{
    return ( destCallSignalAddress_ );
}


void	    	    	    
H323AdmissionEvent::destAlias(
    const H225_ArrayOf_AliasAddress & destAlias__
)
{
    destAlias_ = new H225_ArrayOf_AliasAddress(destAlias__);
}


Sptr<H225_ArrayOf_AliasAddress>
H323AdmissionEvent::destAlias() const
{
    return ( destAlias_ );
}


void  	    	    	    
H323AdmissionEvent::destEndpointInfo(Sptr<EndpointInfo> destEndpointInfo__)
{
    destEndpointInfo_ = destEndpointInfo__;
}


Sptr<EndpointInfo>  	    
H323AdmissionEvent::destEndpointInfo() const
{
    return ( destEndpointInfo_ );
}


void	    	    	    
H323AdmissionEvent::srcCallSignalAddress(
    const IPAddress & srcCallSignalAddress
)
{
    srcCallSignalAddress_ = new IPAddress(srcCallSignalAddress);
}


Sptr<IPAddress>
H323AdmissionEvent::srcCallSignalAddress() const
{
    return ( srcCallSignalAddress_ );
}


void	    	    	    
H323AdmissionEvent::srcAlias(
    const H225_ArrayOf_AliasAddress & srcAlias__
)
{
    srcAlias_ = new H225_ArrayOf_AliasAddress(srcAlias__);
}


Sptr<H225_ArrayOf_AliasAddress>
H323AdmissionEvent::srcAlias() const
{
    return ( srcAlias_ );
}


void  	    	    	    
H323AdmissionEvent::srcEndpointInfo(Sptr<EndpointInfo> srcEndpointInfo__)
{
    srcEndpointInfo_ = srcEndpointInfo__;
}


Sptr<EndpointInfo>  	    
H323AdmissionEvent::srcEndpointInfo() const
{
    return ( srcEndpointInfo_ );
}


void    
H323AdmissionEvent::srcDisplay(const string & display)
{
    srcDisplay_ = display;
}


string  
H323AdmissionEvent::srcDisplay() const
{
    return ( srcDisplay_ );
}


const char* const   	    
H323AdmissionEvent::name() const
{
    return ( "H323AdmissionEvent" );
}


ostream &   	    	    
H323AdmissionEvent::writeTo(ostream & out) const
{
    out << name() << ", ";

    H323CallEvent::writeTo(out);
    
    if ( destCallSignalAddress_ != 0 )
    {
    	out << ", destCallSignalAddress = " << *destCallSignalAddress_;
    }
    if ( destAlias_ != 0 )
    {
    	out << ", destAlias = " << *destAlias_;
    }
    if ( destEndpointInfo_ != 0 )
    {
    	out << ", destEndpointInfo = " << *destEndpointInfo_;
    }
    if ( srcCallSignalAddress_ != 0 )
    {
    	out << ", srcCallSignalAddress = " << *srcCallSignalAddress_;
    }
    if ( srcAlias_ != 0 )
    {
    	out << ", srcAlias = " << *srcAlias_;
    }
    if ( srcEndpointInfo_ != 0 )
    {
    	out << ", srcEndpointInfo = " << *srcEndpointInfo_;
    }

    return ( out );
}
