#if !defined(H323_ADMISSION_EVENT_DOT_H)
#define H323_ADMISSION_EVENT_DOT_H

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const H323AdmissionEvent_hxx_Version = 
    "$Id: H323AdmissionEvent.hxx,v 1.11 2001/06/01 06:07:07 icahoon Exp $";


#include "H323CallEvent.hxx"
#include "Sptr.hxx"
#include "EndpointInfo.hxx"
#include "IPAddress.hxx"
#include "CallIdentifier.hxx"
#include <string>


/** Vovida Open Communication Application Library<br><br>
 */
namespace Vocal
{


/** H.323 specific internal events.<br><br>
 */
namespace H323Events
{


using Vocal::Transport::IPAddress;
using Vocal::SipH323Info::EndpointInfo;


/** Admission event.<br><br>
 *
 *  @see    Vocal::H323Events::H323CallEvent
 *  @see    Vocal::SipH323Info::EndpointInfo
 *  @see    Vocal::Transport::IPAddress
 */
class H323AdmissionEvent : public Vocal::H323Events::H323CallEvent
{
   
    public:


    	/** Create an event given the reply fifo.
	 */
	H323AdmissionEvent(FifoBase< Sptr<Event> > & fifo);


    	/** Create an event given the associated call identifier
	 *  and the reply fifo.
	 */
    	H323AdmissionEvent(
	    const CallIdentifier    	&   callId, 
	    FifoBase< Sptr<Event> > 	&   fifo
	);


    	/** Copy constructor.
	 */
	H323AdmissionEvent(const H323AdmissionEvent &);


    	/** Virtual destructor
	 */
	virtual     ~H323AdmissionEvent();


    	/** Assignment operator
	 */
	H323AdmissionEvent & operator=(const H323AdmissionEvent &);


    	/** If this is set when sent to the gatekeeper, the gatekeeper
    	 *  will attempt to find the EndpointInfo for the destination
    	 *  based on the call signal address.
	 *
    	 *  This should be set on incoming calls from SIP.
    	 *  This has priority over the destAlias.
	 */
	void	    	destCallSignalAddress(const IPAddress &);


    	/** Access the destination call signaling address.
	 */
	Sptr<IPAddress> destCallSignalAddress() const;


    	/** If this is set when sent to gatekeeper, the gatekeeper
    	 *  will attempt to find the EndpointInfo for the destination
    	 *  based on the alias. 
    	 *  
    	 *  This should be set on incoming calls from the SIP.
	 */	
	void	    	destAlias(const H225_ArrayOf_AliasAddress &);
	

	/** Access the destination alias.
    	 */
	Sptr<H225_ArrayOf_AliasAddress>   destAlias() const;


    	/** If the destCallSignalAddress or destAlias was set, upon 
    	 *  return of the event, the endpointInfo for the destination 
    	 *  should be available if the gatekeeper knows about the 
    	 *  endpoint.
	 */
	void	    	destEndpointInfo(Sptr<EndpointInfo>);


    	/** Access the endpoint information for the destination.
	 */
    	Sptr<EndpointInfo>  destEndpointInfo() const;


    	/** If this is set when sent to the gatekeeper, the gatekeeper
    	 *  will attempt to find the EndpointInfo for the source
    	 *  based on the call signal address.
    	 *  
    	 *  This should be set on outgoing calls, since NetMeeting
    	 *  sends an incorrect address.
	 */
	void	    	srcCallSignalAddress(const IPAddress &);


    	/** Access the source call signaling address.
	 */
	Sptr<IPAddress> srcCallSignalAddress() const;


    	/** The source alias address. Used as payload.
	 */	
	void	    	srcAlias(const H225_ArrayOf_AliasAddress &);
	

	/** Access the source alias.
    	 */
	Sptr<H225_ArrayOf_AliasAddress>   srcAlias() const;


    	/** If the srcCallSignalAddress was set, upon return of the 
    	 *  event, the endpointInfo for the source should be available
    	 *  if the gatekeeper knows about the endpoint.
	 */	
    	void	    	srcEndpointInfo(Sptr<EndpointInfo>);
	
	
	/** Access the source endpoint information.
	 */
	Sptr<EndpointInfo>  srcEndpointInfo() const;


        /** The source display. Used as payload.
         */
        void    srcDisplay(const string & display);
        

        /** Access the source endpoint display.
         */
        string  srcDisplay() const;
        

    	/** The name of this class of event, "H323AdmissionEvent".
	 */
    	virtual const char* const   name() const;
	
	
	/** Write the event to an ostream.
	 */
    	virtual ostream &   	    writeTo(ostream &) const;
	

    private:

	Sptr<IPAddress>     	    	destCallSignalAddress_;
    	Sptr<H225_ArrayOf_AliasAddress> destAlias_;
	Sptr<EndpointInfo>  	    	destEndpointInfo_;
	Sptr<IPAddress>     	    	srcCallSignalAddress_;
    	Sptr<H225_ArrayOf_AliasAddress> srcAlias_;
	Sptr<EndpointInfo>  	    	srcEndpointInfo_;
        string                          srcDisplay_;        
};


} // namespace H323Events
} // namespace Vocal


#endif // !defined(H323_ADMISSION_EVENT_DOT_H)
