
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const H323CallEvent_cxx_Version = 
    "$Id: H323CallEvent.cxx,v 1.9 2001/03/21 03:41:40 icahoon Exp $";


#include "H323CallEvent.hxx"


using Vocal::H323Events::H323CallEvent;
using Vocal::H323Events::H323Event;
using Vocal::SipH323Info::CallIdentifier;


H323CallEvent::H323CallEvent(FifoBase< Sptr<Event> > & fifo)
    :	H323Event(fifo)
{
}


H323CallEvent::H323CallEvent(
    const CallIdentifier    	&   callId, 
    FifoBase< Sptr<Event> > 	&   fifo
)
    :	H323Event(fifo),
    	callIdentifier_(callId)
{
}


H323CallEvent::H323CallEvent(const H323CallEvent & src)
    :	H323Event(src),
    	callIdentifier_(src.callIdentifier_)
{
}

					
H323CallEvent::~H323CallEvent()
{
}


H323CallEvent & 
H323CallEvent::operator=(const H323CallEvent & src)
{
    if ( this != &src )
    {
    	H323Event::operator=(src);
    	callIdentifier_ = src.callIdentifier_;
    }
    return ( *this );
}


void	    	    	
H323CallEvent::callIdentifier(const CallIdentifier & callId)
{
    callIdentifier_ = callId;
}


const CallIdentifier &	    
H323CallEvent::callIdentifier() const
{
    return ( callIdentifier_ );
}


const char* const   	    
H323CallEvent::name() const
{
    return ( "H323CallEvent" );
}


ostream &           	    
H323CallEvent::writeTo(ostream & out) const
{
    H323Event::writeTo(out);
    out << ",\ncallId = {\n\t"
        << " conferenceId = ";
                 
    callIdentifier_.conferenceIdentifier().PrintOn(out); 

    out << ", callId = ";
    
    callIdentifier_.callIdentifier().PrintOn(out);

    return ( out << setprecision(0) << "\n}\n" );
}
