#if !defined(H323_EVENT_DOT_H)
#define H323_EVENT_DOT_H

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const H323Event_hxx_Version = 
    "$Id: H323Event.hxx,v 1.10 2001/06/01 06:07:07 icahoon Exp $";


#include "Event.hxx"
#include "Sptr.hxx"
#include "FifoBase.h"
#include "TransportCommon.hxx"


/** Vovida Open Communication Application Library<br><br>
 */
namespace Vocal
{


/** H.323 specific internal events.<br><br>
 */
namespace H323Events
{


using Vocal::Signaling::Event;


/** Common class for all H.323 events.<br><br>
 *
 *  @see    Vocal::ReturnCode
 *  @see    Vocal::Signaling::Event
 *  @see    FifoBase
 *  @see    Sptr
 */
class H323Event : public Vocal::Signaling::Event
{

    public:


    	/** Create the event given the reply fifo.
	 */
    	H323Event(FifoBase< Sptr<Event> > &);


    	/** Copy constructor.
	 */
    	H323Event(const H323Event &);
					

    	/** Virtual destructor.
	 */
	virtual     	~H323Event();


    	/** Assignment operator.
	 */
    	H323Event & 	operator=(const H323Event &);


    	/** Set the returnCode when using this event as the reply.
	 */
    	void	    	returnCode(ReturnCode);


    	/** Access the returnCode associated with the reply.
	 */
	ReturnCode  	returnCode() const;


    	/** Access the fifo to send the reply to this event.
	 */
	FifoBase< Sptr<Event> > &   replyFifo() const;


    	/** The name of this class of event, "H323Event".
	 */
    	virtual const char* const   name() const;


    	/** Write the event to an ostream.
	 */
	virtual ostream &           writeTo(ostream &) const;

    private:
    	    	ReturnCode  	    	    returnCode_;
		FifoBase< Sptr<Event> > *   fifo_;	    
};


} // namespace H323Events
} // namespace Vocal


#endif // !defined(H323_EVENT_DOT_H)
