#if !defined(VOCAL_ADMISSION_POLICY_HXX)
#define VOCAL_ADMISSION_POLICY_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const AdmissionPolicy_hxx_Version = 
    "$Id: AdmissionPolicy.hxx,v 1.11 2001/10/13 01:15:53 icahoon Exp $";


#include "ptlib.h"
#include "h323pdu.h"
#include "RASPolicy.hxx"
#include "Sptr.hxx"
#include "IPAddress.hxx"
#include "EndpointInfo.hxx"


/** Vovida Open Communication Application Library<br><br>
 */
namespace Vocal
{


/** H.323 Gatekeeper<br><br>
 */
namespace H323GK
{


using Vocal::Transport::IPAddress;
using Vocal::SipH323Info::EndpointInfo;


/** Base policy for determining how to handle RAS admission and disengage
 *  messages (ARQ, ACF, ARJ, DRQ, DCF, and DRJ).<br><br>
 *
 *  The default behavior is to do nothing, rather to let the user override the
 *  methods.
 *
 *  @see    Vocal::H323GK::RASPolicy
 *  @see    Vocal::H323GK::RASGKProtocol
 *  @see    Vocal::SipH323Info::GatekeeperInfo
 *  @see    Vocal::SipH323Info::EndpointInfoMap
 *  @see    Vocal::SipH323Info::EndpointInfo
 *  @see    Vocal::Services::Service
 *  @see    Vocal::Transport::IPAddress
 */
class AdmissionPolicy : public Vocal::H323GK::RASPolicy
{

    public:

    	
    	/** Construct given the controling service, the associated ras gk
	 *  protocol, the configuration information for the gatekeeper and
	 *  the table of registered endpoints.
	 */
    	AdmissionPolicy(
	    Service &,
	    RASGKProtocol &,
	    GatekeeperInfo &,
	    EndpointInfoMap &
	);


    	/** Virtual destructor
	 */
	virtual		    ~AdmissionPolicy();
			

    	/** Callback to handle an incoming admission request.
	 */
    	virtual void        processAdmissionRequest(
	    	    	    	Sptr<H323RasPDU>    rasPDU, 
			    	Sptr<IPAddress>     remoteAddr);


    	/** Callback to handle an incoming disengage request.
	 */
    	virtual void   	    processDisengageRequest(
	    	    	    	Sptr<H323RasPDU>    rasPDU,
			    	Sptr<IPAddress>     remoteAddr);
			    

    	/** Callback to handle an incoming disengage response.
	 */
    	virtual void   	    processDisengageResponse(
	    	    	    	Sptr<H323RasPDU>    rasPDU,
			    	Sptr<IPAddress>     remoteAddr);
	    
    	
	/** To initiate an outgoing disengage request.
	 */
    	virtual void	    disengage(
				H225_DisengageReason 	    &	reason,
				Sptr<EndpointInfo>  	    	endpointInfo,
			    	Sptr<IPAddress>     	    	remoteAddr);

    	/** Write the policy to an ostream.
	 */
	virtual ostream &   writeTo(ostream &) const;
};


} // namespace H323GK
} // namespace Vocal


#endif // !defined(VOCAL_ADMISSION_POLICY_HXX)
