#if !defined(VOCAL_DEFAULT_RAS_POLICY_BUILDER_HXX)
#define VOCAL_DEFAULT_RAS_POLICY_BUILDER_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const DefaultRASPolicyBuilder_hxx_Version = 
    "$Id: DefaultRASPolicyBuilder.hxx,v 1.9 2001/10/13 01:15:53 icahoon Exp $";


#include "ptlib.h"
#include "h323pdu.h"
#include <string>
#include "IPAddress.hxx"
#include "RASPolicyBuilder.hxx"


/** Vovida Open Communication Application Library<br><br>
 */
namespace Vocal
{


/** H.323 Gatekeeper<br><br>
 */
namespace H323GK
{


/** Default RAS Policy factory.<br><br>
 *
 *  This creates a DefaultDiscoverypolicy, a DefaultRegistrationPolicy
 *  and a DefaultAdmissionPolicy.
 *
 *  If the default policies are extended, then you need to create
 *  an accompanying ras policy builder to create those policies.
 *
 *  @see    Vocal::H323GK::RASPolicyBuilder
 *  @see    Vocal::H323GK::DefaultDiscoveryPolicy
 *  @see    Vocal::H323GK::DefaultRegistrationPolicy
 *  @see    Vocal::H323GK::DefaultAdmissionPolicy
 *  @see    Vocal::H323GK::DiscoveryPolicy
 *  @see    Vocal::H323GK::RegistrationPolicy
 *  @see    Vocal::H323GK::AdmissionPolicy
 *  @see    Vocal::H323GK::LocationPolicy
 *  @see    Vocal::H323GK::ResourcesAvailablePolicy
 *  @see    Vocal::H323GK::RASGKProtocol
 *  @see    Vocal::SipH323Info::GatekeeperInfo
 *  @see    Vocal::SipH323Info::EndpointInfoMap
 *  @see    Vocal::Signaling::Service
 */
class DefaultRASPolicyBuilder : public Vocal::H323GK::RASPolicyBuilder
{

    public:


    	/** Default constructor
	 */
    	DefaultRASPolicyBuilder();


    	/** Virtual destructor
	 */
	virtual 		    ~DefaultRASPolicyBuilder();
			

    	/** Create the DefaultDiscoveryPolicy. It's the user's responsibility
	 *  to destroy the policy.
	 */
    	virtual DiscoveryPolicy *   	buildDiscoveryPolicy(
	    	    	    	    	    Service &,
	    	    	    	    	    RASGKProtocol &,
					    GatekeeperInfo &,
					    EndpointInfoMap &
					);


    	/** Create the DefaultRegistrationPolicy. It's the user's responsibility
	 *  to destroy the policy.
	 */
    	virtual RegistrationPolicy *	buildRegistrationPolicy(
	    	    	    	    	    Service &,
	    	    	    	    	    RASGKProtocol &,
					    GatekeeperInfo &,
					    EndpointInfoMap &
					);

    	
	/** Create the DefaultAdmissionPolicy. It's the user's responsibility
	 *  to destroy the policy.
	 */
	virtual AdmissionPolicy *	buildAdmissionPolicy(
	    	    	    	    	    Service &,
	    	    	    	    	    RASGKProtocol &,
					    GatekeeperInfo &,
					    EndpointInfoMap &
					);


	/** Create the DefaultLocationPolicy. It's the user's responsibility
	 *  to destroy the policy.
	 */
	virtual LocationPolicy *	buildLocationPolicy(
	    	    	    	    	    Service &,
	    	    	    	    	    RASGKProtocol &,
					    GatekeeperInfo &,
					    EndpointInfoMap &
					);


	/** Create the DefaultResourcesAvaliablePolicy. It's the user's 
         *  responsibility to destroy the policy.
	 */
	virtual ResourcesAvailablePolicy * buildResourcesAvailablePolicy(
	    	    	    	    	    Service &,
	    	    	    	    	    RASGKProtocol &,
					    GatekeeperInfo &,
					    EndpointInfoMap &
					);
};


} // namespace H323GK
} // namespace Vocal


#endif // !defined(VOCAL_DEFAULT_RAS_POLICY_BUILDER_HXX)
