#if !defined(VOCAL_H323_GATEKEEPER_HXX)
#define VOCAL_H323_GATEKEEPER_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const H323Gatekeeper_hxx_Version = 
    "$Id: H323Gatekeeper.hxx,v 1.9 2001/10/26 21:47:54 icahoon Exp $";


#include "PollFifoService.hxx"
#include "PollFifo.hxx"
#include "Sptr.hxx"
#include "Event.hxx"
#include "EndpointInfoMap.hxx"


// Forward declaration
//
namespace Vocal {
    namespace Services {
    	class ControlEvent;
    }
    namespace SipH323Info {
    	class GatekeeperInfo;
    }
}


/** Vovida Open Communication Application Library<br><br>
 */
namespace Vocal
{


/** H.323 Gatekeeper<br><br>
 */
namespace H323GK
{


using Vocal::Services::Event;
using Vocal::Services::ControlEvent;
using Vocal::SipH323Info::GatekeeperInfo;
using Vocal::SipH323Info::EndpointInfoMap;


// Forward declaration
//
class RASPolicyBuilder;
class RASMCastGKProtocol;
class RASGKProtocol;
class DiscoveryPolicy;
class RegistrationPolicy;
class AdmissionPolicy;
class LocationPolicy;
class ResourcesAvailablePolicy;


/** The H323Gatekeeper service.<br><br>
 *
 *  @see    Vocal::H323GK::RASPolicyBuilder
 *  @see    Vocal::H323GK::RASMCastGKProtocol
 *  @see    Vocal::H323GK::RASGKProtocol
 *  @see    Vocal::H323GK::DiscoveryPolicy
 *  @see    Vocal::H323GK::RegistrationPolicy
 *  @see    Vocal::H323GK::AdmissionPolicy
 *  @see    Vocal::H323GK::LocationPolicy
 *  @see    Vocal::H323GK::ResourcesAvailablePolicy
 *  @see    Vocal::SipH323Info::GatekeeperInfo
 *  @see    Vocal::SipH323Info::EndpointInfoMap
 *  @see    Vocal::Services::ControlEvent
 *  @see    Vocal::Services::PollFifoService
 *  @see    FifoBase
 *  @see    Sptr
 *  @see    Event
 */
class H323Gatekeeper : public Vocal::Services::PollFifoService
{

    public:


    	/** Create the service given the ras policy builder and the 
	 *  gatekeeper configuration info.
	 */
    	H323Gatekeeper(
	    RASPolicyBuilder    	&   rasPolicyBuilder,
	    GatekeeperInfo	    	&   gatekeeperInfo
	);


    	/** Virtual destructor
	 */
	virtual ~H323Gatekeeper();


    	/** Start the service when a start event is sent. The gatekeeper
	 *  doesn't process messages until this is sent.
	 */
	ReturnCode  onStart();


    	/** Stop the service when a stop event is sent. The service can
	 *  be restarted. The gatekeeper doesn't process messages when 
	 *  stopped.
	 */
	ReturnCode  onStop();


    	/** Shutdown the service, exiting the thread.
	 */
	ReturnCode  onShutdown();


    private:

    	void	    clear();

    	RASPolicyBuilder            &   myRasPolicyBuilder;
	GatekeeperInfo	    	    &   myGatekeeperInfo;
	EndpointInfoMap     	        myEndpointInfoMap;

    	RASMCastGKProtocol  	    *   myRasMcastGkProtocol;
    	RASGKProtocol  	    	    *   myRasGkProtocol;

	DiscoveryPolicy	    	    *   myDiscoveryPolicy;
	RegistrationPolicy  	    *   myRegistrationPolicy;
	AdmissionPolicy     	    *   myAdmissionPolicy;
        LocationPolicy              *   myLocationPolicy;
        ResourcesAvailablePolicy    *   myResourcesAvailablePolicy;
};


} // namespace H323GK
} // namespace Vocal


#endif // !defined(VOCAL_H323_GATEKEEPER_HXX)
