/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const Alias_cxx_Version = 
    "$Id: Alias.cxx,v 1.7 2000/12/18 23:47:24 bko Exp $";


#include "Alias.hxx"
#include <cstring>


using Vocal::SipH323Info::Alias;
using Vocal::Transport::IPAddress;


Alias::Alias()
    :	e164_("")
{
}


Alias::Alias(const H225_ArrayOf_AliasAddress 	& alias__)
    :	alias_(alias__)
{
    updatePhone();
}


Alias::Alias(const Alias & src)
    :	alias_(src.alias_),
    	e164_(src.e164_)
{
}


Alias::~Alias()
{
}


Alias &    	    	    	    
Alias::operator=(const Alias & src)
{
    if ( this != &src )
    {
    	alias_ = src.alias_;
	e164_ = src.e164_;
    }
    return ( *this );
}


void
Alias::alias(const H225_ArrayOf_AliasAddress & alias__)
{
    alias_ = alias__;
    updatePhone();
}


const H225_ArrayOf_AliasAddress &   
Alias::alias() const
{
    return ( alias_ );
}


void	    	    	    	    
Alias::user(const string & user_)
{
    int     i = findIndex(H225_AliasAddress::e_h323_ID);

    if ( i < 0 )
    {
	i = alias_.GetSize();
	alias_.SetSize(i + 1);
    }

    alias_[i].SetTag(H225_AliasAddress::e_h323_ID);
    (PASN_BMPString &)alias_[i] = (PString)user_.c_str();
}


string	    	    	    	    
Alias::user() const
{
    string  	user_;

    int     	i = findIndex(H225_AliasAddress::e_h323_ID);

    if ( i >= 0 )
    {
    	PASN_BMPString & bmph323 = (PASN_BMPString &)alias_[i];
    	user_ = string((const char*)((PString)bmph323));
    }    

    return ( user_ );
}


bool
Alias::hasUser() const
{
    return ( findIndex(H225_AliasAddress::e_h323_ID) < 0 ? false : true );
}


void	    	    	    	    
Alias::phone(const string & phone_)
{
    string validChars = "0123456789#*";

    e164_ = phone_;
    
    for(size_t i = 0; i < e164_.size(); i++)
    {
    	char	c = e164_.at(i);

	if ( c == 'p' )
	{
	    e164_.replace(i, 1, ',');
	}
	else if ( validChars.find(c) == string::npos )
	{
	    e164_.erase(i, 1);
	    i--;
	}
    }
    
    int     i = findIndex(H225_AliasAddress::e_e164);

    if ( i < 0 )
    {
	i = alias_.GetSize();
	alias_.SetSize(i + 1);
    }

    alias_[i].SetTag(H225_AliasAddress::e_e164);
    (PASN_IA5String &)alias_[i] = (PString)phone_.c_str();
}


string	    	    	    	    
Alias::phone() const
{
    return ( e164_ );
}


bool    	    	
Alias::hasPhone() const
{
    return ( findIndex(H225_AliasAddress::e_e164) < 0 ? false : true );
}


void
Alias::transport(const IPAddress & transport_)
{
    int     i = findIndex(H225_AliasAddress::e_transportID);

    if ( i < 0 )
    {
	i = alias_.GetSize();
	alias_.SetSize(i + 1);
    }

    alias_[i].SetTag(H225_AliasAddress::e_transportID);

    H225_TransportAddress   	    &	transAddr = alias_[i];

    transAddr.SetTag(H225_TransportAddress::e_ipAddress);
    
    H225_TransportAddress_ipAddress & 	ipAddr = transAddr;

    memcpy( &ipAddr.m_ip[0], 
    	    &(transport_.getIPAddress().sin_addr.s_addr), 
	    4);

    ipAddr.m_port = transport_.getPort();
}


IPAddress   	    	    	    
Alias::transport() const
{
    IPAddress	transport_;

    int     	i = findIndex(H225_AliasAddress::e_transportID);

    if ( i >= 0 )
    {
    	H225_TransportAddress	&   transAddr = alias_[i];

	if ( transAddr.GetTag() == H225_TransportAddress::e_ipAddress )
	{
    	    H225_TransportAddress_ipAddress & ipAddr = transAddr;
            transport_.setIP(ipAddr.m_ip.GetPointer());
	    transport_.setPort(ipAddr.m_port);
	}
    }    

    return ( transport_ );
}
	

bool    	    	
Alias::hasTransport() const
{
    return ( findIndex(H225_AliasAddress::e_transportID) < 0 ? false : true );
}


void	    	    	    	    
Alias::url(const string & url_)
{
    int     i = findIndex(H225_AliasAddress::e_url_ID);

    if ( i < 0 )
    {
	i = alias_.GetSize();
	alias_.SetSize(i + 1);
    }

    alias_[i].SetTag(H225_AliasAddress::e_url_ID);
    (PASN_IA5String &)alias_[i] = (PString)url_.c_str();
}


string	    	    	    	    
Alias::url() const
{
    string  url_;

    int     	i = findIndex(H225_AliasAddress::e_h323_ID);

    if ( i >= 0 )
    {
	PASN_IA5String &ia5url = (PASN_IA5String &)alias_[i];
	url_ = string((const char*)((PString)ia5url));
    }    
    
    return ( url_ );
}


bool
Alias::hasUrl() const
{
    return ( findIndex(H225_AliasAddress::e_url_ID) < 0 ? false : true );
}


void	    	    	    	    
Alias::email(const string & email_)
{
    int     i = findIndex(H225_AliasAddress::e_email_ID);

    if ( i < 0 )
    {
	i = alias_.GetSize();
	alias_.SetSize(i + 1);
    }

    alias_[i].SetTag(H225_AliasAddress::e_email_ID);
    (PASN_IA5String &)alias_[i] = (PString)email_.c_str();
}


string	    	    	    	    
Alias::email() const
{
    string  email_;

    int     	i = findIndex(H225_AliasAddress::e_email_ID);

    if ( i >= 0 )
    {
	PASN_IA5String &ia5email = (PASN_IA5String &)alias_[i];
	email_ = string((const char*)((PString)ia5email));
    }    
    
    return ( email_ );
}
		

bool
Alias::hasEmail() const
{
    return ( findIndex(H225_AliasAddress::e_email_ID) < 0 ? false : true );
}


int 	    	    	    	    
Alias::findIndex(unsigned int choice) const
{
    int     i, aliasSize = alias_.GetSize();
    bool    found = false;

    for ( i = 0; i < aliasSize && !found; i++ )
    {
    	if ( alias_[i].GetTag() == choice )
	{
	    found = true;
	    break;
	}
    }

    return ( found ? i : -1 );
}


bool	    	    	
Alias::isEmpty() const
{
    return ( alias_.GetSize() == 0 );
}


void	    	    	
Alias::clear()
{
    alias_.SetSize(0);
    e164_ = "";
}


bool	    	    	    	    
Alias::operator<(const Alias & src) const
{
    if	( this == &src )
    {
	return ( false );
    }
    return ( e164_ < src.e164_ );
}


bool	    	    	    	    
Alias::operator==(const Alias & src) const
{
    if ( this == &src )
    {
    	return ( true );
    }

    if ( e164_ == "" || src.e164_ == "" )
    {
    	return ( false );
    }

    return ( e164_ == src.e164_ );
}


ostream &           	    
Alias::writeTo(ostream & out) const
{
    return ( out << e164_ );
}


void
Alias::updatePhone()
{
    if ( e164_ == "" )
    {
	int 	i = findIndex(H225_AliasAddress::e_e164);

	if ( i >= 0 )
	{
	    PASN_IA5String & e164 = (PASN_IA5String &)alias_[i];
	    e164_ = string((const char*)((PString)e164));
	}    
    }
}
