/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const CallIdentifier_cxx_Version = 
    "$Id: CallIdentifier.cxx,v 1.10 2000/12/18 23:47:24 bko Exp $";


#include "CallIdentifier.hxx"
#include "VLog.hxx"
#include <string.h>


using Vocal::SipH323Info::CallIdentifier;
using Vocal::Logging::VLog;


CallIdentifier::CallIdentifier()
{
}


CallIdentifier::CallIdentifier(
    const H225_ConferenceIdentifier 	&   confId,
    const H225_CallIdentifier 	    	&   callId
)
    :	confId_(confId),
    	callId_(callId)
{
}


CallIdentifier::CallIdentifier(const CallIdentifier & src)
    :	confId_(src.confId_),
	callId_(src.callId_)
{
}


CallIdentifier::~CallIdentifier()
{
}


CallIdentifier &    	    
CallIdentifier::operator=(const CallIdentifier & src)
{
    if ( this != &src )
    {
    	confId_ = src.confId_;
	callId_ = src.callId_;
    }
    return ( *this );
}


H225_ConferenceIdentifier   
CallIdentifier::conferenceIdentifier() const
{
    return ( confId_ );
}


H225_CallIdentifier 	    
CallIdentifier::callIdentifier() const
{
    return ( callId_ );
}


bool	    	    	    
CallIdentifier::operator<(const CallIdentifier & src) const
{
    size_t  	    leftSize = confId_.GetSize(),
    	    	    rightSize = src.confId_.GetSize();
		    
    assert( leftSize == rightSize );
    
    const BYTE  *   left = confId_,
    	    	*   right = src.confId_;

    int result = memcmp(left, right, leftSize);
    
    if ( result == 0 )
    {
	leftSize = callId_.m_guid.GetSize();
    	rightSize = src.callId_.m_guid.GetSize();

	assert( leftSize == rightSize );

    	left = callId_.m_guid;
    	right = src.callId_.m_guid;

	return ( memcmp(left, right, leftSize) < 0 );
    }
    
    return ( result < 0 );
}

	
bool	    	    	    
CallIdentifier::operator==(const CallIdentifier & src) const
{
    size_t  	    leftSize = confId_.GetSize(),
    	    	    rightSize = src.confId_.GetSize();
		    
    assert( leftSize == rightSize );
    
    const BYTE  *   left = confId_,
    	    	*   right = src.confId_;

    int result = memcmp(left, right, leftSize);
    
    if ( result != 0 )
    {
	leftSize = callId_.m_guid.GetSize();
    	rightSize = src.callId_.m_guid.GetSize();

	assert( leftSize == rightSize );

    	left = callId_.m_guid;
    	right = src.callId_.m_guid;

	return ( memcmp(left, right, leftSize) == 0 );
    }
    
    return ( false );
}


ostream &   
CallIdentifier::writeTo(ostream & out) const
{
    int indent	    	= out.precision() + 2;
    int fieldIndent 	= 22;
    
    out << setw(indent+fieldIndent) << "conferenceIdentifier: "
    	<< setprecision(indent)     << confId_ << '\n'
	<< setw(indent+fieldIndent) << "callIdentifier:       "
    	<< setprecision(indent)     << callId_ << '\n';

    return ( out );
}

