/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const RouterInfo_cxx_Version = 
    "$Id: RouterInfo.cxx,v 1.9 2001/05/22 08:06:14 icahoon Exp $";


#include "RouterInfo.hxx"
#include "Lock.hxx"


using Vocal::SipH323Info::RouterInfo;
using Vocal::Transport::IPAddress;
using Vocal::Process::Lock;


RouterInfo::RouterInfo(
    const IPAddress 	&   csAddr,
    bool	    	    routerMediaCtrl,
    bool	    	    remoteClnt
)
{
    callSignalAddress(csAddr);
    remoteClient(remoteClnt);
    routerMediaControl(routerMediaCtrl);
    h225ProtocolID("0.0.8.2250.0.2");
}


RouterInfo::~RouterInfo()
{
}


void	    	    
RouterInfo::callSignalAddress(const IPAddress & csAddr)
{
    Lock    lock(mutex_); (void)lock;
    
    callSignalAddress_ = csAddr;
}


IPAddress   	    
RouterInfo::callSignalAddress() const
{
    Lock    lock(mutex_); (void)lock;
    
    return ( callSignalAddress_ );
}


void	    	    
RouterInfo::remoteClient(bool remoteClnt)
{
    Lock    lock(mutex_); (void)lock;

    remoteClient_ = remoteClnt;
}


bool	    	    
RouterInfo::remoteClient() const
{
    Lock    lock(mutex_); (void)lock;

    return ( remoteClient_ );
}


void	    	    
RouterInfo::routerMediaControl(bool routerMediaCtrl)
{
    Lock    lock(mutex_); (void)lock;

    routerMediaControl_ = routerMediaCtrl;
}


bool	    	    
RouterInfo::routerMediaControl() const
{
    Lock    lock(mutex_); (void)lock;

    return ( routerMediaControl_ );
}


string	    	    
RouterInfo::h225ProtocolID() const
{
    Lock    lock(mutex_); (void)lock;
    
    return ( h225ProtocolID_ );
}


void
RouterInfo::h225ProtocolID(const string & h225pid)
{
    Lock    lock(mutex_); (void)lock;
    
    h225ProtocolID_ = h225pid;
}
