/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const H323SipFeature_cxx_Version = 
    "$Id: H323SipFeature.cxx,v 1.12 2001/02/14 03:45:58 icahoon Exp $";


#include "H323SipFeature.hxx"
#include "H323SipState.hxx"


using Vocal::SipH323Proxy::H323SipFeature;
using Vocal::SipH323Proxy::H323SipState;
using Vocal::SipH323Info::SipProxyInfo;


H323SipFeature::H323SipFeature(
    SipProxyInfo    	    	&   sipProxyInfo
)
{
    h323SipState_ = new H323SipState(sipProxyInfo);
    
    h323SipState_->me(h323SipState_);
    
    addState(h323SipState_);
}

    
H323SipFeature::~H323SipFeature() 
{
}

    
void	    	    	
H323SipFeature::sipStack(
    Sptr<SipTransceiver>     	sipstack
)
{
    assert( h323SipState_ != 0 );
    h323SipState_->sipStack(sipstack);
}


void	    	    	
H323SipFeature::callTranslatorFifo(
    FifoBase< Sptr<Event> > &	callTransFifo
)
{
    assert( h323SipState_ != 0 );
    h323SipState_->callTranslatorFifo(callTransFifo);
}


void	    	    	
H323SipFeature::serviceFifo(
    FifoBase< Sptr<Event> > &	servFifo
)
{
    assert( h323SipState_ != 0 );
    h323SipState_->serviceFifo(servFifo);
}


const char* const 
H323SipFeature::name() const 
{
    return ( "H323SipFeature" );
}
