#if !defined(H323_SIP_STATE_DOT_H)
#define H323_SIP_STATE_DOT_H

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const H323SipState_hxx_Version = 
    "$Id: H323SipState.hxx,v 1.14 2001/02/14 03:45:58 icahoon Exp $";


#include "State.hxx"
#include "SipTransceiver.hxx"
#include "FifoBase.h"
#include "Event.hxx"
#include "Sptr.hxx"
#include "H323SipRegistrationMap.hxx"
#include "H323SipOpAckIn.hxx"
#include "H323SipOpAckOut.hxx"
#include "H323SipOpByeIn.hxx"
#include "H323SipOpByeOut.hxx"
#include "H323SipOpCancelIn.hxx"
#include "H323SipOpCancelOut.hxx"
#include "H323SipOpInviteIn.hxx"
#include "H323SipOpInviteOut.hxx"
#include "H323SipOpRegisterIn.hxx"
#include "H323SipOpRegisterOut.hxx"
#include "H323SipOpStatusIn.hxx"
#include "H323SipOpStatusOut.hxx"


// Forward declaration
//
namespace Vocal {
    namespace SipH323Info {
    	class SipProxyInfo;
    }
}


/** Vovida Open Communication Application Library<br><br>
 */
namespace Vocal
{


/** Sip to H.323 Proxy<br><br>
 */
namespace SipH323Proxy
{


/** A very simple state, with an operator for each direction of a 
 *  SIP message.<br><br>
 *
 *  @see    Vocal::SipH323Info::SipProxyInfo
 *  @see    Vocal::SipH323Proxy::H323SipRegistrationMap
 *  @see    Vocal::SipH323Proxy::H323SipOpAckIn
 *  @see    Vocal::SipH323Proxy::H323SipOpAckOut
 *  @see    Vocal::SipH323Proxy::H323SipOpByeIn
 *  @see    Vocal::SipH323Proxy::H323SipOpByeOut
 *  @see    Vocal::SipH323Proxy::H323SipOpCancelIn
 *  @see    Vocal::SipH323Proxy::H323SipOpCancelOut
 *  @see    Vocal::SipH323Proxy::H323SipOpInviteIn
 *  @see    Vocal::SipH323Proxy::H323SipOpInviteOut
 *  @see    Vocal::SipH323Proxy::H323SipOpRegisterIn
 *  @see    Vocal::SipH323Proxy::H323SipOpRegisterOut
 *  @see    Vocal::SipH323Proxy::H323SipOpStatusIn
 *  @see    Vocal::SipH323Proxy::H323SipOpStatusOut
 */
class H323SipState : public State
{
    public:

    	/** Create given the sip proxy configuration information.
    	 */
    	H323SipState(SipProxyInfo &);


    	/**  Virtual destructor.
    	 */
    	virtual    ~H323SipState();


    	/** Set the sip transceiver.
    	 */
    	void	   sipStack(Sptr<SipTransceiver>);


    	/** Attach the call translator's fifo.
    	 */
    	void	    callTranslatorFifo(FifoBase< Sptr<Event> > &);


    	/** Attach the service fifo.
    	 */
    	void	    serviceFifo(FifoBase< Sptr<Event> > &);


        /** Override the default state's process.
         */
        Sptr < State > process(const Sptr < SipProxyEvent > event);


    	/** Name of this class of State, "H323SipState".
    	 */
    	const 	char* const 	name() const;

        
        /** A hack to get around the silly Sptrs. I can't wait until
         *  I rip out the sip base code.
         */
        void    me(Sptr<State>);
        

    private:

        void                    createOperators();

        SipProxyInfo 	    	    	    	    &	sipProxyInfo_;

    	H323SipRegistrationMap     	    	    	registrationMap_;

        Sptr<State>                                     me_;

        // event operators
        //
 	Sptr<H323SipOpAckIn>                            opAckIn_;
	Sptr<H323SipOpAckOut>	    	    	    	opAckOut_;
 	Sptr<H323SipOpByeIn>                            opByeIn_;
	Sptr<H323SipOpByeOut>                           opByeOut_;
 	Sptr<H323SipOpCancelIn>                         opCancelIn_;
    	Sptr<H323SipOpCancelOut>    	    	    	opCancelOut_;
    	Sptr<H323SipOpInviteIn>     	    	    	opInviteIn_;
	Sptr<H323SipOpInviteOut>    	    	    	opInviteOut_;
    	Sptr<H323SipOpRegisterIn>     	    	    	opRegisterIn_;
	Sptr<H323SipOpRegisterOut>  	    	    	opRegisterOut_;
    	Sptr<H323SipOpStatusIn>     	    	    	opStatusIn_;
    	Sptr<H323SipOpStatusOut>     	    	    	opStatusOut_;

    	// Suppress copying.
	//
    	H323SipState(const H323SipState &);
        H323SipState &     operator=(const H323SipState &);
};


} // namespace SipH323Proxy
} // namespace Vocal

    
#endif // H323_SIP_STATE_DOT_H
