#if !defined(OUTGOING_SIP_EVENT_EXCH_DOT_HXX)
#define OUTGOING_SIP_EVENT_EXCH_DOT_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const OutgoingSipEventExch_hxx_Version = 
    "$Id: OutgoingSipEventExch.hxx,v 1.2 2001/04/03 07:51:14 icahoon Exp $";


#include "EventObserver.hxx"
#include "Sptr.hxx"
#include "Event.hxx"
#include "FifoBase.h"
#include "Service.hxx"
#include "SipProxyEvent.hxx"


/** Vovida Open Communication Application Library<br><br>
 */
namespace Vocal
{


/** Sip to H.323 Proxy<br><br>
 */
namespace SipH323Proxy
{

using Vocal::Services::Event;
using Vocal::Services::Service;

/** This exchange receives incoming OutgoingSipEvents from the EventSubject
 *  and converts them to SipProxyEvent and places them on the worker thread's
 *  fifo.
 */
class OutgoingSipEventExch
    :   public Vocal::Behavioral::EventObserver< Sptr<Event> >
{
    public:
    
    
        /** Create 
         */
        OutgoingSipEventExch(
            Service                         &   service,
            FifoBase< Sptr<SipProxyEvent> > &   workerFifo
        );
        

        /** Virtual destructor
         */
        virtual ~OutgoingSipEventExch();


    	/** EventObserver callback. Returns true if the event
	 *  is a outgoing sip event.
	 */
    	virtual bool	    interestedIn(const Sptr<Event> &);



    	/** EventObserver callback. This processes the outgoing sip event.
         *  This basically does a translation operation, translating
         *  OutgoingSipEvents to SipEvents, and queueing the outbound
         *  SipEvent in the worker thread's fifo.
	 */
    	virtual void	    onEvent(Sptr<Event>);


        /** Dump to an ostream.
         */
        ostream & writeTo(ostream &) const;
    
    private:


        /** The service that will deliver us events. We need to register
         *  and unregister with this service.
         */    
        Service                         &   myService;


        /** Fifo for outbound SipEvents.
         */    
        FifoBase< Sptr<SipProxyEvent> > &   myWorkerFifo;
};


} // namespace SipH323Proxy
} // namespace Vocal


#endif // !defined(OUTGOING_SIP_EVENT_EXCH_DOT_HXX)
