#!/bin/sh
#
# $Id$
#
# ====================================================================
# The Vovida Software License, Version 1.0 
# 
# Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. The names "VOCAL", "Vovida Open Communication Application Library",
#    and "Vovida Open Communication Application Library (VOCAL)" must
#    not be used to endorse or promote products derived from this
#    software without prior written permission. For written
#    permission, please contact vocal\@vovida.org.
# 
# 4. Products derived from this software may not be called "VOCAL", nor
#    may "VOCAL" appear in their name, without prior written
#    permission of Vovida Networks, Inc.
# 
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
# WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
# NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
# NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
# IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
# OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
# USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
# DAMAGE.
# 
# ====================================================================
# 
# This software consists of voluntary contributions made by Vovida
# Networks, Inc. and many individuals on behalf of Vovida Networks,
# Inc.  For more information on Vovida Networks, Inc., please see
# <http://www.vovida.org/>.
#
# $Log$
# Revision 1.4  2000/08/12 00:14:43  icahoon
# added license
#
# Revision 1.3  2000/07/24 22:34:37  icahoon
# moved to vocal
#
# Revision 1.2  2000/07/05 22:58:43  hyu
# Merging BRANCH_CURD into HEAD.
#
# Revision 1.1.2.1  2000/06/20 23:29:52  icahoon
# added html interface for allinone server
#
#

# disable filename globbing
set -f

echo Content-type: text/html
echo

CMD=""
CMD_NAME="op"
CMD_VALUE=""

export HOSTNAME=`/bin/uname -n`
export HOST=${HOSTNAME%%.*}

# Reasonable defaults
#
FLAGS="dr"
LOG_LEVEL="LOG_DEBUG"
PSERVER_HOST=$HOST
PSERVER_PORT=6005
PORT=7061

ERROR=0

# parse the query string
#
if [ "$QUERY_STRING" != "" ]
then
    while [ "$QUERY_STRING" != "" ]
    do
    
    	CMD=${QUERY_STRING%%&*}
    	CMD_NAME=${CMD%%=*}
    	CMD_VALUE=${CMD#*=}

    	QUERY_STRING=${QUERY_STRING#*&}
	
	case $CMD_NAME in
	
    	    flags )
	    	if [ "$CMD_VALUE" != "" ]
		then
	    	    FLAGS=$CMD_VALUE
		fi
	    	;;
		
	    log_level )
	    	if [ "$CMD_VALUE" != "" ]
		then
		    LOG_LEVEL=$CMD_VALUE
		fi
		;;
		
	    pserver_host )
	    	if [ "$CMD_VALUE" != "" ]
		then
		    PSERVER_HOST=$CMD_VALUE
		fi
		;;

	    pserver_port )
	    	if [ "$CMD_VALUE" != "" ]
		then
		    PSERVER_PORT=$CMD_VALUE
		fi
		;;
	    
	    port )
	    	if [ "$CMD_VALUE" != "" ]
		then
    	    	    PORT=$CMD_VALUE
		fi
		;;
		
	    op )
	    	QUERY_STRING=""
		;;

    	    * )
	    	ERROR=1
	    	;;	    	    
	esac
    done
fi

if [ $ERROR -eq 1 ]
then

cat  << EOF
    <html>
    <body>
	Error
    </body>
    </html>
EOF
exit

fi


if [ -d config ]
then

    echo $FLAGS > config/flags
    echo $LOG_LEVEL > config/log_level
    echo $PSERVER_HOST > config/pserver_host
    echo $PSERVER_PORT > config/pserver_port
    echo $PORT > config/port

else

cat << EOF
    <html>
    <body>
    	Error
    </body>
    </html>
EOF
exit

fi

cat << EOF
<html>

<!-- #include virtual="/vocal/common/copyright"-->

<head> 
    <title>Feature Server Update</title> 
    <link rel="stylesheet" href="/vocal/common/style.css" type="text/css">
</head>

<body>
<table border="0" cellspacing="0" cellpadding="10" width="100%">
     <tr>
	<td colspan="4" valign="bottom" align="left" bgcolor="#D2C8B6">
	<h3>Feature Server Update</h3>
	</td>
    </tr>

    <tr>
	<td colspan="4" bgcolor="#FFFFFF">
	    &nbsp;
	</td>
    </tr>

    <tr>
    	<th bgcolor="#FFFFFF" td align="left" valign="top">
    	    Flags
	</th>

	<td colspan="3" bgcolor="#FFFFFF" td align="left" valign="top">
EOF
	
echo "	    `cat config/flags` &nbsp;"

cat << EOF    
    	</td>
    </tr>

    <tr>
    	<th bgcolor="#FFFFFF" td align="left" valign="top">
    	    Log Level
	</th>
	
	<td colspan="3" bgcolor="#FFFFFF" td align="left" valign="top">
EOF

echo "	    `cat config/log_level` &nbsp;"

cat << EOF    
    	</td>
    </tr>
    
    <tr>
    	<th bgcolor="#FFFFFF" td align="left" valign="top">
    	    Provisioning Server Host
	</th>
	
	<td colspan="3" bgcolor="#FFFFFF" td align="left" valign="top">
EOF

echo "	    `cat config/pserver_host` &nbsp;"

cat << EOF    
    	</td>
    </tr>

    <tr>
    	<th bgcolor="#FFFFFF" td align="left" valign="top">
    	    Provisioning Server Port
	</th>
	
	<td colspan="3" bgcolor="#FFFFFF" td align="left" valign="top">
EOF

echo "	    `cat config/pserver_port` &nbsp;"

cat << EOF    
    	</td>
    </tr>

    <tr>
    	<th bgcolor="#FFFFFF" td align="left" valign="top">
    	    Port
	</th>
	
	<td colspan="3" bgcolor="#FFFFFF" td align="left" valign="top">
EOF

echo "	    `cat config/port` &nbsp;"

cat << EOF    
    	</td>
    </tr>

</table>
</body>

</html>

EOF

