#!/bin/sh
#
# $Id: start.cgi,v 1.8 2000/08/12 00:15:36 icahoon Exp $
#
# ====================================================================
# The Vovida Software License, Version 1.0 
# 
# Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. The names "VOCAL", "Vovida Open Communication Application Library",
#    and "Vovida Open Communication Application Library (VOCAL)" must
#    not be used to endorse or promote products derived from this
#    software without prior written permission. For written
#    permission, please contact vocal\@vovida.org.
# 
# 4. Products derived from this software may not be called "VOCAL", nor
#    may "VOCAL" appear in their name, without prior written
#    permission of Vovida Networks, Inc.
# 
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
# WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
# NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
# NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
# IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
# OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
# USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
# DAMAGE.
# 
# ====================================================================
# 
# This software consists of voluntary contributions made by Vovida
# Networks, Inc. and many individuals on behalf of Vovida Networks,
# Inc.  For more information on Vovida Networks, Inc., please see
# <http://www.vovida.org/>.
#
# $Log: start.cgi,v $
# Revision 1.8  2000/08/12 00:15:36  icahoon
# added license
#
# Revision 1.7  2000/08/01 21:42:08  icahoon
# fixes for installation into /usr/local/vocal
#
# Revision 1.6  2000/07/29 09:02:42  icahoon
# minor changes
#
# Revision 1.5  2000/07/24 22:34:44  icahoon
# moved to vocal
#
# Revision 1.4  2000/07/05 22:58:46  hyu
# Merging BRANCH_CURD into HEAD.
#
# Revision 1.3.8.3  2000/06/29 21:30:53  icahoon
# minor changes
#
# Revision 1.3.8.2  2000/06/23 08:44:37  icahoon
# minor fixes
#
# Revision 1.3.8.1  2000/06/15 21:28:50  icahoon
# mostly registration updates, some router / translator additions
#
# Revision 1.3.2.2  2000/06/12 23:10:04  icahoon
# support for marshal command line options
#
# Revision 1.3.2.1  2000/06/06 18:05:43  icahoon
# support sipport, support SIGTEM, remove timeout
#
# Revision 1.3  2000/05/27 08:09:01  icahoon
# removed leading whitespace from top output
#
# Revision 1.2  2000/05/27 02:31:35  icahoon
# changes sigh323csgw to run as a daemon, so picking stdout.log now.
#
# Revision 1.1  2000/05/27 01:47:42  icahoon
# Prototype web interface to the sip to h323 call signalling gateway
#
#

# disable filename globbing
set -f

ARGS=""
CMDLINE=0

if [ "$1" = "-nc" ]
then
    shift
    CMDLINE=1
fi


GKID=""
EPTID=""
LOGLVL=""
SIPPORT=""
SIP_REMOTE=""
SIP_REMOTE_TYPE=""
MEDIA_CONTROL=""

if [ -f config/gkid ]
then

    GKID=`cat config/gkid`
    if [ "$GKID" != "" ]
    then
    	GKID="-g $GKID"
	ARGS="$ARGS $GKID"
    fi
fi

if [ -r config/eptid ]
then

    EPTID=`cat config/eptid`
    if [ "$EPTID" != "" ]
    then
    	EPTID="-e $EPTID"
	ARGS="$ARGS $EPTID"
    fi
fi

if [ -r config/loglvl ]
then

    LOGLVL=`cat config/loglvl`
    if [ "$LOGLVL" != "" ]
    then
    	LOGLVL="-l $LOGLVL"
	ARGS="$ARGS $LOGLVL"
    fi
fi

if [ -r config/sipport ]
then

    SIPPORT=`cat config/sipport`
    if [ "$SIPPORT" != "" ]
    then
    	SIPPORT="-p $SIPPORT"
	ARGS="$ARGS $SIPPORT"
    fi
fi

if [ -r config/sipremote ]
then

    SIP_REMOTE=`cat config/sipremote`
    if [ "$SIP_REMOTE" != "" ]
    then
    	SIP_REMOTE="-a $SIP_REMOTE"
	ARGS="$ARGS $SIP_REMOTE"
    fi
fi

if [ -r config/sipremotetype ]
then

    SIP_REMOTE_TYPE=`cat config/sipremotetype`
    if [ "$SIP_REMOTE_TYPE" = "SIP_User_Agent" ]
    then
	ARGS="$ARGS -c"
    fi
fi

if [ -r config/mediacontrol ]
then

    MEDIA_CONTROL=`cat config/mediacontrol`
    if [ "$MEDIA_CONTROL" != "Between_endpoints" ]
    then
	ARGS="$ARGS -m"
    fi
fi

if [ $CMDLINE -eq 0 ]
then

    echo Content-type: text/html
    echo

    cat << EOF
<html>

<!-- #include virtual="/vocal/common/copyright"-->

<head> 
	<title>Sip To H.323 Call Signalling Gateway Start</title> 
	<link rel="stylesheet" href="/vocal/common/style.css" type="text/css">
</head>

<body>
<table border="0" cellspacing="0" cellpadding="10" width="100%">
    <tr>
	<td colspan="4" valign="bottom" align="left" bgcolor="#D2C8B6">
	<h3>Sip To H.323 Call Signalling Gateway Start</h3>
	</td>
    </tr>

    <tr>
	<td colspan="4" bgcolor="#FFFFFF">
    	    &nbsp;
	</td>
    </tr>

    <tr>
	<td colspan="4" bgcolor="#FFFFFF">
<pre>
EOF

fi

TOP=`top b n 1 | grep siph323csgw | head -1`
TOP=`echo $TOP`
PROCESS=${TOP%% *}

if [ "$PROCESS" != "" ]
then
    echo Already started.
else
    echo Starting...
    echo "./siph323csgw -o log $ARGS &"
    ./siph323csgw -o log $ARGS &
fi

if [ $CMDLINE -eq 0 ]
then

    echo '<hr><br>'
    echo '<ul>'
    sleep 1
    if [ -r ./stdout.log ]
    then
    	cat ./stdout.log
    fi
    echo '<br>'
    top b n 1 | head -n 9 -
    top b n 1 | grep siph323csgw
    echo '</ul>'
    cat <<EOF
</pre>		
	</td>
    </tr>

    <tr>
	<td colspan="4" bgcolor="#FFFFFF">
	    &nbsp;
	</td>
    </tr>

</table>
</body>
</html>
EOF

fi


