/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const EvtCCSentAddress_cxx_Version = 
    "$Id: EvtCCSentAddress.cxx,v 1.9 2001/10/24 21:26:38 icahoon Exp $";


#include <cassert>

#include "VLog.hxx"

#include "EvtCCSentAddress.hxx"
#include "FeatureAcceptCC.hxx"

using namespace router;
using Vocal::Logging::VLog;

EvtCCSentAddress::
EvtCCSentAddress(SMCall* sm)
    : EvtUserAcceptCC(sm)
{
}


void EvtCCSentAddress::
inIdle(FeatureAcceptCC& acceptcc)
{
    const string    fn("EvtCCSentAddress::inIdle");
    VLog            log(fn);

    VDEBUG(log) << fn << VDEBUG_END(log);

    acceptcc.setState(&PatternAcceptCC::inPublished, "inPublished");
}


void EvtCCSentAddress::
inOther(FeatureAcceptCC& acceptcc)
{
    const string    fn("EvtCCSentAddress::inOther");
    VLog            log(fn);

    VDEBUG(log) << fn << VDEBUG_END(log);

    // Ignore
    VINFO(log)  << fn << ": Sent H245 address for call " 
                << acceptcc.getSM() << VINFO_END(log);
}


#ifndef DISABLE_WRITER

ostream& EvtCCSentAddress::
writeTo(ostream& out) const
{
    out << "EvtCCSentAddress";
    return out;
}

#endif
