/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const EvtCEOTransferReq_cxx_Version = 
    "$Id: EvtCEOTransferReq.cxx,v 1.9 2001/10/24 21:26:38 icahoon Exp $";


#include <cassert>

#include "VLog.hxx"

#include "H323Consts.hxx"
#include "EvtCEOTransferReq.hxx"
#include "FeatureCEO.hxx"
#include "FeatureH245.hxx"
#include "EvtMsdRequest.hxx"

using namespace router;
using Vocal::Logging::VLog;

EvtCEOTransferReq::
EvtCEOTransferReq(SMCall* sm)
    : EvtUserCEO2(sm)
{
}


// Transfer request is processed independently of state
void EvtCEOTransferReq::
inOther(FeatureCEO& ceo)
{
    const string    fn("EvtCEOTransferReq::inOther");
    VLog            log(fn);

    VDEBUG(log) << fn << VDEBUG_END(log);

    // Send TCS message
    ceo.out_SQ = (ceo.out_SQ + 1) % 256;
    tcs_.m().m_sequenceNumber = ceo.out_SQ;
    tcs_.m().m_protocolIdentifier = H245ProtocolID;
    ceo.getSM()->getCallCtlConnection()->send(tcs_);

    ceo.setState(&PatternCEO::inWait, "inWait");
}


void EvtCEOTransferReq::
inPaused(FeatureH245& h245)
{
    const string    fn("EvtCEOTransferReq::inPaused");
    VLog            log(fn);

    VDEBUG(log) << fn << VDEBUG_END(log);

    h245.gotTCSAck = false;

    if (isNonZero())
    {
	// Start MSD procedure
	if (!h245.msdStarted)
	{
	    h245.msdCompleted = false;
	    EvtMsdRequest msd(h245.getSM());
	    msd.start();
	}

	h245.setState(&PatternH245::inTCE, "inTCE");
    }
    else
    {
	h245.msdCompleted = false;
    }
}


void EvtCEOTransferReq::
inTPaused(FeatureH245& h245)
{
    const string    fn("EvtCEOTransferReq::inTPaused");
    VLog            log(fn);

    VDEBUG(log) << fn << VDEBUG_END(log);

    h245.gotTCSAck = false;

    if (!isNonZero())
    {
	h245.msdCompleted = false;
	h245.setState(&PatternH245::inPaused, "inPaused");
    }
}


void EvtCEOTransferReq::
inTCE(FeatureH245& h245)
{
    const string    fn("EvtCEOTransferReq::inTCE");
    VLog            log(fn);

    VDEBUG(log) << fn << VDEBUG_END(log);

    h245.gotTCSAck = false;

    if (!isNonZero())
    {
	h245.msdCompleted = false;
	h245.setState(&PatternH245::inPaused, "inPaused");
    }
}


void EvtCEOTransferReq::
inOpen(FeatureH245& h245)
{
    const string    fn("EvtCEOTransferReq::inOpen");
    VLog            log(fn);

    VDEBUG(log) << fn << VDEBUG_END(log);

    h245.gotTCSAck = false;

    if (isNonZero())
    {
	h245.setState(&PatternH245::inTCE, "inTCE");
    }
    else
    {
	h245.msdCompleted = false;
	h245.setState(&PatternH245::inPaused, "inPaused");
    }
}


void EvtCEOTransferReq::
inOther(FeatureH245& h245)
{
    const string    fn("EvtCEOTransferReq::inOther(FeatureH245&)");
    VLog            log(fn);

    VDEBUG(log) << fn << VDEBUG_END(log);

    h245.gotTCSAck = false;

    VWARN(log) << fn << ": Unexpected EvtCEOTransferReq event for call "
               << h245.getSM() << VWARN_END(log);
}


bool EvtCEOTransferReq::
isNonZero()
{
    // (depends from H.245 parser)
    return
	tcs_.m().HasOptionalField(
	    H245_TerminalCapabilitySet::e_multiplexCapability) ||
	tcs_.m().HasOptionalField(
	    H245_TerminalCapabilitySet::e_capabilityTable) ||
	tcs_.m().HasOptionalField(
	    H245_TerminalCapabilitySet::e_capabilityDescriptors);
}


#ifndef DISABLE_WRITER

ostream& EvtCEOTransferReq::
writeTo(ostream& out) const
{
    out << "EvtCEOTransferReq";
    return out;
}

#endif
