/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const EvtEndSession_cxx_Version = 
    "$Id: EvtEndSession.cxx,v 1.12 2001/10/24 21:26:39 icahoon Exp $";


#include <cassert>

#include "VLog.hxx"

#include "EvtEndSession.hxx"
#include "FeatureCallCtlSession.hxx"

using namespace router;
using Vocal::Logging::VLog;

EvtEndSession::
EvtEndSession(RouterCallCtl* sm)
    : EvtH245CallCtlSessionCommand(sm)
{
}


void EvtEndSession::
inOpened(FeatureCallCtlSession& callctlsession)
{
    const string    fn("EvtEndSession::inOpened");
    VLog            log(fn);

    VDEBUG(log) << fn << VDEBUG_END(log);

    callctlsession.setState(&PatternCallCtlSession::inClosed, "inClosed");

    // Send endsession
    BH245EndSession es;
    es.m().SetTag(H245_EndSessionCommand::e_disconnect);
    callctlsession.getSM()->getCallCtlConnection()->send(es);

    bool terminatePatch = false;

    // Send releaseComplete if tunneling is used
    if (callctlsession.getSM()->getCallSigConnection()->h245Tunneling())
    {
        terminatePatch = true;
        callctlsession.getSM()->getCallSigConnection()->sendReleaseComplete(
            H225_ReleaseCompleteReason::e_undefinedReason);
    }

    // Terminate call
    callctlsession.getSM()->terminate();

    // This solves the problem of handling tunneled EndSession command
    if (terminatePatch)
    {
        throw H245EndSession();
    }
}


void EvtEndSession::
inWaitEndSession(FeatureCallCtlSession& callctlsession)
{
    const string    fn("EvtEndSession::inWaitEndSession");
    VLog            log(fn);

    VDEBUG(log) << fn << VDEBUG_END(log);

    callctlsession.setState(&PatternCallCtlSession::inClosed, "inClosed");

    // Terminate call
    callctlsession.getSM()->terminate();
}


void EvtEndSession::
inOther(FeatureCallCtlSession& callctlsession)
{
    const string    fn("EvtEndSession::inOther");
    VLog            log(fn);

    VDEBUG(log) << fn << VDEBUG_END(log);

    // Ignore
    VWARN(log) << fn << ": Got unexpected EndSession for call "
               << callctlsession.getSM() << VWARN_END(log);
}


EvtH245B* EvtEndSession::
clone_impl()
{
    return new EvtEndSession(*this);
}


EvtEndSession::TagType EvtEndSession::
getTag() const throw()
{
    // (depends from H.245 parser)
    return H245Command::e_endSessionCommand;
}


void EvtEndSession::
updateID()
{
    // Ignore
}


#ifndef DISABLE_WRITER

ostream& EvtEndSession::
writeTo(ostream& out) const
{
    out << "EvtEndSession";
    EvtH245CallCtlSessionCommand::writeTo(out);
    return out;
}

#endif
