/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const EvtMSDeterminationRelease_cxx_Version = 
    "$Id: EvtMSDeterminationRelease.cxx,v 1.12 2001/10/24 21:26:39 icahoon Exp $";


#include <cassert>

#include "VLog.hxx"

#include "EvtMSDeterminationRelease.hxx"
#include "FeatureMsd.hxx"

using namespace router;
using Vocal::Logging::VLog;

EvtMSDeterminationRelease::
EvtMSDeterminationRelease(RouterCallCtl* sm)
    : EvtCallCtlMsdIndication(sm)
{
}


void EvtMSDeterminationRelease::
inIdle(FeatureMsd& msd)
{
    const string    fn("EvtMSDeterminationRelease::inIdle");
    VLog            log(fn);

    VDEBUG(log) << fn << VDEBUG_END(log);

    // Ignore
    VINFO(log) << fn << ": Got MSD Release for call "
               << msd.getSM() << VINFO_END(log);
}


void EvtMSDeterminationRelease::
inOther(FeatureMsd& msd)
{
    const string    fn("EvtMSDeterminationRelease::inOther");
    VLog            log(fn);

    VDEBUG(log) << fn << VDEBUG_END(log);

    VWARN(log) << fn 
        << ": Remote sees no response from local MSDE"
        << VWARN_END(log);

    msd.setState(&PatternMsd::inIdle, "inIdle");
    msd.callbackReject_.start();
}


EvtH245B* EvtMSDeterminationRelease::
clone_impl()
{
    return new EvtMSDeterminationRelease(*this);
}


H245IndicationTag EvtMSDeterminationRelease::
getTag() const throw()
{
    // (depends from H.245 parser)
    return H245Indication::e_masterSlaveDeterminationRelease;
}


#ifndef DISABLE_WRITER

ostream& EvtMSDeterminationRelease::
writeTo(ostream& out) const
{
    out << "EvtMSDeterminationRelease";
    EvtCallCtlMsdIndication::writeTo(out);
    return out;
}

#endif
