#ifndef INCLUDED_EvtQ931Msg_hxx
#define INCLUDED_EvtQ931Msg_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const EvtQ931Msg_hxx_Version = 
"$Id: EvtQ931Msg.hxx,v 1.8 2000/12/18 23:47:35 bko Exp $";

#include "mte/EventStatic.hxx"
#include "Q931Defs.hxx"
#include "EvtQ931.hxx"

namespace router
{

    /// Event containing Q.931 message
    class EvtQ931MsgB: public EvtQ931
    {
    public:
        typedef H225Tag::Choices TagType;

        /// Copy Q.931 message to the event
        void setMessage(const Q931Base& message);

        /// Get message type for the Q.931 message
        TagType getType() const throw() { return getTag(); };

        Q931Base& getQ931Base() throw();
        const Q931Base& getQ931Base() const throw();

        H225TheMessge& getSelMessage() throw();
        const H225TheMessge& getSelMessage() const throw();

        Q931& getQ931() throw();
        const Q931& getQ931() const throw();

    protected:
#ifndef DISABLE_WRITER
        virtual ostream& writeTo(ostream& out) const;
#endif

    private:
        /// Get tag for the Q.931 message (call-back)
        virtual TagType getTag() const throw() = 0;

        /// Update call signalling ID (call-back)
        virtual void updateID();

        Q931Base message_; // TODO: change to SmartPtr
    };

    typedef mte::EventStatic<EvtQ931MsgB> EvtQ931B;

    class EvtQ931Msg: public EvtQ931B
    {
        typedef EvtQ931B BaseType;
    };
}

#endif
