#ifndef INCLUDED_FeatureRTD_hxx
#define INCLUDED_FeatureRTD_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const FeatureRTD_hxx_Version = 
"$Id: FeatureRTD.hxx,v 1.6 2000/12/18 23:47:37 bko Exp $";

#include "Sptr.hxx"

#include "mte/Feature.hxx"

#include "PatternRTD.hxx"
#include "EvtRTDTimer.hxx"
#include "EntryRTD.hxx"
#include "ExitRTD.hxx"

namespace router
{

    class SMCall;
    
    /// Roundtrip delay determination state variables
    class RTDData
    {
    private:
        RTDData() : out_SQ(0) {}
        
        int out_SQ;

        // These classes are allowed to access state variables:
        friend class FeatureRTD;
        friend class EvtRTDTransferReq; // TODO
        friend class EvtRTDResponse; // TODO
    };

    /// Feature for roundtrip delay determination procedure
    class FeatureRTD: public mte::Feature<PatternRTD, FeatureRTD&>
    {
    public:
        /// Construct feature and assign pointer to call state machine
        FeatureRTD(SMCall* sm);
        
        virtual ~FeatureRTD();

        /// Return pointer to the state machine
        SMCall* getSM() const;

        /// Reset feature
        virtual void reset();

        /// Start timer
        void startTimer();

        /// Stop timer
        void stopTimer();

#ifndef DISABLE_WRITER
        virtual ostream& writeTo(ostream& out) const;
#endif

    private:
        /// Process event (implementation)
        virtual void process_impl(PatternRTD& pattern);

#ifndef DISABLE_WRITER
        char* getStateName() const;
#endif
        
        SMCall* const sm_;
        Sptr<EvtRTDTimer> timer_;
        EntryRTD entry_;
        ExitRTD exit_;
    };
    
    //----------------------------
    // Inline  methods
    //----------------------------

    inline SMCall* FeatureRTD::
    getSM() const
    {
        return sm_;
    }

}

#endif
