#if !defined(H245_PROTOCOL_DOT_H)
#define H245_PROTOCOL_DOT_H

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const H245Protocol_hxx_Version = 
"$Id$";

#include "TPKTClientProtocol.hxx"
#include "TPKTClientSocket.hxx"
#include "Sptr.hxx"
#include "SesH245.hxx"
#include "RouterCallCtl.hxx"
#include "ConnectionCallCtl.hxx"

namespace Vocal
{
    namespace Services {
    	class PollFifoService;
    }
}

namespace router
{
    using Vocal::Transport::TPKTClientSocket;
    using Vocal::Transport::TPKTClientProtocol;
    using Vocal::Services::PollFifoService;
    using Vocal::IO::FileDescriptor;
    using Vocal::Transport::ConnectionBrokenException;
    using Vocal::SystemException;
    
    class H245Protocol : public TPKTClientProtocol
    {
    public:
        H245Protocol(
            PollFifoService& service,
    	    Sptr<TPKTClientSocket> h245Socket,
            SesH245& h245EventSrc,
            RouterCallCtl& callCtlRouter,
            ConnectionCallCtl& callCtlconnection,
		    const char*	name = 0
            );

    	virtual ~H245Protocol();

        virtual const FileDescriptor& getFileDescriptor() const;

    	virtual TPKTClientSocket& getTPKTClientSocket();
		
        virtual void onOutgoingAvailable();

    	virtual void onTPKTArrived(vector<u_int8_t> & message);

        virtual void onDisconnect(ConnectionBrokenException*)
            throw ( Vocal::Transport::ProtocolException );

        virtual void onError(SystemException*)
            throw ( Vocal::Transport::ProtocolException );

        virtual void handleUnknownException();

        void terminate();

        void ignore();

    private:
    	PollFifoService      	    	    	&   service_;
        Sptr<TPKTClientSocket>      	    	    h245Socket_;
        SesH245                       	    	&   h245EventSrc_;
        RouterCallCtl                 	    	&   callCtlRouter_;
        ConnectionCallCtl             	    	&   connection_;
        bool                                	    isIgnored_;
    };

}

#endif // !defined(Q931_PROTOCOL_DOT_H)
