/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const H245ServerProtocol_cxx_Version = 
    "$Id$";

#include "VLog.hxx"

#include "PollFifoService.hxx"

#include "H245ServerProtocol.hxx"
#include "EvtCCTCPAccepted.hxx"

#include "ConnectionCallSig.hxx"
#include "CallIdentifier.hxx"
#include "CallTrace.hxx"

using namespace router;
using Vocal::Logging::VLog;

H245ServerProtocol::
H245ServerProtocol(
    SMCall& sm,
    PollFifoService& service,
    router::SesH245& h245EventSrc,
    router::RouterCallCtl& callCtlRouter,
    ConnectionCallCtl& callCtlConnection,
    H245Protocol** h245PassiveProtocol,
    const char* name = 0
)
    : TPKTServerProtocol(name ? name : "H245ServerProtocol"),
      sm_(sm),
      service_(service),
      h245EventSrc_(h245EventSrc),
      callCtlRouter_(callCtlRouter),
      connection_(callCtlConnection),
      h245PassiveProtocol_(h245PassiveProtocol),
      tpktServerSocket_("H245ServerSocket",
                        "H245ClientSocket")
{
    awaitIncoming(true);
    service_.registerProtocol(*this);
}


H245ServerProtocol::
~H245ServerProtocol()
{
    service_.unregisterProtocol(*this);
}


const FileDescriptor& H245ServerProtocol::
getFileDescriptor() const
{
    return tpktServerSocket_;
}


TPKTServerSocket& H245ServerProtocol::
getTPKTServerSocket()
{
    return tpktServerSocket_;
}


void H245ServerProtocol::
onAcceptedConnection(Sptr<TPKTClientSocket> h245socket)
{
    H245Protocol* h245Protocol = new H245Protocol(
        service_,
        h245socket, 
        h245EventSrc_,
        callCtlRouter_,
        connection_,
        "H245Protocol");

    #ifndef DISABLE_WRITER
    VLog    log;
    CALLTRACE(log, CT_IN, h245socket->getRemoteAddress(), CT_TCP, "H.245_Connection")
    	<< "{\n" << setprecision(0);

    if ( log.logInfo().isOn() )
    {
    	sm_.getCallSigConnection()->getCallID().writeTo(log.logInfo().get());
    }

    CALLTRACE_CONT(log) << "}" << CALLTRACE_END(log);
    #endif

    if (*h245PassiveProtocol_)
    {
        // Already accepted a connection, delete it and use a new one

        VLog log;
        VWARN(log)
            << "Accepting new control connection and removing an old one"
            << VWARN_END(log);

        delete *h245PassiveProtocol_;
        *h245PassiveProtocol_ = 0;
    }

    *h245PassiveProtocol_ = h245Protocol;

    EvtCCTCPAccepted evt(&sm_);
    evt.start();
}
