/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const SesH245_cxx_Version = 
    "$Id: SesH245.cxx,v 1.9 2001/03/09 06:54:42 icahoon Exp $";


#include "VLog.hxx"

#include "SesH245.hxx"

using namespace router;
using Vocal::Logging::VLog;


bool SesH245::
add(BEvtH245Msg* event)
{
    EvtH245Request* request = dynamic_cast<EvtH245Request*>(event);

    if (request)
    {
        return requests_.add(request);
    }

    EvtH245Response* response = dynamic_cast<EvtH245Response*>(event);

    if (response)
    {
        return responses_.add(response);
    }

    EvtH245Command* command = dynamic_cast<EvtH245Command*>(event);

    if (command)
    {
        return commands_.add(command);
    }

    EvtH245Indication* indication = dynamic_cast<EvtH245Indication*>(event);

    if (indication)
    {
        return indications_.add(indication);
    }

    return false;
}


void SesH245::
generate(H245Message& message, ConnectionCallCtl* connection)
{
    VLog log;
    VDEBUG(log) << "SesH245::generate" << VDEBUG_END(log);

    // (depends from H.245 parser)

    switch (message.GetTag())
    {
    case H245Message::e_request:
    {
        H245Request& requestMsg = message;
        VDEBUG(log) << "Generating H245 request event for tag = "
                    << requestMsg.GetTag()
                    << VDEBUG_END(log);
        EvtH245B* request = requests_.clone(requestMsg.GetTag());

        VDEBUG(log) << "Result address = "
                    << request
                    << VDEBUG_END(log);

        if (request)
        {
            request->setMessage(message);
            request->setConnection(connection);
            request->start();
            request->destroy();
        }
	else
	{
	    VWARN(log) << "Unknown message type = "
		       << requestMsg.GetTag()
		       << VWARN_END(log);
	}

        break;
    }
    case H245Message::e_response:
    {
        H245Response& responseMsg = message;
        VDEBUG(log) << "Generating H245 response event for tag = "
                    << responseMsg.GetTag()
                    << VDEBUG_END(log);
        EvtH245B* response = responses_.clone(responseMsg.GetTag());

        VDEBUG(log) << "Result address = "
                    << response
                    << VDEBUG_END(log);

        if (response)
        {
            response->setMessage(message);
            response->setConnection(connection);
            response->start();
            response->destroy();
        }
	else
	{
	    VWARN(log) << "Unknown message type = "
		       << responseMsg.GetTag()
		       << VWARN_END(log);
	}

        break;
    }
    case H245Message::e_command:
    {
        H245Command& commandMsg = message;
        VDEBUG(log) << "Generating H245 command event for tag = "
                    << commandMsg.GetTag()
                    << VDEBUG_END(log);
        EvtH245B* command = commands_.clone(commandMsg.GetTag());

        VDEBUG(log) << "Result address = "
                    << command
                    << VDEBUG_END(log);

        if (command)
        {
            command->setMessage(message);
            command->setConnection(connection);
            command->start();
            command->destroy();
        }
	else
	{
	    VWARN(log) << "Unknown message type = "
		       << commandMsg.GetTag()
		       << VWARN_END(log);
	}

        break;
    }
    case H245Message::e_indication:
    {
        H245Indication& indicationMsg = message;
        VDEBUG(log) << "Generating H245 indication event for tag = "
                    << indicationMsg.GetTag()
                    << VDEBUG_END(log);
        EvtH245B* indication =
            indications_.clone(indicationMsg.GetTag());

        VDEBUG(log) << "Result address = "
                    << indication
                    << VDEBUG_END(log);

        if (indication)
        {
            indication->setMessage(message);
            indication->setConnection(connection);
            indication->start();
            indication->destroy();
        }
	else
	{
	    VWARN(log) << "Unknown message type = "
		       << indicationMsg.GetTag()
		       << VWARN_END(log);
	}

        break;
    }
    default:
        // Ignore
        VWARN(log) << "Got unknown H245 message with tag = "
                   << message.GetTag() << VWARN_END(log);
        break;
    }
}
