#if !defined(VOCAL_SIP_TO_H323_CALLSIG_GW_HXX)
#define VOCAL_SIP_TO_H323_CALLSIG_GW_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const SipToH323CallSigGW_hxx_Version = 
    "$Id: SipToH323CallSigGW.hxx,v 1.2 2001/05/24 09:59:08 icahoon Exp $";


#include "ServiceApplication.hxx"


namespace Vocal {
    namespace SipH323Info {
        class SipProxyInfo;
        class GatekeeperInfo;
        class RouterInfo;
    }
    namespace SipH323Proxy {
        class H323SipProxyService;
    }
    namespace H323GK {
        class H323Gatekeeper;
    }
    namespace Configuration {
        class Configuration;
    }
    class H323CallTranslator;
}
    

/** Vovida Open Communication Application Library<br><br>
 */
namespace Vocal
{


/** SIP to H.323 Call Signaling Gateway<br><br>
 */
namespace SipH323CSGW
{

	
class SipToH323RASPolicyBuilder;


using Vocal::Services::ServiceApplication;
using Vocal::SipH323Info::SipProxyInfo;
using Vocal::SipH323Info::GatekeeperInfo;
using Vocal::SipH323Info::RouterInfo;
using Vocal::SipH323Info::SipProxyInfo;
using Vocal::SipH323Proxy::H323SipProxyService;
using Vocal::Configuration::Config;
using namespace Vocal::H323GK;


/** Sip To H323 Call Signaling Gateway.<br><br>
 *
 *  @see    Vocal::Services::ServiceApplication
 *  @see    Vocal::SipH323Info::SipProxyInfo
 *  @see    Vocal::SipH323Info::GatekeeperInfo
 *  @see    Vocal::SipH323Info::RouterInfo
 *  @see    Vocal::SipH323CSGW::SipToH323RASPolicyBuilder
 *  @see    Vocal::H323GK::H323Gatekeeper
 *  @see    Vocal::SipH323Proxy::H323SipProxyService
 */
class SipToH323CallSigGW : public Vocal::Services::ServiceApplication
{

    public:


    	/** Create the application.
    	 */
    	SipToH323CallSigGW();


    	/** Destroy the application.
    	 */
	~SipToH323CallSigGW();


    private:
    
    
        /** Provides a SipToH323Config.
         */
        virtual Config *    createConfig() const;


        /** Provide the filename of the log. 
         *  
         */
        virtual string  logFile() const;
         
        
        /** Creates SipToH323CallSigGW specific items.
         */
        virtual void    userInit();


        /** Cleans up SipToH323CallSigGW specific items.
         */
        virtual void    userUninit();
        

    	SipProxyInfo	    	    	    *	myIncomingSipProxyInfo;
    	SipProxyInfo	    	    	    *	myOutgoingSipProxyInfo;
	GatekeeperInfo	    	    	    *	myGkInfo;
	RouterInfo  	    	    	    *	myRouterInfo;
	
	SipToH323RASPolicyBuilder    	    *	myRasPolicyBuilder;

    	H323SipProxyService   	    	    *	myIncomingSipProxy;
    	H323SipProxyService   	    	    *	myOutgoingSipProxy;
	H323Gatekeeper	    	     	    *	myH323Gk;
    	H323CallTranslator  	    	    *	myH323CallTranslator;
};


} // namespace SipH323CSGW
} // namespace Vocal


#endif // !defined(VOCAL_SIP_TO_H323_CALLSIG_GW_HXX)
