#if !defined(VOCAL_SIP_TO_H323_CONFIG_HXX)
#define VOCAL_SIP_TO_H323_CONFIG_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const SipToH323Config_hxx_Version = 
    "$Id: SipToH323Config.hxx,v 1.3 2001/08/17 23:03:54 icahoon Exp $";


#include "Config.hxx"
#include "IPAddress.hxx"
#include "EndpointInfo.hxx"
#include <list>


/** Vovida Open Communication Application Library<br><br>
 */
namespace Vocal
{


/** SIP to H.323 Call Signaling Gateway<br><br>
 */
namespace SipH323CSGW
{


using Vocal::Transport::IPAddress;
using Vocal::SipH323Info::EndpointInfo;


/** 
 *
 */
class SipToH323Config : public Vocal::Configuration::Config
{

    public:


    	/** Create the application.
    	 */
    	SipToH323Config();


    	/** Destroy the application.
    	 */
	~SipToH323Config();


        /** Provide the usage to the user.
         */
        virtual void        usage() const;
        

        /** Provide the error message when the config file parsing fails.
         */
        virtual void        loadCfgFileErrorMsg(ReturnCode) const;
        
    
        /** The user has the ability to modify the options before
         *  it is used to parse the command line.
         */
        virtual ReturnCode  preParse();
        
        
        /** The user has the ability to query the options after
         *  it has been used to parse the command line.
         */
        virtual ReturnCode  postParse();


        /** The directory where the log file goes.
         */
        string  outputDirectory() const;
        
        
        /** The assigned gatekeeper identifier.
         */
        string   gatekeeperId() const;
        
        
        /** The endpoint identifier prefix.
         */
        string  endpointId() const;
        
        
        /** The port (and port + 1) assigned for the sip proxy.
         */
        u_int16_t   sipPort() const;


        /** The address of the remote sip entity
         */
        IPAddress   sipRemote() const;
        

        /** If true the remote sip entity is a client, thus h.323 
         *  registrations will not be translated. This is false
         *  by default.
         */
        bool        sipRemoteClient() const;
        
        
        /** The list of statically registered gateways.
         */
        list< Sptr<EndpointInfo> >  gateways() const;


    	/** If we need to dump it to a log.
	 */
        virtual ostream &   writeTo(ostream &) const;


    private:

        ReturnCode      loadGateways();
    
        string          outputDirectoryStr;
        string          gatekeeperIdStr;
        string          endpointIdStr;
        string          sipPortStr;
        string          sipRemoteStr;
        string          sipRemoteClientStr;
        string          gatewaysStr;
        
        char            outputDirectoryChar;
        char            gatekeeperIdChar;
        char            endpointIdChar;
        char            sipPortChar;
        char            sipRemoteChar;
        char            sipRemoteClientChar;


        string                      myOutputDirectory;
        string                      myGatekeeperId;
        string                      myEndpointId;
        u_int16_t                   mySipPort;
        IPAddress                   mySipRemote;
        bool                        mySipRemoteClient;
        list< Sptr<EndpointInfo> >  myGateways;        
};


} // namespace SipH323CSGW
} // namespace Vocal


#endif // !defined(VOCAL_SIP_TO_H323_CONFIG_HXX)
