/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const SipToH323RASPolicyBuilder_cxx_Version = 
    "$Id: SipToH323RASPolicyBuilder.cxx,v 1.2 2001/08/22 20:36:28 icahoon Exp $";


#include "SipToH323RASPolicyBuilder.hxx"
#include "SipToH323RegistrationPolicy.hxx"
#include "SipToH323AdmissionPolicy.hxx"


using namespace Vocal::SipH323CSGW;


SipToH323RASPolicyBuilder::SipToH323RASPolicyBuilder()
    :	sipProxyFifo_(0)
{
}


SipToH323RASPolicyBuilder::~SipToH323RASPolicyBuilder()
{
}


RegistrationPolicy *	
SipToH323RASPolicyBuilder::buildRegistrationPolicy(
    Service  	    	    &	service,
    RASGKProtocol   	    & 	rasGkProtocol,
    GatekeeperInfo  	    &	gatekeeperInfo,
    EndpointInfoMap 	    &	endpointInfoMap
)
{
    assert( sipProxyFifo_ != 0 );
    
    return ( new SipToH323RegistrationPolicy(
    	    	    	    	service,
    	    	    	    	rasGkProtocol, 
    	    	    	    	gatekeeperInfo,
    	    	    	    	endpointInfoMap,
				*sipProxyFifo_) );
}


AdmissionPolicy *	
SipToH323RASPolicyBuilder::buildAdmissionPolicy(
    Service 	    	    &	service,
    RASGKProtocol   	    &	rasGkProtocol,
    GatekeeperInfo  	    &	gatekeeperInfo,
    EndpointInfoMap 	    &	endpointInfoMap
)
{
    assert( sipProxyFifo_ != 0 );
    
    return ( new SipToH323AdmissionPolicy(
    	    	    	    	service,
    	    	    	    	rasGkProtocol, 
    	    	    	    	gatekeeperInfo,
    	    	    	    	endpointInfoMap) );
}


LocationPolicy *	
SipToH323RASPolicyBuilder::buildLocationPolicy(
    Service 	    	    &	service,
    RASGKProtocol   	    &	rasGkProtocol,
    GatekeeperInfo  	    &	gatekeeperInfo,
    EndpointInfoMap 	    &	endpointInfoMap
)
{
    assert( sipProxyFifo_ != 0 );
    
    // The location functionality isn't used by the siph323csgw.
    //
    return ( 0 );
}


void	    	    	    	
SipToH323RASPolicyBuilder::connectToSipProxy(
    FifoBase< Sptr<Event> > &	    sipProxyFifo
)
{
    sipProxyFifo_ = &sipProxyFifo;
}
					
