#ifndef __ADDRCONV_H_
#define __ADDRCONV_H_

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const Address_hxx_Version = 
"$Id: Address.hxx,v 1.22 2000/12/20 01:47:42 vzubarev Exp $";

#include "SipUrl.hxx"
#include "ptlib.h"
#include "ptlib/ipsock.h"
#include "asner.h"
#include "Data.hxx"
#include "h225.h"
#include "h245.h"
#include "IPAddress.hxx"
#include "XlatorException.hxx"
#include "Writer.hxx"

class H225_ArrayOf_AliasAddress;

namespace Translator
{
    using Vocal::Transport::IPAddress;

    typedef H225_ArrayOf_AliasAddress H323Address;
    typedef SipUrl SipAddress;

    ///
    class Address : public Vocal::IO::Writer
    {
    private:
        
	/// default number of alias addresses
	enum { alias_size = 5 };
	
	/// flag for Setup msg (used in port assignment)
	bool isSetup;
	
	/// primary container for the address
        H323Address address;
	
	/// checks whether address is a dotted quad ip
	bool isIp(const std::string &) const;
	
	/// checks syntax
	bool checkSyntax(const char *) const;
	
	/// converts address to SipAddress
	SipAddress retreiveSip() const throw (XlatorException);
	
	/// converts address to H323Address
	H323Address retreiveH323() const;
	
	/// converts address to IPAddress
	IPAddress retreiveIP() const throw (XlatorException);
	
	// converts address to IP6Address
	//IP6Address retreiveIP6() const throw (XlatorException);
	
	/// assigns SipAddress to address
	void assign(const SipAddress &s_address_c) throw (XlatorException);
	
	/// assigns H323Address to address
        void assign(const H323Address &h_address);
	
	/// assigns IPAddress to address
	void assign(const IPAddress &ip_address) throw (XlatorException);
	
	// assigns IP6Address to address (stub for future IPv6 addresses)
	//void assign(const IP6Address &ip6_address) throw (XlatorException);
	
	/// assigns Address to Address
	void assign(const Address &);
	
    public:
    
        /// default constructor
	Address() {}
	
        /// constructor for H323Address
	Address(const H323Address &h_address) { assign (h_address); }
        
	/// constructor for SipAddress
	Address(const SipAddress &s_address_c) { assign (s_address_c); }
	
	/// constructor for IPAddress
	Address(const IPAddress &ip_address) { assign (ip_address); }
	
	/// constructor of H245 Transport Address
	Address(const H245_UnicastAddress_iPAddress &ipAddress) { setH245_Transport(ipAddress); }
	
	/// constructor of H225 Transport Address
	Address(const H225_TransportAddress_ipAddress &ipAddress) { setH225_Transport(ipAddress); }

	// constructor for IP6Address
	//
	//Address(const IP6Address &ip6_address) { assign (ip6_address); }
	
	/// copy constructor
	Address(const Address &a) { assign(a.address); }
	
	/// you know what this is :)
	virtual ~Address() {}

	///
	void setH225_Transport(const H225_TransportAddress_ipAddress &ipAddress);
	
	/** 
	   the only function which works as an assignment operator
	   it flushes the the entire address and sets only one field of
	   the AliasAddress - Transport Address
	*/
	void setH245_Transport(const H245_UnicastAddress_iPAddress &ipAddress);
	
	/// set User Id
	void setUser(const std::string &user);
	
	/// set User Id
	void setUser(const H323Address &user) throw (XlatorException);

	/// set e_e164 field of the address
	void setPhone(const H323Address &h_address) throw (XlatorException);
	
	/// set e_e164 field of the address
	void setPhone(const std::string &phone) throw (XlatorException);
	
	/// set IP address (Transport Address field)
	void setTransport(const IPAddress &ipAddress) throw (XlatorException);
	
	//
	//void setVal(const IP6Address &ip6_address) { assign (ip6_address); }

	/// assignment operators for different address types
	Address &operator=(const H323Address &h_address) { assign (h_address); return *this; }
        
	///
	Address &operator=(const SipAddress &s_address_c) { assign (s_address_c); return *this; }
	
	///
	Address &operator=(const IPAddress &ip_address) { assign (ip_address); return *this; }
	
	//
	//Address &operator=(const IP6Address &ip6_address) { assign (ip6_address); return *this; }

	///
	Address &operator=(const H245_UnicastAddress_iPAddress &ipAddress);
	
	///
	Address &operator=(const H225_TransportAddress_ipAddress &ipAddress);
	
	///
	Address &operator=(const Address &a) { if(&a != this)assign (a.address); return *this; }
	
	/// Comparison (only compares e_e164 fields)
	bool operator==(const Address &a_address) const;
	
	///
	bool operator<(const Address &a_address) const;
	
	///
	bool operator>(const Address &a_address) const;
	
	/// type cast operators
	operator SipAddress() const throw (XlatorException) { return retreiveSip(); }
	
	///
	operator H323Address() const { return retreiveH323(); }
	
	///
	operator IPAddress() const throw (XlatorException) { return retreiveIP(); }
	
	//
	//operator IP6Address() const { return retreiveIP6(); }
	
	///
	operator H245_UnicastAddress_iPAddress() const throw (XlatorException)
	         { return getH245_Transport(); }

        ///
	operator H225_TransportAddress_ipAddress() const throw (XlatorException)
	         { return getH225_Transport(); }
	
	/// accessors for different address types
	H323Address getH323Address() const { return retreiveH323(); }
        
	///
	SipAddress getSipAddress() const { return retreiveSip(); }
	
	///
	IPAddress getIPAddress() const throw (XlatorException) { return retreiveIP(); }
	
	///
	H245_UnicastAddress_iPAddress getH245_Transport() const throw (XlatorException);
	
	///
	H225_TransportAddress_ipAddress getH225_Transport() const throw (XlatorException);
	
	//
	//IP6Address getIP6Address() const { return retreiveIP6(); }
	
	///
	std::string getUser() const throw (XlatorException);
	
	///
	std::string getPhone() const throw (XlatorException);
	
	///
	bool hasPhone() const;
	
	///
	bool hasIPAddress() const;
	
	///
	bool isEmpty() const;
	
	///
	void clear();

    	/// For logging
	virtual ostream & writeTo(ostream &) const;
    };
}
#endif
