#ifndef __CAPABSET_H_
#define __CAPABSET_H_

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const CapabilitySet_hxx_Version = 
"$Id: CapabilitySet.hxx,v 1.11 2001/05/13 09:06:38 icahoon Exp $";

#include <vector>
#include "H323_CapTable.hxx"
#include "XlatorException.hxx"

namespace Translator
{

    ///
    class CapabilitySet
    {
    public:
        
	///
        class CapabilityDescrVec
	{
	public:
	   
	   /// Alternative Capability Set
	   typedef std::vector<int> AltCapSet;
	   
	   /// Capability descriptor
	   typedef std::vector<AltCapSet> CapDescriptor;
	
	   /// Vector of capability descriptors
	   mutable std::vector<CapDescriptor> capDescrVec;
	
	public:
	   
	   ///
	   CapabilityDescrVec() {}
	   
	   ///
	   ~CapabilityDescrVec() {}
	   
	   /// add element to AltCapSet - capDescrVec[i][j]
	   void addCapability(int cap, int i = -1, int j = -1) throw (XlatorException);
	
	   /// add entire AltCapSet - capDescrVec[i][j]
	   void addAltCapSet(const AltCapSet &, int i = -1, int j = -1) throw (XlatorException);
	   
	   /// add entire CapDescriptor - capDescrVec[i]
	   void addCapDescriptor(const CapDescriptor&, int i = -1) throw (XlatorException);

	   /// remove entire CapDescriptor - capDescrVec[i]
	   void removeCapDescriptor(int i) throw (XlatorException);

	   /// get AltCapSet element  - capDescrVec[i][j][h]
	   int getCapability(int i, int j, int h) const throw (XlatorException);
	   
	   /** get the number of AltCapSet for a given Capability
	       in a current CapabilityDescriptor (curDescrN),
	       starting from index
	   */
	   int getCapabilityIndex(int cap, int index, int curDescrN) const throw (XlatorException);
	
	   /// get entire AltCapSet - capDescrVec[i][j]
	   const AltCapSet &getAltCapSet(int i, int j) const throw (XlatorException);
	   
	   /// get capDescrVec size (number of CapDescriptors)
	   int getCapDescrVecSize() const;

	   /// get capDescrVec[i] size (number of AltCapSets)
	   int getCapDescrSize(int i) const;
	
	   /// get capDescrVec[i][j] size (number of capabilities)
	   int getAltCapSize(int i, int j) const;
	   
	   /// sort all AltCapSets (before Intersection)
	   void sortCapDescrVec() const;
	   
	   /// clear the entire capDescrVec out
	   void clear();
	   
	   /// assignment
	   CapabilityDescrVec &operator=(const CapabilityDescrVec&);
	};

    public:
	/// Capability Table
       	H323_CapTable h323CapTable;
	
	/// Receive Capabilities
	CapabilityDescrVec capRecv;
	
	/// Transmit Capabilities
	CapabilityDescrVec capTransm;

        ///
	CapabilitySet() {}
	
	///
	virtual ~CapabilitySet() {}
    };
}

#endif
