#ifndef __CAPABTRANSL_H_
#define __CAPABTRANSL_H_

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const CapabilityTranslator_hxx_Version = 
"$Id: CapabilityTranslator.hxx,v 1.2 2002/03/01 22:51:07 bko Exp $";

#include <iostream>
#include <string>
#include <vector>
#include <iterator>
#include <map>

#include "Sdp2Session.hxx"
#include "ptlib.h"
#include "h245.h"
#include "IPAddress.hxx"
#include "CallTranslatorEM.hxx"
#if 0
#include "GlobalCapTable.hxx"
#include "Capability.hxx"
#include "CapabilitySetSip.hxx"
#include "CapabilitySetH323.hxx"
#include "CapabilitySetIntersect.hxx"
#include "XlatorException.hxx"
#include "H323_CapTable.hxx"
#endif
#include "CapabilityMap.hxx"

namespace CapabilityTranslator
{
    typedef H245_TerminalCapabilitySet TCS;
    typedef H245_OpenLogicalChannel OLC;
    typedef H245_OpenLogicalChannelAck OLCAck;
    typedef H245_CloseLogicalChannel CLC;
    typedef H245_RequestChannelClose RCC;
    typedef H245_OpenLogicalChannelReject OLCReject;
    
    typedef Vocal::SDP::SdpSession SDPS;

    ///
    class CapabilityTranslator
    {
    private:

    public:
	///
	CapabilityTranslator();
	
	///
	virtual ~CapabilityTranslator();

	/// set Translator to Master or Slave
	void setMaster(bool mI);
	
	/** set the indication that H245 Control Channel has been opened, 
	    now it's okay to send H.245 messages
	*/
	void setControlChannel(bool ccI);
	
	/** switch to the Phase C of H.245 procedures (per spec)
	    okay to send any H.245 messages
	*/
	void setPhaseC(bool pcI);
	
	/** address resolving for SDP message, result
	    of EvtResolve
	*/
	void resolved(std::string hostName, Transport::IPAddress address);
	
	/// start of FastConnect procedures
	void enterFS();
	
	/// handle OLC structure coming in FastConnect message
	void handleFSOLC(const OLC&);
	
	/// termination of FastConnect procedures
	void exitFS();
	
	/// Set capabilities, transport ID of Sip side (from SDP msg)
	void handleSDP(SDPS &msgSDP);
	
	/// handle empty SDP (received in re-INVITE)
	void handleEmptySDP();

	/// Set capabilities of H323 Side (from TCS msg)
	void handleTCS(const TCS &msgTCS);
	
	/** Set TransportIDs (get from OLC msg) of H323 side for a given media session
	    if returns false - OLC msg is bad - should issue CloseChannel for this channel
	*/
	void handleOLC(const OLC&);
	
	/** Set TransportIDs (get from OLCAck msg) of H323 side for a given media session
	    if returns false - OLCAck msg is bad - should issue CloseChannel for this channel
	*/
	void handleOLCAck(const OLCAck&);
	
	/** Handle CloseLogicalChannel message
	    if returns false - OLCAck msg is bad - should issue CloseChannel for this channel
	*/
	void handleCLC(const CLC&);

	/** Handle RequestCloseChannel message
	    if returns false - CLC msg is bad
	*/
	void handleRCC(const RCC&);

	/** Handle OLCReject message
	    if returns false - RCC msg is bad
	*/
	void handleOLCReject(const OLCReject&);
	
	/// close Incoming logical channel
	void closeInLC(unsigned);
	
	/// close Outgoing logical channel
	void closeOutLC(unsigned);
	
	/** TODO: to be investigated
	    per Evgeny's proposal:
	       cancel the effect of last handleSDP() call
	*/
	void cancelSDP();
	void confirmSDP();
	
	/// generate EvtSendSDP immediately
	void sendSDP();
	
	/** TODO: needs further investigation
	    send TCS (ignore if no SDP or SDP 0 received)
	    0 - send TCS as is
	    1 - recommended to simplify TCS
	    2 - recommended to simplify TCS significantly
	*/
	void sendTCS(int);
	
	/// register an event with CapabilityTranslator
	void add(translator::CallTranslatorEM* em);
	
	/********* Call-Back events ********/
	// TODO: 
	// right now this is just basically the list of events
	// which need to be registered with CapabilityTranslator
	// via add() method
	// exact implementation to be investigated and updated
	// per mte module studying
	
	
	/// address resovle event
	//EvtResolve;
	
	// send FS OLC
	
	/// received TCS 0
	//EvtZeroTCS;
	
	/// received SDP 0
	//EvtZeroSDP;
	
	/** prepare to send SDP message
	    it will be sent asynchroniously
	    with re-INVITE or synchroniously with 200 or ACK
	*/
	//EvtSendSDP;
	
	/// send TCS
	//router::EvtCEOTransferReq;
	
	/// send OLC
	//EvtSendOLC;
	
	/// send CLC
	//router::EvtLCOReleaseReq;
	
	/// send OLCAck
	//router::EvtLCIEstablishResp;
	
	/// send OLCReject
	//router::EvtLCIReleaseReq;
	
	/// send RCC
	//router::EvtRCORequest;
	
	/// create Incoming LC on FastConnect procedure
	//router::EvtCreatInLC;
	
	/// create Outgoing LC on FastConnect procedure
	//router::EvtCreatOutLC;
	
	/// force control channel opening after Fast Connect
	//router::EvtForceCCOpen;
	
	/** SDP parsing procedure completed, no async resovler is
	    needed
	*/
	//EvtSDPParseDone;

        Translator::CapabilityMap map_;
        translator::CallTranslatorEM* em_;
        bool gotOlc_;
        bool gotAck_;
        bool gotSDP_;
        bool gotCC_;
    };
}

#endif
