/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const EvtSendSDP_cxx_Version = 
    "$Id: EvtSendSDP.cxx,v 1.3 2001/10/24 21:26:43 icahoon Exp $";


#include <cassert>

#include "VLog.hxx"
#include "Sptr.hxx"

#include "SipSdp.hxx"
#include "AckMsg.hxx"
#include "StatusMsg.hxx"
#include "InviteMsg.hxx"

#include "EvtSendSDP.hxx"
#include "FeatureTrCallSig.hxx"
#include "TrConnectionSIP.hxx"
#include "TrConnectionSDP.hxx"
#include "CapabilityTranslator.hxx"

using namespace translator;
using Vocal::Logging::VLog;


EvtSendSDP::
EvtSendSDP(TrSMCall* sm)
    : EvtUserTrCallSigSDP(sm)
{
}


void EvtSendSDP::
inSDPH2S(FeatureTrCallSig& trcallsig)
{
    const string    fn("EvtSendSDP::inSDPH2S");
    VLog            log(fn);
    
    VDEBUG(log) << fn << VDEBUG_END(log);

    trcallsig.getSM()->getSDPConnection()->send(sdp_);

    trcallsig.setState(&PatternTrCallSig::inSending, "inSending");

    Sptr<AckMsg> sipAck = new AckMsg(*(trcallsig.status_));
    trcallsig.getSM()->getSIPConnection()->send(sipAck);

    trcallsig.setState(&PatternTrCallSig::inConnected, "inConnected");
}


void EvtSendSDP::
inSDPInvite(FeatureTrCallSig& trcallsig)
{
    const string    fn("EvtSendSDP::inSDPInvite");
    VLog            log(fn);
    
    VDEBUG(log) << fn << VDEBUG_END(log);

    trcallsig.getSM()->getSDPConnection()->send(sdp_);

    trcallsig.setState(&PatternTrCallSig::inSending, "inSending");

    Sptr<StatusMsg> sip200ok = new StatusMsg(*(trcallsig.invite_), 200);
    trcallsig.getSM()->getSIPConnection()->send(sip200ok);
    trcallsig.status_ = sip200ok;

    trcallsig.setState(&PatternTrCallSig::inWaitAck, "inWaitAck");
}


void EvtSendSDP::
inWaitAck(FeatureTrCallSig& trcallsig)
{
    const string    fn("EvtSendSDP::inWaitAck");
    VLog            log(fn);
    
    VDEBUG(log) << fn << VDEBUG_END(log);

    if (change_)
    {
        trcallsig.getSM()->getSDPConnection()->send(sdp_);
    }
}


void EvtSendSDP::
inConnected(FeatureTrCallSig& trcallsig)
{
    const string    fn("EvtSendSDP::inConnected");
    VLog            log(fn);
    
    VDEBUG(log) << fn << VDEBUG_END(log);

    if (change_)
    {
        trcallsig.getSM()->getSDPConnection()->send(sdp_);

        trcallsig.setState(&PatternTrCallSig::inSending, "inSending");

        Sptr<InviteMsg> sipinvite = new InviteMsg(*(trcallsig.status_));
        trcallsig.getSM()->getSIPConnection()->send(sipinvite);
        trcallsig.invite_ = sipinvite;

        trcallsig.setState(&PatternTrCallSig::inWait200, "inWait200");
    }
    else
    {
        // No change in SDP - no need to reInvite
    }
}


void EvtSendSDP::
inSending(FeatureTrCallSig& trcallsig)
{
    const string    fn("EvtSendSDP::inSending");
    VLog            log(fn);
    
    VDEBUG(log) << fn << VDEBUG_END(log);

    // Do nothing to prevent nested invocation of EvtSendSDP
}


void EvtSendSDP::
inOther(FeatureTrCallSig& trcallsig)
{
    const string    fn("EvtSendSDP::inOther(FeatureTrCallSig&)");
    VLog            log(fn);
    
    VDEBUG(log) << fn << VDEBUG_END(log);

    // Ignore
}


void EvtSendSDP::
inOther(FeatureTrSDP& trsdp)
{
    const string    fn("EvtSendSDP::inOther(FeatureTrSDP&)");
    VLog            log(fn);
    
    VDEBUG(log) << fn << VDEBUG_END(log);

    // send(sdp_) moved to the call signalling
    // trsdp.getSM()->getSDPConnection()->send(sdp_);
}


#ifndef DISABLE_WRITER

ostream& EvtSendSDP::
writeTo(ostream& out) const
{
    out << "EvtSendSDP";
    return out;
}

#endif
