/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const EvtTrACF_cxx_Version = 
    "$Id: EvtTrACF.cxx,v 1.22 2001/10/24 21:26:43 icahoon Exp $";


#include <assert.h>

#include "VLog.hxx"

#include "ValAssistant.h"
#include "InviteMsg.hxx"

#include "RouterInfo.hxx"
#include "Address.hxx"

#include "TSMMediator.hxx"
#include "EvtTrACF.hxx"
#include "FeatureTrCallSig.hxx"
#include "TrConnectionRas.hxx"
#include "EvtCallInit.hxx"
#include "TranslatorConsts.hxx"

using namespace translator;
using Vocal::Logging::VLog;
using Vocal::SUCCESS;


EvtTrACF::
EvtTrACF(TrRouterRas* sm)
    : EvtRasTrCallSigMessage(sm)
{
}


void EvtTrACF::
inWaitOutAdmission(FeatureTrCallSig& trcallsig)
{
    const string    fn("EvtTrACF::inWaitOutAdmission");
    VLog            log(fn);
    
    VDEBUG(log) << fn << VDEBUG_END(log);

    assert(this);
    assert(trcallsig.getSM());

    if  (   contents_->returnCode() == SUCCESS
        &&  contents_->destEndpointInfo() != 0
        )
    {
        // Confirmed

        // (depends from H.323/SIP parser)

        router::EvtCallInit init(trcallsig.getSM()->getH323Call());

        // Fill in Call data in 'init'
        Translator::Address addressTranslator;

        // From: field (take it from the invite message)
        //SipUrl sipUrl = trcallsig.invite_->getFrom().getUrl();
        //addressTranslator = sipUrl;

       Sptr<BaseUrl> baseUrl = trcallsig.invite_->getFrom().getUrl();
       Sptr<SipUrl> sipUrl;

       if(baseUrl.getPtr()!=0)
       {
         if(baseUrl->getType()==SIP_URL)
         {
               sipUrl.dynamicCast(baseUrl);
         }
         else
         {
               VEMERG(log) << fn << ": baseUrl is not SipUrl, cannot proceed..." << VEMERG_END(log);
         }
       }
       else
       {
               VEMERG(log) << fn << ": baseUrl is a null pointer, cannot proceed..." << VEMERG_END(log);
       }

        addressTranslator = *sipUrl;                                       

        init.fromAliases_ = addressTranslator.getH323Address();

        if (addressTranslator.hasIPAddress())
        {
            init.hasFromIP_ = true;
            init.fromIP_ = addressTranslator.getIPAddress();

        }
        else
        {
            init.hasFromIP_ = false;
        }

        if (addressTranslator.hasPhone())
        {
            init.fromDisplay_ = addressTranslator.getPhone();
        }
        else
        {
            init.fromDisplay_ = "Unknown number";
        }

        // To: field (take it from the gk response)
        init.toAliases_ = contents_->destEndpointInfo()->alias();
        init.hasToIP_ = true;
        init.toIP_ = contents_->destEndpointInfo()->callSignalAddress();

        // It should work this way but it doesn't - bug in sip stack?
        //trcallsig.invite_->setTo(
        //    SipTo(Address(contents_->destEndpointInfo()->callSignalAddress())));

        // Translate callID
        VDEBUG(log) << fn << ": Translating callID" << VDEBUG_END(log);
        Data dcid = trcallsig.invite_->getCallId().getCallIdData();
        std::string cid = dcid.convertString();
        VDEBUG(log) << fn << ": callID=" << cid << VDEBUG_END(log);

        if (cid.size() > 32 && cid[32] == '.')
        {
            VDEBUG(log) << fn << ": cid[32]=='.'" << VDEBUG_END(log);
            cid[32] = 0;
            unsigned char uc[256];

            if (convertToUnsigned(cid.c_str(), uc))
            {
                VDEBUG(log) << fn << ": confid converted" << VDEBUG_END(log);
                PASN_OctetString pos;
                pos.SetValue(uc, 16);
                init.confid_ = H323GloballyUniqueID(pos);
                init.hasConfid_ = true;

                if (cid.size() > 65 && cid[65] == '@')
                {
                    VDEBUG(log) << fn << ": cid[65]=='@'" << VDEBUG_END(log);
                    cid[65] = 0;

                    if (convertToUnsigned(cid.c_str() + 33, uc))
                    {
                        VDEBUG(log) << fn << ": callid converted" << VDEBUG_END(log);
                        PASN_OctetString pos;
                        pos.SetValue(uc, 16);
                        init.cid_ = H323GloballyUniqueID(pos);
                        init.hasCid_ = true;
                    }
                }
            }
        }

        trcallsig.setState(&PatternTrCallSig::inConnectingS2H, "inConnectingS2H");

        init.start();
    }
    else
    {
        // Rejected
        trcallsig.setReason(msip::respNotFound);
        trcallsig.getSM()->complete();
    }
}


void EvtTrACF::
inWaitInAdmission(FeatureTrCallSig& trcallsig)
{
    const string    fn("EvtTrACF::inWaitInAdmission");
    VLog            log(fn);
    
    VDEBUG(log) << fn << VDEBUG_END(log);

    if (contents_->srcEndpointInfo() != 0)
    {
        // Confirmed

        // (depends from H.323/SIP parser)

        trcallsig.setState(&PatternTrCallSig::inConnectingH2S, "inConnectingH2S");

        // From: field (take it from the gk response)
        Translator::Address addressTranslator;
        
        if ( contents_->srcAlias() != 0 )
        {
            addressTranslator = *contents_->srcAlias();
            
            if ( !addressTranslator.hasPhone() )
            {
                addressTranslator = contents_->srcEndpointInfo()->alias();
            }
        }
        else
        {
            addressTranslator = contents_->srcEndpointInfo()->alias();
        }
       
                
        addressTranslator.setTransport(Vocal::Transport::IPAddress());

        //SipFrom from = SipFrom(addressTranslator);
	
	Sptr<BaseUrl> baseFrom = new SipUrl(addressTranslator);
	SipFrom from = SipFrom(baseFrom);
        
        from.setDisplayName(Data(contents_->srcDisplay()));

        if (!(trcallsig.getSM()->getH323Call()->getRouterInfo()->routerMediaControl()))
        {
            VDEBUG(log) << fn << ": Setting 'CanRecvH245Address' token" << VDEBUG_END(log);
            from.setTokenDetails(CanRecvH245Address, CanRecvH245AddressValue);
        }

        trcallsig.invite_->setFrom(from);

        trcallsig.getSM()->getSIPConnection()->send(trcallsig.invite_);
    }
    else
    {
        // Rejected
        trcallsig.getSM()->complete();
    }
}


void EvtTrACF::
inOther(FeatureTrCallSig& trcallsig)
{
    const string    fn("EvtTrACF::inOther");
    VLog            log(fn);
    
    VDEBUG(log) << fn << VDEBUG_END(log);

    VWARN(log) << fn << ": Got unexpected admission confirm/reject message for call "
               << trcallsig.getSM() << VWARN_END(log);
}


#ifndef DISABLE_WRITER

ostream& EvtTrACF::
writeTo(ostream& out) const
{
    out << "EvtTrACF";
    return out;
}

#endif
