/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const EvtTrConnect_cxx_Version = 
    "$Id: EvtTrConnect.cxx,v 1.16 2001/10/24 21:26:43 icahoon Exp $";


#include <assert.h>

#include "VLog.hxx"

#include "RouterInfo.hxx"

#include "TSMMediator.hxx"
#include "EvtTrConnect.hxx"
#include "FeatureTrCallSig.hxx"
#include "CapabilityTranslator.hxx"
#include "TrConnectionSDP.hxx"
#include "TrConnectionSIP.hxx"
#include "TranslatorConsts.hxx"

using namespace translator;
using Vocal::Logging::VLog;

EvtTrConnect::
EvtTrConnect(router::RouterCallSig* tr, TrSMCall* sm)
    : mte::EvtBind11<router::EvtConnect, EvtDirectTrCallSig>(tr, sm)
{
}


void EvtTrConnect::
inConnectingS2H(FeatureTrCallSig& trcallsig)
{
    const string    fn("EvtTrConnect::inConnectingS2H");
    VLog            log(fn);
    
    VDEBUG(log) << fn << VDEBUG_END(log);

    TrSMCall* sm = trcallsig.getSM();
    bool direct =
        !(sm->getH323Call()->getRouterInfo()->routerMediaControl())
        && sm->needH245Patch()
        && sm->getH323Call()->getCallSigConnection()->haveRemoteH245();

    if (direct)
    {
        // (depends from SIP parser)

        Sptr<StatusMsg> sip200ok = new StatusMsg(*(trcallsig.invite_), 200);

        SipTo to = sip200ok->getTo();

        if (direct)
        {
            VDEBUG(log) << fn << ": Setting 'h245Address' token" << VDEBUG_END(log);
            to.setTokenDetails(
                H245Address,
                trcallsig.getSM()->getH323Call()->getCallSigConnection()->
                getRemoteH245().getIPAndPortAsString());
        }

        sip200ok->setTo(to);

        trcallsig.status_ = sip200ok;
        trcallsig.setState(&PatternTrCallSig::inWaitAck, "inWaitAck");

        trcallsig.getSM()->getSIPConnection()->send(sip200ok);
    }
    else
    {
        trcallsig.setState(&PatternTrCallSig::inSDPInvite, "inSDPInvite");
    }
}


void EvtTrConnect::
inOther(FeatureTrCallSig& trcallsig)
{
    const string    fn("EvtTrConnect::inOther");
    VLog            log(fn);
    
    VDEBUG(log) << fn << VDEBUG_END(log);

    // Ignore
    VWARN(log) << fn << ": Got unexpected Connect message for call "
               << trcallsig.getSM() << VWARN_END(log);
}


router::EvtQ931B* EvtTrConnect::
clone_impl()
{
    return new EvtTrConnect(*this);
}


void EvtTrConnect::
updateRoute()
{
    assert(getConnection());
    assert(getConnection()->getSM());

    TSMMediator& m = dynamic_cast<TSMMediator&>(
        getConnection()->getSM()->getMaster()
    );

    setDirectTarget(&(m.getSM()));
}


#ifndef DISABLE_WRITER

ostream& EvtTrConnect::
writeTo(ostream& out) const
{
    out << "EvtTrConnect";
    mte::EvtBind11<router::EvtConnect, EvtDirectTrCallSig>::writeTo(out);
    return out;
}

#endif
