
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const GSMCapability_cxx_Version = 
"$Id: GSMCapability.cxx,v 1.9 2001/01/15 21:54:22 vzubarev Exp $";
#include "ptlib.h"
#include "asner.h"
#include "GSMCapability.hxx"

using namespace Translator;

GSMCapability::GSMCapability(
                   Rate r, 
		   bool cfNoise, 
		   bool scrmbl, 
		   unsigned int aUnitSize)
  : AudioCapability(GSM_SMPLS_PER_FRAME, 
                    GSM_FRMS_PER_PACKET, 
		    GSM_CLCK_RATE),
    rate(r),
    comfortNoise(cfNoise),
    scrambled(scrmbl),
    audioUnitSize(aUnitSize) {} // 160 samples per Unit (20 ms with this clock rate); 
                                // default packet size 4 Units per Packet
                                // 8000 hz - clockrate
				// in this case audioUnitSize we interpret as 
				// packetSize


AudioCapability * GSMCapability::clone() const
{
  return new GSMCapability(rate, comfortNoise, scrambled, audioUnitSize);
}


const unsigned GSMCapability::gsmSubType[3] =
  { H245_AudioCapability::e_gsmFullRate, 
    H245_AudioCapability::e_gsmHalfRate, 
    H245_AudioCapability::e_gsmEnhancedFullRate };


unsigned GSMCapability::getSubType() const
{
  return gsmSubType[rate];
}



bool GSMCapability::getParamSendMsg(H245_AudioCapability & cap,
                                       unsigned packetSize) const
{
  cap.SetTag(gsmSubType[rate]);

  H245_GSMAudioCapability & capGSM = cap;
  PASN_Integer & aUnS = capGSM.m_audioUnitSize;
  PASN_Boolean & cN = capGSM.m_comfortNoise;
  PASN_Boolean & scr = capGSM.m_scrambled;

  aUnS = audioUnitSize;
  cN = comfortNoise;
  scr = scrambled;
  
  return true;
}


bool GSMCapability::setParamRecvMsg(const H245_AudioCapability & cap,
                                        unsigned & packetSize)
{
  switch (cap.GetTag()) {
    case H245_AudioCapability::e_gsmFullRate :
      rate = GSMCapability::FullRate;
      break;

    case H245_AudioCapability::e_gsmHalfRate :
      rate = GSMCapability::HalfRate;
      break;

    case H245_AudioCapability::e_gsmEnhancedFullRate :
      rate = GSMCapability::EnhancedFullRate;
      break;

    default :
      return false;
  }

  const H245_GSMAudioCapability & capGSM = cap;
  const PASN_Integer & aUnS = capGSM.m_audioUnitSize;
  const PASN_Boolean & cN = capGSM.m_comfortNoise;
  const PASN_Boolean & scr = capGSM.m_scrambled;

  audioUnitSize = aUnS;
  comfortNoise = cN;
  scrambled = scr;

  // the size of the packet (in units)
  packetSize = audioUnitSize;
  return true;
}
