#ifndef INCLUDED_EvtRouted_hxx
#define INCLUDED_EvtRouted_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const EvtRouted_hxx_Version = 
"$Id: EvtRouted.hxx,v 1.7 2000/12/18 23:48:39 bko Exp $";

namespace mte
{

    /// Event routed to state machine using router object
    template<class SMEvt, class ER>
    class EvtRouted: public ER, public SMEvt
    {
    public:
        typedef typename ER::TargetType TargetType;
        typedef typename SMEvt::SMType SM;
        typedef typename ER::Src Src;

        /// Construct routed event assigning pointer to event router
        EvtRouted(TargetType* pRouter = 0);
        
        /// Process event (state machine in not yet found)
        bool process(Src& src);

        /// Process event with given state machine
        virtual void process(SM& stateMachine);
    };
    
    //----------------------------
    // Inline and template methods
    //----------------------------

    template<class SMEvt, class ER>
    EvtRouted<SMEvt, ER>::
    EvtRouted(TargetType* pRouter)
        : ER(pRouter)
    {
    }

    template<class SMEvt, class ER>
    inline bool EvtRouted<SMEvt, ER>::
    process(Src& src)
    {
        assert(this);

        return route(src);
    }

    template<class SMEvt, class ER>
    void EvtRouted<SMEvt, ER>::
    process(SM& stateMachine)
    {
        assert(this);

        SMEvt::process(stateMachine);
    }

}

#endif
