#ifndef MESSAGE_HANDLER_HXX
#define MESSAGE_HANDLER_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const MessageHandler_hxx_Version = "$Id: MessageHandler.hxx,v 1.7 2001/02/07 02:35:12 hlo Exp $";



/*
    The MessageHandler class is to provide a mechanism to process the restart in 
    progress request (rsip) from an endpoint.  It could be from an endpoint
    (with the full address) or from a Gateway with the wildcard address.
 
    - The processMsg() method takes a MgcCommand to categorize into 3 cases:
    1. SINGLE = means the message from a single endpoint.
    2. WILDCARD = means the message from a Gateway with the 
       wildcard (*@<ip address>).
    3. NOMATCH = means there is no such endpoint configured in the system.
 
    - The queueInEventData() method will take care of pushing a message into
    the stack.
 
 */

#include "Sptr.hxx"
#include "cpLog.h"
#include "endpoint.hxx"

static const char* const MessageHandlerVersion = "$Id: MessageHandler.hxx,v 1.7 2001/02/07 02:35:12 hlo Exp $";

typedef enum {
    SINGLE = 0,
    WILDCARD,
    NOMATCH
} RSIPType;

class MgcCommand;
class MessageHandler
{
    public:
        /// default constructor
        MessageHandler();
        /// Destructor
        ~MessageHandler();

        /// This is to process the MgcCommand getting
        /// from the stack
        void processMsg(Sptr < MgcCommand > cmd);

    protected:

    private:

        /// verify the matching address between the two id
        int addressMatched(const string id1, const string id2);

        /// prepare and send event data of each matched endpoint
        /// into the stack
        void processWildcard();

        /// prepare and send event data into stack
        void queueInEventData();

        /// return SINGLE, WILDCARD, or NOMATCH
        /// currently support epId format: "111@aaaaaa.aaa"
        /// or "*@aaaaa.aaa" or "*@*"
        RSIPType getType(const string epId);

        Sptr < MgcCommand > itsCmd;
        EndpointMap::iterator epIter;

};

#endif // MESSAGE_HANDLER_HXX
