#ifndef CALL_AGENT_HXX
#define CALL_AGENT_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const callAgent_h_Version = "$Id: callAgent.hxx,v 1.2 2001/08/03 21:46:56 icahoon Exp $";


#include <string>
#include <map>
#include <list>

#include "MgcCommand.hxx"
#include "MgcGateway.hxx"
#include "Fifo.h"
#include "callEvent.hxx"
#include "SipMsg.hxx"

using namespace Vocal;

void queueInSipMessage(SipMsg &);
void startReceiveSipMessage(int);

#define GATEWAY_MAX 12

#define ENDPOINT_MAX 1000

#define CALL_DATA_MAX 1200

#define CP_THREAD_MAX 12
#define RECEIVE_PORT_MAX CP_THREAD_MAX
#define INPUT_Q_MAX CP_THREAD_MAX
#define Q_EVENT_DATA_MAX INPUT_Q_MAX * 200

///
typedef enum
{
    StackEvent = 0,
    InternalEvent,
    TimeoutEvent,
    SipStackEvent,
    QueueEventTypeMax
} QueueEventType;


#define CALL_ID_LEN_MAX 33

typedef struct endpoint Endpoint;
typedef struct gateway Gateway;
typedef struct callData CallData;
typedef struct cpEventData CpEventData;


#if FENCEPOST_DEBUGGING_CODE
#define FENCEPOST(x) int x;
#else
#define FENCEPOST(x)
#endif

///
typedef struct
{
    FENCEPOST(blank_0)
    int id;
    FENCEPOST(blank_1)
    QueueEventType type;
    FENCEPOST(blank_2)
    Sptr < MgcCommand > cmd;
    SipMsg* SIPcmd;   // for SIP message
    FENCEPOST(blank_3)
    Endpoint* ep;
    FENCEPOST(blank_4)
    Gateway* gw;
    FENCEPOST(blank_5)
    CallData* callData;
    FENCEPOST(blank_6)
    CallEvent internalEvent;
    FENCEPOST(blank_7)
    char callIdentifier[CALL_ID_LEN_MAX];
    FENCEPOST(blank_8)
    int featureId;
    FENCEPOST(blank_9)
}
QueueEventData;

///
typedef enum
{
    Residential = 0,
    GatewayTypeMax
} GatewayType;

///
typedef struct gateway
{
    string configFile;
    GatewayType type;

    MgcGateway* asyncStack;
    MgcGatewayHandle handle;

    Fifo < QueueEventData* > * iq;
    list < CpEventData* > * edPool;

    // The following fields are set according to config file
    string domainName;
    string gatewayName;
    int remotePort;
    string address;           // Proxy address (same as domainName if no proxy)
    int port;                 //       port    (same as remotePort if no proxy)
    int agentPort;

    int cpThread;
}
Gateway;

///
typedef struct
{
    int localPort [RECEIVE_PORT_MAX];
    int localPortCount;

    char* sipMyHost;
    int sipMyPort;
    char* sipProxyHost;
    int sipProxyPort;
    char* otherCaHost;
    int otherCaPort;

    char* provisioningHost1;
    int provisioningPort1;
    char* provisioningHost2;
    int provisioningPort2;

    int startupDelay;

    char* earlyRtp;

    int registerRetrans;

    char* dialPlan;

    char* conferenceServer;

    int threadCount;

    int endpointsMax;
    int callsMax;

    int gatewayCount;      // used as index for filling out gws[]
    int endpointCount;     // used as index for filling out eps[]
}

CallAgent;


/// Shared global data
extern CallAgent ca;
///
extern Fifo < QueueEventData* > iqs [];
///
extern int connectCount [];
///
extern Endpoint eps [];
///
typedef map < string, Endpoint* > EndpointMap ;
///
extern EndpointMap epMap;
///
extern Gateway gws [];
///
extern map < string, Gateway* > gwMap;
///
extern CallData call [];
///
extern list < CallData* > callDataList;
///
extern map < string, Endpoint* > dnMap;
///
extern QueueEventData* newQueueEventData (void);
///
extern void freeQueueEventData (QueueEventData*);
///
extern QueueEventData qEventData [];
///
extern int agentPortIndex (int);

/* CALL_AGENT_H */
#endif
