#ifndef POTSTATEENGINE_HXX
#define POTSTATEENGINE_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const potStateEngine_h_Version = "$Id: potStateEngine.hxx,v 1.2 2002/02/27 22:19:19 bko Exp $";


#include "stateEngine.hxx"
#include "endpoint.hxx"
#include "Sdp2Session.hxx"

#define UdpPortMax 9999

/**
 * Call State-Event Function Matrix for Plain Old Telephone(POT) endpoint type
 */

extern StateEngine potStateEngine;

/**
 * Call event handling functions
 * They must have the same return type and arguments
 */
extern void* initRestart (void*);
extern void* idleRestart (void*);
extern void* initResponse (void*);
extern void* callEventError (void*);
extern void* callReject (void*);
extern void* callEventIgnore (void*);
extern void* newCall (void*);
extern void* receiveCall (void*);
extern void* restartTransient (void*);
extern void* restartHalfCall (void*);
extern void* dialingComplete (void*);
extern void* connectionReady (void*);
extern void* connectionReady2 (void*);
extern void* restartOutCall (void*);
extern void* ringBack (void*);
extern void* disconnect (void*);
extern void* clearCall (void*);
extern void* featureInit(void*);
extern void* featureIdle(void*);
extern void* featureDialing(void*);
extern void* featureDigitsCollected(void*);
extern void* featureCallInitiated(void*);
extern void* featureFarEndAlerted(void*);
extern void* featureReceivedCall(void*);
extern void* featureAlerting(void*);
extern void* featureConnected(void*);
extern void* featureDisconnecting(void*);
extern void* connected (void*);
extern void* restartInCall (void*);
extern void* glare (void*);
extern void* ringing (void*);
extern void* answer (void*);
extern void* restartConnected (void*);
extern void* dtmfDigit (void*);
extern void* restartDisonnect (void*);
extern void* disconnectResponse (void*);
extern void* disconnectRelease (void*);
extern void* modifyConnect(void*);
extern void* originalCall(void*);
extern void cpStackEvent (Endpoint*, Sptr < MgcCommand > );

///
typedef enum
{
    LinePackageRequestOnHookTransition,     // Line Package requested events
    LinePackageRequestOffHookTransition,
    LinePackageRequestFlashHook,

    DTMFPackageRequestWildcardMatch,        // DTMF Package requested events

    GenericPackageSignalRingBackToneOn,     // Generic Package signal requests

    LinePackageSignalDialToneOn,            // Line Package signal requests
    LinePackageSignalRingingOn,
    LinePackageSignalBusyToneOn,
    LinePackageCallWaitingTone,
    LinePackageSignalErrorTone,

    DTMFPackageSignalDTMF_0,                // DTMF Package signal requests
    DTMFPackageSignalDTMF_1,
    DTMFPackageSignalDTMF_2,
    DTMFPackageSignalDTMF_3,
    DTMFPackageSignalDTMF_4,
    DTMFPackageSignalDTMF_5,
    DTMFPackageSignalDTMF_6,
    DTMFPackageSignalDTMF_7,
    DTMFPackageSignalDTMF_8,
    DTMFPackageSignalDTMF_9,
    DTMFPackageSignalDTMF_Hash,
    DTMFPackageSignalDTMF_Star,
    DTMFPackageSignalDTMF_A,
    DTMFPackageSignalDTMF_B,
    DTMFPackageSignalDTMF_C,
    DTMFPackageSignalDTMF_D

} RequestOrSignalEvent;

///
void addRequestOrSignalEvent( MgcCommand*, RequestOrSignalEvent );

/* POTSTATEENGINE_H */
#endif
